@@ -454,6 +454,17 @@ class CatalogServiceThriftIf : public CatalogServiceIf {
     VLOG_RPC << "GetLatestCompactions(): response=" << ThriftDebugStringNoThrow(resp);
   }
 
+  void SetEventProcessorStatus(TSetEventProcessorStatusResponse& resp,
+      const TSetEventProcessorStatusRequest& req) override {
+    VLOG_RPC << "SetEventProcessorStatus(): request=" << ThriftDebugString(req);
+    Status status = AcceptRequest(req.protocol_version);
+    if (status.ok()) {
+      status = catalog_server_->catalog()->SetEventProcessorStatus(req, &resp);
+    }
+    if (!status.ok()) LOG(ERROR) << status.GetDetail();
+    VLOG_RPC << "SetEventProcessorStatus(): response=" << ThriftDebugStringNoThrow(resp);
+  }
+
  private:
   CatalogServer* catalog_server_;
   string server_address_;
@@ -477,7 +488,7 @@ CatalogServer::CatalogServer(MetricGroup* metrics)
   : protocol_version_(CatalogServiceVersion::V2),
     thrift_iface_(new CatalogServiceThriftIf(this)),
     thrift_serializer_(FLAGS_compact_catalog_topic), metrics_(metrics),
-    is_active_(!FLAGS_enable_catalogd_ha),
+    is_active_(!FLAGS_enable_catalogd_ha), is_ha_determined_(!FLAGS_enable_catalogd_ha),
     topic_updates_ready_(false), last_sent_catalog_version_(0L),
     catalog_objects_max_version_(0L) {
   topic_processing_time_metric_ = StatsMetric<double>::CreateAndRegister(metrics,
@@ -717,6 +728,8 @@ void CatalogServer::UpdateActiveCatalogd(bool is_registration_reply,
       catalog_->RegenerateServiceId();
     }
   }
+
+  is_ha_determined_ = true;
 }
 
 bool CatalogServer::IsActive() {
@@ -1221,6 +1234,8 @@ void CatalogServer::GetCatalogOpRecords(const TGetOperationUsageResponse& respon
       document);
   document->AddMember("inflight_catalog_operations", inflight_catalog_ops,
       document->GetAllocator());
+  document->AddMember("num_inflight_catalog_ops",
+      response.in_flight_catalog_operations.size(), document->GetAllocator());
   Value finished_catalog_ops(kArrayType);
   CatalogOpListToJson(response.finished_catalog_operations, &finished_catalog_ops,
       document);
