@@ -129,9 +129,32 @@ class CatalogServer {
     return protocol_version_;
   }
 
+  /// Blocks until this catalog is ready to process requests and does not time out.
+  ///
+  /// If catalog HA is not enabled, the catalog being ready is indicated by the variable
+  /// last_sent_catalog_version_ having a value greater than 0.
+  ///
+  /// If catalog HA is enabled, waits until the active and standby catalogd daemons have
+  /// been determined. In the active daemon, this function will not return until the
+  /// last_sent_catalog_version_ variable is greater than 0. In the standby daemon, this
+  /// function returns as soon as the daemon has been determined to be the standby.
+  ///
+  /// Returns `true` or `false` indicating if this catalogd is the active catalogd.
+  /// If catalog HA is not enabled, returns `true`.
+  bool WaitForCatalogReady();
+
+  // Initializes workload management by creating or upgrading the necessary database and
+  // tables. Does not check if the current catalogd is active or if workload management is
+  // enabled.
+  Status InitWorkloadManagement();
+
  private:
   friend class CatalogServiceThriftIf;
 
+  // Defines the milliseconds to sleep during the loop that waits for the first catalog
+  // update to take place.
+  static const int WM_INIT_CHECK_SLEEP_MS = 5000;
+
   /// Protocol version of the Catalog Service.
   CatalogServiceVersion::type protocol_version_;
 
@@ -194,13 +217,17 @@ class CatalogServer {
   std::unique_ptr<Thread> catalog_metrics_refresh_thread_;
 
   /// Protects is_active_, active_catalogd_version_checker_,
-  /// catalog_update_cv_, pending_topic_updates_, catalog_objects_to/from_version_, and
-  /// last_sent_catalog_version.
+  /// catalog_update_cv_, pending_topic_updates_, catalog_objects_to/from_version_,
+  /// last_sent_catalog_version, and is_ha_determined_.
   std::mutex catalog_lock_;
 
   /// Set to true if this catalog instance is active.
   bool is_active_;
 
+  /// Set to true after active catalog has been determined. Will be true if catalog ha
+  /// is not enabled.
+  bool is_ha_determined_;
+
   /// Object to track the version of received active catalogd.
   boost::scoped_ptr<ActiveCatalogdVersionChecker> active_catalogd_version_checker_;
 
@@ -364,6 +391,10 @@ class CatalogServer {
   /// value) pairs, one for each Hadoop configuration value.
   void HadoopVarzHandler(const Webserver::WebRequest& req,
       rapidjson::Document* document);
+
+  /// Indicates if the catalog has finished initialization. If last_sent_catalog_version_
+  /// is greater than 0, returns `true`, otherwise returns `false`.
+  inline bool IsCatalogInitialized();
 };
 
 }
