@@ -46,6 +46,8 @@ DEFINE_int32(max_nonhdfs_partitions_parallel_load, 20,
 DEFINE_int32(initial_hms_cnxn_timeout_s, 120,
     "Number of seconds catalogd will wait to establish an initial connection to the HMS "
     "before exiting.");
+DEFINE_bool(enable_reading_puffin_stats, false, "Impala will only read Iceberg Puffin "
+    "stats files if this flag is set to true.");
 
 Catalog::Catalog() {
   JniMethodDescriptor methods[] = {
@@ -74,6 +76,7 @@ Catalog::Catalog() {
     {"getNullPartitionName", "()[B", &get_null_partition_name_id_},
     {"getLatestCompactions", "([B)[B", &get_latest_compactions_id_},
     {"getAllHadoopConfigs", "()[B", &get_hadoop_configs_id_},
+    {"setEventProcessorStatus", "([B)[B", &set_event_processor_status_id_},
   };
 
   JNIEnv* jni_env = JniUtil::GetJNIEnv();
@@ -236,3 +239,8 @@ Status Catalog::GetLatestCompactions(
     const TGetLatestCompactionsRequest& req, TGetLatestCompactionsResponse* resp) {
   return JniUtil::CallJniMethod(catalog_, get_latest_compactions_id_, req, resp);
 }
+
+Status Catalog::SetEventProcessorStatus(
+    const TSetEventProcessorStatusRequest& req, TSetEventProcessorStatusResponse* resp) {
+  return JniUtil::CallJniMethod(catalog_, set_event_processor_status_id_, req, resp);
+}
