@@ -971,11 +971,11 @@ CodegenAnyVal CodegenAnyVal::CreateFromReadWriteInfo(
 
     llvm::Value* ptr_null = llvm::Constant::getNullValue(ptr->getType());
     llvm::PHINode* ptr_phi = LlvmCodeGen::CreateBinaryPhiNode(builder, ptr, ptr_null,
-        non_null_incoming_block, read_write_info.null_block());
+        non_null_incoming_block, read_write_info.null_block(), "ptr_phi");
 
     llvm::Value* len_null = llvm::ConstantInt::get(len->getType(), 0);
     llvm::PHINode* len_phi = LlvmCodeGen::CreateBinaryPhiNode(builder, len, len_null,
-        non_null_incoming_block, read_write_info.null_block());
+        non_null_incoming_block, read_write_info.null_block(), "len_phi");
 
     result.SetPtr(ptr_phi);
     result.SetLen(len_phi);
@@ -984,22 +984,22 @@ CodegenAnyVal CodegenAnyVal::CreateFromReadWriteInfo(
         read_write_info.GetTimeAndDate().time_of_day->getType(), 0);
     llvm::PHINode* time_of_day_phi = LlvmCodeGen::CreateBinaryPhiNode(builder,
         read_write_info.GetTimeAndDate().time_of_day, time_of_day_null,
-        non_null_incoming_block, read_write_info.null_block());
+        non_null_incoming_block, read_write_info.null_block(), "time_of_day_phi");
 
     llvm::Value* date_null = llvm::ConstantInt::get(
         read_write_info.GetTimeAndDate().date->getType(), 0);
-    llvm::PHINode* date_phi = LlvmCodeGen::CreateBinaryPhiNode(builder,
-        read_write_info.GetTimeAndDate().date, date_null, non_null_incoming_block,
-        read_write_info.null_block());
+    llvm::PHINode* date_phi =
+        LlvmCodeGen::CreateBinaryPhiNode(builder, read_write_info.GetTimeAndDate().date,
+            date_null, non_null_incoming_block, read_write_info.null_block(), "date_phi");
 
     result.SetTimeOfDay(time_of_day_phi);
     result.SetDate(date_phi);
   } else {
     llvm::Value* null = llvm::Constant::getNullValue(
         read_write_info.GetSimpleVal()->getType());
-    llvm::PHINode* val_phi = LlvmCodeGen::CreateBinaryPhiNode(builder,
-        read_write_info.GetSimpleVal(), null, non_null_incoming_block,
-        read_write_info.null_block());
+    llvm::PHINode* val_phi =
+        LlvmCodeGen::CreateBinaryPhiNode(builder, read_write_info.GetSimpleVal(), null,
+            non_null_incoming_block, read_write_info.null_block(), "val_phi");
 
     result.SetVal(val_phi);
   }
@@ -1065,29 +1065,28 @@ void CodegenAnyVal::StructChildToReadWriteInfo(
   // 'AnyVal's). As this code deals with values in 'AnyVal's, for BOOLEANS we use i8,
   // which is the LLVM type corresponding to 'bool', and for other types we use the slot
   // type.
-  llvm::Type* child_type = type.type == TYPE_BOOLEAN ?
+  llvm::Type* child_type = type.type == TYPE_BOOLEAN || type.type == TYPE_CHAR ?
     codegen->i8_type() : slot_type;
   llvm::Value* cast_child_ptr = builder->CreateBitCast(child_ptr,
       child_type->getPointerTo(), "cast_child_ptr");
 
   switch (type.type) {
-    case TYPE_STRING:
-    case TYPE_VARCHAR:
     case TYPE_CHAR: {
+      llvm::Value* len = codegen->GetI32Constant(type.len);
+      read_write_info->SetPtrAndLen(cast_child_ptr, len);
+      break;
+    }
+    case TYPE_STRING:
+    case TYPE_VARCHAR: {
       llvm::Function* str_ptr_fn = codegen->GetFunction(
             IRFunction::STRING_VALUE_PTR, false);
       llvm::Value* ptr = builder->CreateCall(str_ptr_fn,
             llvm::ArrayRef<llvm::Value*>({cast_child_ptr}), "ptr");
 
-      llvm::Value* len;
-      if (type.type == TYPE_CHAR) {
-        len = codegen->GetI32Constant(type.len);
-      } else {
-        llvm::Function* str_len_fn = codegen->GetFunction(
-            IRFunction::STRING_VALUE_LEN, false);
-        len = builder->CreateCall(str_len_fn,
-            llvm::ArrayRef<llvm::Value*>({cast_child_ptr}), "len");
-      }
+      llvm::Function* str_len_fn = codegen->GetFunction(
+          IRFunction::STRING_VALUE_LEN, false);
+      llvm::Value* len = builder->CreateCall(str_len_fn,
+          llvm::ArrayRef<llvm::Value*>({cast_child_ptr}), "len");
       read_write_info->SetPtrAndLen(ptr, len);
       break;
     }
