@@ -310,7 +310,7 @@ DEFINE_string(blacklisted_dbs, "sys,information_schema",
     " create, or drop databases which are blacklisted.");
 DEFINE_string(blacklisted_tables, "",
     "Comma separated full names (in format: <db>.<table>) of blacklisted tables. "
-    "Configure which tables to be skipped for loading (in startup and reseting metadata "
+    "Configure which tables to be skipped for loading (in startup and resetting metadata "
     "of the table). Users can't access, create, or drop tables which are blacklisted");
 
 DEFINE_double_hidden(invalidate_tables_gc_old_gen_full_threshold, 0.6, "The threshold "
@@ -352,9 +352,9 @@ DEFINE_int32(num_check_authorization_threads, 1,
     "The number of threads used to check authorization for the user when executing show "
     "tables/databases. This configuration is applicable only when authorization is "
     "enabled. A value of 1 disables multi-threaded execution for checking authorization."
-    "However, a small value of larger than 1 may limit the parallism of FE requests when "
-    "checking authorization with a high concurrency. The value must be in the range of "
-    "1 to 128.");
+    "However, a small value of larger than 1 may limit the parallelism of FE requests "
+    "when checking authorization with a high concurrency. The value must be in the range "
+    "of 1 to 128.");
 
 DEFINE_bool_hidden(use_customized_user_groups_mapper_for_ranger, false,
     "If true, use the customized user-to-groups mapper when performing authorization via"
@@ -417,14 +417,19 @@ DEFINE_int64(update_catalogd_rpc_resend_interval_ms, 100, "(Advanced) Interval (
     "statestore has failed to send the RPC to the subscriber.");
 
 DEFINE_int32(iceberg_reload_new_files_threshold, 100, "(Advanced) If during a table "
-    "refresh the number of new files are greater than this, catalogd will completely "
-    "reload all file metadata. If number of new files are less or equal to this, "
-    "catalogd will only load the metadata of the newly added files.");
+    "refresh the number of new files are greater than this, catalogd will use a "
+    "recursive file listing to load file metadata. If number of new files are less or "
+    "equal to this, catalogd will load the file metadata one by one.");
 
 DEFINE_bool(iceberg_allow_datafiles_in_table_location_only, true, "If true, Impala "
     "does not allow Iceberg data file locations outside of the table directory during "
     "reads");
 
+DEFINE_bool(iceberg_always_allow_merge_on_read_operations, false, "Impala can only "
+    "write delete files with 'merge-on-read'. If this flag is true, Impala allows "
+    "executing DELETE, UPDATE and MERGE operations on Iceberg tables even if the table "
+    "property is 'copy-on-write'.");
+
 // Host and port of Statestore Service
 DEFINE_string(state_store_host, "localhost",
     "hostname where StatestoreService is running");
