@@ -505,6 +505,7 @@ Status GroupingAggregator::AddBatchStreaming(
   }
 
   TPrefetchMode::type prefetch_mode = state->query_options().prefetch_mode;
+  int64_t num_row_out_batch_old = out_batch->num_rows();
   GroupingAggregatorConfig::AddBatchStreamingImplFn fn
       = add_batch_streaming_impl_fn_.load();
   if (fn != nullptr) {
@@ -515,8 +516,8 @@ Status GroupingAggregator::AddBatchStreaming(
         child_batch, out_batch, ht_ctx_.get(), remaining_capacity));
   }
   *eos = (streaming_idx_ == 0);
-
-  num_rows_returned_ += out_batch->num_rows();
+  DCHECK_GE(out_batch->num_rows(), num_row_out_batch_old);
+  num_rows_returned_ += out_batch->num_rows() - num_row_out_batch_old;
   COUNTER_SET(num_passthrough_rows_, num_rows_returned_);
   return Status::OK();
 }
@@ -576,7 +577,7 @@ int GroupingAggregator::GroupingExprsVarlenSize() {
   for (int expr_idx : string_grouping_exprs_) {
     StringValue* sv = reinterpret_cast<StringValue*>(ht_ctx_->ExprValue(expr_idx));
     // Avoid branching by multiplying length by null bit.
-    varlen_size += sv->len * !ht_ctx_->ExprValueNull(expr_idx);
+    varlen_size += sv->Len() * !ht_ctx_->ExprValueNull(expr_idx);
   }
   return varlen_size;
 }
@@ -603,9 +604,10 @@ void GroupingAggregator::CopyGroupingValues(
     // ptr and len were already copied to the fixed-len part of string value
     StringValue* sv = reinterpret_cast<StringValue*>(
         intermediate_tuple->GetSlot(slot_desc->tuple_offset()));
-    memcpy(buffer, sv->ptr, sv->len);
-    sv->ptr = reinterpret_cast<char*>(buffer);
-    buffer += sv->len;
+    if (sv->IsSmall()) continue;
+    memcpy(buffer, sv->Ptr(), sv->Len());
+    sv->SetPtr(reinterpret_cast<char*>(buffer));
+    buffer += sv->Len();
   }
 }
 
