@@ -103,6 +103,10 @@ struct HdfsFileDesc {
   /// Whether file is erasure coded.
   bool is_erasure_coded = false;
 
+  /// Fragment instance assignment from the scheduler (used for maintaining
+  /// deterministic assignments when that mode is enabled).
+  TUniqueId fragment_instance_id;
+
   /// Some useful typedefs for creating HdfsFileDesc related data structures.
   /// This is a pair for partition ID and filename which uniquely identifies a file.
   typedef pair<int64_t, std::string> PartitionFileKey;
@@ -180,7 +184,7 @@ class ScanRangeSharedState {
   /// for which initial scan ranges need to be issued. Returns nullptr if the instance
   /// has not been assigned anything.
   std::vector<HdfsFileDesc*>* GetFilesForIssuingScanRangesForInstance(
-      const TUniqueId fragment_instance_id) {
+      const TUniqueId& fragment_instance_id) {
     auto it = file_assignment_per_instance_.find(fragment_instance_id);
     if (it == file_assignment_per_instance_.end()) return nullptr;
     return &it->second;
@@ -781,10 +785,6 @@ class HdfsScanNodeBase : public ScanNode {
   RuntimeProfile::Counter* hdfs_open_file_timer_ = nullptr;
   RuntimeProfile::SummaryStatsCounter* initial_range_ideal_reservation_stats_ = nullptr;
   RuntimeProfile::SummaryStatsCounter* initial_range_actual_reservation_stats_ = nullptr;
-  RuntimeProfile::SummaryStatsCounter* compressed_bytes_read_per_column_counter_ =
-      nullptr;
-  RuntimeProfile::SummaryStatsCounter* uncompressed_bytes_read_per_column_counter_ =
-      nullptr;
 
   /// HDFS read thread concurrency bucket: bucket[i] refers to the number of sample
   /// taken where there are i concurrent hdfs read thread running. Created in Open().
@@ -819,6 +819,12 @@ class HdfsScanNodeBase : public ScanNode {
   /// Pointer to the scan range related state that is shared across all node instances.
   ScanRangeSharedState* shared_state_ = nullptr;
 
+  /// Whether mt_dop uses deterministic scan range assignment
+  /// If true, each fragment instance has its own list of scan ranges.
+  /// If false, the fragment instances get scan ranges from the shared queue.
+  /// Not used for mt_dop=0.
+  bool deterministic_scanrange_assignment_;
+
   /// Utility class for handling file metadata.
   FileMetadataUtils file_metadata_utils_;
 
