@@ -363,8 +363,10 @@ Status HdfsOrcScanner::Open(ScannerContext* context) {
 
   bool is_table_full_acid = scan_node_->hdfs_table()->IsTableFullAcid();
   schema_resolver_.reset(
-      new OrcSchemaResolver(*scan_node_->hdfs_table(), &reader_->getType(), filename(),
-          is_table_full_acid, state_->query_options().orc_schema_resolution));
+      new OrcSchemaResolver(*scan_node_->hdfs_table(), file_metadata_utils_,
+          &reader_->getType(), filename(), is_table_full_acid,
+          state_->query_options().orc_schema_resolution));
+  RETURN_IF_ERROR(schema_resolver_->Init());
   bool is_file_full_acid = schema_resolver_->HasFullAcidV2Schema();
   acid_original_file_ = is_table_full_acid && !is_file_full_acid;
   if (is_table_full_acid) {
@@ -1003,6 +1005,7 @@ Status HdfsOrcScanner::AssembleRows(RowBatch* row_batch) {
         end_of_stripe_ = true;
         return Status::OK();
       }
+      COUNTER_ADD(scan_node_->rows_read_counter(), orc_root_batch_->numElements);
       num_rows_read += orc_root_batch_->numElements;
     }
 
@@ -1011,7 +1014,6 @@ Status HdfsOrcScanner::AssembleRows(RowBatch* row_batch) {
     continue_execution &= !scan_node_->ReachedLimitShared() && !context_->cancelled();
   }
   stripe_rows_read_ += num_rows_read;
-  COUNTER_ADD(scan_node_->rows_read_counter(), num_rows_read);
   // Merge Scanner-local counter into HdfsScanNode counter and reset.
   COUNTER_ADD(scan_node_->collection_items_read_counter(), coll_items_read_counter_);
   coll_items_read_counter_ = 0;
