@@ -21,12 +21,6 @@
 
 namespace impala {
 
-void CollectionColumnReader::Close(RowBatch* row_batch) {
-  for (ParquetColumnReader* child_reader : children_) {
-    child_reader->Close(row_batch);
-  }
-}
-
 bool CollectionColumnReader::NextLevels() {
   DCHECK(!children_.empty());
   DCHECK_LE(rep_level_, new_collection_rep_level());
@@ -58,7 +52,7 @@ bool CollectionColumnReader::ReadValue(MemPool* pool, Tuple* tuple) {
       CollectionValue* slot = tuple->GetCollectionSlot(tuple_offset_);
       *slot = CollectionValue();
     } else {
-      tuple->SetNull(null_indicator_offset_);
+      SetNullSlot(tuple);
     }
     return CollectionColumnReader::NextLevels();
   }
@@ -84,9 +78,12 @@ bool CollectionColumnReader::ReadValueBatch(MemPool* pool, int max_values,
       continue_execution = NextLevels();
       continue;
     }
-    // Fill in position slot if applicable
-    if (pos_slot_desc_ != nullptr) {
-      ReadPositionNonBatched(tuple->GetBigIntSlot(pos_slot_desc()->tuple_offset()));
+    // Fill in position slots if applicable
+    if (pos_slot_desc() != nullptr) {
+      ReadItemPositionNonBatched(tuple->GetBigIntSlot(pos_slot_desc()->tuple_offset()));
+    } else if (file_pos_slot_desc() != nullptr) {
+      ReadFilePositionNonBatched(
+          tuple->GetBigIntSlot(file_pos_slot_desc()->tuple_offset()));
     }
     continue_execution = ReadValue(pool, tuple);
     ++val_count;
