@@ -214,6 +214,12 @@ class ParquetColumnReader {
     return pos_slot_desc_ != nullptr || file_pos_slot_desc_ != nullptr;
   }
 
+  void ReadItemPositionBatched(int16_t rep_level, int64_t* pos) {
+    // Reset position counter if we are at the start of a new parent collection.
+    if (rep_level <= max_rep_level() - 1) pos_current_value_ = 0;
+    *pos = pos_current_value_++;
+  }
+
  protected:
   HdfsParquetScanner* parent_;
   const SchemaNode& node_;
@@ -225,6 +231,9 @@ class ParquetColumnReader {
   const SlotDescriptor* pos_slot_desc_ = nullptr;
   const SlotDescriptor* file_pos_slot_desc_ = nullptr;
 
+  /// Index within the file of the first row in the row group.
+  int64_t row_group_first_row_ = 0;
+
   /// The next value to write into the position slot, if there is one. 64-bit int because
   /// the pos slot is always a BIGINT Set to ParquetLevel::INVALID_POS when this column
   /// reader does not have a current rep and def level (i.e. before the first NextLevels()
@@ -298,7 +307,9 @@ class BaseScalarColumnReader : public ParquetColumnReader {
       const SlotDescriptor* slot_desc)
     : ParquetColumnReader(parent, node, slot_desc),
       col_chunk_reader_(parent, node.element->name,
-        slot_desc != nullptr ? slot_desc->id() : -1, PageReaderValueMemoryType()) {
+        slot_desc != nullptr ? slot_desc->id() : -1, PageReaderValueMemoryType(),
+        max_rep_level() > 0,
+        max_def_level() > 0) {
     DCHECK_GE(node_.col_idx, 0) << node_.DebugString();
   }
 
@@ -376,6 +387,12 @@ class BaseScalarColumnReader : public ParquetColumnReader {
   template <bool ADVANCE_REP_LEVEL>
   bool NextLevels();
 
+  /// Returns file position of current row ('current_row_' is the index of the row
+  /// within the row group).
+  int64_t FilePositionOfCurrentRow() const {
+    return row_group_first_row_ + current_row_;
+  }
+
  protected:
   // Friend parent scanner so it can perform validation (e.g. ValidateEndOfRowGroup())
   friend class HdfsParquetScanner;
@@ -398,7 +415,11 @@ class BaseScalarColumnReader : public ParquetColumnReader {
   ParquetLevelDecoder rep_levels_{false};
 
   /// Page encoding for values of the current data page. Cached here for perf. Set in
-  /// InitDataPage().
+  /// InitDataPageDecoders().
+  ///
+  /// Parquet V2 deprecated PLAIN_DICTIONARY and RLE_DICTIONARY should be used instead.
+  /// In this member PLAIN_DICTIONARY is used both for pages with PLAIN_DICTIONARY and
+  /// RLE_DICTIONARY as the encodings mean the same.
   parquet::Encoding::type page_encoding_ = parquet::Encoding::PLAIN_DICTIONARY;
 
   /// Num values remaining in the current data page
@@ -414,7 +435,8 @@ class BaseScalarColumnReader : public ParquetColumnReader {
   /// Metadata for the column for the current row group.
   const parquet::ColumnMetaData* metadata_ = nullptr;
 
-  /// Index of the current top-level row. It is updated together with the rep/def levels.
+  /// Index of the current top-level row within the row group. It is updated together
+  /// with the rep/def levels.
   /// When updated, and its value is N, it means that we already processed the Nth row
   /// completely, hence the initial value is '-1', because '0' would mean that we already
   /// processed the first (zeroeth) row.
@@ -509,7 +531,10 @@ class BaseScalarColumnReader : public ParquetColumnReader {
   /// decompressed data page. Decoders can initialize state from here. The caller must
   /// validate the input such that 'size' is non-negative and that 'data' has at least
   /// 'size' bytes remaining.
-  virtual Status InitDataPage(uint8_t* data, int size) = 0;
+  virtual Status InitDataDecoder(uint8_t* data, int size) = 0;
+
+  /// Initializes decoders for rep/def levels and data.
+  Status InitDataPageDecoders(const ParquetColumnChunkReader::DataPageInfo& page_info);
 
   ParquetColumnChunkReader::ValueMemoryType PageReaderValueMemoryType() {
     if (slot_desc_ == nullptr) {
@@ -590,7 +615,7 @@ class BaseScalarColumnReader : public ParquetColumnReader {
   int64_t LastRowIdxInCurrentPage() const {
     DCHECK(!candidate_data_pages_.empty());
     int64_t num_rows =
-        parent_->file_metadata_.row_groups[parent_->group_idx_].num_rows;
+        parent_->file_metadata_.row_groups[parent_->row_group_idx_].num_rows;
     // Find the next valid page.
     int page_idx = candidate_data_pages_[candidate_page_idx_] + 1;
     while (page_idx < offset_index_.page_locations.size()) {
@@ -685,7 +710,7 @@ inline void ParquetColumnReader::ReadFilePositionNonBatched(int64_t* file_pos) {
   DCHECK_GE(def_level_, 0);
   DCHECK_GE(def_level_, def_level_of_immediate_repeated_ancestor()) <<
       "Caller should have called NextLevels() until we are ready to read a value";
-  *file_pos = LastProcessedRow() + 1;
+  *file_pos = row_group_first_row_ + LastProcessedRow() + 1;
 }
 
 // Change 'val_count' to zero to exercise IMPALA-5197. This verifies the error handling
