@@ -26,16 +26,16 @@
 #include <gutil/strings/numbers.h>
 #include <gutil/strings/substitute.h>
 
+#include "common/names.h"
 #include "exprs/anyval-util.h"
 #include "exprs/cast-format-expr.h"
 #include "exprs/decimal-functions.h"
+#include "gutil/strings/numbers.h"
 #include "runtime/string-value.inline.h"
 #include "runtime/timestamp-value.h"
 #include "runtime/timestamp-value.inline.h"
 #include "util/string-parser.h"
 
-#include "common/names.h"
-
 using namespace impala;
 using namespace impala_udf;
 using namespace datetime_parse_util;
@@ -59,6 +59,27 @@ const int MAX_BOOLEAN_CHARS = 1;
 
 namespace {
 
+template <class T>
+constexpr const char* TypeToName() {
+  if constexpr (std::is_same_v<T, int8_t>) {
+    return "TINYINT";
+  } else if constexpr (std::is_same_v<T, int16_t>) {
+    return "SMALLINT";
+  } else if constexpr (std::is_same_v<T, int32_t>) {
+    return "INT";
+  } else if constexpr (std::is_same_v<T, int64_t>) {
+    return "BIGINT";
+  } else if constexpr (std::is_same_v<T, float>) {
+    return "FLOAT";
+  } else if constexpr (std::is_same_v<T, double>) {
+    return "DOUBLE";
+  } else {
+    // This function doesn't support other types.
+    static_assert(!std::is_same_v<T, T>);
+    return nullptr;
+  }
+}
+
 // This struct is used as a helper in the validation of casts. For a cast from 'FROM_TYPE'
 // to 'TO_TYPE' it provides compile time constants about what type of conversion it is.
 // These constants are used in the template specifications of the 'Validate()' function to
@@ -158,14 +179,20 @@ bool Validate(FROM_TYPE from, FunctionContext* ctx) {
   }
 
   if (UNLIKELY(!is_ok)) {
+    constexpr const char* FROM_TYPE_NAME = TypeToName<FROM_TYPE>();
+    constexpr const char* TO_TYPE_NAME = TypeToName<TO_TYPE>();
+    string err;
     if (std::isnan(from)) {
-      ctx->SetError("NaN value cannot be converted to integer type.");
+      err = Substitute("NaN value of type $0 cannot be converted to $1.",
+          FROM_TYPE_NAME, TO_TYPE_NAME);
     } else if (!std::isfinite(from)) {
-      ctx->SetError("Non-finite value cannot be converted to integer type.");
+      err = Substitute("Non-finite value of type $0 cannot be converted to $1.",
+          FROM_TYPE_NAME, TO_TYPE_NAME);
     } else {
-      ctx->SetError("Out-of-range floating point value cannot be converted to "
-          "integer type.");
+      err = Substitute("Converting value $0 of type $1 to $2 failed, "
+          "value out of range for destination type.", from, FROM_TYPE_NAME, TO_TYPE_NAME);
     }
+    ctx->SetError(err.c_str());
   }
 
   return is_ok;
@@ -194,8 +221,10 @@ bool Validate(FROM_TYPE from, FunctionContext* ctx) {
   const bool is_ok = in_range || std::isnan(from) || !std::isfinite(from);
 
   if (UNLIKELY(!is_ok)) {
-      ctx->SetError(
-          "Out-of-range double value cannot be converted to float.");
+    const string err = Substitute("Converting value $0 of type $1 to $2 failed, "
+        "value out of range for destination type.", from, TypeToName<FROM_TYPE>(),
+        TypeToName<TO_TYPE>());
+    ctx->SetError(err.c_str());
   }
 
   return is_ok;
@@ -309,31 +338,28 @@ CAST_EXACT_NUMERIC_TO_STRING(SmallIntVal, MAX_SMALLINT_CHARS, FastInt32ToBufferL
 CAST_EXACT_NUMERIC_TO_STRING(IntVal, MAX_INT_CHARS, FastInt32ToBufferLeft);
 CAST_EXACT_NUMERIC_TO_STRING(BigIntVal, MAX_BIGINT_CHARS, FastInt64ToBufferLeft);
 
-
-#define CAST_FLOAT_TO_STRING(float_type, format) \
-  StringVal CastFunctions::CastToStringVal(FunctionContext* ctx, const float_type& val) { \
-    if (val.is_null) return StringVal::null(); \
-    /* val.val could be -nan, return "nan" instead */ \
-    if (std::isnan(val.val)) return StringVal("nan"); \
-    /* Add 1 to MAX_FLOAT_CHARS since snprintf adds a trailing '\0' */ \
-    StringVal sv(ctx, MAX_FLOAT_CHARS + 1); \
-    if (UNLIKELY(sv.is_null)) { \
-      DCHECK(!ctx->impl()->state()->GetQueryStatus().ok()); \
-      return sv; \
-    } \
-    sv.len = snprintf(reinterpret_cast<char*>(sv.ptr), sv.len, format, val.val); \
-    DCHECK_GT(sv.len, 0); \
-    DCHECK_LE(sv.len, MAX_FLOAT_CHARS); \
-    AnyValUtil::TruncateIfNecessary(ctx->GetReturnType(), &sv); \
-    return sv; \
+#define CAST_FLOAT_TO_STRING(float_type, convert_method, buffer_size)          \
+  StringVal CastFunctions::CastToStringVal(                                    \
+      FunctionContext* ctx, const float_type& val) {                           \
+    if (val.is_null) return StringVal::null();                                 \
+    /* val.val could be -nan, return "nan" instead */                          \
+    if (std::isnan(val.val)) return StringVal("nan");                          \
+    StringVal sv(ctx, buffer_size);                                            \
+    if (UNLIKELY(sv.is_null)) {                                                \
+      DCHECK(!ctx->impl()->state()->GetQueryStatus().ok());                    \
+      return sv;                                                               \
+    }                                                                          \
+    sv.len = strlen(convert_method(val.val, reinterpret_cast<char*>(sv.ptr))); \
+    DCHECK_GT(sv.len, 0);                                                      \
+    DCHECK_LE(sv.len, MAX_FLOAT_CHARS);                                        \
+    AnyValUtil::TruncateIfNecessary(ctx->GetReturnType(), &sv);                \
+    return sv;                                                                 \
   }
 
-// Floats have up to 9 significant digits, doubles up to 17
-// (see http://en.wikipedia.org/wiki/Single-precision_floating-point_format
-// and http://en.wikipedia.org/wiki/Double-precision_floating-point_format)
-CAST_FLOAT_TO_STRING(FloatVal, "%.9g");
-CAST_FLOAT_TO_STRING(DoubleVal, "%.17g");
-
+// Convert a double or float to a string and produce the exact same original precision.
+// See gutil/strings/numbers.h and gutil/strings/numbers.cc for more details.
+CAST_FLOAT_TO_STRING(FloatVal, FloatToBuffer, kFloatToBufferSize);
+CAST_FLOAT_TO_STRING(DoubleVal, DoubleToBuffer, kDoubleToBufferSize);
 
 StringVal CastFunctions::CastToStringVal(FunctionContext* ctx, const TimestampVal& val) {
   DCHECK(ctx != nullptr);
