@@ -31,6 +31,7 @@
 #include "codegen/llvm-codegen.h"
 #include "exprs/anyval-util.h"
 #include "exprs/scalar-expr-evaluator.h"
+#include "runtime/fragment-state.h"
 #include "runtime/hdfs-fs-cache.h"
 #include "runtime/lib-cache.h"
 #include "runtime/runtime-state.h"
@@ -52,8 +53,8 @@ using std::pair;
 ScalarFnCall::ScalarFnCall(const TExprNode& node)
   : ScalarExpr(node),
     vararg_start_idx_(node.__isset.vararg_start_idx ? node.vararg_start_idx : -1),
-    prepare_fn_(NULL),
-    close_fn_(NULL),
+    prepare_fn_(),
+    close_fn_(),
     scalar_fn_(NULL) {
   DCHECK_NE(fn_.binary_type, TFunctionBinaryType::JAVA);
 }
@@ -61,17 +62,17 @@ ScalarFnCall::ScalarFnCall(const TExprNode& node)
 Status ScalarFnCall::LoadPrepareAndCloseFn(LlvmCodeGen* codegen) {
   if (fn_.scalar_fn.__isset.prepare_fn_symbol) {
     RETURN_IF_ERROR(GetFunction(codegen, fn_.scalar_fn.prepare_fn_symbol,
-        reinterpret_cast<void**>(&prepare_fn_)));
+        &prepare_fn_));
   }
   if (fn_.scalar_fn.__isset.close_fn_symbol) {
     RETURN_IF_ERROR(GetFunction(codegen, fn_.scalar_fn.close_fn_symbol,
-        reinterpret_cast<void**>(&close_fn_)));
+        &close_fn_));
   }
   return Status::OK();
 }
 
 Status ScalarFnCall::Init(
-    const RowDescriptor& desc, bool is_entry_point, RuntimeState* state) {
+    const RowDescriptor& desc, bool is_entry_point, FragmentState* state) {
   // Initialize children first.
   RETURN_IF_ERROR(ScalarExpr::Init(desc, is_entry_point, state));
 
@@ -213,12 +214,13 @@ Status ScalarFnCall::OpenEvaluator(FunctionContext::FunctionStateScope scope,
   }
   fn_ctx->impl()->SetNonConstantArgs(move(non_constant_args));
 
-  if (prepare_fn_ != nullptr) {
+  const impala_udf::UdfPrepare prepare_fn = prepare_fn_.load();
+  if (prepare_fn != nullptr) {
     if (scope == FunctionContext::FRAGMENT_LOCAL) {
-      prepare_fn_(fn_ctx, FunctionContext::FRAGMENT_LOCAL);
+      prepare_fn(fn_ctx, FunctionContext::FRAGMENT_LOCAL);
       if (fn_ctx->has_error()) return Status(fn_ctx->error_msg());
     }
-    prepare_fn_(fn_ctx, FunctionContext::THREAD_LOCAL);
+    prepare_fn(fn_ctx, FunctionContext::THREAD_LOCAL);
     if (fn_ctx->has_error()) return Status(fn_ctx->error_msg());
   }
 
@@ -228,11 +230,12 @@ Status ScalarFnCall::OpenEvaluator(FunctionContext::FunctionStateScope scope,
 void ScalarFnCall::CloseEvaluator(FunctionContext::FunctionStateScope scope,
     RuntimeState* state, ScalarExprEvaluator* eval) const {
   DCHECK_GE(fn_ctx_idx_, 0);
-  if (close_fn_ != NULL) {
+  const impala_udf::UdfClose close_fn = close_fn_.load();
+  if (close_fn != NULL) {
     FunctionContext* fn_ctx = eval->fn_context(fn_ctx_idx_);
-    close_fn_(fn_ctx, FunctionContext::THREAD_LOCAL);
+    close_fn(fn_ctx, FunctionContext::THREAD_LOCAL);
     if (scope == FunctionContext::FRAGMENT_LOCAL) {
-      close_fn_(fn_ctx, FunctionContext::FRAGMENT_LOCAL);
+      close_fn(fn_ctx, FunctionContext::FRAGMENT_LOCAL);
     }
   }
   ScalarExpr::CloseEvaluator(scope, state, eval);
@@ -329,15 +332,7 @@ Status ScalarFnCall::GetCodegendComputeFnImpl(LlvmCodeGen* codegen, llvm::Functi
     llvm::Function* child_fn = NULL;
     vector<llvm::Value*> child_fn_args;
     // Set 'child_fn' to the codegen'd function, sets child_fn == NULL if codegen fails
-    Status status = children_[i]->GetCodegendComputeFn(codegen, false, &child_fn);
-    if (UNLIKELY(!status.ok())) {
-      DCHECK(child_fn == NULL);
-      // Set 'child_fn' to the interpreted function
-      child_fn = GetStaticGetValWrapper(children_[i]->type(), codegen);
-      // First argument to interpreted function is children_[i]
-      llvm::Type* expr_ptr_type = codegen->GetStructPtrType<ScalarExpr>();
-      child_fn_args.push_back(codegen->CastPtrToLlvmPtr(expr_ptr_type, children_[i]));
-    }
+    RETURN_IF_ERROR(children_[i]->GetCodegendComputeFn(codegen, false, &child_fn));
     child_fn_args.push_back(eval);
     child_fn_args.push_back(row);
 
@@ -345,22 +340,63 @@ Status ScalarFnCall::GetCodegendComputeFnImpl(LlvmCodeGen* codegen, llvm::Functi
     DCHECK(child_fn != NULL);
     llvm::Type* arg_type = CodegenAnyVal::GetUnloweredType(codegen, children_[i]->type());
     llvm::Value* arg_val_ptr;
+#ifdef __aarch64__
+    PrimitiveType col_type = children_[i]->type().type;
+#endif
     if (i < NumFixedArgs()) {
+#ifndef __aarch64__
       // Allocate space to store 'child_fn's result so we can pass the pointer to the UDF.
       arg_val_ptr = codegen->CreateEntryBlockAlloca(builder, arg_type, "arg_val_ptr");
       udf_args.push_back(arg_val_ptr);
+#else
+      if (col_type != TYPE_BOOLEAN and col_type != TYPE_TINYINT
+          and col_type != TYPE_SMALLINT) {
+        arg_val_ptr = codegen->CreateEntryBlockAlloca(builder, arg_type, "arg_val_ptr");
+        udf_args.push_back(arg_val_ptr);
+      }
+#endif
     } else {
       // Store the result of 'child_fn' in varargs_buffer[i].
       arg_val_ptr =
           builder.CreateConstGEP1_32(varargs_buffer, i - NumFixedArgs(), "arg_val_ptr");
     }
+#ifndef __aarch64__
     DCHECK_EQ(arg_val_ptr->getType(), arg_type->getPointerTo());
     // The result of the call must be stored in a lowered AnyVal
     llvm::Value* lowered_arg_val_ptr = builder.CreateBitCast(arg_val_ptr,
         CodegenAnyVal::GetLoweredPtrType(codegen, children_[i]->type()),
         "lowered_arg_val_ptr");
+#else
+    llvm::Value* lowered_arg_val_ptr;
+    if (col_type == TYPE_BOOLEAN or col_type == TYPE_TINYINT
+        or col_type == TYPE_SMALLINT) {
+      lowered_arg_val_ptr = codegen->CreateEntryBlockAlloca(builder,
+          CodegenAnyVal::GetLoweredType(codegen, children_[i]->type()), 1,
+          FunctionContextImpl::VARARGS_BUFFER_ALIGNMENT, "lowered_arg_val_ptr");
+    } else {
+      lowered_arg_val_ptr = builder.CreateBitCast(arg_val_ptr,
+          CodegenAnyVal::GetLoweredPtrType(codegen, children_[i]->type()),
+          "lowered_arg_val_ptr");
+    }
+#endif
     CodegenAnyVal::CreateCall(
         codegen, &builder, child_fn, child_fn_args, "arg_val", lowered_arg_val_ptr);
+#ifdef __aarch64__
+    if (col_type == TYPE_BOOLEAN or col_type == TYPE_TINYINT
+        or col_type == TYPE_SMALLINT) {
+      if (i < NumFixedArgs()) {
+        arg_val_ptr = builder.CreateTruncOrBitCast(lowered_arg_val_ptr,
+            CodegenAnyVal::GetUnloweredPtrType(codegen, children_[i]->type()),
+            "arg_val_ptr");
+        udf_args.push_back(arg_val_ptr);
+      } else {
+        llvm::Value* tmp_ptr = builder.CreateTruncOrBitCast(lowered_arg_val_ptr,
+            CodegenAnyVal::GetUnloweredPtrType(codegen, children_[i]->type()),
+            "tmp_ptr");
+        builder.CreateStore(builder.CreateLoad(tmp_ptr), arg_val_ptr);
+      }
+    }
+#endif
   }
 
   if (vararg_start_idx_ != -1) {
@@ -388,11 +424,15 @@ Status ScalarFnCall::GetCodegendComputeFnImpl(LlvmCodeGen* codegen, llvm::Functi
   return Status::OK();
 }
 
-Status ScalarFnCall::GetFunction(LlvmCodeGen* codegen, const string& symbol, void** fn) {
+Status ScalarFnCall::GetFunction(LlvmCodeGen* codegen, const string& symbol,
+    CodegenFnPtrBase* fn) {
   if (fn_.binary_type == TFunctionBinaryType::NATIVE
       || fn_.binary_type == TFunctionBinaryType::BUILTIN) {
-    return LibCache::instance()->GetSoFunctionPtr(
-        fn_.hdfs_location, symbol, fn_.last_modified_time, fn, &cache_entry_);
+    void* raw_fn;
+    const Status status = LibCache::instance()->GetSoFunctionPtr(
+        fn_.hdfs_location, symbol, fn_.last_modified_time, &raw_fn, &cache_entry_);
+    fn->store(raw_fn);
+    return status;
   } else {
     DCHECK_EQ(fn_.binary_type, TFunctionBinaryType::IR);
     DCHECK(codegen != NULL);
