@@ -57,7 +57,9 @@ uint64_t StringFunctions::re2_mem_limit_ = 8 << 20;
 //  - 1-indexed positions
 //  - supported negative positions (count from the end of the string)
 //  - [optional] len.  No len indicates longest substr possible
-StringVal StringFunctions::Substring(FunctionContext* context,
+// Marks as IR_ALWAYS_INLINE since this is called in other overloads.
+// So the overloads can also replace GetConstFnAttr() calls in codegen.
+IR_ALWAYS_INLINE StringVal StringFunctions::Substring(FunctionContext* context,
     const StringVal& str, const BigIntVal& pos, const BigIntVal& len) {
   if (str.is_null || pos.is_null || len.is_null) return StringVal::null();
   if (context->impl()->GetConstFnAttr(FunctionContextImpl::UTF8_MODE)) {
@@ -153,8 +155,8 @@ StringVal StringFunctions::Space(FunctionContext* context, const BigIntVal& len)
   if (len.val <= 0) return StringVal();
   if (len.val > StringVal::MAX_LENGTH) {
     context->SetError(Substitute(ERROR_CHARACTER_LIMIT_EXCEEDED,
-         "space() result",
-         PrettyPrinter::Print(StringVal::MAX_LENGTH, TUnit::BYTES)).c_str());
+        "space() result",
+        PrettyPrinter::Print(StringVal::MAX_LENGTH, TUnit::BYTES)).c_str());
     return StringVal::null();
   }
   StringVal result(context, len.val);
@@ -257,7 +259,7 @@ StringVal StringFunctions::Rpad(FunctionContext* context, const StringVal& str,
 
 IntVal StringFunctions::Length(FunctionContext* context, const StringVal& str) {
   if (str.is_null) return IntVal::null();
-  if (context->impl()->GetConstFnAttr(FunctionContextImpl::UTF8_MODE, 0)) {
+  if (context->impl()->GetConstFnAttr(FunctionContextImpl::UTF8_MODE)) {
     return Utf8Length(context, str);
   }
   return IntVal(str.len);
@@ -288,7 +290,10 @@ IntVal StringFunctions::Utf8Length(FunctionContext* context, const StringVal& st
   return IntVal(CountUtf8Chars(str.ptr, str.len));
 }
 
-StringVal StringFunctions::Lower(FunctionContext* context, const StringVal& str) {
+// Marks as IR_ALWAYS_INLINE since this is called in MaskFunctions::MaskHash().
+// So the caller can also replace GetConstFnAttr() calls in codegen.
+IR_ALWAYS_INLINE StringVal StringFunctions::Lower(FunctionContext* context,
+    const StringVal& str) {
   if (str.is_null) return StringVal::null();
   if (context->impl()->GetConstFnAttr(FunctionContextImpl::UTF8_MODE)) {
     return LowerUtf8(context, str);
@@ -879,8 +884,10 @@ IntVal StringFunctions::Ascii(FunctionContext* context, const StringVal& str) {
   return IntVal((str.len == 0) ? 0 : static_cast<int32_t>(str.ptr[0]));
 }
 
-IntVal StringFunctions::Instr(FunctionContext* context, const StringVal& str,
-    const StringVal& substr, const BigIntVal& start_position,
+// Marks as IR_ALWAYS_INLINE since this is called in other overloads.
+// So the overloads can also replace GetConstFnAttr() calls in codegen.
+IR_ALWAYS_INLINE IntVal StringFunctions::Instr(FunctionContext* context,
+    const StringVal& str, const StringVal& substr, const BigIntVal& start_position,
     const BigIntVal& occurrence) {
   if (str.is_null || substr.is_null || start_position.is_null || occurrence.is_null) {
     return IntVal::null();
@@ -1248,8 +1255,8 @@ StringVal StringFunctions::Concat(
 
   if (total_size > StringVal::MAX_LENGTH) {
     context->SetError(Substitute(ERROR_CHARACTER_LIMIT_EXCEEDED,
-         "Concatenated string length",
-         PrettyPrinter::Print(StringVal::MAX_LENGTH, TUnit::BYTES)).c_str());
+        "Concatenated string length",
+        PrettyPrinter::Print(StringVal::MAX_LENGTH, TUnit::BYTES)).c_str());
     return StringVal::null();
   }
 
@@ -1296,8 +1303,8 @@ StringVal StringFunctions::ConcatWs(FunctionContext* context, const StringVal& s
 
   if (total_size > StringVal::MAX_LENGTH) {
     context->SetError(Substitute(ERROR_CHARACTER_LIMIT_EXCEEDED,
-         "Concatenated string length",
-         PrettyPrinter::Print(StringVal::MAX_LENGTH, TUnit::BYTES)).c_str());
+        "Concatenated string length",
+        PrettyPrinter::Print(StringVal::MAX_LENGTH, TUnit::BYTES)).c_str());
     return StringVal::null();
   }
 
@@ -1428,7 +1435,7 @@ StringVal StringFunctions::ParseUrlKey(FunctionContext* ctx, const StringVal& ur
 
   StringValue result;
   if (!UrlParser::ParseUrlKey(StringValue::FromStringVal(url), url_part,
-                              StringValue::FromStringVal(key), &result)) {
+      StringValue::FromStringVal(key), &result)) {
     // url is malformed, or url_part is invalid.
     if (url_part == UrlParser::INVALID) {
       stringstream ss;
@@ -1708,8 +1715,8 @@ DoubleVal StringFunctions::JaroSimilarity(
   }
   double m = static_cast<double>(matching_characters);
   double jaro_similarity = 1.0 / 3.0  * ( m / static_cast<double>(s1len)
-                                        + m / static_cast<double>(s2len)
-                                        + (m - transpositions) / m );
+      + m / static_cast<double>(s2len)
+      + (m - transpositions) / m );
 
   ctx->Free(reinterpret_cast<uint8_t*>(s1_matching));
   ctx->Free(reinterpret_cast<uint8_t*>(s2_matching));
@@ -1727,27 +1734,27 @@ DoubleVal StringFunctions::JaroDistance(
 }
 
 DoubleVal StringFunctions::JaroWinklerDistance(FunctionContext* ctx,
-      const StringVal& s1, const StringVal& s2) {
+    const StringVal& s1, const StringVal& s2) {
   return StringFunctions::JaroWinklerDistance(ctx, s1, s2,
-    DoubleVal(0.1), DoubleVal(0.7));
+      DoubleVal(0.1), DoubleVal(0.7));
 }
 
 DoubleVal StringFunctions::JaroWinklerDistance(FunctionContext* ctx,
-      const StringVal& s1, const StringVal& s2,
-      const DoubleVal& scaling_factor) {
+    const StringVal& s1, const StringVal& s2,
+    const DoubleVal& scaling_factor) {
   return StringFunctions::JaroWinklerDistance(ctx, s1, s2,
-    scaling_factor, DoubleVal(0.7));
+      scaling_factor, DoubleVal(0.7));
 }
 
 // Based on https://en.wikipedia.org/wiki/Jaro%E2%80%93Winkler_distance
 // Implements Jaro-Winkler distance
 // Extended with boost_theshold: Winkler's modification only applies if Jaro exceeds it
 DoubleVal StringFunctions::JaroWinklerDistance(FunctionContext* ctx,
-      const StringVal& s1, const StringVal& s2,
-      const DoubleVal& scaling_factor, const DoubleVal& boost_threshold) {
+    const StringVal& s1, const StringVal& s2,
+    const DoubleVal& scaling_factor, const DoubleVal& boost_threshold) {
 
   DoubleVal jaro_winkler_similarity = StringFunctions::JaroWinklerSimilarity(
-    ctx, s1, s2, scaling_factor, boost_threshold);
+      ctx, s1, s2, scaling_factor, boost_threshold);
 
   if (jaro_winkler_similarity.is_null) return DoubleVal::null();
   if (jaro_winkler_similarity.val == -1.0) return DoubleVal(-1.0);
@@ -1755,24 +1762,24 @@ DoubleVal StringFunctions::JaroWinklerDistance(FunctionContext* ctx,
 }
 
 DoubleVal StringFunctions::JaroWinklerSimilarity(FunctionContext* ctx,
-      const StringVal& s1, const StringVal& s2) {
+    const StringVal& s1, const StringVal& s2) {
   return StringFunctions::JaroWinklerSimilarity(ctx, s1, s2,
-    DoubleVal(0.1), DoubleVal(0.7));
+      DoubleVal(0.1), DoubleVal(0.7));
 }
 
 DoubleVal StringFunctions::JaroWinklerSimilarity(FunctionContext* ctx,
-      const StringVal& s1, const StringVal& s2,
-      const DoubleVal& scaling_factor) {
+    const StringVal& s1, const StringVal& s2,
+    const DoubleVal& scaling_factor) {
   return StringFunctions::JaroWinklerSimilarity(ctx, s1, s2,
-    scaling_factor, DoubleVal(0.7));
+      scaling_factor, DoubleVal(0.7));
 }
 
 // Based on https://en.wikipedia.org/wiki/Jaro%E2%80%93Winkler_distance
 // Implements Jaro-Winkler similarity
 // Extended with boost_theshold: Winkler's modification only applies if Jaro exceeds it
 DoubleVal StringFunctions::JaroWinklerSimilarity(FunctionContext* ctx,
-      const StringVal& s1, const StringVal& s2,
-      const DoubleVal& scaling_factor, const DoubleVal& boost_threshold) {
+    const StringVal& s1, const StringVal& s2,
+    const DoubleVal& scaling_factor, const DoubleVal& boost_threshold) {
 
   constexpr int MAX_PREFIX_LENGTH = 4;
   int s1len = s1.len;
@@ -1806,12 +1813,12 @@ DoubleVal StringFunctions::JaroWinklerSimilarity(FunctionContext* ctx,
     int common_length = std::min(MAX_PREFIX_LENGTH, std::min(s1len, s2len));
     int common_prefix = 0;
     while (common_prefix < common_length &&
-           s1.ptr[common_prefix] == s2.ptr[common_prefix]) {
+        s1.ptr[common_prefix] == s2.ptr[common_prefix]) {
       common_prefix++;
     }
 
     jaro_winkler_similarity += common_prefix * scaling_factor.val *
-      (1.0 - jaro_similarity.val);
+        (1.0 - jaro_similarity.val);
   }
   return DoubleVal(jaro_winkler_similarity);
 }
@@ -1874,8 +1881,8 @@ IntVal StringFunctions::DamerauLevenshtein(
         l_cost = 1;
       }
       d[i][j] = std::min(d[i - 1][j - 1] + l_cost, // substitution
-                         std::min(d[i][j - 1] + 1, // insertion
-                                  d[i - 1][j] + 1) // deletion
+          std::min(d[i][j - 1] + 1, // insertion
+              d[i - 1][j] + 1) // deletion
       );
       if (i > 1 && j > 1 && s1.ptr[i - 1] == s2.ptr[j - 2]
           && s1.ptr[i - 2] == s2.ptr[j - 1]) {
@@ -1927,4 +1934,38 @@ StringVal StringFunctions::PrettyPrintMemory(FunctionContext* context,
   return prettyPrint(context, bytes, TUnit::BYTES);
 }
 
+// The state is a bool used to reduce excessive logs.
+void StringFunctions::AesPrepare(FunctionContext* context,
+    FunctionContext::FunctionStateScope scope) {
+  if (scope != FunctionContext::THREAD_LOCAL) return;
+  bool* state = reinterpret_cast<bool*>(context->Allocate(sizeof(bool)));
+  if (state == nullptr) {
+    context->AddWarning("Failed to allocate memory for function state.");
+    return;
+  }
+  *state = false;
+  context->SetFunctionState(scope, state);
+}
+
+void StringFunctions::AesClose(FunctionContext* context,
+    FunctionContext::FunctionStateScope scope) {
+  if (scope != FunctionContext::THREAD_LOCAL) return;
+  bool* state = reinterpret_cast<bool*>(
+      context->GetFunctionState(FunctionContext::THREAD_LOCAL));
+  if (state != nullptr) {
+    context->Free(reinterpret_cast<uint8_t*>(state));
+    context->SetFunctionState(scope, nullptr);
+  }
+}
+
+// Implementation details and comments are provided in string-functions.cc file.
+StringVal StringFunctions::AesDecrypt(FunctionContext* ctx, const StringVal& expr,
+    const StringVal& key, const StringVal& mode, const StringVal& iv) {
+  return AesDecryptImpl(ctx, expr, key, mode, iv);
+}
+
+StringVal StringFunctions::AesEncrypt(FunctionContext* ctx, const StringVal& expr,
+    const StringVal& key, const StringVal& mode, const StringVal& iv) {
+  return AesEncryptImpl(ctx, expr, key, mode, iv);
+}
 }
