@@ -19,6 +19,7 @@
 
 #include <algorithm>
 #include <cinttypes>
+#include <cmath>
 #include <cstdint>
 #include <memory>
 #include <mutex>
@@ -28,11 +29,12 @@
 #include <utility>
 #include <vector>
 
-#include <boost/bind.hpp>
 #include <gflags/gflags.h>
 
 #include "kudu/gutil/dynamic_annotations.h"
+#include "kudu/gutil/integral_types.h"
 #include "kudu/gutil/map-util.h"
+#include "kudu/gutil/port.h"
 #include "kudu/gutil/stringprintf.h"
 #include "kudu/gutil/strings/split.h"
 #include "kudu/gutil/strings/substitute.h"
@@ -61,19 +63,21 @@ DEFINE_int32(maintenance_manager_num_threads, 1,
              "For spinning disks, the number of threads should "
              "not be above the number of devices.");
 TAG_FLAG(maintenance_manager_num_threads, stable);
+DEFINE_validator(maintenance_manager_num_threads,
+                 [](const char* /*n*/, int32 v) { return v > 0; });
 
 DEFINE_int32(maintenance_manager_polling_interval_ms, 250,
-       "Polling interval for the maintenance manager scheduler, "
-       "in milliseconds.");
+             "Polling interval for the maintenance manager scheduler, "
+             "in milliseconds.");
 TAG_FLAG(maintenance_manager_polling_interval_ms, hidden);
 
 DEFINE_int32(maintenance_manager_history_size, 8,
-       "Number of completed operations the manager keeps track of.");
+             "Number of completed operations the manager keeps track of.");
 TAG_FLAG(maintenance_manager_history_size, hidden);
 
 DEFINE_bool(enable_maintenance_manager, true,
-       "Enable the maintenance manager, which runs flush, compaction, and "
-       "garbage collection operations on tablets.");
+            "Enable the maintenance manager, which runs flush, compaction, "
+            "and garbage collection operations on tablets.");
 TAG_FLAG(enable_maintenance_manager, unsafe);
 
 DEFINE_int64(log_target_replay_size_mb, 1024,
@@ -84,7 +88,7 @@ DEFINE_int64(log_target_replay_size_mb, 1024,
 TAG_FLAG(log_target_replay_size_mb, experimental);
 
 DEFINE_int64(data_gc_min_size_mb, 0,
-             "The (exclusive) minimum number of megabytes of ancient data on "
+             "The (exclusive) minimum number of mebibytes of ancient data on "
              "disk, per tablet, needed to prioritize deletion of that data.");
 TAG_FLAG(data_gc_min_size_mb, experimental);
 
@@ -110,6 +114,11 @@ TAG_FLAG(max_priority_range, advanced);
 TAG_FLAG(max_priority_range, experimental);
 TAG_FLAG(max_priority_range, runtime);
 
+DEFINE_int32(maintenance_manager_inject_latency_ms, 0,
+             "Injects latency into maintenance thread. For use in tests only.");
+TAG_FLAG(maintenance_manager_inject_latency_ms, runtime);
+TAG_FLAG(maintenance_manager_inject_latency_ms, unsafe);
+
 namespace kudu {
 
 MaintenanceOpStats::MaintenanceOpStats() {
@@ -123,14 +132,15 @@ void MaintenanceOpStats::Clear() {
   logs_retained_bytes_ = 0;
   data_retained_bytes_ = 0;
   perf_improvement_ = 0;
+  workload_score_ = 0;
   last_modified_ = MonoTime();
 }
 
 MaintenanceOp::MaintenanceOp(string name, IOUsage io_usage)
     : name_(std::move(name)),
+      io_usage_(io_usage),
       running_(0),
-      cancel_(false),
-      io_usage_(io_usage) {
+      cancel_(false) {
 }
 
 MaintenanceOp::~MaintenanceOp() {
@@ -161,20 +171,25 @@ const MaintenanceManager::Options MaintenanceManager::kDefaultOptions = {
   .history_size = 0,
 };
 
-MaintenanceManager::MaintenanceManager(const Options& options,
-                                       string server_uuid)
-  : server_uuid_(std::move(server_uuid)),
-    num_threads_(options.num_threads <= 0 ?
-                 FLAGS_maintenance_manager_num_threads : options.num_threads),
-    cond_(&lock_),
-    shutdown_(false),
-    polling_interval_ms_(options.polling_interval_ms <= 0 ?
-          FLAGS_maintenance_manager_polling_interval_ms :
-          options.polling_interval_ms),
-    running_ops_(0),
-    completed_ops_count_(0),
-    rand_(GetRandomSeed32()),
-    memory_pressure_func_(&process_memory::UnderMemoryPressure) {
+MaintenanceManager::MaintenanceManager(
+    const Options& options,
+    string server_uuid,
+    const scoped_refptr<MetricEntity>& metric_entity)
+    : server_uuid_(std::move(server_uuid)),
+      num_threads_(options.num_threads > 0
+                   ? options.num_threads
+                   : FLAGS_maintenance_manager_num_threads),
+      polling_interval_(MonoDelta::FromMilliseconds(
+          options.polling_interval_ms > 0
+              ? options.polling_interval_ms
+              : FLAGS_maintenance_manager_polling_interval_ms)),
+      cond_(&lock_),
+      shutdown_(false),
+      running_ops_(0),
+      completed_ops_count_(0),
+      rand_(GetRandomSeed32()),
+      memory_pressure_func_(&process_memory::UnderMemoryPressure),
+      metrics_(CHECK_NOTNULL(metric_entity)) {
   CHECK_OK(ThreadPoolBuilder("MaintenanceMgr")
                .set_min_threads(num_threads_)
                .set_max_threads(num_threads_)
@@ -191,10 +206,9 @@ MaintenanceManager::~MaintenanceManager() {
 
 Status MaintenanceManager::Start() {
   CHECK(!monitor_thread_);
-  RETURN_NOT_OK(Thread::Create("maintenance", "maintenance_scheduler",
-      boost::bind(&MaintenanceManager::RunSchedulerThread, this),
-      &monitor_thread_));
-  return Status::OK();
+  return Thread::Create("maintenance", "maintenance_scheduler",
+                        [this]() { this->RunSchedulerThread(); },
+                        &monitor_thread_);
 }
 
 void MaintenanceManager::Shutdown() {
@@ -218,42 +232,64 @@ void MaintenanceManager::Shutdown() {
   }
 }
 
+void MaintenanceManager::MergePendingOpRegistrationsUnlocked() {
+  lock_.AssertAcquired();
+  OpMapType ops_to_register;
+  {
+    std::lock_guard<simple_spinlock> l(registration_lock_);
+    ops_to_register = std::move(ops_pending_registration_);
+    ops_pending_registration_.clear();
+  }
+  for (auto& op_and_stats : ops_to_register) {
+    auto* op = op_and_stats.first;
+    op->cond_.reset(new ConditionVariable(&running_instances_lock_));
+    VLOG_AND_TRACE_WITH_PREFIX("maintenance", 1) << "Registered " << op->name();
+  }
+  ops_.insert(ops_to_register.begin(), ops_to_register.end());
+}
+
 void MaintenanceManager::RegisterOp(MaintenanceOp* op) {
   CHECK(op);
-  std::lock_guard<Mutex> guard(lock_);
-  CHECK(!op->manager_) << "Tried to register " << op->name()
-                       << ", but it is already registered.";
-  pair<OpMapTy::iterator, bool> val
-    (ops_.insert(OpMapTy::value_type(op, MaintenanceOpStats())));
-  CHECK(val.second) << "Tried to register " << op->name()
-                    << ", but it already exists in ops_.";
-  op->manager_ = shared_from_this();
-  op->cond_.reset(new ConditionVariable(&lock_));
-  VLOG_AND_TRACE_WITH_PREFIX("maintenance", 1) << "Registered " << op->name();
+  {
+    std::lock_guard<simple_spinlock> l(registration_lock_);
+    CHECK(!op->manager_) << "Tried to register " << op->name()
+                        << ", but it is already registered.";
+    EmplaceOrDie(&ops_pending_registration_, op, MaintenanceOpStats());
+    op->manager_ = shared_from_this();
+  }
+  // If we can take 'lock_', add to 'ops_' immediately.
+  if (lock_.try_lock()) {
+    MergePendingOpRegistrationsUnlocked();
+    lock_.unlock();
+  }
 }
 
 void MaintenanceManager::UnregisterOp(MaintenanceOp* op) {
+  CHECK(op->manager_.get() == this) << "Tried to unregister " << op->name()
+      << ", but it is not currently registered with this maintenance manager.";
+  op->CancelAndDisable();
+
+  // While the op is running, wait for it to be finished.
+  {
+    std::lock_guard<Mutex> guard(running_instances_lock_);
+    if (op->running_ > 0) {
+      VLOG_AND_TRACE_WITH_PREFIX("maintenance", 1)
+          << Substitute("Waiting for op $0 to finish so we can unregister it", op->name());
+    }
+    while (op->running_ > 0) {
+      op->cond_->Wait();
+    }
+  }
+
+  // Remove the op from 'ops_', and if it wasn't there, erase it from
+  // 'ops_pending_registration_'.
   {
     std::lock_guard<Mutex> guard(lock_);
-    CHECK(op->manager_.get() == this) << "Tried to unregister " << op->name()
-        << ", but it is not currently registered with this maintenance manager.";
-    auto iter = ops_.find(op);
-    CHECK(iter != ops_.end()) << "Tried to unregister " << op->name()
-                              << ", but it was never registered";
-    // While the op is running, wait for it to be finished.
-    if (iter->first->running_ > 0) {
-      VLOG_AND_TRACE_WITH_PREFIX("maintenance", 1) << "Waiting for op " << op->name()
-                                                   << " to finish so we can unregister it.";
+    if (ops_.erase(op) == 0) {
+      std::lock_guard<simple_spinlock> l(registration_lock_);
+      const auto num_erased_ops = ops_pending_registration_.erase(op);
+      CHECK_GT(num_erased_ops, 0);
     }
-    op->CancelAndDisable();
-    while (iter->first->running_ > 0) {
-      op->cond_->Wait();
-      iter = ops_.find(op);
-      CHECK(iter != ops_.end()) << "Tried to unregister " << op->name()
-          << ", but another thread unregistered it while we were "
-          << "waiting for it to complete";
-    }
-    ops_.erase(iter);
   }
   VLOG_WITH_PREFIX(1) << "Unregistered op " << op->name();
   op->cond_.reset();
@@ -271,93 +307,116 @@ void MaintenanceManager::RunSchedulerThread() {
     return;
   }
 
-  MonoDelta polling_interval = MonoDelta::FromMilliseconds(polling_interval_ms_);
-
-  std::unique_lock<Mutex> guard(lock_);
-
   // Set to true if the scheduler runs and finds that there is no work to do.
   bool prev_iter_found_no_work = false;
 
   while (true) {
-    // We'll keep sleeping if:
-    //    1) there are no free threads available to perform a maintenance op.
-    // or 2) we just tried to schedule an op but found nothing to run.
-    // However, if it's time to shut down, we want to do so immediately.
-    while (CouldNotLaunchNewOp(prev_iter_found_no_work)) {
-      cond_.WaitFor(polling_interval);
-      prev_iter_found_no_work = false;
-    }
-    if (shutdown_) {
-      VLOG_AND_TRACE_WITH_PREFIX("maintenance", 1) << "Shutting down maintenance manager.";
-      return;
+    MaintenanceOp* op = nullptr;
+    string op_note;
+    {
+      std::unique_lock<Mutex> guard(lock_);
+      // Upon each iteration, we should have dropped and reacquired 'lock_'.
+      // Register any ops that may have been buffered for registration while the
+      // lock was last held.
+      MergePendingOpRegistrationsUnlocked();
+
+      // We'll keep sleeping if:
+      //    1) there are no free threads available to perform a maintenance op.
+      // or 2) we just tried to schedule an op but found nothing to run.
+      // However, if it's time to shut down, we want to do so immediately.
+      while (CouldNotLaunchNewOp(prev_iter_found_no_work)) {
+        cond_.WaitFor(polling_interval_);
+        prev_iter_found_no_work = false;
+      }
+      if (shutdown_) {
+        VLOG_AND_TRACE_WITH_PREFIX("maintenance", 1) << "Shutting down maintenance manager.";
+        return;
+      }
+
+      if (PREDICT_FALSE(FLAGS_maintenance_manager_inject_latency_ms > 0)) {
+        LOG(WARNING) << "Injecting " << FLAGS_maintenance_manager_inject_latency_ms
+                     << "ms of latency into maintenance thread";
+        SleepFor(MonoDelta::FromMilliseconds(FLAGS_maintenance_manager_inject_latency_ms));
+      }
+
+      // Find the best op. If we found no work to do, then we should sleep
+      // before trying again to schedule. Otherwise, we can go right into trying
+      // to find the next op.
+      {
+        auto best_op_and_why = FindBestOp();
+        op = best_op_and_why.first;
+        op_note = std::move(best_op_and_why.second);
+      }
+      if (op) {
+        // While 'running_instances_lock_' is held, check one more time for
+        // whether the op is cancelled. This ensures that we don't attempt to
+        // launch an op that has been destructed in UnregisterOp(). See
+        // KUDU-3268 for more details.
+        std::lock_guard<Mutex> guard(running_instances_lock_);
+        if (op->cancelled()) {
+          VLOG_AND_TRACE_WITH_PREFIX("maintenance", 2)
+              << "picked maintenance operation that has been cancelled";
+          continue;
+        }
+        IncreaseOpCount(op);
+        prev_iter_found_no_work = false;
+      } else {
+        VLOG_AND_TRACE_WITH_PREFIX("maintenance", 2)
+            << "no maintenance operations look worth doing";
+        prev_iter_found_no_work = true;
+        continue;
+      }
     }
 
-    // If we found no work to do, then we should sleep before trying again to schedule.
-    // Otherwise, we can go right into trying to find the next op.
-    prev_iter_found_no_work = !FindAndLaunchOp(&guard);
+    // Prepare the maintenance operation.
+    DCHECK(op);
+    if (!op->Prepare()) {
+      LOG_WITH_PREFIX(INFO) << "Prepare failed for " << op->name()
+                            << ". Re-running scheduler.";
+      metrics_.SubmitOpPrepareFailed();
+      std::lock_guard<Mutex> guard(running_instances_lock_);
+      DecreaseOpCountAndNotifyWaiters(op);
+      continue;
+    }
+
+    LOG_AND_TRACE_WITH_PREFIX("maintenance", INFO)
+        << Substitute("Scheduling $0: $1", op->name(), op_note);
+    // Submit the maintenance operation to be run on the "MaintenanceMgr" pool.
+    CHECK_OK(thread_pool_->Submit([this, op]() { this->LaunchOp(op); }));
   }
 }
 
-bool MaintenanceManager::FindAndLaunchOp(std::unique_lock<Mutex>* guard) {
-  // Find the best op.
-  auto best_op_and_why = FindBestOp();
-  auto* op = best_op_and_why.first;
-  const auto& note = best_op_and_why.second;
-
-  if (!op) {
-    VLOG_AND_TRACE_WITH_PREFIX("maintenance", 2)
-        << "No maintenance operations look worth doing.";
-    return false;
-  }
-
-  // Prepare the maintenance operation.
-  IncreaseOpCount(op);
-  guard->unlock();
-  bool ready = op->Prepare();
-  guard->lock();
-  if (!ready) {
-    LOG_WITH_PREFIX(INFO) << "Prepare failed for " << op->name()
-                          << ". Re-running scheduler.";
-    DecreaseOpCount(op);
-    op->cond_->Signal();
-    return true;
-  }
-
-  LOG_AND_TRACE_WITH_PREFIX("maintenance", INFO)
-      << Substitute("Scheduling $0: $1", op->name(), note);
-  // Run the maintenance operation.
-  CHECK_OK(thread_pool_->SubmitFunc(boost::bind(&MaintenanceManager::LaunchOp, this, op)));
-
-  return true;
-}
-
-// Finding the best operation goes through four filters:
-// - If there's an Op that we can run quickly that frees log retention, we run it.
-// - If we've hit the overall process memory limit (note: this includes memory that the Ops cannot
-//   free), we run the Op with the highest RAM usage.
-// - If there are Ops that are retaining logs past our target replay size, we run the one that has
-//   the highest retention (and if many qualify, then we run the one that also frees up the
-//   most RAM).
-// - Finally, if there's nothing else that we really need to do, we run the Op that will improve
-//   performance the most.
+// Finding the best operation goes through some filters:
+// - If there's an Op that we can run quickly that frees log retention, run it
+//   (e.g. GCing WAL segments).
+// - If we've hit the overall process memory limit (note: this includes memory
+//   that the Ops cannot free), we run the Op that retains the most WAL
+//   segments, which will free memory (e.g. MRS or DMS flush).
+// - If there Ops that are retaining logs past our target replay size, we
+//   run the one that has the highest retention, and if many qualify, then we
+//   run the one that also frees up the most RAM (e.g. MRS or DMS flush).
+// - If there are Ops that we can run that free disk space, run whichever frees
+//   the most space (e.g. GCing ancient deltas).
+// - Finally, if there's nothing else that we really need to do, we run the Op
+//   that will improve performance the most.
 //
-// The reason it's done this way is that we want to prioritize limiting the amount of resources we
-// hold on to. Low IO Ops go first since we can quickly run them, then we can look at memory usage.
-// Reversing those can starve the low IO Ops when the system is under intense memory pressure.
-//
-// In the third priority we're at a point where nothing's urgent and there's nothing we can run
-// quickly.
-// TODO(wdberkeley) We currently optimize for freeing log retention but we could consider having
-// some sort of sliding priority between log retention and RAM usage. For example, is an Op that
-// frees 128MB of log retention and 12MB of RAM always better than an op that frees 12MB of log
-// retention and 128MB of RAM? Maybe a more holistic approach would be better.
+// In general, we want to prioritize limiting the amount of expensive resources
+// we hold onto. Low IO ops that free WAL disk space are preferred, followed by
+// ops that free memory, then ops that free data disk space, then ops that
+// improve performance.
 pair<MaintenanceOp*, string> MaintenanceManager::FindBestOp() {
+  SCOPED_LOG_SLOW_EXECUTION(WARNING, 10000, "finding best maintenance operation");
   TRACE_EVENT0("maintenance", "MaintenanceManager::FindBestOp");
 
   if (!HasFreeThreads()) {
     return {nullptr, "no free threads"};
   }
 
+  const auto start_time = MonoTime::Now();
+  SCOPED_CLEANUP({
+    metrics_.SubmitOpFindDuration(MonoTime::Now() - start_time);
+  });
+
   int64_t low_io_most_logs_retained_bytes = 0;
   MaintenanceOp* low_io_most_logs_retained_bytes_op = nullptr;
 
@@ -391,8 +450,8 @@ pair<MaintenanceOp*, string> MaintenanceManager::FindBestOp() {
                         op->name(), logs_retained_bytes);
     }
 
-    // We prioritize ops that can free more logs, but when it's the same we pick
-    // the one that also frees up the most memory.
+    // We prioritize ops that can free more logs, but when it's the same we
+    // pick the one that also frees up the most memory.
     const auto ram_anchored = stats.ram_anchored();
     if (std::make_pair(logs_retained_bytes, ram_anchored) >
         std::make_pair(most_logs_retained_bytes,
@@ -411,7 +470,8 @@ pair<MaintenanceOp*, string> MaintenanceManager::FindBestOp() {
                         op->name(), data_retained_bytes);
     }
 
-    const auto perf_improvement = PerfImprovement(stats.perf_improvement(), op->priority());
+    const auto perf_improvement =
+        AdjustedPerfScore(stats.perf_improvement(), stats.workload_score(), op->priority());
     if ((!best_perf_improvement_op) ||
         (perf_improvement > best_perf_improvement)) {
       best_perf_improvement_op = op;
@@ -455,7 +515,8 @@ pair<MaintenanceOp*, string> MaintenanceManager::FindBestOp() {
     return {most_logs_retained_bytes_ram_anchored_op, std::move(note)};
   }
 
-  // Look at ops that we can run quickly that free up data on disk.
+  // Look at ops that free up data on disk. To avoid starvation of
+  // performance-improving ops, we might skip freeing disk space.
   if (most_data_retained_bytes_op &&
       most_data_retained_bytes > FLAGS_data_gc_min_size_mb * 1024 * 1024) {
     if (!best_perf_improvement_op || best_perf_improvement <= 0 ||
@@ -474,18 +535,24 @@ pair<MaintenanceOp*, string> MaintenanceManager::FindBestOp() {
   return {nullptr, "no ops with positive improvement"};
 }
 
-double MaintenanceManager::PerfImprovement(double perf_improvement, int32_t priority) const {
+double MaintenanceManager::AdjustedPerfScore(double perf_improvement,
+                                             double workload_score,
+                                             int32_t priority) {
+  if (perf_improvement == 0) {
+    return 0;
+  }
+  double perf_score = perf_improvement + workload_score;
   if (priority == 0) {
-    return perf_improvement;
+    return perf_score;
   }
 
   priority = std::max(priority, -FLAGS_max_priority_range);
   priority = std::min(priority, FLAGS_max_priority_range);
-  return perf_improvement * std::pow(FLAGS_maintenance_op_multiplier, priority);
+  return perf_score * std::pow(FLAGS_maintenance_op_multiplier, priority);
 }
 
 void MaintenanceManager::LaunchOp(MaintenanceOp* op) {
-  int64_t thread_id = Thread::CurrentThreadId();
+  const auto thread_id = Thread::CurrentThreadId();
   OpInstance op_instance;
   op_instance.thread_id = thread_id;
   op_instance.name = op->name();
@@ -497,21 +564,30 @@ void MaintenanceManager::LaunchOp(MaintenanceOp* op) {
   }
 
   SCOPED_CLEANUP({
-    op->RunningGauge()->Decrement();
+    // To avoid timing distortions in case of lock contention, it's important
+    // to take a snapshot of 'now' right after the operation completed
+    // before acquiring any locks in the code below.
+    const auto now = MonoTime::Now();
 
-    std::lock_guard<Mutex> l(lock_);
+    op->RunningGauge()->Decrement();
     {
       std::lock_guard<Mutex> lock(running_instances_lock_);
       running_instances_.erase(thread_id);
-    }
-    op_instance.duration = MonoTime::Now() - op_instance.start_mono_time;
-    completed_ops_[completed_ops_count_ % completed_ops_.size()] = op_instance;
-    completed_ops_count_++;
 
-    op->DurationHistogram()->Increment(op_instance.duration.ToMilliseconds());
-    DecreaseOpCount(op);
-    op->cond_->Signal();
-    cond_.Signal(); // Wake up scheduler.
+      op_instance.duration = now - op_instance.start_mono_time;
+      op->DurationHistogram()->Increment(op_instance.duration.ToMilliseconds());
+
+      DecreaseOpCountAndNotifyWaiters(op);
+    }
+    cond_.Signal(); // wake up the scheduler
+
+    // Add corresponding entry into the completed_ops_ container.
+    {
+      std::lock_guard<simple_spinlock> lock(completed_ops_lock_);
+      completed_ops_[completed_ops_count_ % completed_ops_.size()] =
+          std::move(op_instance);
+      ++completed_ops_count_;
+    }
   });
 
   scoped_refptr<Trace> trace(new Trace);
@@ -530,11 +606,13 @@ void MaintenanceManager::LaunchOp(MaintenanceOp* op) {
                                       trace->MetricsAsJSON());
 }
 
-void MaintenanceManager::GetMaintenanceManagerStatusDump(MaintenanceManagerStatusPB* out_pb) {
+void MaintenanceManager::GetMaintenanceManagerStatusDump(
+    MaintenanceManagerStatusPB* out_pb) {
   DCHECK(out_pb != nullptr);
   std::lock_guard<Mutex> guard(lock_);
+  MergePendingOpRegistrationsUnlocked();
   for (const auto& val : ops_) {
-    MaintenanceManagerStatusPB_MaintenanceOpPB* op_pb = out_pb->add_registered_operations();
+    auto* op_pb = out_pb->add_registered_operations();
     MaintenanceOp* op(val.first);
     const MaintenanceOpStats& stats(val.second);
     op_pb->set_name(op->name());
@@ -544,11 +622,13 @@ void MaintenanceManager::GetMaintenanceManagerStatusDump(MaintenanceManagerStatu
       op_pb->set_ram_anchored_bytes(stats.ram_anchored());
       op_pb->set_logs_retained_bytes(stats.logs_retained_bytes());
       op_pb->set_perf_improvement(stats.perf_improvement());
+      op_pb->set_workload_score(stats.workload_score());
     } else {
       op_pb->set_runnable(false);
       op_pb->set_ram_anchored_bytes(0);
       op_pb->set_logs_retained_bytes(0);
       op_pb->set_perf_improvement(0.0);
+      op_pb->set_workload_score(0.0);
     }
   }
 
@@ -560,13 +640,18 @@ void MaintenanceManager::GetMaintenanceManagerStatusDump(MaintenanceManagerStatu
   }
 
   // The latest completed op will be dumped at first.
-  for (int n = 1; n <= completed_ops_.size(); n++) {
-    int64_t i = completed_ops_count_ - n;
-    if (i < 0) break;
-    const auto& completed_op = completed_ops_[i % completed_ops_.size()];
+  {
+    std::lock_guard<simple_spinlock> lock(completed_ops_lock_);
+    for (int n = 1; n <= completed_ops_.size(); ++n) {
+      if (completed_ops_count_ < n) {
+        break;
+      }
+      size_t i = completed_ops_count_ - n;
+      const auto& completed_op = completed_ops_[i % completed_ops_.size()];
 
-    if (!completed_op.name.empty()) {
-      *out_pb->add_completed_operations() = completed_op.DumpToPB();
+      if (!completed_op.name.empty()) {
+        *out_pb->add_completed_operations() = completed_op.DumpToPB();
+      }
     }
   }
 }
@@ -576,21 +661,25 @@ string MaintenanceManager::LogPrefix() const {
 }
 
 bool MaintenanceManager::HasFreeThreads() {
-  return num_threads_ - running_ops_ > 0;
+  return num_threads_ > running_ops_;
 }
 
 bool MaintenanceManager::CouldNotLaunchNewOp(bool prev_iter_found_no_work) {
+  lock_.AssertAcquired();
   return (!HasFreeThreads() || prev_iter_found_no_work || disabled_for_tests()) && !shutdown_;
 }
 
-void MaintenanceManager::IncreaseOpCount(MaintenanceOp *op) {
-  op->running_++;
-  running_ops_++;
+void MaintenanceManager::IncreaseOpCount(MaintenanceOp* op) {
+  running_instances_lock_.AssertAcquired();
+  ++running_ops_;
+  ++op->running_;
 }
 
-void MaintenanceManager::DecreaseOpCount(MaintenanceOp *op) {
-  op->running_--;
-  running_ops_--;
+void MaintenanceManager::DecreaseOpCountAndNotifyWaiters(MaintenanceOp* op) {
+  running_instances_lock_.AssertAcquired();
+  --running_ops_;
+  --op->running_;
+  op->cond_->Signal();
 }
 
 } // namespace kudu
