@@ -127,17 +127,17 @@ DEFINE_string(trusted_domain, "",
     "<username>:<password> where the password is not used and can be left blank.");
 
 DEFINE_bool(trusted_domain_use_xff_header, false,
-    "If set to true, this uses the 'X-Forwarded-For' HTML header to check for origin "
+    "If set to true, this uses the 'X-Forwarded-For' HTTP header to check for origin "
     "while attempting to verify if the connection request originated from a trusted "
     "domain. Only used if '--trusted_domain' is specified. Warning: Only use this if you "
     "trust the incoming connection to have this set correctly.");
 
 DEFINE_bool(trusted_domain_empty_xff_header_use_origin, false,
-    "If set to true and the 'X-Forwarded-For' HTML header value is empty in the request, "
+    "If set to true and the 'X-Forwarded-For' HTTP header value is empty in the request, "
     "then the origin of the the underlying transport is used while attempting to "
     "verify if the connection request originated from a trusted domain. Only used "
     "if '--trusted_domain' and '--trusted_domain_use_xff_header' flags are specified. "
-    "Warning: In case the 'X-Forwarded-For' HTML header is empty or not in the request, "
+    "Warning: In case the 'X-Forwarded-For' HTTP header is empty or not in the request, "
     "this flag allows a fallback to the default behavior in trusted domain check "
     "(where '--trusted_domain' flag is specified, but '--trusted_domain_use_xff_header' "
     "flag is not set).");
@@ -148,6 +148,14 @@ DEFINE_bool(trusted_domain_strict_localhost, true,
     "important for security, because reverse DNS can resolve other non-local addresses "
     "to localhost.");
 
+DEFINE_bool(use_xff_address_as_origin, false,
+    "If set to true use the address in 'X-Forwarded-For' HTTP header as origin of the "
+    "connection. If XFF header is not set then the Peer Address on the underlying socket "
+    "is used as the origin. If XFF header has multiple addresses corresponding to the "
+    "various intermediate proxies, then the first address is used as the origin. XFF "
+    "header with multiple IP addresses must be comma separated. Only use this if you "
+    "trust the incoming connection to have the XFF header set correctly.");
+
 // This flag must be used with caution to avoid security risks.
 DEFINE_string(trusted_auth_header, "",
     "If set as non empty string, Impala will look for this header in the HTTP headers. "
@@ -210,6 +218,57 @@ DEFINE_bool_hidden(jwt_allow_without_tls, false,
     "When this configuration is set to true, Impala allows JWT authentication on "
     "unsecure channel. This should be only enabled for testing, or development for which "
     "TLS is handled by proxy.");
+
+// OAuth functions
+// If set, Impala will support OAuth based authentication.
+// header.
+DEFINE_bool(oauth_token_auth, false,
+    "When true, read the OAuth token out of the HTTP Header and extract user name from "
+    "the token payload.");
+// The last segment of an OAuth token is the signature, which is used to verify that the
+// token was signed by the sender and not altered in any way. By default, it's required
+// to validate the signature of the OAuth tokens. Otherwise it may expose security issue.
+DEFINE_bool(oauth_jwt_validate_signature, true,
+    "When true, validate the signature of OAuth token with pre-installed JWKS."
+    "This should only be set to false for development / testing");
+// JWKS contains the public keys used by the signing party to the clients that need to
+// validate signatures. It represents cryptographic keys in JSON data structure.
+DEFINE_string(oauth_jwks_file_path, "",
+    "File path of the pre-installed JSON Web Key Set (JWKS) for OAuth verification");
+// This specifies the URL for OAuth to be downloaded.
+DEFINE_string(oauth_jwks_url, "", "URL of the OAuth Endpoint for token verification");
+// Enables retrieving the OAuth JWKS from the specified URL without verifying the
+// presented TLS certificate from the server.
+DEFINE_bool(oauth_jwks_verify_server_certificate, true,
+    "Specifies if the TLS certificate of the JWKS server is verified when retrieving "
+    "the JWKS from the specified JWKS URL.  A certificate is considered valid if a "
+    "trust chain can be established for it, and if the certificate has a common name or "
+    "SAN that matches the server's hostname. This should only be set to false for "
+    "development / testing.");
+// Enables defining a custom pem bundle file containing root certificates to trust.
+DEFINE_string(oauth_jwks_ca_certificate, "", "File path of a pem bundle of root ca "
+    "certificates that will be trusted when retrieving the JWKS from the "
+    "specified JWKS URL.");
+DEFINE_int32(oauth_jwks_update_frequency_s, 60,
+    "(Advanced) The time in seconds to wait for refreshing the OAuth token "
+    "from the OAuth URL.");
+DEFINE_int32(oauth_jwks_pulling_timeout_s, 10,
+    "(Advanced) The time in seconds for connection timed out when verifying OAuth token "
+    "from the specified OAuth server.");
+// This specifies the custom claim in the OAuth token that contains the "username" for
+// the session.
+DEFINE_string(oauth_jwt_custom_claim_username, "username",
+    "Custom claim of the token that "
+    "contains the username");
+// If set, Impala allows OAuth authentication on unsecure channel.
+// OAuth is only secure when used with TLS. But in some deployment scenarios, TLS is
+// handled by proxy so that it does not show up as TLS to Impala.
+DEFINE_bool_hidden(oauth_allow_without_tls, false,
+    "When this configuration is set to true, Impala allows OAuth authentication on "
+    "unsecure channel. This should be only enabled for testing, or development for which "
+    "TLS is handled by proxy.");
+// End OAuth
+
 DEFINE_bool(enable_group_filter_check_for_authenticated_kerberos_user, false,
     "If this configuration is set to true, Impala checks the provided "
     "LDAP group filter, if any, with the authenticated Kerberos user. "
@@ -218,6 +277,9 @@ DEFINE_bool(enable_group_filter_check_for_authenticated_kerberos_user, false,
     "is the same Active Directory server). "
     "The default value is false, which provides backwards-compatible behavior.");
 
+DEFINE_string_hidden(test_cookie, "",
+    "Adds Set-Cookie: <test_cookie> to BasicAuth for testing.");
+
 namespace impala {
 
 // Sasl callbacks.  Why are these here?  Well, Sasl isn't that bright, and
@@ -614,8 +676,8 @@ bool CookieAuth(ThriftServer::ConnectionContext* connection_context,
   return false;
 }
 
-bool GetUsernameFromBasicAuthHeader(
-    ThriftServer::ConnectionContext* connection_context, string& auth_header) {
+static bool GetUsernameFromBasicAuthHeader(
+    ThriftServer::ConnectionContext* connection_context, string auth_header) {
   string stripped_basic_auth_token;
   StripWhiteSpace(&auth_header);
   bool got_basic_auth =
@@ -633,23 +695,43 @@ bool GetUsernameFromBasicAuthHeader(
   return true;
 }
 
-bool TrustedDomainCheck(ThriftServer::ConnectionContext* connection_context,
-    const AuthenticationHash& hash, const std::string& origin, string auth_header) {
+bool SetOrigin(
+    ThriftServer::ConnectionContext* connection_context, const std::string& origin) {
+  connection_context->http_origin = origin;
+  if (FLAGS_use_xff_address_as_origin) {
+    std::string origin_client;
+    Status status = GetXFFOriginClientAddress(origin, origin_client);
+    if (!status.ok()) {
+      LOG(ERROR) << "Error parsing XFF header: " << status;
+    } else {
+      connection_context->network_address = MakeNetworkAddress(origin_client);
+    }
+  }
+  return false;
+}
+
+static bool TrustedDomainCheck(ThriftServer::ConnectionContext* connection_context,
+    const AuthenticationHash& hash, const std::string& origin,
+    const string& auth_header) {
   if (!IsTrustedDomain(origin, FLAGS_trusted_domain,
           FLAGS_trusted_domain_strict_localhost)) {
     return false;
   }
 
-  if (!GetUsernameFromBasicAuthHeader(connection_context, auth_header)) return false;
+  if (!GetUsernameFromBasicAuthHeader(connection_context, auth_header)) {
+    return false;
+  }
   // Create a cookie to return.
   connection_context->return_headers.push_back(
       Substitute("Set-Cookie: $0", GenerateCookie(connection_context->username, hash)));
   return true;
 }
 
-bool HandleTrustedAuthHeader(ThriftServer::ConnectionContext* connection_context,
-    const AuthenticationHash& hash, string auth_header) {
-  if (!GetUsernameFromBasicAuthHeader(connection_context, auth_header)) return false;
+static bool HandleTrustedAuthHeader(ThriftServer::ConnectionContext* connection_context,
+    const AuthenticationHash& hash, const string& auth_header) {
+  if (!GetUsernameFromBasicAuthHeader(connection_context, auth_header)) {
+    return false;
+  }
   // Create a cookie to return.
   connection_context->return_headers.push_back(
       Substitute("Set-Cookie: $0", GenerateCookie(connection_context->username, hash)));
@@ -674,6 +756,10 @@ bool BasicAuth(ThriftServer::ConnectionContext* connection_context,
     // Create a cookie to return.
     connection_context->return_headers.push_back(
         Substitute("Set-Cookie: $0", GenerateCookie(username, hash)));
+    if (!FLAGS_test_cookie.empty()) {
+      connection_context->return_headers.push_back(
+          Substitute("Set-Cookie: $0", FLAGS_test_cookie));
+    }
     return true;
   }
   connection_context->return_headers.push_back("WWW-Authenticate: Basic");
@@ -691,11 +777,14 @@ bool JWTTokenAuth(ThriftServer::ConnectionContext* connection_context,
     return false;
   }
   if (FLAGS_jwt_validate_signature) {
-    status = JWTHelper::GetInstance()->Verify(decoded_token.get());
+    status =  ExecEnv::GetInstance()->GetJWTHelperInstance()->Verify(decoded_token.get());
     if (!status.ok()) {
       LOG(ERROR) << "Error verifying JWT token received from: "
                  << TNetworkAddressToString(connection_context->network_address)
                  << " Error: " << status;
+      connection_context->return_headers.push_back(
+          Substitute("WWW-Authenticate: Bearer error=\"invalid_token\",\
+error_description=\"$0 \"", status.GetDetail()));
       return false;
     }
   }
@@ -715,6 +804,46 @@ bool JWTTokenAuth(ThriftServer::ConnectionContext* connection_context,
   return true;
 }
 
+bool OAuthTokenAuth(ThriftServer::ConnectionContext* connection_context,
+    const AuthenticationHash& hash, const string& token) {
+  JWTHelper::UniqueJWTDecodedToken decoded_token;
+  Status status = JWTHelper::Decode(token, decoded_token);
+  if (!status.ok()) {
+    LOG(ERROR) << "Error decoding OAuth token received from: "
+               << TNetworkAddressToString(connection_context->network_address)
+               << " Error: " << status;
+    return false;
+  }
+  if (FLAGS_oauth_jwt_validate_signature) {
+    status = ExecEnv::GetInstance()->GetOAuthHelperInstance()->Verify(
+        decoded_token.get());
+    if (!status.ok()) {
+      LOG(ERROR) << "Error verifying OAuth token received from: "
+                 << TNetworkAddressToString(connection_context->network_address)
+                 << " Error: " << status;
+      connection_context->return_headers.push_back(
+          Substitute("WWW-Authenticate: Bearer error=\"invalid_token\",\
+error_description=\"$0 \"", status.GetDetail()));
+      return false;
+    }
+  }
+
+  DCHECK(!FLAGS_oauth_jwt_custom_claim_username.empty());
+  string username;
+  status = JWTHelper::GetCustomClaimUsername(
+      decoded_token.get(), FLAGS_oauth_jwt_custom_claim_username, username);
+  if (!status.ok()) {
+    LOG(ERROR) << "Error extracting username from OAuth token received from: "
+               << TNetworkAddressToString(connection_context->network_address)
+               << " Error: " << status;
+    return false;
+  }
+  connection_context->username = username;
+  // TODO: cookies are not added, but are not needed right now
+
+  return true;
+}
+
 // Performs a step of SPNEGO auth for the HTTP transport and sets the username and
 // kerberos_user_principal on 'connection_context' if auth is successful.
 // 'header_token' is the value from an 'Authorization: Negotiate" header.
@@ -752,11 +881,10 @@ bool NegotiateAuth(ThriftServer::ConnectionContext* connection_context,
                     << TNetworkAddressToString(connection_context->network_address)
                     << ": " << spnego_status.ToString();
       } else {
+        string short_user = GetShortUsernameFromKerberosPrincipal(username);
         if (FLAGS_enable_ldap_auth &&
             FLAGS_enable_group_filter_check_for_authenticated_kerberos_user) {
 
-          string short_user = GetShortUsernameFromKerberosPrincipal(username);
-
           LOG(INFO) << "Checking LDAP group filters for "
                     << "username \"" << short_user << "\" "
                     << "parsed from user principal \""
@@ -777,6 +905,7 @@ bool NegotiateAuth(ThriftServer::ConnectionContext* connection_context,
         connection_context->username = username;
         // Save the username as Kerberos user principal in the connection context.
         connection_context->kerberos_user_principal = username;
+        connection_context->kerberos_user_short = short_user;
         // Create a cookie to return.
         connection_context->return_headers.push_back(
             Substitute("Set-Cookie: $0", GenerateCookie(username, hash)));
@@ -1274,7 +1403,7 @@ Status SecureAuthProvider::Start() {
 Status SecureAuthProvider::GetServerTransportFactory(
     ThriftServer::TransportType underlying_transport_type, const std::string& server_name,
     MetricGroup* metrics, std::shared_ptr<TTransportFactory>* factory) {
-  DCHECK(!principal_.empty() || has_ldap_ || has_saml_ || has_jwt_);
+  DCHECK(!principal_.empty() || has_ldap_ || has_saml_ || has_jwt_ || has_oauth_);
 
   if (underlying_transport_type == ThriftServer::HTTP) {
     bool has_kerberos = !principal_.empty();
@@ -1283,7 +1412,7 @@ Status SecureAuthProvider::GetServerTransportFactory(
     bool check_trusted_auth_header = !FLAGS_trusted_auth_header.empty();
     factory->reset(new THttpServerTransportFactory(server_name, metrics, has_ldap_,
         has_kerberos, use_cookies, check_trusted_domain, check_trusted_auth_header,
-        has_saml_, has_jwt_));
+        has_saml_, has_jwt_, has_oauth_));
     return Status::OK();
   }
 
@@ -1364,8 +1493,8 @@ void SecureAuthProvider::SetupConnectionContext(
   TSocket* socket = nullptr;
   switch (underlying_transport_type) {
     case ThriftServer::BINARY: {
-      TBufferedTransport* buffered_transport =
-          down_cast<TBufferedTransport*>(input_transport);
+      ThriftServer::BufferedTransport* buffered_transport =
+          down_cast<ThriftServer::BufferedTransport*>(input_transport);
       TSaslServerTransport* sasl_transport = down_cast<TSaslServerTransport*>(
           buffered_transport->getUnderlyingTransport().get());
       socket = down_cast<TSocket*>(sasl_transport->getUnderlyingTransport().get());
@@ -1375,6 +1504,8 @@ void SecureAuthProvider::SetupConnectionContext(
         // Save the username as Kerberos user principal in the connection context
         // if the actual auth mechanism is Kerberos.
         connection_ptr->kerberos_user_principal = connection_ptr->username;
+        connection_ptr->kerberos_user_short =
+            GetShortUsernameFromKerberosPrincipal(connection_ptr->username);
       }
       break;
     }
@@ -1414,14 +1545,20 @@ void SecureAuthProvider::SetupConnectionContext(
         callbacks.validate_saml2_bearer_fn =
             std::bind(ValidateSaml2Bearer, connection_ptr.get(), hash_);
       }
-      if (has_jwt_) {
+      if (has_jwt_ ) {
         callbacks.jwt_token_auth_fn =
             std::bind(JWTTokenAuth, connection_ptr.get(), hash_, std::placeholders::_1);
       }
+      if (has_oauth_) {
+        callbacks.oauth_token_auth_fn =
+            std::bind(OAuthTokenAuth, connection_ptr.get(), hash_, std::placeholders::_1);
+      }
       if (!FLAGS_trusted_auth_header.empty()) {
         callbacks.trusted_auth_header_handle_fn = std::bind(
             HandleTrustedAuthHeader, connection_ptr.get(), hash_, std::placeholders::_1);
       }
+      callbacks.set_http_origin_fn = std::bind(SetOrigin, connection_ptr.get(),
+          std::placeholders::_1);
       http_input_transport->setCallbacks(callbacks);
       http_output_transport->setCallbacks(callbacks);
       socket = down_cast<TSocket*>(http_input_transport->getUnderlyingTransport().get());
@@ -1468,8 +1605,8 @@ void NoAuthProvider::SetupConnectionContext(
   TSocket* socket = nullptr;
   switch (underlying_transport_type) {
     case ThriftServer::BINARY: {
-      TBufferedTransport* buffered_transport =
-          down_cast<TBufferedTransport*>(input_transport);
+      ThriftServer::BufferedTransport* buffered_transport =
+          down_cast<ThriftServer::BufferedTransport*>(input_transport);
       socket = down_cast<TSocket*>(buffered_transport->getUnderlyingTransport().get());
       break;
     }
@@ -1483,9 +1620,12 @@ void NoAuthProvider::SetupConnectionContext(
           HttpPathFn, connection_ptr.get(), "", std::placeholders::_1,
           std::placeholders::_2, std::placeholders::_3);
       callbacks.return_headers_fn = std::bind(ReturnHeaders, connection_ptr.get());
+      callbacks.set_http_origin_fn = std::bind(SetOrigin, connection_ptr.get(),
+          std::placeholders::_1);
       http_input_transport->setCallbacks(callbacks);
       http_output_transport->setCallbacks(callbacks);
       socket = down_cast<TSocket*>(http_input_transport->getUnderlyingTransport().get());
+
       break;
     }
     default:
@@ -1539,6 +1679,20 @@ Status AuthManager::Init() {
     }
   }
 
+  if (FLAGS_oauth_token_auth) {
+    if (!IsExternalTlsConfigured()) {
+      if (!FLAGS_oauth_allow_without_tls) {
+        return Status("OAuth authentication should be only used with TLS enabled.");
+      }
+      LOG(WARNING) << "OAuth authentication is used without TLS.";
+    }
+    if (FLAGS_oauth_jwt_custom_claim_username.empty()) {
+      return Status(
+          "OAuth authentication requires oauth_jwt_custom_claim_username to be "
+          "specified.");
+    }
+  }
+
   // Get all of the flag validation out of the way
   if (FLAGS_enable_ldap_auth) {
     RETURN_IF_ERROR(
@@ -1623,6 +1777,10 @@ Status AuthManager::Init() {
       LOG(INFO) << "External communication can be also authenticated with JWT";
       sap->InitJwt();
     }
+    if (FLAGS_oauth_token_auth) {
+      LOG(INFO) << "External communication can be also authenticated with OAuth";
+      sap->InitOauth();
+    }
   } else {
     external_auth_provider_.reset(new NoAuthProvider());
     LOG(INFO) << "External communication is not authenticated for binary protocols";
@@ -1632,6 +1790,12 @@ Status AuthManager::Init() {
       sap->InitSaml();
       LOG(INFO) << "External communication is authenticated for hs2-http protocol with "
                    "SAML2 SSO";
+    } else if (FLAGS_oauth_token_auth) {
+      SecureAuthProvider* sap = nullptr;
+      external_http_auth_provider_.reset(sap = new SecureAuthProvider(false));
+      sap->InitOauth();
+      LOG(INFO)
+          << "External communication is authenticated for hs2-http protocol with Oauth";
     } else if (use_jwt) {
       SecureAuthProvider* sap = nullptr;
       external_http_auth_provider_.reset(sap = new SecureAuthProvider(false));
