@@ -59,6 +59,24 @@ class AuthProvider;
 /// TODO: shutdown is buggy (which only harms tests)
 class ThriftServer {
  public:
+
+   /// Override TBufferedTransport::close() to avoid calling flush() which is not safe
+   /// in TSSLSocket if it is in error state. See IMPALA-13680 / THRIFT-5846 for details.
+   class BufferedTransport :
+      public apache::thrift::transport::TBufferedTransport {
+
+    public:
+      BufferedTransport(std::shared_ptr<apache::thrift::transport::TTransport>& transport,
+          uint32_t sz, std::shared_ptr<apache::thrift::TConfiguration>&& config)
+      : apache::thrift::transport::TBufferedTransport(transport, sz, config) {}
+
+      // base implementation:
+      // https://github.com/apache/thrift/blob/d078721e44fea7713832ae5d0f5d9ca67317f19e/lib/cpp/src/thrift/transport/TBufferTransports.h#L367
+      virtual void close() override {
+        transport_->close();
+      }
+  };
+
   /// Transport factory that wraps transports in a buffered transport with a customisable
   /// buffer-size and optionally in another transport from a provided factory. A larger
   /// buffer is usually more efficient, as it allows the underlying transports to perform
@@ -81,7 +99,7 @@ class ThriftServer {
       VerifyMaxMessageSizeInheritance(trans.get(), wrapped.get());
       std::shared_ptr<apache::thrift::transport::TTransport> buffered_wrapped =
           std::shared_ptr<apache::thrift::transport::TTransport>(
-              new apache::thrift::transport::TBufferedTransport(
+              new BufferedTransport(
                   wrapped, buffer_size_, wrapped->getConfiguration()));
       VerifyMaxMessageSizeInheritance(wrapped.get(), buffered_wrapped.get());
       return buffered_wrapped;
@@ -106,6 +124,9 @@ class ThriftServer {
     Username username;
     Username do_as_user;
     TNetworkAddress network_address;
+    /// If using hs2-http protocol, this is the origin of the session
+    /// as recorded in the X-Forwarded-For http message header.
+    std::string http_origin;
     std::string server_name;
     /// Used to pass HTTP headers generated by the input transport to the output transport
     /// to be returned.
@@ -117,6 +138,9 @@ class ThriftServer {
     // Used in case of Kerberos authentication only to store the authenticated Kerberos
     // user principal
     std::string kerberos_user_principal;
+    // Used in case of Kerberos authentication only to store the authenticated Kerberos
+    // user principal's short name
+    std::string kerberos_user_short;
   };
 
   /// Interface class for receiving connection creation / termination events.
@@ -301,6 +325,14 @@ class ThriftServer {
           kudu::security::SecurityDefaults::kDefaultTlsCipherSuites,
       bool disable_tls12 = false);
 
+  /// Sets keepalive options for the client TCP connections. Keepalive is only enabled
+  /// if probe_period_s > 0. These are the three standard keepalive settings for Linux:
+  /// If a client connection is idle for probe_period_s seconds, it starts sending
+  /// keepalives. If it doesn't hear back, it retries every retry_period_s seconds until
+  /// it exceeds retry_count.
+  void SetKeepAliveOptions(int32_t probe_period_s, int32_t retry_period_s,
+      int32_t retry_count);
+
   /// Creates the server socket on which this server listens. May be SSL enabled. Returns
   /// OK unless there was a Thrift error.
   Status CreateSocket(std::shared_ptr<apache::thrift::transport::TServerSocket>* socket);
@@ -400,6 +432,11 @@ class ThriftServer {
   TransportType transport_type_;
 
   bool is_external_facing_;
+
+  /// Keepalive options for client connections.
+  int32_t keepalive_probe_period_s_ = 0;
+  int32_t keepalive_retry_period_s_ = 0;
+  int32_t keepalive_retry_count_ = 0;
 };
 
 /// Helper class to build new ThriftServer instances.
@@ -496,6 +533,19 @@ class ThriftServerBuilder {
     return *this;
   }
 
+  /// Sets keepalive options for the client TCP connections. Keepalive is only enabled
+  /// if probe_period_s > 0. These are the three standard keepalive settings for Linux:
+  /// If a client connection is idle for probe_period_s seconds, it starts sending
+  /// keepalives. If it doesn't hear back, it retries every retry_period_s seconds until
+  /// it exceeds retry_count.
+  ThriftServerBuilder& keepalive(int32_t probe_period_s,
+      int32_t retry_period_s, int32_t retry_count) {
+    keepalive_probe_period_s_ = probe_period_s;
+    keepalive_retry_period_s_ = retry_period_s;
+    keepalive_retry_count_ = retry_count;
+    return *this;
+  }
+
   /// Constructs a new ThriftServer and puts it in 'server', if construction was
   /// successful, returns an error otherwise. In the error case, 'server' will not have
   /// been set and will not need to be freed, otherwise the caller assumes ownership of
@@ -510,6 +560,8 @@ class ThriftServerBuilder {
           version_, certificate_, private_key_, pem_password_cmd_, cipher_list_,
           tls_ciphersuites_, disable_tls12_));
     }
+    ptr->SetKeepAliveOptions(keepalive_probe_period_s_, keepalive_retry_period_s_,
+        keepalive_retry_count_);
     (*server) = ptr.release();
     return Status::OK();
   }
@@ -538,6 +590,9 @@ class ThriftServerBuilder {
       kudu::security::SecurityDefaults::kDefaultTlsCipherSuites;
   bool disable_tls12_ = false;
   bool is_external_facing_ = true;
+  int32_t keepalive_probe_period_s_ = 0;
+  int32_t keepalive_retry_period_s_ = 0;
+  int32_t keepalive_retry_count_ = 0;
 };
 
 /// Contains a map from string for --ssl_minimum_version to Thrift's SSLProtocol.
