@@ -178,7 +178,7 @@ class BufferPool::FreeBufferArena : public CacheLineAligned {
 
   /// Compute a sum over all the lists in the arena. Does not lock the arena.
   int64_t SumOverSizes(
-      std::function<int64_t(PerSizeLists* lists, int64_t buffer_size)> compute_fn);
+      const std::function<int64_t(PerSizeLists* lists, int64_t buffer_size)>& compute_fn);
 
   BufferAllocator* const parent_;
 
@@ -496,7 +496,7 @@ int64_t BufferPool::BufferAllocator::FreeToSystem(vector<BufferHandle>&& buffers
 }
 
 int64_t BufferPool::BufferAllocator::SumOverArenas(
-    std::function<int64_t(FreeBufferArena* arena)> compute_fn) const {
+    const std::function<int64_t(FreeBufferArena* arena)>& compute_fn) const {
   int64_t total = 0;
   for (const unique_ptr<FreeBufferArena>& arena : per_core_arenas_) {
     total += compute_fn(arena.get());
@@ -761,7 +761,7 @@ int BufferPool::FreeBufferArena::GetFreeListSize(int64_t len) {
 }
 
 int64_t BufferPool::FreeBufferArena::SumOverSizes(
-    std::function<int64_t(PerSizeLists* lists, int64_t buffer_size)> compute_fn) {
+    const std::function<int64_t(PerSizeLists* lists, int64_t buffer_size)>& compute_fn) {
   int64_t total = 0;
   for (int i = 0; i < NumBufferSizes(); ++i) {
     int64_t buffer_size = (1L << i) * parent_->min_buffer_len_;
