@@ -92,7 +92,7 @@ class ClientCacheHelper {
   /// returned client will not be present in the per-host cache.
   //
   /// If there is an error creating the new client, *client_key will be NULL.
-  Status GetClient(const TNetworkAddress& address, ClientFactory factory_method,
+  Status GetClient(const TNetworkAddress& address, const ClientFactory& factory_method,
       ClientKey* client_key) WARN_UNUSED_RESULT;
 
   /// Returns a newly-opened client in client_key. May reopen the existing client, or may
@@ -101,7 +101,7 @@ class ClientCacheHelper {
   /// Returns an error status and sets 'client_key' to NULL if a new client cannot
   /// created.
   Status ReopenClient(
-      ClientFactory factory_method, ClientKey* client_key) WARN_UNUSED_RESULT;
+      const ClientFactory& factory_method, ClientKey* client_key) WARN_UNUSED_RESULT;
 
   /// Returns a client to the cache. Upon return, *client_key will be NULL, and the
   /// associated client will be available in the per-host cache.
@@ -130,11 +130,12 @@ class ClientCacheHelper {
   template <class T> friend class ClientCache;
   /// Private constructor so that only ClientCache can instantiate this class.
   ClientCacheHelper(uint32_t num_tries, uint64_t wait_ms, int32_t send_timeout_ms,
-      int32_t recv_timeout_ms)
+      int32_t recv_timeout_ms, int32_t conn_timeout_ms)
       : num_tries_(num_tries),
         wait_ms_(wait_ms),
         send_timeout_ms_(send_timeout_ms),
         recv_timeout_ms_(recv_timeout_ms),
+        conn_timeout_ms_(conn_timeout_ms),
         metrics_enabled_(false) { }
 
   /// There are three lock categories - the cache-wide lock (cache_lock_), the locks for a
@@ -193,6 +194,10 @@ class ClientCacheHelper {
   /// Time to wait for the underlying socket to receive data, e.g., for an RPC response.
   const int32_t recv_timeout_ms_;
 
+  /// Time to wait for setting up underlying TSocket connection. The default value
+  /// equals 0, which is same as the default value of TSocket.connTimeout_.
+  const int32_t conn_timeout_ms_;
+
   /// True if metrics have been registered (i.e. InitMetrics() was called)), and *_metric_
   /// are valid pointers.
   bool metrics_enabled_;
@@ -204,7 +209,7 @@ class ClientCacheHelper {
   IntGauge* total_clients_metric_;
 
   /// Create a new client for specific address in 'client' and put it in client_map_
-  Status CreateClient(const TNetworkAddress& address, ClientFactory factory_method,
+  Status CreateClient(const TNetworkAddress& address, const ClientFactory& factory_method,
       ClientKey* client_key) WARN_UNUSED_RESULT;
 };
 
@@ -410,7 +415,7 @@ class ClientCache {
   typedef ThriftClient<T> Client;
 
   ClientCache(const std::string& service_name = "", bool enable_ssl = false)
-      : client_cache_helper_(1, 0, 0, 0) {
+      : client_cache_helper_(1, 0, 0, 0, 0) {
     client_factory_ = boost::bind<ThriftClientImpl*>(
         boost::mem_fn(&ClientCache::MakeClient), this, _1, _2, service_name, enable_ssl);
   }
@@ -418,11 +423,13 @@ class ClientCache {
   /// Create a ClientCache where connections are tried num_tries times, with a pause of
   /// wait_ms between attempts. The underlying TSocket's send and receive timeouts of
   /// each connection can also be set. If num_tries == 0, retry connections indefinitely.
-  /// A send/receive timeout of 0 means there is no timeout.
+  /// A send/receive/connect timeout of 0 means there is no timeout.
   ClientCache(uint32_t num_tries, uint64_t wait_ms, int32_t send_timeout_ms = 0,
       int32_t recv_timeout_ms = 0, const std::string& service_name = "",
-      bool enable_ssl = false)
-      : client_cache_helper_(num_tries, wait_ms, send_timeout_ms, recv_timeout_ms) {
+      bool enable_ssl = false, int32_t conn_timeout_ms = 0)
+      : client_cache_helper_(
+          num_tries, wait_ms, send_timeout_ms, recv_timeout_ms, conn_timeout_ms) {
+    DCHECK_GE(conn_timeout_ms, 0);
     client_factory_ = boost::bind<ThriftClientImpl*>(
         boost::mem_fn(&ClientCache::MakeClient), this, _1, _2, service_name, enable_ssl);
   }
