@@ -131,6 +131,7 @@ void Coordinator::BackendState::SetRpcParams(const DebugOptions& debug_options,
   fragment_info->__isset.fragment_instance_ctxs = true;
   fragment_info->fragment_instance_ctxs.resize(
       backend_exec_params_.instance_params().size());
+  DCHECK_GT(fragment_info->fragment_instance_ctxs.size(), 0);
   for (int i = 0; i < backend_exec_params_.instance_params().size(); ++i) {
     TPlanFragmentInstanceCtx& instance_ctx = fragment_info->fragment_instance_ctxs[i];
     PlanFragmentInstanceCtxPB* instance_ctx_pb = request->add_fragment_instance_ctxs();
@@ -181,6 +182,10 @@ void Coordinator::BackendState::SetRpcParams(const DebugOptions& debug_options,
     // table construction.
     instance_ctx.__set_filters_produced(produced_it->second);
   }
+  DCHECK_GT(fragment_info->fragments.size(), 0);
+  DCHECK_EQ(fragment_info->fragments.size(), request->fragment_ctxs_size());
+  DCHECK_EQ(fragment_info->fragment_instance_ctxs.size(),
+      request->fragment_instance_ctxs_size());
 }
 
 void Coordinator::BackendState::SetExecError(
@@ -327,6 +332,8 @@ void Coordinator::BackendState::ExecAsync(const DebugOptions& debug_options,
     }
     request.set_query_ctx_sidecar_idx(query_ctx_sidecar_idx);
 
+    CopyFilepathToHostsMappingToRequest(&request);
+
     VLOG_FILE << "making rpc: ExecQueryFInstances"
               << " host=" << impalad_address() << " query_id=" << PrintId(query_id_);
 
@@ -342,6 +349,25 @@ done:
   exec_done_cv_.NotifyAll();
 }
 
+void Coordinator::BackendState::CopyFilepathToHostsMappingToRequest(
+    ExecQueryFInstancesRequestPB* request) const {
+  DCHECK(request != nullptr);
+  google::protobuf::Map<int32, FilepathToHostsMapPB>* by_node_filepath_to_hosts =
+      request->mutable_by_node_filepath_to_hosts();
+  for (const auto& it_nodes : exec_params_.query_schedule().by_node_filepath_to_hosts()) {
+    google::protobuf::Map<string, FilepathToHostsListPB>* filepath_to_hosts =
+        (*by_node_filepath_to_hosts)[it_nodes.first].mutable_filepath_to_hosts();
+    for (const auto& it_files : it_nodes.second.filepath_to_hosts()) {
+      (*filepath_to_hosts)[it_files.first].set_is_relative(
+          it_files.second.is_relative());
+      for (const auto& it_host : it_files.second.hosts()) {
+        auto* hosts = (*filepath_to_hosts)[it_files.first].add_hosts();
+        *hosts = it_host;
+      }
+    }
+  }
+}
+
 Status Coordinator::BackendState::GetStatus(bool* is_fragment_failure,
     TUniqueId* failed_instance_id) {
   lock_guard<mutex> l(lock_);
