@@ -166,8 +166,7 @@ inline bool SlotDescriptor::IsChildOfStruct() const {
 
 ColumnDescriptor::ColumnDescriptor(const TColumnDescriptor& tdesc)
   : name_(tdesc.name),
-    type_(ColumnType::FromThrift(tdesc.type)),
-    aux_type_(tdesc.type) {
+    type_(ColumnType::FromThrift(tdesc.type)) {
   if (tdesc.__isset.icebergFieldId) {
     field_id_ = tdesc.icebergFieldId;
     // Get key and value field_id for Iceberg table column with Map type.
@@ -262,10 +261,9 @@ HdfsTableDescriptor::HdfsTableDescriptor(const TTableDescriptor& tdesc, ObjectPo
     const TIcebergPartitionSpec& spec = iceberg_partition_specs_[iceberg_spec_id_];
     DCHECK_EQ(spec.spec_id, iceberg_spec_id_);
     for (const TIcebergPartitionField& spec_field : spec.partition_fields) {
-      if (spec_field.transform.transform_type == TIcebergPartitionTransformType::VOID) {
-        continue;
-      }
-      iceberg_non_void_partition_names_.push_back(spec_field.field_name);
+      auto transform_type = spec_field.transform.transform_type;
+      if (transform_type == TIcebergPartitionTransformType::VOID) continue;
+      iceberg_non_void_partition_fields_.push_back(spec_field);
     }
     iceberg_parquet_compression_codec_ = tdesc.icebergTable.parquet_compression_codec;
     iceberg_parquet_row_group_size_ = tdesc.icebergTable.parquet_row_group_size;
@@ -344,6 +342,16 @@ string KuduTableDescriptor::DebugString() const {
   return out.str();
 }
 
+SystemTableDescriptor::SystemTableDescriptor(const TTableDescriptor& tdesc)
+  : TableDescriptor(tdesc), table_name_(tdesc.systemTable.table_name) {}
+
+string SystemTableDescriptor::DebugString() const {
+  stringstream out;
+  out << "SystemTable(" << TableDescriptor::DebugString() << " table=" << table_name_
+      << ")";
+  return out.str();
+}
+
 TupleDescriptor::TupleDescriptor(const TTupleDescriptor& tdesc)
   : id_(tdesc.id),
     byte_size_(tdesc.byteSize),
@@ -599,6 +607,9 @@ Status DescriptorTbl::CreateTblDescriptorInternal(const TTableDescriptor& tdesc,
     case TTableType::KUDU_TABLE:
       *desc = pool->Add(new KuduTableDescriptor(tdesc));
       break;
+    case TTableType::SYSTEM_TABLE:
+      *desc = pool->Add(new SystemTableDescriptor(tdesc));
+      break;
     default:
       DCHECK(false) << "invalid table type: " << tdesc.tableType;
   }
@@ -642,7 +653,7 @@ Status DescriptorTbl::CreateInternal(ObjectPool* pool, const TDescriptorTable& t
     if (slot_d->type().IsStructType() && children_tuple_descriptor != nullptr &&
         children_tuple_descriptor->getMasterTuple() == nullptr) {
       TupleDescriptor* master_tuple = parent;
-      // If this struct is nested into another structs then get the topmost tuple for the
+      // If this struct is nested into other struct(s) then get the topmost tuple for the
       // master.
       if (parent->getMasterTuple() != nullptr) master_tuple = parent->getMasterTuple();
       children_tuple_descriptor->setMasterTuple(master_tuple);
@@ -699,9 +710,18 @@ void SlotDescriptor::CodegenLoadAnyVal(CodegenAnyVal* any_val, llvm::Value* raw_
     case TYPE_STRING:
     case TYPE_VARCHAR: {
       // Convert StringValue to StringVal
-      llvm::Value* string_value = builder->CreateLoad(raw_val_ptr, "string_value");
-      any_val->SetPtr(builder->CreateExtractValue(string_value, 0, "ptr"));
-      any_val->SetLen(builder->CreateExtractValue(string_value, 1, "len"));
+      llvm::Function* str_ptr_fn = codegen->GetFunction(
+          IRFunction::STRING_VALUE_PTR, false);
+      llvm::Function* str_len_fn = codegen->GetFunction(
+          IRFunction::STRING_VALUE_LEN, false);
+
+      llvm::Value* ptr = builder->CreateCall(str_ptr_fn,
+          llvm::ArrayRef<llvm::Value*>({raw_val_ptr}), "ptr");
+      llvm::Value* len = builder->CreateCall(str_len_fn,
+          llvm::ArrayRef<llvm::Value*>({raw_val_ptr}), "len");
+
+      any_val->SetPtr(ptr);
+      any_val->SetLen(len);
       break;
     }
     case TYPE_CHAR:
@@ -814,7 +834,7 @@ void SlotDescriptor::CodegenSetNullIndicator(
 // Example IR for materializing a string column with non-NULL 'pool'. Includes the part
 // that is generated by CodegenAnyVal::ToReadWriteInfo().
 //
-// Produced for the following query:
+// Produced for the following query as part of the @MaterializeExprs() function.
 //   select string_col from functional_orc_def.alltypes order by string_col limit 2;
 //
 //   ; [insert point starts here]
@@ -903,7 +923,7 @@ void SlotDescriptor::CodegenWriteToSlotHelper(
     const CodegenAnyValReadWriteInfo& read_write_info,
     llvm::Value* main_tuple_llvm_struct_ptr, llvm::Value* tuple_llvm_struct_ptr,
     llvm::Value* pool_val,
-    NonWritableBasicBlock insert_before) const {
+    const NonWritableBasicBlock& insert_before) const {
   DCHECK(main_tuple_llvm_struct_ptr->getType()->isPointerTy());
   DCHECK(main_tuple_llvm_struct_ptr->getType()->getPointerElementType()->isStructTy());
   DCHECK(tuple_llvm_struct_ptr->getType()->isPointerTy());
@@ -918,7 +938,7 @@ void SlotDescriptor::CodegenWriteToSlotHelper(
     CodegenStoreStructToNativePtr(read_write_info, main_tuple_llvm_struct_ptr,
         slot, pool_val, insert_before);
   } else {
-    CodegenStoreNonNullAnyVal(read_write_info, slot, pool_val, this);
+    CodegenStoreNonNullAnyVal(read_write_info, slot, pool_val, this, insert_before);
 
     // We only need this branch if we are not a struct, because for structs, the last leaf
     // (non-struct) field will add this branch.
@@ -934,7 +954,7 @@ void SlotDescriptor::CodegenWriteToSlotHelper(
 void SlotDescriptor::CodegenStoreStructToNativePtr(
     const CodegenAnyValReadWriteInfo& read_write_info, llvm::Value* main_tuple_ptr,
     llvm::Value* struct_slot_ptr, llvm::Value* pool_val,
-    NonWritableBasicBlock insert_before) const {
+    const NonWritableBasicBlock& insert_before) const {
   DCHECK(type_.IsStructType());
   DCHECK(children_tuple_descriptor_ != nullptr);
   DCHECK(read_write_info.type().IsStructType());
@@ -1002,6 +1022,7 @@ CodegenAnyValReadWriteInfo CodegenAnyValToReadWriteInfo(CodegenAnyVal& any_val,
     case TYPE_STRUCT:
       DCHECK(false) << "Invalid type for this function. "
                     << "Call 'StoreStructToNativePtr()' instead.";
+      break;
     default:
       DCHECK(false) << "NYI: " << rwi.type().DebugString();
       break;
@@ -1012,23 +1033,24 @@ CodegenAnyValReadWriteInfo CodegenAnyValToReadWriteInfo(CodegenAnyVal& any_val,
 
 void SlotDescriptor::CodegenStoreNonNullAnyVal(CodegenAnyVal& any_val,
       llvm::Value* raw_val_ptr, llvm::Value* pool_val,
-      const SlotDescriptor* slot_desc) {
+      const SlotDescriptor* slot_desc, const NonWritableBasicBlock& insert_before) {
   CodegenAnyValReadWriteInfo rwi = CodegenAnyValToReadWriteInfo(any_val, pool_val);
-  CodegenStoreNonNullAnyVal(rwi, raw_val_ptr, pool_val, slot_desc);
+  CodegenStoreNonNullAnyVal(rwi, raw_val_ptr, pool_val, slot_desc, insert_before);
 }
 
 void SlotDescriptor::CodegenStoreNonNullAnyVal(
     const CodegenAnyValReadWriteInfo& read_write_info, llvm::Value* raw_val_ptr,
-    llvm::Value* pool_val, const SlotDescriptor* slot_desc) {
+    llvm::Value* pool_val, const SlotDescriptor* slot_desc,
+    const NonWritableBasicBlock& insert_before) {
   LlvmBuilder* builder = read_write_info.builder();
   const ColumnType& type = read_write_info.type();
   switch (type.type) {
     case TYPE_STRING:
     case TYPE_VARCHAR:
-    case TYPE_ARRAY: // CollectionVal has same memory layout as StringVal.
-    case TYPE_MAP: { // CollectionVal has same memory layout as StringVal.
+    case TYPE_ARRAY:
+    case TYPE_MAP: {
       CodegenWriteStringOrCollectionToSlot(read_write_info, raw_val_ptr,
-          pool_val, slot_desc);
+          pool_val, slot_desc, insert_before);
       break;
     }
     case TYPE_CHAR:
@@ -1064,6 +1086,7 @@ void SlotDescriptor::CodegenStoreNonNullAnyVal(
     case TYPE_STRUCT:
       DCHECK(false) << "Invalid type for this function. "
                     << "Call 'StoreStructToNativePtr()' instead.";
+      break;
     default:
       DCHECK(false) << "NYI: " << type.DebugString();
       break;
@@ -1099,42 +1122,401 @@ void SlotDescriptor::CodegenSetToNull(const CodegenAnyValReadWriteInfo& read_wri
   }
 }
 
+// Example IR for materializing an int and an array<string> column with non-NULL 'pool'.
+// Includes the part that is generated by CodegenAnyVal::ToReadWriteInfo().
+//
+// Produced for the following query as part of the @MaterializeExprs() function.
+//   select id, arr_string_1d from functional_parquet.collection_tbl order by id limit 2;
+//
+//   ; [insert point starts here]
+//   br label %entry1
+//
+// entry1:                               ; preds = %entry
+//   %is_null = trunc i64 %src to i1
+//   br i1 %is_null, label %null, label %non_null
+//
+// non_null:                             ; preds = %entry1
+//   %1 = ashr i64 %src, 32
+//   %2 = trunc i64 %1 to i32
+//   %slot = getelementptr inbounds <{ %"struct.impala::CollectionValue", i32, i8 }>,
+//       <{ %"struct.impala::CollectionValue", i32, i8 }>* %tuple, i32 0, i32 1
+//   store i32 %2, i32* %slot
+//   br label %end_write
+//
+// null:                                 ; preds = %entry1
+//   %3 = bitcast <{ %"struct.impala::CollectionValue", i32, i8 }>* %tuple to i8*
+//   %null_byte_ptr = getelementptr inbounds i8, i8* %3, i32 16
+//   %null_byte = load i8, i8* %null_byte_ptr
+//   %null_bit_set = or i8 %null_byte, 2
+//   store i8 %null_bit_set, i8* %null_byte_ptr
+//   br label %end_write
+//
+// end_write:                            ; preds = %null, %non_null
+//   %4 = getelementptr %"class.impala::ScalarExprEvaluator"*,
+//       %"class.impala::ScalarExprEvaluator"** %slot_materialize_expr_evals, i32 1
+//   %expr_eval2 = load %"class.impala::ScalarExprEvaluator"*,
+//       %"class.impala::ScalarExprEvaluator"** %4
+//   %src3 = call { i64, i8* } @GetSlotRef.4(
+//       %"class.impala::ScalarExprEvaluator"* %expr_eval2,
+//       %"class.impala::TupleRow"* %row)
+//   br label %entry4
+//
+// entry4:                               ; preds = %end_write
+//   %5 = extractvalue { i64, i8* } %src3, 0
+//   %is_null7 = trunc i64 %5 to i1
+//   br i1 %is_null7, label %null6, label %non_null5
+//
+// non_null5:                            ; preds = %entry4
+//   %src8 = extractvalue { i64, i8* } %src3, 1
+//   %6 = extractvalue { i64, i8* } %src3, 0
+//   %7 = ashr i64 %6, 32
+//   %8 = trunc i64 %7 to i32
+//   %slot10 = getelementptr inbounds <{ %"struct.impala::CollectionValue", i32, i8 }>,
+//       <{ %"struct.impala::CollectionValue", i32, i8 }>* %tuple, i32 0, i32 0
+//   %9 = insertvalue %"struct.impala::CollectionValue" zeroinitializer, i32 %8, 1
+//   %coll_tuple_byte_len = mul i32 %8, 13
+//   %10 = sext i32 %coll_tuple_byte_len to i64
+//   %new_coll_val_ptr = call i8* @_ZN6impala7MemPool8AllocateILb0EEEPhli(
+//       %"class.impala::MemPool"* %pool, i64 %10, i32 8)
+//   call void @llvm.memcpy.p0i8.p0i8.i32(i8* %new_coll_val_ptr, i8* %src8,
+//       i32 %coll_tuple_byte_len, i32 0, i1 false)
+//   %11 = insertvalue %"struct.impala::CollectionValue" %9, i8* %new_coll_val_ptr, 0
+//   store i32 0, i32* %item_index_addr
+//   br label %loop_condition_block
+//
+// null6:                                ; preds = %entry4
+//   %12 = bitcast <{ %"struct.impala::CollectionValue", i32, i8 }>* %tuple to i8*
+//   %null_byte_ptr14 = getelementptr inbounds i8, i8* %12, i32 16
+//   %null_byte15 = load i8, i8* %null_byte_ptr14
+//   %null_bit_set16 = or i8 %null_byte15, 1
+//   store i8 %null_bit_set16, i8* %null_byte_ptr14
+//   br label %end_write9
+//
+// loop_condition_block:                 ; preds = %loop_increment_block, %non_null5
+//   %item_index = load i32, i32* %item_index_addr
+//   %continue_loop = icmp slt i32 %item_index, %8
+//   br i1 %continue_loop, label %loop_body_block, label %loop_exit_block
+//
+// loop_body_block:                      ; preds = %loop_condition_block
+//   %children_tuple_array = bitcast i8* %new_coll_val_ptr
+//       to <{ %"class.impala::StringValue", i8 }>*
+//   %children_tuple = getelementptr inbounds <{ %"class.impala::StringValue", i8 }>,
+//       <{ %"class.impala::StringValue", i8 }>* %children_tuple_array, i32 %item_index
+//   %13 = bitcast <{ %"class.impala::StringValue", i8 }>* %children_tuple to i8*
+//   %null_byte_ptr11 = getelementptr inbounds i8, i8* %13, i32 12
+//   %null_byte12 = load i8, i8* %null_byte_ptr11
+//   %null_mask = and i8 %null_byte12, 1
+//   %is_null13 = icmp ne i8 %null_mask, 0
+//   br i1 %is_null13, label %next_block_after_child_is_written,
+//       label %child_non_null_block
+//
+// loop_increment_block:                 ; preds = %next_block_after_child_is_written
+//   %item_index_incremented = add i32 %item_index, 1
+//   store i32 %item_index_incremented, i32* %item_index_addr
+//   br label %loop_condition_block
+//
+// loop_exit_block:                      ; preds = %loop_condition_block
+//   store %"struct.impala::CollectionValue" %11,
+//       %"struct.impala::CollectionValue"* %slot10
+//   br label %end_write9
+//
+// child_non_null_block:                 ; preds = %loop_body_block
+//   %child_str_or_coll_value_addr = getelementptr inbounds
+//       <{ %"class.impala::StringValue", i8 }>,
+//       <{ %"class.impala::StringValue", i8 }>* %children_tuple, i32 0, i32 0
+//   %child_str_or_coll_value_ptr = call i8* @_ZNK6impala11StringValue5IrPtrEv(
+//       %"class.impala::StringValue"* %child_str_or_coll_value_addr)
+//   %child_str_or_coll_value_len = call i32 @_ZNK6impala11StringValue5IrLenEv(
+//       %"class.impala::StringValue"* %child_str_or_coll_value_addr)
+//   %14 = sext i32 %child_str_or_coll_value_len to i64
+//   %new_ptr = call i8* @_ZN6impala7MemPool8AllocateILb0EEEPhli(
+//       %"class.impala::MemPool"* %pool, i64 %14, i32 8)
+//   call void @llvm.memcpy.p0i8.p0i8.i32(i8* %new_ptr, i8* %child_str_or_coll_value_ptr,
+//       i32 %child_str_or_coll_value_len, i32 0, i1 false)
+//   call void @_ZN6impala11StringValue8IrAssignEPci(
+//       %"class.impala::StringValue"* %child_str_or_coll_value_addr, i8* %new_ptr,
+//       i32 %child_str_or_coll_value_len)
+//   br label %next_block_after_child_is_written
+//
+// next_block_after_child_is_written:    ; preds = %child_non_null_block, %loop_body_block
+//   br label %loop_increment_block
+//
+// end_write9:                           ; preds = %null6, %loop_exit_block
+//   ; [insert point ends here]
 void SlotDescriptor::CodegenWriteStringOrCollectionToSlot(
+    const CodegenAnyValReadWriteInfo& read_write_info,
+    llvm::Value* slot_ptr, llvm::Value* pool_val, const SlotDescriptor* slot_desc,
+    const NonWritableBasicBlock& insert_before) {
+  const ColumnType& type = read_write_info.type();
+  if (type.IsStringType()) {
+    CodegenWriteStringToSlot(read_write_info, slot_ptr, pool_val, slot_desc);
+  } else {
+    DCHECK(type.IsCollectionType());
+    CodegenWriteCollectionToSlot(read_write_info, slot_ptr, pool_val, slot_desc,
+        insert_before);
+  }
+}
+
+namespace {
+constexpr int COLL_VALUE_PTR_IDX = 0;
+constexpr int COLL_VALUE_LEN_IDX = 1;
+
+llvm::Value* CodegenStrOrCollValueGetPtr(LlvmCodeGen* codegen, LlvmBuilder* builder,
+    llvm::Value* str_or_coll_value_addr, const string& name = "") {
+  if (str_or_coll_value_addr->getType() ==
+      codegen->GetStructType<StringValue>()->getPointerTo()) {
+    llvm::Function* str_ptr_fn = codegen->GetFunction(
+        IRFunction::STRING_VALUE_PTR, false);
+    return builder->CreateCall(str_ptr_fn,
+        llvm::ArrayRef<llvm::Value*>({str_or_coll_value_addr}), name);
+  } else {
+    DCHECK(str_or_coll_value_addr->getType() ==
+        codegen->GetStructType<CollectionValue>()->getPointerTo());
+    llvm::Value* ptr_addr = builder->CreateStructGEP(nullptr, str_or_coll_value_addr,
+        COLL_VALUE_PTR_IDX, name + "_addr");
+    return builder->CreateLoad(ptr_addr, name);
+  }
+}
+
+llvm::Value* CodegenStrOrCollValueGetLen(LlvmCodeGen* codegen, LlvmBuilder* builder,
+    llvm::Value* str_or_coll_value_addr, const string& name = "") {
+  if (str_or_coll_value_addr->getType() ==
+      codegen->GetStructType<StringValue>()->getPointerTo()) {
+    llvm::Function* str_len_fn = codegen->GetFunction(
+        IRFunction::STRING_VALUE_LEN, false);
+    return builder->CreateCall(str_len_fn,
+        llvm::ArrayRef<llvm::Value*>({str_or_coll_value_addr}), name);
+  } else {
+    DCHECK(str_or_coll_value_addr->getType() ==
+        codegen->GetStructType<CollectionValue>()->getPointerTo());
+    llvm::Value* len_addr = builder->CreateStructGEP(nullptr, str_or_coll_value_addr,
+        COLL_VALUE_LEN_IDX, name + "_addr");
+    return builder->CreateLoad(len_addr, name);
+  }
+}
+
+llvm::Value* CodegenCollValueSetPtr(LlvmCodeGen* codegen, LlvmBuilder* builder,
+    llvm::Value* str_or_coll_value, llvm::Value* ptr, const string& name = "") {
+  DCHECK(str_or_coll_value->getType() == codegen->GetStructType<CollectionValue>());
+  return builder->CreateInsertValue(str_or_coll_value, ptr, COLL_VALUE_PTR_IDX,
+      name);
+}
+
+llvm::Value* CodegenCollValueSetLen(LlvmCodeGen* codegen, LlvmBuilder* builder,
+    llvm::Value* str_or_coll_value, llvm::Value* len, const string& name = "") {
+  DCHECK(str_or_coll_value->getType() == codegen->GetStructType<CollectionValue>());
+  return builder->CreateInsertValue(str_or_coll_value, len, COLL_VALUE_LEN_IDX,
+      name);
+}
+} /* anonymous namespace */
+
+void SlotDescriptor::CodegenWriteCollectionToSlot(
+    const CodegenAnyValReadWriteInfo& read_write_info,
+    llvm::Value* slot_ptr, llvm::Value* pool_val, const SlotDescriptor* slot_desc,
+    const NonWritableBasicBlock& insert_before) {
+  LlvmCodeGen* codegen = read_write_info.codegen();
+  LlvmBuilder* builder = read_write_info.builder();
+  const ColumnType& type = read_write_info.type();
+  DCHECK(type.IsCollectionType());
+
+  // Convert to 'CollectionValue'.
+  llvm::Type* raw_type = codegen->GetSlotType(type);
+  llvm::Value* coll_value = llvm::Constant::getNullValue(raw_type);
+  coll_value = CodegenCollValueSetLen(codegen, builder, coll_value,
+      read_write_info.GetPtrAndLen().len);
+  if (pool_val != nullptr) {
+    llvm::Value* num_tuples = read_write_info.GetPtrAndLen().len;
+    DCHECK(slot_desc != nullptr) << "SlotDescriptor needed to calculate the size of "
+        << "the collection for copying.";
+    // For a 'CollectionValue', 'len' is not the byte size of the whole data but the
+    // number of items, so we have to multiply it with the byte size of the item tuple
+    // to get the data size.
+    int item_tuple_byte_size = slot_desc->children_tuple_descriptor()->byte_size();
+    llvm::Value* byte_len = builder->CreateMul(num_tuples,
+        codegen->GetI32Constant(item_tuple_byte_size), "coll_tuple_byte_len");
+
+    // Allocate a 'new_ptr' from 'pool_val' and copy the data from 'read_write_info->ptr'.
+    llvm::Value* new_ptr = codegen->CodegenMemPoolAllocate(
+        builder, pool_val, byte_len, "new_coll_val_ptr");
+    codegen->CodegenMemcpy(builder, new_ptr, read_write_info.GetPtrAndLen().ptr,
+        byte_len);
+    coll_value = CodegenCollValueSetPtr(codegen, builder, coll_value, new_ptr);
+
+    slot_desc->CodegenWriteCollectionItemsToSlot(codegen, builder, new_ptr,
+        read_write_info.GetPtrAndLen().len, pool_val, insert_before);
+  } else {
+    coll_value = CodegenCollValueSetPtr(codegen, builder, coll_value,
+        read_write_info.GetPtrAndLen().ptr);
+  }
+  builder->CreateStore(coll_value, slot_ptr);
+}
+
+void SlotDescriptor::CodegenWriteStringToSlot(
     const CodegenAnyValReadWriteInfo& read_write_info,
     llvm::Value* slot_ptr, llvm::Value* pool_val, const SlotDescriptor* slot_desc) {
   LlvmCodeGen* codegen = read_write_info.codegen();
   LlvmBuilder* builder = read_write_info.builder();
   const ColumnType& type = read_write_info.type();
-  DCHECK(type.IsStringType() || type.IsCollectionType());
+  DCHECK(type.IsStringType());
 
-  // Convert to StringValue/CollectionValue
-  llvm::Type* raw_type = codegen->GetSlotType(type);
-  llvm::Value* str_or_coll_value = llvm::Constant::getNullValue(raw_type);
-  str_or_coll_value = builder->CreateInsertValue(
-      str_or_coll_value, read_write_info.GetPtrAndLen().len, 1);
+  llvm::Value* ptr = read_write_info.GetPtrAndLen().ptr;
+  llvm::Value* len = read_write_info.GetPtrAndLen().len;
   if (pool_val != nullptr) {
-    llvm::Value* len = read_write_info.GetPtrAndLen().len;
-    if (type.IsCollectionType()) {
-      DCHECK(slot_desc != nullptr) << "SlotDescriptor needed to calculate the size of "
-          << "the collection for copying.";
-      // For a 'CollectionValue', 'len' is not the byte size of the whole data but the
-      // number of items, so we have to multiply it with the byte size of the item tuple
-      // to get the data size.
-      int item_tuple_byte_size = slot_desc->children_tuple_descriptor()->byte_size();
-      len = builder->CreateMul(len, codegen->GetI32Constant(item_tuple_byte_size));
-    }
-
-    // Allocate a 'new_ptr' from 'pool_val' and copy the data from
-    // 'read_write_info->ptr'
+    // Allocate a 'new_ptr' from 'pool_val' and copy the data from 'read_write_info->ptr'.
     llvm::Value* new_ptr = codegen->CodegenMemPoolAllocate(
         builder, pool_val, len, "new_ptr");
-    codegen->CodegenMemcpy(builder, new_ptr, read_write_info.GetPtrAndLen().ptr, len);
-    str_or_coll_value = builder->CreateInsertValue(str_or_coll_value, new_ptr, 0);
-  } else {
-    str_or_coll_value = builder->CreateInsertValue(
-        str_or_coll_value, read_write_info.GetPtrAndLen().ptr, 0);
+    codegen->CodegenMemcpy(builder, new_ptr, ptr, len);
+    ptr = new_ptr;
   }
-  builder->CreateStore(str_or_coll_value, slot_ptr);
+  llvm::Function* str_assign_fn = codegen->GetFunction(
+      IRFunction::STRING_VALUE_ASSIGN, false);
+  builder->CreateCall(str_assign_fn,
+      llvm::ArrayRef<llvm::Value*>({slot_ptr, ptr, len}));
+}
+
+void SlotDescriptor::CodegenWriteCollectionItemsToSlot(LlvmCodeGen* codegen,
+    LlvmBuilder* builder, llvm::Value* collection_value_ptr, llvm::Value* num_tuples,
+    llvm::Value* pool_val, const NonWritableBasicBlock& insert_before) const {
+  DCHECK(pool_val != nullptr);
+  // We construct a while-like loop using basic blocks and conditional branches to iterate
+  // through the items of the collection, recursively.
+  llvm::Function* fn = builder->GetInsertBlock()->getParent();
+  llvm::BasicBlock* loop_condition_block = insert_before.CreateBasicBlockBefore(
+      codegen->context(), "loop_condition_block", fn);
+  llvm::BasicBlock* loop_body_block = insert_before.CreateBasicBlockBefore(
+      codegen->context(), "loop_body_block", fn);
+  llvm::BasicBlock* loop_increment_block = insert_before.CreateBasicBlockBefore(
+      codegen->context(), "loop_increment_block", fn);
+  llvm::BasicBlock* loop_exit_block = insert_before.CreateBasicBlockBefore(
+      codegen->context(), "loop_exit_block", fn);
+
+  // Initialise the loop counter.
+  llvm::Value* item_index_addr = codegen->CreateEntryBlockAlloca(
+      *builder, codegen->i32_type(), "item_index_addr");
+  builder->CreateStore(codegen->GetI32Constant(0), item_index_addr);
+
+  builder->CreateBr(loop_condition_block);
+
+  // Loop condition block
+  builder->SetInsertPoint(loop_condition_block);
+  llvm::Value* item_index = builder->CreateLoad(item_index_addr, "item_index");
+  llvm::Value* continue_loop = builder->CreateICmpSLT(
+      item_index, num_tuples, "continue_loop");
+  builder->CreateCondBr(continue_loop, loop_body_block, loop_exit_block);
+
+  // Loop body
+  builder->SetInsertPoint(loop_body_block);
+  CodegenWriteCollectionItemLoopBody(codegen, builder, collection_value_ptr, num_tuples,
+      item_index, fn, insert_before, pool_val);
+  builder->CreateBr(loop_increment_block);
+
+  // Loop increment
+  builder->SetInsertPoint(loop_increment_block);
+  llvm::Value* item_index_incremented = builder->CreateAdd(
+      item_index, codegen->GetI32Constant(1), "item_index_incremented");
+  builder->CreateStore(item_index_incremented, item_index_addr);
+  builder->CreateBr(loop_condition_block);
+
+  // Loop exit
+  builder->SetInsertPoint(loop_exit_block);
+}
+
+void SlotDescriptor::CodegenWriteCollectionItemLoopBody(LlvmCodeGen* codegen,
+    LlvmBuilder* builder, llvm::Value* collection_value_ptr, llvm::Value* num_tuples,
+    llvm::Value* item_index, llvm::Function* fn,
+    const NonWritableBasicBlock& insert_before, llvm::Value* pool_val) const {
+  DCHECK(pool_val != nullptr);
+  const TupleDescriptor* children_tuple_desc = children_tuple_descriptor();
+  DCHECK(children_tuple_desc != nullptr);
+
+  llvm::Type* children_tuple_struct_type = children_tuple_desc->GetLlvmStruct(codegen);
+  DCHECK(children_tuple_struct_type != nullptr);
+  llvm::PointerType* children_tuple_type = codegen->GetPtrType(
+      children_tuple_struct_type);
+
+  llvm::Value* children_tuple_array = builder->CreateBitCast(collection_value_ptr,
+      children_tuple_type, "children_tuple_array");
+  llvm::Value* children_tuple = builder->CreateInBoundsGEP(children_tuple_array,
+      item_index, "children_tuple");
+
+  CodegenWriteCollectionIterateOverChildren(codegen, builder, children_tuple,
+      children_tuple, fn, insert_before, pool_val);
+}
+
+void SlotDescriptor::CodegenWriteCollectionIterateOverChildren(LlvmCodeGen* codegen,
+    LlvmBuilder* builder, llvm::Value* master_tuple, llvm::Value* children_tuple,
+    llvm::Function* fn, const NonWritableBasicBlock& insert_before,
+    llvm::Value* pool_val) const {
+  DCHECK(pool_val != nullptr);
+  const TupleDescriptor* children_tuple_desc = children_tuple_descriptor();
+  DCHECK(children_tuple_desc != nullptr);
+
+  for (const SlotDescriptor* child_slot_desc : children_tuple_desc->slots()) {
+    DCHECK(child_slot_desc != nullptr);
+
+    const ColumnType& child_type = child_slot_desc->type();
+    if (child_type.IsVarLenStringType() || child_type.IsCollectionType()) {
+      child_slot_desc->CodegenWriteCollectionVarlenChild(codegen, builder, master_tuple,
+          children_tuple, fn, insert_before, pool_val);
+    } else if (child_type.IsStructType()) {
+      child_slot_desc->CodegenWriteCollectionStructChild(codegen, builder,
+          master_tuple, children_tuple, fn, insert_before, pool_val);
+    }
+  }
+}
+
+void SlotDescriptor::CodegenWriteCollectionStructChild(LlvmCodeGen* codegen,
+    LlvmBuilder* builder, llvm::Value* master_tuple, llvm::Value* tuple,
+    llvm::Function* fn, const NonWritableBasicBlock& insert_before,
+    llvm::Value* pool_val) const {
+  DCHECK(type().IsStructType());
+
+  const TupleDescriptor* children_tuple_desc = children_tuple_descriptor();
+  DCHECK(children_tuple_desc != nullptr);
+
+  llvm::Value* children_tuple = builder->CreateStructGEP(nullptr, tuple,
+      llvm_field_idx(), "struct_children_tuple");
+
+  // TODO IMPALA-12775: Check whether the struct itself is NULL.
+  CodegenWriteCollectionIterateOverChildren(codegen, builder, master_tuple,
+      children_tuple, fn, insert_before, pool_val);
+}
+
+void SlotDescriptor::CodegenWriteCollectionVarlenChild(LlvmCodeGen* codegen,
+    LlvmBuilder* builder, llvm::Value* master_tuple, llvm::Value* children_tuple,
+    llvm::Function* fn, const NonWritableBasicBlock& insert_before,
+    llvm::Value* pool_val) const {
+  DCHECK(pool_val != nullptr);
+  DCHECK(type_.IsVarLenStringType() || type_.IsCollectionType());
+
+  llvm::BasicBlock* child_non_null_block = insert_before.CreateBasicBlockBefore(
+      codegen->context(), "child_non_null_block", fn);
+  llvm::BasicBlock* child_written_block = insert_before.CreateBasicBlockBefore(
+      codegen->context(), "next_block_after_child_is_written", fn);
+
+  llvm::Value* child_is_null = CodegenIsNull(codegen, builder, master_tuple);
+  builder->CreateCondBr(child_is_null, child_written_block, child_non_null_block);
+
+  // Note: Although the input of CodegenWriteStringOrCollectionToSlot() is a '*Val', not a
+  // '*Value', the items of a collection are still '*Value' objects, because the pointer
+  // of the collection points to an array of tuples (the items). 'StringValue' has Small
+  // String Optimisation, but smallness is not preserved here: even if the 'StringValue'
+  // was originally small, the new copy will be a long string.
+  builder->SetInsertPoint(child_non_null_block);
+  llvm::Value* child_str_or_coll_value_slot = builder->CreateStructGEP(nullptr,
+      children_tuple, llvm_field_idx(), "child_str_or_coll_value_addr");
+  llvm::Value* child_str_or_coll_value_ptr = CodegenStrOrCollValueGetPtr(codegen, builder,
+      child_str_or_coll_value_slot, "child_str_or_coll_value_ptr");
+  llvm::Value* child_str_or_coll_value_len = CodegenStrOrCollValueGetLen(codegen, builder,
+      child_str_or_coll_value_slot, "child_str_or_coll_value_len");
+
+  CodegenAnyValReadWriteInfo child_rwi(codegen, builder, type());
+  child_rwi.SetPtrAndLen(child_str_or_coll_value_ptr, child_str_or_coll_value_len);
+
+  CodegenWriteStringOrCollectionToSlot(child_rwi, child_str_or_coll_value_slot, pool_val,
+      this, insert_before);
+  builder->CreateBr(child_written_block);
+  builder->SetInsertPoint(child_written_block);
 }
 
 llvm::Value* SlotDescriptor::CodegenToTimestampValue(
@@ -1225,6 +1607,7 @@ llvm::StructType* TupleDescriptor::CreateLlvmStructTypeFromFieldTypes(
   // 16 bytes although their useful payload is only 12 bytes.
   llvm::StructType* tuple_struct = llvm::StructType::get(codegen->context(),
       llvm::ArrayRef<llvm::Type*>(field_types), true);
+  DCHECK(tuple_struct != nullptr);
   const llvm::DataLayout& data_layout = codegen->execution_engine()->getDataLayout();
   const llvm::StructLayout* layout = data_layout.getStructLayout(tuple_struct);
   for (SlotDescriptor* slot: slots()) {
