@@ -60,6 +60,8 @@
 #include "util/default-path-handlers.h"
 #include "util/hdfs-bulk-ops.h"
 #include "util/impalad-metrics.h"
+#include "util/jwt-util-internal.h"
+#include "util/jwt-util.h"
 #include "util/mem-info.h"
 #include "util/memory-metrics.h"
 #include "util/metrics.h"
@@ -149,6 +151,7 @@ DECLARE_string(state_store_host);
 DECLARE_int32(state_store_port);
 DECLARE_string(state_store_2_host);
 DECLARE_int32(state_store_2_port);
+DECLARE_string(cluster_membership_topic_id);
 
 DECLARE_string(debug_actions);
 DECLARE_string(ssl_client_ca_certificate);
@@ -200,7 +203,7 @@ using namespace impala;
 /// Helper method to forward cluster membership updates to the frontend.
 /// For additional details see comments for PopulateExecutorMembershipRequest()
 /// in cluster-membership-mgr.cc
-void SendClusterMembershipToFrontend(ClusterMembershipMgr::SnapshotPtr& snapshot,
+void SendClusterMembershipToFrontend(const ClusterMembershipMgr::SnapshotPtr& snapshot,
     const vector<TExecutorGroupSet>& expected_exec_group_sets, Frontend* frontend) {
   TUpdateExecutorMembershipRequest update_req;
 
@@ -283,9 +286,12 @@ ExecEnv::ExecEnv(int krpc_port, int subscriber_port, int webserver_port,
   } else if (FLAGS_is_executor) {
     subscriber_type = TStatestoreSubscriberType::EXECUTOR;
   }
-  // Set StatestoreSubscriber::subscriber_id as hostname + krpc_port.
-  statestore_subscriber_.reset(new StatestoreSubscriber(
-      Substitute("impalad@$0:$1", FLAGS_hostname, FLAGS_krpc_port), subscriber_address,
+  // Set StatestoreSubscriber::subscriber_id as cluster_id + hostname + krpc_port.
+  string subscriber_id = Substitute("impalad@$0:$1", FLAGS_hostname, FLAGS_krpc_port);
+  if (!FLAGS_cluster_membership_topic_id.empty()) {
+    subscriber_id = FLAGS_cluster_membership_topic_id + '-' + subscriber_id;
+  }
+  statestore_subscriber_.reset(new StatestoreSubscriber(subscriber_id, subscriber_address,
       statestore_address, statestore2_address, metrics_.get(), subscriber_type));
 
   if (FLAGS_is_coordinator) {
@@ -511,7 +517,7 @@ Status ExecEnv::Init() {
   RETURN_IF_ERROR(cluster_membership_mgr_->Init());
   if (FLAGS_is_coordinator && frontend_ != nullptr) {
     cluster_membership_mgr_->RegisterUpdateCallbackFn(
-        [this](ClusterMembershipMgr::SnapshotPtr snapshot) {
+        [this](const ClusterMembershipMgr::SnapshotPtr& snapshot) {
           SendClusterMembershipToFrontend(snapshot,
               this->cluster_membership_mgr()->GetExpectedExecGroupSets(),
               this->frontend());
@@ -530,6 +536,9 @@ Status ExecEnv::Init() {
     AiFunctions::set_api_key(api_key);
   }
 
+  jwt_helper_ = new JWTHelper();
+  oauth_helper_ = new JWTHelper();
+
   return Status::OK();
 }
 
@@ -582,7 +591,7 @@ void ExecEnv::SetImpalaServer(ImpalaServer* server) {
   });
   if (FLAGS_is_coordinator) {
     cluster_membership_mgr_->RegisterUpdateCallbackFn(
-        [server](ClusterMembershipMgr::SnapshotPtr snapshot) {
+        [server](const ClusterMembershipMgr::SnapshotPtr& snapshot) {
           std::unordered_set<BackendIdPB> current_backend_set;
           for (const auto& it : snapshot->current_backends) {
             current_backend_set.insert(it.second.backend_id());
@@ -592,7 +601,7 @@ void ExecEnv::SetImpalaServer(ImpalaServer* server) {
   }
   if (FLAGS_is_executor && !TestInfo::is_test()) {
     cluster_membership_mgr_->RegisterUpdateCallbackFn(
-        [](ClusterMembershipMgr::SnapshotPtr snapshot) {
+        [](const ClusterMembershipMgr::SnapshotPtr& snapshot) {
           std::unordered_set<BackendIdPB> current_backend_set;
           for (const auto& it : snapshot->current_backends) {
             current_backend_set.insert(it.second.backend_id());
