@@ -25,7 +25,7 @@
 #include "kudu/util/net/sockaddr.h"
 #include "kudu/util/trace.h"
 
-#include "exec/kudu-util.h"
+#include "exec/kudu/kudu-util.h"
 #include "runtime/exec-env.h"
 #include "runtime/krpc-data-stream-recvr.h"
 #include "runtime/mem-tracker.h"
@@ -366,7 +366,7 @@ void KrpcDataStreamMgr::RespondToTimedOutSender(const std::unique_ptr<ContextTyp
   TUniqueId finst_id;
   finst_id.__set_lo(request->dest_fragment_instance_id().lo());
   finst_id.__set_hi(request->dest_fragment_instance_id().hi());
-  string remote_addr = Substitute(" $0", ctx->rpc_context->remote_address().host());
+  string remote_addr = Substitute(" $0", ctx->rpc_context->remote_address().ToString());
   ErrorMsg msg(TErrorCode::DATASTREAM_SENDER_TIMEOUT, remote_addr, PrintId(finst_id),
       ctx->request->dest_node_id());
   VLOG_QUERY << msg.msg();
