@@ -39,6 +39,7 @@
 #include "runtime/descriptors.h"
 #include "runtime/exec-env.h"
 #include "runtime/fragment-state.h"
+#include "runtime/iceberg-position-delete-collector.h"
 #include "runtime/mem-tracker.h"
 #include "runtime/raw-value.inline.h"
 #include "runtime/row-batch.h"
@@ -50,6 +51,7 @@
 #include "util/debug-util.h"
 #include "util/network-util.h"
 #include "util/pretty-printer.h"
+#include "util/ubsan.h"
 
 #include "gen-cpp/data_stream_service.pb.h"
 #include "gen-cpp/data_stream_service.proxy.h"
@@ -116,9 +118,8 @@ void KrpcDataStreamSenderConfig::Close() {
 // A datastream sender may send row batches to multiple destinations. There is one
 // channel for each destination.
 //
-// Clients can call TransmitData() to directly send a serialized row batch to the
-// destination or it can call AddRow() to accumulate rows in an internal row batch
-// to certain capacity before sending it. The underlying RPC layer is implemented
+// Clients can call TransmitData() to send a serialized (and optionally
+// compressed) row batch to the destination. The underlying RPC layer is implemented
 // with KRPC, which provides interfaces for asynchronous RPC calls. Normally, the
 // calls above will return before the RPC has completed but they may block if there
 // is already an in-flight RPC.
@@ -129,7 +130,7 @@ void KrpcDataStreamSenderConfig::Close() {
 // can be swapped with outbound_batch_ once the previous in-flight RPC is finished.
 // This means that the client can serialize the next row batch while the current row batch
 // is being sent. outbound_batch_ is only used in the partitioned case - in the
-// unpartitioned case the shared outbound_batch_ holds the data for the the in-flight RPC.
+// unpartitioned case the shared outbound_batch_ holds the data for the in-flight RPC.
 //
 // Upon completion of a RPC, the callback TransmitDataCompleteCb() is invoked. If the RPC
 // fails due to remote service's queue being full, TransmitDataCompleteCb() will schedule
@@ -140,8 +141,7 @@ void KrpcDataStreamSenderConfig::Close() {
 // release resources. Teardown() will cancel any in-flight RPC and wait for the
 // completion callback to be called before returning. It's expected that the execution
 // thread to flush all buffered row batches and send the end-of-stream message (by
-// calling FlushBatches(), SendEosAsync() and WaitForRpc()) before closing the data
-// stream sender.
+// SendEosAsync() and WaitForRpc()) before closing the data stream sender.
 //
 // Note that the RPC payloads are owned solely by the channel and the KRPC layer will
 // relinquish references of them before the completion callback is invoked so it's
@@ -174,7 +174,7 @@ class KrpcDataStreamSender::Channel : public CacheLineAligned {
 
   // Initializes the channel.
   // Returns OK if successful, error indication otherwise.
-  Status Init(RuntimeState* state, std::shared_ptr<CharMemTrackerAllocator> allocator);
+  Status Init(RuntimeState* state, const shared_ptr<CharMemTrackerAllocator>& allocator);
 
   // Serializes the given row batch and send it to the destination. If the preceding
   // RPC is in progress, this function may block until the previous RPC finishes.
@@ -191,24 +191,13 @@ class KrpcDataStreamSender::Channel : public CacheLineAligned {
   // can be reused.
   Status TransmitData(std::unique_ptr<OutboundRowBatch>* outbound_batch, bool swap_batch);
 
-  // Copies a single row into this channel's row batch and flushes the row batch once
-  // it reaches capacity. This call may block if the row batch's capacity is reached
-  // and the preceding RPC is still in progress. Returns error status if serialization
-  // failed or if the preceding RPC failed. Return OK otherwise.
-  Status ALWAYS_INLINE AddRow(TupleRow* row);
-
   // Shutdowns the channel and frees the row batch allocation. Any in-flight RPC will
-  // be cancelled. It's expected that clients normally call FlushBatches(), SendEosAsync()
+  // be cancelled. It's expected that clients normally call SendEosAsync()
   // and WaitForRpc() before calling Teardown() to flush all buffered row batches to
   // destinations. Teardown() may be called without flushing the channel in cases such
   // as cancellation or error.
   void Teardown(RuntimeState* state);
 
-  // Flushes any buffered row batches. Return error status if the TransmitData() RPC
-  // fails. The RPC is sent asynchrononously. WaitForRpc() must be called to wait
-  // for the RPC. This should be only called from a fragment executor thread.
-  Status FlushBatches();
-
   // Sends the EOS RPC to close the channel. Return error status if sending the EOS RPC
   // failed. The RPC is sent asynchrononously. WaitForRpc() must be called to wait for
   // the RPC. This should be only called from a fragment executor thread.
@@ -219,12 +208,17 @@ class KrpcDataStreamSender::Channel : public CacheLineAligned {
   // Returns OK otherwise. This should be only called from a fragment executor thread.
   Status WaitForRpc();
 
+  int RowBatchCapacity() const { return row_batch_capacity_; }
+  int CalculateRowBatchCapacity() const;
+
   // The type for a RPC worker function.
   typedef boost::function<Status()> DoRpcFn;
 
   bool IsLocal() const { return is_local_; }
+  KrpcDataStreamSender* GetParent() { return parent_; }
 
  private:
+  friend KrpcDataStreamSender::IcebergPositionDeleteChannel;
   // The parent data stream sender owning this channel. Not owned.
   KrpcDataStreamSender* parent_;
 
@@ -241,9 +235,7 @@ class KrpcDataStreamSender::Channel : public CacheLineAligned {
   // True if the target fragment instance runs within the same process.
   const bool is_local_;
 
-  // The row batch for accumulating rows copied from AddRow().
-  // Only used if the partitioning scheme is "KUDU" or "HASH_PARTITIONED".
-  scoped_ptr<RowBatch> batch_;
+  int row_batch_capacity_ = -1;
 
   // Owns an outbound row batch that can be referenced by the in-flight RPC. Contains
   // a RowBatchHeaderPB and the buffers for the serialized tuple offsets and data.
@@ -297,6 +289,9 @@ class KrpcDataStreamSender::Channel : public CacheLineAligned {
   // TODO: Fix IMPALA-3990
   bool remote_recvr_closed_ = false;
 
+  // Returns the serialization batch from 'parent'.
+  unique_ptr<OutboundRowBatch>* GetSerializationBatch();
+
   // Returns true if the channel should terminate because the parent sender
   // has been closed or cancelled.
   bool ShouldTerminate() const { return shutdown_ || parent_->state_->is_cancelled(); }
@@ -331,7 +326,7 @@ class KrpcDataStreamSender::Channel : public CacheLineAligned {
   // In which case, MarkDone() will be called with the error status and the RPC is
   // considered complete. 'status' is the error status passed by KRPC code in case the
   // callback was aborted.
-  void RetryCb(DoRpcFn rpc_fn, const kudu::Status& status);
+  void RetryCb(const DoRpcFn& rpc_fn, const kudu::Status& status);
 
   // A callback function called from KRPC reactor threads upon completion of an in-flight
   // TransmitData() RPC. This is called when the remote server responds to the RPC or
@@ -382,23 +377,30 @@ class KrpcDataStreamSender::Channel : public CacheLineAligned {
 };
 
 Status KrpcDataStreamSender::Channel::Init(
-    RuntimeState* state, std::shared_ptr<CharMemTrackerAllocator> allocator) {
-  // TODO: take into account of var-len data at runtime.
-  int capacity =
-      max(1, parent_->per_channel_buffer_size_ / max(row_desc_->GetRowSize(), 1));
-  batch_.reset(new RowBatch(row_desc_, capacity, parent_->mem_tracker()));
+    RuntimeState* state, const shared_ptr<CharMemTrackerAllocator>& allocator) {
+  row_batch_capacity_ = CalculateRowBatchCapacity();
 
   // Create a DataStreamService proxy to the destination.
   RETURN_IF_ERROR(DataStreamService::GetProxy(address_, hostname_, &proxy_));
 
   // Init outbound_batch_.
-  outbound_batch_.reset(new OutboundRowBatch(allocator));
+  outbound_batch_.reset(new OutboundRowBatch(*allocator));
 
   return Status::OK();
 }
 
+int KrpcDataStreamSender::Channel::CalculateRowBatchCapacity() const {
+  // TODO: take into account of var-len data at runtime.
+  return
+      max(1, parent_->per_channel_buffer_size_ / max(row_desc_->GetRowSize(), 1));
+}
+
 void KrpcDataStreamSender::Channel::MarkDone(const Status& status) {
-  if (UNLIKELY(!status.ok())) COUNTER_ADD(parent_->rpc_failure_counter_, 1);
+  if (UNLIKELY(!status.ok())) {
+    COUNTER_ADD(parent_->rpc_failure_counter_, 1);
+  } else {
+    COUNTER_ADD(parent_->rpc_success_counter_, 1);
+  }
   rpc_status_ = status;
   rpc_in_flight_ = false;
   rpc_in_flight_batch_ = nullptr;
@@ -467,7 +469,7 @@ Status KrpcDataStreamSender::Channel::WaitForRpcLocked(std::unique_lock<SpinLock
 }
 
 void KrpcDataStreamSender::Channel::RetryCb(
-    DoRpcFn rpc_fn, const kudu::Status& cb_status) {
+    const DoRpcFn& rpc_fn, const kudu::Status& cb_status) {
   COUNTER_ADD(parent_->rpc_retry_counter_, 1);
   std::unique_lock<SpinLock> l(lock_);
   DCHECK(rpc_in_flight_);
@@ -597,6 +599,8 @@ Status KrpcDataStreamSender::Channel::TransmitData(
            << " #rows=" << outbound_batch->get()->header()->num_rows();
   std::unique_lock<SpinLock> l(lock_);
   RETURN_IF_ERROR(WaitForRpcLocked(&l));
+  // Measure the time needed after getting the lock.
+  SCOPED_TIMER(parent_->transmit_data_timer_);
   DCHECK(!rpc_in_flight_);
   DCHECK(rpc_in_flight_batch_ == nullptr);
   // If the remote receiver is closed already, there is no point in sending anything.
@@ -613,39 +617,33 @@ Status KrpcDataStreamSender::Channel::TransmitData(
 }
 
 Status KrpcDataStreamSender::Channel::SerializeAndSendBatch(RowBatch* batch) {
-  unique_ptr<OutboundRowBatch>* serialization_batch = &parent_->serialization_batch_;
-  DCHECK(serialization_batch->get() != nullptr);
-  // Reads 'rpc_in_flight_batch_' without acquiring 'lock_', so reads can be racey.
-  ANNOTATE_IGNORE_READS_BEGIN();
-  DCHECK(serialization_batch->get() != rpc_in_flight_batch_);
-  ANNOTATE_IGNORE_READS_END();
+  unique_ptr<OutboundRowBatch>* serialization_batch = GetSerializationBatch();
   RETURN_IF_ERROR(parent_->SerializeBatch(batch, serialization_batch->get(), !is_local_));
   // Swap serialization_batch with outbound_batch_ once the old RPC is finished.
   RETURN_IF_ERROR(TransmitData(serialization_batch, true /*swap_batch*/));
   return Status::OK();
 }
 
-Status KrpcDataStreamSender::Channel::SendCurrentBatch() {
-  RETURN_IF_ERROR(SerializeAndSendBatch(batch_.get()));
-  batch_->Reset();
-  return Status::OK();
+unique_ptr<OutboundRowBatch>* KrpcDataStreamSender::Channel::GetSerializationBatch() {
+  unique_ptr<OutboundRowBatch>* serialization_batch = &parent_->serialization_batch_;
+  DCHECK(serialization_batch->get() != nullptr);
+  // Reads 'rpc_in_flight_batch_' without acquiring 'lock_', so reads can be racey.
+  ANNOTATE_IGNORE_READS_BEGIN();
+  DCHECK(serialization_batch->get() != rpc_in_flight_batch_);
+  ANNOTATE_IGNORE_READS_END();
+  return serialization_batch;
 }
 
-inline Status KrpcDataStreamSender::Channel::AddRow(TupleRow* row) {
-  if (batch_->AtCapacity()) {
-    // batch_ is full, let's send it.
-    RETURN_IF_ERROR(SendCurrentBatch());
+Status KrpcDataStreamSender::PartitionRowCollector::SendCurrentBatch() {
+  if (collector_batch_->IsEmpty()) {
+    DCHECK_EQ(num_rows_, 0);
+    return Status::OK();
   }
-  TupleRow* dest = batch_->GetRow(batch_->AddRow());
-  const vector<TupleDescriptor*>& descs = row_desc_->tuple_descriptors();
-  for (int i = 0; i < descs.size(); ++i) {
-    if (UNLIKELY(row->GetTuple(i) == nullptr)) {
-      dest->SetTuple(i, nullptr);
-    } else {
-      dest->SetTuple(i, row->GetTuple(i)->DeepCopy(*descs[i], batch_->tuple_data_pool()));
-    }
-  }
-  batch_->CommitLastRow();
+  num_rows_ = 0;
+  RETURN_IF_ERROR(channel_->GetParent()->PrepareBatchForSend(
+      collector_batch_.get(), !channel_->IsLocal()));
+  RETURN_IF_ERROR(channel_->TransmitData(&collector_batch_, true /*swap_batch*/));
+  collector_batch_->Reset();
   return Status::OK();
 }
 
@@ -696,23 +694,7 @@ Status KrpcDataStreamSender::Channel::DoEndDataStreamRpc() {
   return Status::OK();
 }
 
-Status KrpcDataStreamSender::Channel::FlushBatches() {
-  VLOG_RPC << "Channel::FlushBatches() fragment_instance_id="
-           << PrintId(fragment_instance_id_) << " dest_node=" << dest_node_id_
-           << " #rows= " << batch_->num_rows();
-
-  // We can return an error here and not go on to send the EOS RPC because the error that
-  // we returned will be sent to the coordinator who will then cancel all the remote
-  // fragments including the one that this sender is sending to.
-  if (batch_->num_rows() > 0) RETURN_IF_ERROR(SendCurrentBatch());
-  return Status::OK();
-}
-
 Status KrpcDataStreamSender::Channel::SendEosAsync() {
-  VLOG_RPC << "Channel::SendEosAsync() fragment_instance_id="
-           << PrintId(fragment_instance_id_) << " dest_node=" << dest_node_id_
-           << " #rows= " << batch_->num_rows();
-  DCHECK_EQ(0, batch_->num_rows()) << "Batches must be flushed";
   {
     std::unique_lock<SpinLock> l(lock_);
     DCHECK(!rpc_in_flight_);
@@ -739,10 +721,67 @@ void KrpcDataStreamSender::Channel::Teardown(RuntimeState* state) {
     rpc_controller_.Cancel();
     while (rpc_in_flight_) rpc_done_cv_.wait(l);
   }
-  batch_.reset();
   outbound_batch_.reset(nullptr);
 }
 
+/// KrpcDataStreamSender's generic partitioning and serialization methods are inefficient
+/// for Iceberg position delete records. This class stores and efficiently serializes
+/// such data, then uses an internal Channel object's TransmitData() to send out
+/// the already serialized outbound row batches.
+class KrpcDataStreamSender::IcebergPositionDeleteChannel {
+ public:
+  IcebergPositionDeleteChannel(KrpcDataStreamSender* parent, Channel* channel,
+      TupleDescriptor* desc) : delete_collector_(desc) {
+    parent_ = parent;
+    channel_ = channel;
+    capacity_ = channel_->RowBatchCapacity();
+  }
+
+  void Prepare(MemTracker* parent_mem_tracker) {
+    delete_collector_.Init(parent_mem_tracker);
+  }
+
+  void Teardown() {
+    delete_collector_.Close();
+  }
+
+  Status AddRow(TupleRow* row) {
+    RETURN_IF_ERROR(delete_collector_.AddRow(row));
+    if (delete_collector_.RowCount() == capacity_) {
+      RETURN_IF_ERROR(Flush());
+    }
+    return Status::OK();
+  }
+
+  Status Flush() {
+    if (delete_collector_.RowCount() == 0) return Status::OK();
+    unique_ptr<OutboundRowBatch>* serialization_batch = channel_->GetSerializationBatch();
+    RETURN_IF_ERROR(ToOutboundRowBatch(serialization_batch->get()));
+    RETURN_IF_ERROR(channel_->TransmitData(serialization_batch, /*swap_batch=*/true));
+    return Status::OK();
+  }
+
+ private:
+  Status ToOutboundRowBatch(OutboundRowBatch* dest) {
+    {
+      SCOPED_TIMER(parent_->serialize_batch_timer_);
+      RETURN_IF_ERROR(delete_collector_.Serialize(dest));
+      constexpr int NUM_TUPLES_PER_ROW = 1;
+      bool compress = !channel_->IsLocal();
+      RETURN_IF_ERROR(dest->PrepareForSend(NUM_TUPLES_PER_ROW,
+          compress ? parent_->compression_scratch_.get(): nullptr));
+      int64_t uncompressed_bytes = RowBatch::GetDeserializedSize(*dest);
+      COUNTER_ADD(parent_->uncompressed_bytes_counter_, uncompressed_bytes);
+    }
+    return Status::OK();
+  }
+
+  KrpcDataStreamSender* parent_;
+  KrpcDataStreamSender::Channel* channel_;
+  int capacity_;
+  IcebergPositionDeleteCollector delete_collector_;
+};
+
 KrpcDataStreamSender::KrpcDataStreamSender(TDataSinkId sink_id, int sender_id,
     const KrpcDataStreamSenderConfig& sink_config, const TDataStreamSink& sink,
     const google::protobuf::RepeatedPtrField<PlanFragmentDestinationPB>& destinations,
@@ -774,11 +813,25 @@ KrpcDataStreamSender::KrpcDataStreamSender(TDataSinkId sink_id, int sender_id,
         destination.krpc_backend(), destination.fragment_instance_id(), sink.dest_node_id,
         per_channel_buffer_size, is_local));
 
+    if (partition_type_  == TPartitionType::HASH_PARTITIONED
+        || sink.output_partition.type == TPartitionType::KUDU) {
+      partition_row_collectors_.emplace_back();
+      partition_row_collectors_.back().channel_ = channels_.back().get();
+    }
+
     if (IsDirectedMode()) {
       DCHECK(host_to_channel_.find(destination.address()) == host_to_channel_.end());
       host_to_channel_[destination.address()] = channels_.back().get();
     }
   }
+  if (IsDirectedMode()) {
+    DCHECK_EQ(row_desc_->tuple_descriptors().size(), 1);
+    TupleDescriptor* tuple_desc = row_desc_->tuple_descriptors()[0];
+    for (unique_ptr<Channel>& ch : channels_) {
+      channel_to_ice_channel_[ch.get()] = make_unique<IcebergPositionDeleteChannel>(
+          this, ch.get(), tuple_desc);
+    }
+  }
 
   if (partition_type_ == TPartitionType::UNPARTITIONED
       || partition_type_ == TPartitionType::RANDOM) {
@@ -803,8 +856,10 @@ Status KrpcDataStreamSender::Prepare(
   RETURN_IF_ERROR(ScalarExprEvaluator::Create(partition_exprs_, state, state->obj_pool(),
       expr_perm_pool_.get(), expr_results_pool_.get(), &partition_expr_evals_));
   serialize_batch_timer_ = ADD_TIMER(profile(), "SerializeBatchTime");
+  transmit_data_timer_ = ADD_TIMER(profile(), "TransmitDataTime");
   rpc_retry_counter_ = ADD_COUNTER(profile(), "RpcRetry", TUnit::UNIT);
   rpc_failure_counter_ = ADD_COUNTER(profile(), "RpcFailure", TUnit::UNIT);
+  rpc_success_counter_ = ADD_COUNTER(profile(), "RpcSuccess", TUnit::UNIT);
   bytes_sent_counter_ = ADD_COUNTER(profile(), "TotalBytesSent", TUnit::BYTES);
   state->AddBytesSentCounter(bytes_sent_counter_);
   bytes_sent_time_series_counter_ =
@@ -828,16 +883,23 @@ Status KrpcDataStreamSender::Prepare(
   string process_address =
        NetworkAddressPBToString(ExecEnv::GetInstance()->krpc_address());
 
-  serialization_batch_.reset(new OutboundRowBatch(char_mem_tracker_allocator_));
+  serialization_batch_.reset(new OutboundRowBatch(*char_mem_tracker_allocator_));
   if (partition_type_ == TPartitionType::UNPARTITIONED) {
-    in_flight_batch_.reset(new OutboundRowBatch(char_mem_tracker_allocator_));
+    in_flight_batch_.reset(new OutboundRowBatch(*char_mem_tracker_allocator_));
   }
 
-  compression_scratch_.reset(new TrackedString(*char_mem_tracker_allocator_.get()));
+  compression_scratch_.reset(new TrackedString(*char_mem_tracker_allocator_));
 
   for (int i = 0; i < channels_.size(); ++i) {
     RETURN_IF_ERROR(channels_[i]->Init(state, char_mem_tracker_allocator_));
   }
+  for (PartitionRowCollector& collector: partition_row_collectors_) {
+    collector.collector_batch_.reset(new OutboundRowBatch(*char_mem_tracker_allocator_));
+    collector.row_batch_capacity_ = collector.channel_->RowBatchCapacity();
+  }
+  for (auto& [ch, ice_ch] : channel_to_ice_channel_) {
+    ice_ch->Prepare(mem_tracker_.get());
+  }
   return Status::OK();
 }
 
@@ -946,7 +1008,8 @@ Status KrpcDataStreamSenderConfig::CodegenHashRow(
 
     // Picks the input value to hash function
     builder.SetInsertPoint(hash_val_block);
-    llvm::PHINode* val_ptr_phi = rwi.CodegenNullPhiNode(native_ptr, null_ptr);
+    llvm::PHINode* val_ptr_phi =
+        rwi.CodegenNullPhiNode(native_ptr, null_ptr, "val_ptr_phi");
 
     // Creates a global constant of the partition expression's ColumnType. It has to be a
     // constant for constant propagation and dead code elimination in 'get_hash_value_fn'
@@ -1013,6 +1076,11 @@ void KrpcDataStreamSenderConfig::Codegen(FragmentState* state) {
         codegen->GetI32Constant(num_channels_), "GetNumChannels");
     DCHECK_EQ(num_replaced, 1);
 
+    num_replaced = codegen->ReplaceCallSitesWithValue(hash_and_add_rows_fn,
+        codegen->GetI32Constant(input_row_desc_->num_tuples_no_inline()),
+        "num_tuples_no_inline");
+    DCHECK_EQ(num_replaced, 1);
+
     // Replace HashRow() with the handcrafted IR function.
     num_replaced = codegen->ReplaceCallSites(hash_and_add_rows_fn,
         hash_row_fn, KrpcDataStreamSender::HASH_ROW_SYMBOL);
@@ -1029,10 +1097,6 @@ void KrpcDataStreamSenderConfig::Codegen(FragmentState* state) {
   AddCodegenStatus(codegen_status, sender_name);
 }
 
-Status KrpcDataStreamSender::AddRowToChannel(const int channel_id, TupleRow* row) {
-  return channels_[channel_id]->AddRow(row);
-}
-
 uint64_t KrpcDataStreamSender::HashRow(TupleRow* row, uint64_t seed) {
   uint64_t hash_val = seed;
   for (ScalarExprEvaluator* eval : partition_expr_evals_) {
@@ -1069,7 +1133,8 @@ Status KrpcDataStreamSender::Send(RuntimeState* state, RowBatch* batch) {
     }
     // At this point no RPCs can still refer to the old in_flight_batch_.
     in_flight_batch_.swap(serialization_batch_);
-  } else if (partition_type_ == TPartitionType::RANDOM || channels_.size() == 1) {
+  } else if (partition_type_ == TPartitionType::RANDOM ||
+      (channels_.size() == 1 && partition_type_ != TPartitionType::DIRECTED)) {
     // Round-robin batches among channels. Wait for the current channel to finish its
     // rpc before overwriting its batch.
     Channel* current_channel = channels_[current_channel_idx_].get();
@@ -1081,7 +1146,6 @@ Status KrpcDataStreamSender::Send(RuntimeState* state, RowBatch* batch) {
     const int num_rows = batch->num_rows();
     const int hash_batch_size = RowBatch::HASH_BATCH_SIZE;
     int channel_ids[hash_batch_size];
-
     for (int batch_start = 0; batch_start < num_rows; batch_start += hash_batch_size) {
       int batch_window_size = min(num_rows - batch_start, hash_batch_size);
       for (int i = 0; i < batch_window_size; ++i) {
@@ -1099,40 +1163,41 @@ Status KrpcDataStreamSender::Send(RuntimeState* state, RowBatch* batch) {
 
       for (int i = 0; i < batch_window_size; ++i) {
         TupleRow* row = batch->GetRow(i + batch_start);
-        RETURN_IF_ERROR(channels_[channel_ids[i]]->AddRow(row));
+        int channel_id = channel_ids[i];
+        PartitionRowCollector& collector = partition_row_collectors_[channel_id];
+        RETURN_IF_ERROR(collector.AppendRow(row, row_desc_));
       }
     }
   } else if (partition_type_ == TPartitionType::DIRECTED) {
     const int num_rows = batch->num_rows();
     char* prev_filename_ptr = nullptr;
-    vector<Channel*> prev_channels;
-    bool skipped_prev_row = false;
+    vector<IcebergPositionDeleteChannel*> prev_channels;
     for (int row_idx = 0; row_idx < num_rows; ++row_idx) {
       DCHECK_EQ(batch->num_tuples_per_row(), 1);
       TupleRow* tuple_row = batch->GetRow(row_idx);
-      Tuple* row = batch->GetRow(row_idx)->GetTuple(0);
-      StringValue* filename_value = row->GetStringSlot(0);
+      Tuple* tuple = batch->GetRow(row_idx)->GetTuple(0);
+      StringValue* filename_value = tuple->GetStringSlot(0);
       DCHECK(filename_value != nullptr);
       StringValue::SimpleString filename_value_ss = filename_value->ToSimpleString();
       if (filename_value_ss.ptr == prev_filename_ptr) {
         // If the filename pointer is the same as the previous one then we can instantly
         // send the row to the same channels as the previous row.
-        DCHECK(skipped_prev_row || !prev_channels.empty() ||
-            (filename_value_ss.len == 0 && prev_channels.empty()));
-        for (Channel* ch : prev_channels) RETURN_IF_ERROR(ch->AddRow(tuple_row));
+        for (IcebergPositionDeleteChannel* ch : prev_channels) {
+          RETURN_IF_ERROR(ch->AddRow(tuple_row));
+        }
         continue;
       }
+      prev_channels.clear();
       prev_filename_ptr = filename_value_ss.ptr;
       string filename(filename_value_ss.ptr, filename_value_ss.len);
 
       const auto filepath_to_hosts_it = filepath_to_hosts_.find(filename);
       if (filepath_to_hosts_it == filepath_to_hosts_.end()) {
-        // This can happen when e.g. compaction removed some data files from a snapshot
-        // but a delete file referencing them remained because it references other data
-        // files that remains in the new snapshot.
-        // Another use-case is table sampling where we read only a subset of the data
-        // files.
-        // A third case is when the delete record is invalid.
+        // This can happen due to file pruning, or when compaction removed some data
+        // files from a snapshot but a delete file referencing them remained because
+        // it references other data files that are still present in the new snapshot.
+        // Another case is table sampling where we read only a subset of the data files.
+        // Or, when simply the delete record is invalid.
         if (UNLIKELY(filename_value_ss.len == 0)) {
           state->LogError(
             ErrorMsg(TErrorCode::GENERAL, "NULL found as file_path in delete file"));
@@ -1140,12 +1205,10 @@ Status KrpcDataStreamSender::Send(RuntimeState* state, RowBatch* batch) {
           VLOG(3) << "Row from delete file refers to a non-existing data file: " <<
               filename;
         }
-        skipped_prev_row = true;
+        DCHECK(prev_channels.empty());
         continue;
       }
-      skipped_prev_row = false;
 
-      prev_channels.clear();
       for (const NetworkAddressPB& host_addr : filepath_to_hosts_it->second) {
         const auto channel_map_it = host_to_channel_.find(host_addr);
         if (channel_map_it == host_to_channel_.end()) {
@@ -1159,8 +1222,10 @@ Status KrpcDataStreamSender::Send(RuntimeState* state, RowBatch* batch) {
           return Status(ss.str());
         }
 
-        prev_channels.push_back(channel_map_it->second);
-        RETURN_IF_ERROR(channel_map_it->second->AddRow(tuple_row));
+        IcebergPositionDeleteChannel* ice_channel =
+            channel_to_ice_channel_[channel_map_it->second].get();
+        prev_channels.push_back(ice_channel);
+        RETURN_IF_ERROR(ice_channel->AddRow(tuple_row));
       }
     }
   } else {
@@ -1209,8 +1274,11 @@ Status KrpcDataStreamSender::FlushFinal(RuntimeState* state) {
   // If we hit an error here, we can return without closing the remaining channels as
   // the error is propagated back to the coordinator, which in turn cancels the query,
   // which will cause the remaining open channels to be closed.
-  for (unique_ptr<Channel>& channel : channels_) {
-    RETURN_IF_ERROR(channel->FlushBatches());
+  for (auto& [ch, ice_ch] : channel_to_ice_channel_) {
+    RETURN_IF_ERROR(ice_ch->Flush());
+  }
+  for (PartitionRowCollector& collector: partition_row_collectors_) {
+    RETURN_IF_ERROR(collector.SendCurrentBatch());
   }
   for (unique_ptr<Channel>& channel : channels_) {
     RETURN_IF_ERROR(channel->WaitForRpc());
@@ -1227,6 +1295,13 @@ Status KrpcDataStreamSender::FlushFinal(RuntimeState* state) {
 void KrpcDataStreamSender::Close(RuntimeState* state) {
   SCOPED_TIMER(profile()->total_time_counter());
   if (closed_) return;
+
+  for (auto& [ch, ice_ch] : channel_to_ice_channel_) {
+    ice_ch->Teardown();
+  }
+
+  partition_row_collectors_.clear();
+
   for (int i = 0; i < channels_.size(); ++i) {
     channels_[i]->Teardown(state);
   }
@@ -1257,6 +1332,19 @@ Status KrpcDataStreamSender::SerializeBatch(
   return Status::OK();
 }
 
+Status KrpcDataStreamSender::PrepareBatchForSend(
+    OutboundRowBatch* batch, bool compress) {
+  DCHECK(partition_type_ == TPartitionType::HASH_PARTITIONED
+      || partition_type_ == TPartitionType::KUDU);
+  SCOPED_TIMER(serialize_batch_timer_);
+  RETURN_IF_ERROR(batch->PrepareForSend(row_desc_->tuple_descriptors().size(),
+      compress ? compression_scratch_.get() : nullptr, true));
+  int64_t uncompressed_bytes = RowBatch::GetDeserializedSize(*batch);
+  COUNTER_ADD(uncompressed_bytes_counter_, uncompressed_bytes);
+  return Status::OK();
+}
+
+
 int64_t KrpcDataStreamSender::GetNumDataBytesSent() const {
   return bytes_sent_counter_->value();
 }
