@@ -20,6 +20,7 @@
 #include <cstring>
 #include <vector>
 
+#include "codegen/impala-ir.h"
 #include "gen-cpp/row_batch.pb.h"
 #include "kudu/util/slice.h"
 #include "runtime/mem-tracker.h"
@@ -29,15 +30,17 @@ namespace impala {
 template <typename K, typename V> class FixedSizeHashTable;
 class MemTracker;
 class RowBatchSerializeTest;
+class RowDescriptor;
 class RuntimeState;
+class Tuple;
+class TupleDescriptor;
+class TupleRow;
 
 /// A KRPC outbound row batch which contains the serialized row batch header and buffers
 /// for holding the tuple offsets and tuple data.
 class OutboundRowBatch {
  public:
-
-  OutboundRowBatch(std::shared_ptr<CharMemTrackerAllocator> allocator)
-    : tuple_data_(*allocator.get()) {}
+  OutboundRowBatch(const CharMemTrackerAllocator& allocator) : tuple_data_(allocator) {}
 
   const RowBatchHeaderPB* header() const { return &header_; }
 
@@ -64,10 +67,42 @@ class OutboundRowBatch {
          header_.has_compression_type();
   }
 
+  // Prepares the outbound row batch for sending over the network. If
+  // 'compression_scratch' is not null, then it also tries to compress the tuple_data,
+  // and swaps tuple_data and compression_scratch if the compressed data is smaller.
+  // If 'used_append_row' is true, assumes that AppendRow() was used to serialize
+  // the batch and the actual size comes from tuple_data_offset_.
+  // Also sets the header.
+  Status PrepareForSend(int num_tuples_per_row, TrackedString* compression_scratch,
+      bool used_append_row = false);
+
+  void Reset();
+
+  bool IsEmpty() { return tuple_offsets_.empty(); }
+
+  inline Status IR_ALWAYS_INLINE AppendRow(
+      const TupleRow* row, const RowDescriptor* row_desc);
+
+  // Returns true if the size limit (also used by RowBatch) is reached.
+  // Only used if the batch is serialized with AppendRow().
+  inline bool ReachedSizeLimit();
+
  private:
+  friend class IcebergPositionDeleteCollector;
   friend class RowBatch;
   friend class RowBatchSerializeBaseline;
 
+  inline bool IR_ALWAYS_INLINE TryAppendTuple(
+      const Tuple* tuple, const TupleDescriptor* desc);
+
+  // Try compressing tuple_data to compression_scratch, swap if compressed data is
+  // smaller.
+  Status TryCompress(TrackedString* compression_scratch, bool* is_compressed);
+
+  // Sets header of this outbound row batch.
+  void SetHeader(int num_rows, int num_tuples_per_row, int64_t uncompressed_size,
+      bool is_compressed);
+
   /// The serialized header which contains the meta-data of the row batch such as the
   /// number of rows and compression scheme used etc.
   RowBatchHeaderPB header_;
@@ -78,6 +113,10 @@ class OutboundRowBatch {
 
   /// Contains the actual data of all the tuples. The data could be compressed.
   TrackedString tuple_data_;
+
+  /// Used only if the row batch is filled with AppendRow(). Marks the offset of the
+  /// next tuple to write in tuple_data_.
+  int tuple_data_offset_ = 0;
 };
 
 }
