@@ -81,8 +81,8 @@ class RowBatchSerializeTest : public testing::Test {
       bool print_batches, bool full_dedup = false) {
     if (print_batches) cout << PrintBatch(batch) << endl;
 
-    TrackedString compression_scratch(*char_mem_tracker_allocator_.get());
-    OutboundRowBatch row_batch(char_mem_tracker_allocator_);
+    TrackedString compression_scratch(*char_mem_tracker_allocator_);
+    OutboundRowBatch row_batch(*char_mem_tracker_allocator_);
     RETURN_IF_ERROR(batch->Serialize(&row_batch, full_dedup, &compression_scratch));
 
     RowBatch deserialized_batch(&row_desc, row_batch, tracker_.get());
@@ -214,7 +214,9 @@ class RowBatchSerializeTest : public testing::Test {
         StringValue* string_value = reinterpret_cast<StringValue*>(slot);
         StringValue* deserialized_string_value =
             reinterpret_cast<StringValue*>(deserialized_slot);
-        EXPECT_NE(string_value->Ptr(), deserialized_string_value->Ptr());
+        if (!deserialized_string_value->IsSmall()) {
+          EXPECT_NE(string_value->Ptr(), deserialized_string_value->Ptr());
+        }
       }
 
       if (type.IsCollectionType()) {
@@ -615,13 +617,13 @@ void RowBatchSerializeTest::TestDupRemoval(bool full_dedup) {
   CreateTuples(tuple_desc, batch->tuple_data_pool(), num_distinct_tuples, 0, 10, &tuples);
   AddTuplesToRowBatch(num_rows, tuples, repeats, batch);
 
-  TrackedString compression_scratch(*char_mem_tracker_allocator_.get());
-  OutboundRowBatch row_batch(char_mem_tracker_allocator_);
+  TrackedString compression_scratch(*char_mem_tracker_allocator_);
+  OutboundRowBatch row_batch(*char_mem_tracker_allocator_);
   EXPECT_OK(batch->Serialize(&row_batch, full_dedup, &compression_scratch));
   // Serialized data should only have one copy of each tuple.
   int64_t total_byte_size = 0; // Total size without duplication
   for (int i = 0; i < tuples.size(); ++i) {
-    total_byte_size += tuples[i]->TotalByteSize(tuple_desc);
+    total_byte_size += tuples[i]->TotalByteSize(tuple_desc, true /*assume_smallify*/);
   }
   EXPECT_EQ(total_byte_size, row_batch.header()->uncompressed_size());
   TestRowBatch(row_desc, batch, false, full_dedup);
@@ -755,8 +757,8 @@ TEST_F(RowBatchSerializeTest, DedupPathologicalFull) {
   EXPECT_TRUE(UseFullDedup(batch));
   LOG(INFO) << "Serializing row batch";
 
-  TrackedString compression_scratch(*char_mem_tracker_allocator_.get());
-  OutboundRowBatch row_batch(char_mem_tracker_allocator_);
+  TrackedString compression_scratch(*char_mem_tracker_allocator_);
+  OutboundRowBatch row_batch(*char_mem_tracker_allocator_);
   EXPECT_OK(batch->Serialize(&row_batch, &compression_scratch));
   LOG(INFO) << "Serialized batch size: " << row_batch.TupleDataAsSlice().size();
   LOG(INFO) << "Serialized batch uncompressed size: "
