@@ -191,6 +191,11 @@ class RowBatch {
     /// Return the current row pointed to by the row pointer.
     TupleRow* IR_ALWAYS_INLINE Get() { return reinterpret_cast<TupleRow*>(row_); }
 
+    /// Return the current row pointed to by the row pointer.
+    const TupleRow* IR_ALWAYS_INLINE Get() const {
+      return reinterpret_cast<const TupleRow*>(row_);
+    }
+
     /// Increment the row pointer and return the next row.
     TupleRow* IR_ALWAYS_INLINE Next() {
       row_ += num_tuples_per_row_;
@@ -198,14 +203,22 @@ class RowBatch {
       return Get();
     }
 
+    void IR_ALWAYS_INLINE Advance(int n) {
+      row_ += n * num_tuples_per_row_;
+      DCHECK_LE((row_ - parent_->tuple_ptrs_) / num_tuples_per_row_, parent_->capacity_);
+    }
+
     /// Returns the index in the RowBatch of the current row. This does an integer
     /// division and so should not be used in hot inner loops.
     int RowNum() { return (row_ - parent_->tuple_ptrs_) / num_tuples_per_row_; }
 
+    /// Returns number of rows remaining.
+    int RemainingRows() { return (row_batch_end_ - row_) / num_tuples_per_row_; }
+
     /// Returns true if the iterator is beyond the last row for read iterators.
     /// Useful for read iterators to determine the limit. Write iterators should use
     /// RowBatch::AtCapacity() instead.
-    bool IR_ALWAYS_INLINE AtEnd() { return row_ >= row_batch_end_; }
+    bool IR_ALWAYS_INLINE AtEnd() const { return row_ >= row_batch_end_; }
 
     /// Returns the row batch which this iterator is iterating through.
     RowBatch* parent() { return parent_; }
@@ -292,6 +305,12 @@ class RowBatch {
     memcpy(dest, src, num_tuples_per_row_ * sizeof(Tuple*));
   }
 
+  /// Copies tuple pointers from another row batch. It is allowed for 'src' to have
+  /// less tuples per row - in this case the prefix is copied and remaining tuples are
+  /// initialized to 0. The caller must ensure that src has enough rows / dst has enough
+  /// capacity.
+  void CopyRows(RowBatch* src, int num_rows, int src_offset, int dst_offset);
+
   /// Copy 'num_rows' rows from 'src' to 'dest' within the batch. Useful for exec
   /// nodes that skip an offset and copied more than necessary.
   void CopyRows(int64_t dest, int64_t src, int64_t num_rows) {
