@@ -22,10 +22,10 @@
 
 namespace impala {
 
-const char* StringValue::LLVM_CLASS_NAME = "struct.impala::StringValue";
+const char* StringValue::LLVM_CLASS_NAME = "class.impala::StringValue";
 
 string StringValue::DebugString() const {
-  return string(ptr, len);
+  return string(Ptr(), Len());
 }
 
 ostream& operator<<(ostream& os, const StringValue& string_value) {
@@ -35,6 +35,8 @@ ostream& operator<<(ostream& os, const StringValue& string_value) {
 uint64_t StringValue::ToUInt64() const {
   unsigned char bytes[8];
   *((uint64_t*)bytes) = 0;
+  uint32_t len = Len();
+  const char* ptr = Ptr();
   int chars_to_copy = (len < 8) ? len : 8;
   for (int i = 0; i < chars_to_copy; i++) {
     bytes[i] = static_cast<unsigned char>(ptr[i]);
@@ -45,7 +47,10 @@ uint64_t StringValue::ToUInt64() const {
       | static_cast<uint64_t>(bytes[6]) << 8 | static_cast<uint64_t>(bytes[7]);
 }
 
-string StringValue::LeastSmallerString() const {
+string StringValue::LargestSmallerString() const {
+  uint32_t len = Len();
+  if (len == 0) return "";
+  const char* ptr = Ptr();
   if (len == 1 && ptr[0] == 0x00) return "";
 
   int i = len - 1;
@@ -74,6 +79,8 @@ string StringValue::LeastSmallerString() const {
 }
 
 string StringValue::LeastLargerString() const {
+  uint32_t len = Len();
+  const char* ptr = Ptr();
   if (len == 0) return string("\00", 1);
   int i = len - 1;
   while (i >= 0 && ptr[i] == (int8_t)0xff) i--;
