@@ -93,6 +93,15 @@ public:
 
   int Len() const { return string_impl_.Len(); }
 
+  /// Returns the number of bytes needed outside the slot itself:
+  /// - if the length is too long to smallify, return length
+  /// - if the length is small enough to smallify:
+  ///   - if assume_smallify is true or the string is already smallified return 0
+  ///   - otherwise (not already smallified and assume_smallify is false) return length
+  int ExternalLen(bool assume_smallify) const {
+    return string_impl_.ExternalLen(assume_smallify);
+  }
+
   /// Sets the length of this String object. Length can only be decreased.
   void SetLen(int len) { return string_impl_.SetLen(len); }
 
@@ -205,4 +214,15 @@ std::ostream& operator<<(std::ostream& os, const StringValue& string_value);
 
 }
 
+/// With this specialization it is possbile to use StringValues in hash-based std
+/// containers (unordered_set, unordered_map) without the need of explicitly
+/// specifying the Hash template parameter.
+namespace std {
+  template <> struct hash<impala::StringValue> {
+    size_t operator()(const impala::StringValue& str) const {
+      return hash_value(str);
+    }
+  };
+}
+
 #endif
