@@ -42,6 +42,7 @@ DECLARE_string(state_store_2_host);
 DECLARE_int32(state_store_2_port);
 DECLARE_int32(state_store_subscriber_port);
 DECLARE_string(hostname);
+DECLARE_string(cluster_membership_topic_id);
 
 namespace impala {
 
@@ -62,9 +63,13 @@ AdmissiondEnv::AdmissiondEnv()
       MakeNetworkAddress(FLAGS_state_store_host, FLAGS_state_store_port);
   TNetworkAddress statestore2_address =
       MakeNetworkAddress(FLAGS_state_store_2_host, FLAGS_state_store_2_port);
-  statestore_subscriber_.reset(new StatestoreSubscriber(
-      Substitute("admissiond@$0", TNetworkAddressToString(admission_service_addr)),
-      subscriber_address, statestore_address, statestore2_address, metrics,
+  string subscriber_id = Substitute("admissiond@$0",
+      TNetworkAddressToString(admission_service_addr));
+  if (!FLAGS_cluster_membership_topic_id.empty()) {
+    subscriber_id = FLAGS_cluster_membership_topic_id + '-' + subscriber_id;
+  }
+  statestore_subscriber_.reset(new StatestoreSubscriber(subscriber_id, subscriber_address,
+      statestore_address, statestore2_address, metrics,
       TStatestoreSubscriberType::ADMISSIOND));
 
   scheduler_.reset(new Scheduler(metrics, request_pool_service()));
@@ -110,7 +115,7 @@ Status AdmissiondEnv::Init() {
 
   RETURN_IF_ERROR(cluster_membership_mgr_->Init());
   cluster_membership_mgr_->RegisterUpdateCallbackFn(
-      [&](ClusterMembershipMgr::SnapshotPtr snapshot) {
+      [&](const ClusterMembershipMgr::SnapshotPtr& snapshot) {
         std::unordered_set<BackendIdPB> current_backends;
         for (const auto& it : snapshot->current_backends) {
           current_backends.insert(it.second.backend_id());
