@@ -31,6 +31,7 @@
 
 DECLARE_int32(num_expected_executors);
 DECLARE_string(expected_executor_group_sets);
+DECLARE_string(cluster_membership_topic_id);
 
 namespace {
 using namespace impala;
@@ -90,14 +91,21 @@ ClusterMembershipMgr::ClusterMembershipMgr(
     current_membership_(std::make_shared<const Snapshot>()),
     statestore_subscriber_(subscriber),
     local_backend_id_(move(local_backend_id)) {
+  if (FLAGS_cluster_membership_topic_id.empty()) {
+    membership_topic_name_ = Statestore::IMPALA_MEMBERSHIP_TOPIC;
+  } else {
+    membership_topic_name_ =
+        FLAGS_cluster_membership_topic_id + '-' + Statestore::IMPALA_MEMBERSHIP_TOPIC;
+  }
   Status status = PopulateExpectedExecGroupSets(expected_exec_group_sets_);
   if(!status.ok()) {
     LOG(FATAL) << "Error populating expected executor group sets: " << status;
   }
   InitMetrics(metrics);
   // Register the metric update function as a callback.
-  RegisterUpdateCallbackFn([this](
-      ClusterMembershipMgr::SnapshotPtr snapshot) { this->UpdateMetrics(snapshot); });
+  RegisterUpdateCallbackFn([this](const ClusterMembershipMgr::SnapshotPtr& snapshot) {
+    this->UpdateMetrics(snapshot);
+  });
 }
 
 void ClusterMembershipMgr::InitMetrics(MetricGroup* metrics) {
@@ -132,7 +140,7 @@ Status ClusterMembershipMgr::Init() {
   StatestoreSubscriber::UpdateCallback cb =
       bind<void>(mem_fn(&ClusterMembershipMgr::UpdateMembership), this, _1, _2);
   Status status = statestore_subscriber_->AddTopic(
-      Statestore::IMPALA_MEMBERSHIP_TOPIC, /* is_transient=*/ true,
+      membership_topic_name_, /* is_transient=*/ true,
       /* populate_min_subscriber_topic_version=*/ false,
       /* filter_prefix= */"", cb);
   if (!status.ok()) {
@@ -162,21 +170,11 @@ ClusterMembershipMgr::SnapshotPtr ClusterMembershipMgr::GetSnapshot() const {
   return state;
 }
 
-static bool is_active_coordinator(const BackendDescriptorPB& be) {
+static inline bool is_active_coordinator(const BackendDescriptorPB& be) {
   return be.has_is_coordinator() && be.is_coordinator() &&
       !(be.has_is_quiescing() && be.is_quiescing());
 }
 
-ExecutorGroup ClusterMembershipMgr::Snapshot::GetCoordinators() const {
-  ExecutorGroup coordinators("all-coordinators");
-  for (const auto& it : current_backends) {
-    if (is_active_coordinator(it.second)) {
-      coordinators.AddExecutor(it.second);
-    }
-  }
-  return coordinators;
-}
-
 vector<TNetworkAddress> ClusterMembershipMgr::Snapshot::GetCoordinatorAddresses() const {
   vector<TNetworkAddress> coordinators;
   for (const auto& it : current_backends) {
@@ -189,6 +187,19 @@ vector<TNetworkAddress> ClusterMembershipMgr::Snapshot::GetCoordinatorAddresses(
   return coordinators;
 }
 
+static inline void _removeCoordIfExists(
+    const std::shared_ptr<ClusterMembershipMgr::Snapshot>& state,
+    const BackendDescriptorPB& be) {
+
+  // The BackendDescriptorPB may be incomplete. Use the backend id to retrieve the actual
+  // backend descriptor so the backend can be removed.
+  const BackendDescriptorPB* actual_be =
+      state->all_coordinators.LookUpBackendDesc(be.backend_id());
+  if (actual_be != nullptr) {
+    state->all_coordinators.RemoveExecutor(*actual_be);
+  }
+}
+
 void ClusterMembershipMgr::UpdateMembership(
     const StatestoreSubscriber::TopicDeltaMap& incoming_topic_deltas,
     vector<TTopicDelta>* subscriber_topic_updates) {
@@ -196,7 +207,7 @@ void ClusterMembershipMgr::UpdateMembership(
 
   // First look to see if the topic we're interested in has an update.
   StatestoreSubscriber::TopicDeltaMap::const_iterator topic =
-      incoming_topic_deltas.find(Statestore::IMPALA_MEMBERSHIP_TOPIC);
+      incoming_topic_deltas.find(membership_topic_name_);
 
   // Ignore spurious messages.
   if (topic == incoming_topic_deltas.end()) return;
@@ -289,6 +300,14 @@ void ClusterMembershipMgr::UpdateMembership(
             RemoveExecutorAndGroup(be_desc, group, new_executor_groups);
           }
         }
+
+        // If a coordinator is not shutdown gracefully, then it will be deleted here.
+        if (be_desc.is_coordinator()) {
+          _removeCoordIfExists(new_state, be_desc);
+        }
+
+        // Note: be_desc is a reference to item.key, thus this erase must come at the end
+        // of the loop where be_desc is initialized.
         new_backend_map->erase(item.key);
       }
       continue;
@@ -338,32 +357,45 @@ void ClusterMembershipMgr::UpdateMembership(
       if (existing.is_quiescing()) DCHECK(be_desc.is_quiescing());
 
       // If the node starts quiescing
-      if (be_desc.is_quiescing() && !existing.is_quiescing() && existing.is_executor()) {
-        // If the backend starts quiescing and it is present in the blacklist, remove it
-        // from the blacklist. If the backend is present in the blacklist, there is no
-        // need to remove it from the executor group because it has already been removed
-        bool blacklisted = new_blacklist->FindAndRemove(be_desc)
-            == ExecutorBlacklist::State::BLACKLISTED;
-        if (blacklisted) {
-          VLOG(1) << "Removing backend " << item.key << " from blacklist (quiescing)";
-          DCHECK(!IsBackendInExecutorGroups(be_desc, *new_executor_groups));
-        } else {
-          // Executor needs to be removed from its groups
-          for (const auto& group : be_desc.executor_groups()) {
-            VLOG(1) << "Removing backend " << item.key << " from group "
-                    << group.DebugString() << " (quiescing)";
-            RemoveExecutorAndGroup(be_desc, group, new_executor_groups);
+      if (be_desc.is_quiescing() && !existing.is_quiescing()) {
+        if (existing.is_executor()) {
+          // If the backend starts quiescing and it is present in the blacklist, remove it
+          // from the blacklist. If the backend is present in the blacklist, there is no
+          // need to remove it from the executor group because it has already been removed
+          bool blacklisted = new_blacklist->FindAndRemove(be_desc)
+              == ExecutorBlacklist::State::BLACKLISTED;
+          if (blacklisted) {
+            VLOG(1) << "Removing backend " << item.key << " from blacklist (quiescing)";
+            DCHECK(!IsBackendInExecutorGroups(be_desc, *new_executor_groups));
+          } else {
+            // Executor needs to be removed from its groups
+            for (const auto& group : be_desc.executor_groups()) {
+              VLOG(1) << "Removing backend " << item.key << " from group "
+                      << group.DebugString() << " (quiescing)";
+              RemoveExecutorAndGroup(be_desc, group, new_executor_groups);
+            }
           }
         }
+
+        if (existing.is_coordinator()) {
+          _removeCoordIfExists(new_state, be_desc);
+        }
       }
       existing = be_desc;
     } else {
       // Create
       new_backend_map->insert(make_pair(item.key, be_desc));
-      if (!be_desc.is_quiescing() && be_desc.is_executor()) {
-        for (const auto& group : be_desc.executor_groups()) {
-          VLOG(1) << "Adding backend " << item.key << " to group " << group.DebugString();
-          FindOrInsertExecutorGroup(group, new_executor_groups)->AddExecutor(be_desc);
+      if (!be_desc.is_quiescing()) {
+        if (be_desc.is_executor()) {
+          for (const auto& group : be_desc.executor_groups()) {
+            VLOG(1) << "Adding backend " << item.key << " to group " <<
+                group.DebugString();
+            FindOrInsertExecutorGroup(group, new_executor_groups)->AddExecutor(be_desc);
+          }
+        }
+
+        if (is_active_coordinator(be_desc)) {
+          new_state->all_coordinators.AddExecutor(be_desc);
         }
       }
       // Since this backend is new, it cannot already be on the blacklist or probation.
@@ -393,16 +425,27 @@ void ClusterMembershipMgr::UpdateMembership(
   if (NeedsLocalBackendUpdate(*new_state, local_be_desc)) {
     // We need to update both the new membership state and the statestore
     (*new_backend_map)[local_backend_id_] = *local_be_desc;
-    for (const auto& group : local_be_desc->executor_groups()) {
-      if (local_be_desc->is_quiescing()) {
-        VLOG(1) << "Removing local backend from group " << group.DebugString();
-        RemoveExecutorAndGroup(*local_be_desc, group, new_executor_groups);
-      } else if (local_be_desc->is_executor()) {
-        VLOG(1) << "Adding local backend to group " << group.DebugString();
-        FindOrInsertExecutorGroup(
-            group, new_executor_groups)->AddExecutor(*local_be_desc);
+    // Could be a coordinator and/or executor, but only executors add the local backend.
+    DCHECK(local_be_desc->is_coordinator() || local_be_desc->is_executor());
+    if (local_be_desc->is_executor()) {
+      for (const auto& group : local_be_desc->executor_groups()) {
+        if (local_be_desc->is_quiescing()) {
+          VLOG(1) << "Removing local backend from group " << group.DebugString();
+          RemoveExecutorAndGroup(*local_be_desc, group, new_executor_groups);
+        } else {
+          VLOG(1) << "Adding local backend to group " << group.DebugString();
+          FindOrInsertExecutorGroup(
+              group, new_executor_groups)->AddExecutor(*local_be_desc);
+        }
       }
     }
+
+    // Add ourself to the list of all coordinators.
+    if (is_active_coordinator(*local_be_desc.get())) {
+      _removeCoordIfExists(new_state, *local_be_desc);
+      new_state->all_coordinators.AddExecutor(*local_be_desc);
+    }
+
     AddLocalBackendToStatestore(*local_be_desc, subscriber_topic_updates);
     DCHECK(CheckConsistency(*new_backend_map, *new_executor_groups, *new_blacklist));
   }
@@ -522,7 +565,7 @@ void ClusterMembershipMgr::AddLocalBackendToStatestore(
 
   subscriber_topic_updates->emplace_back(TTopicDelta());
   TTopicDelta& update = subscriber_topic_updates->back();
-  update.topic_name = Statestore::IMPALA_MEMBERSHIP_TOPIC;
+  update.topic_name = membership_topic_name_;
   update.topic_entries.emplace_back(TTopicItem());
   // Setting this flag allows us to pass the resulting topic update to other
   // ClusterMembershipMgr instances in tests unmodified.
@@ -543,7 +586,7 @@ ClusterMembershipMgr::BeDescSharedPtr ClusterMembershipMgr::GetLocalBackendDescr
   return local_be_desc_fn_ ? local_be_desc_fn_() : nullptr;
 }
 
-void ClusterMembershipMgr::NotifyListeners(SnapshotPtr snapshot) {
+void ClusterMembershipMgr::NotifyListeners(const SnapshotPtr& snapshot) {
   lock_guard<mutex> l(callback_fn_lock_);
   for (const auto& fn : update_callback_fns_) fn(snapshot);
 }
@@ -676,7 +719,7 @@ bool ClusterMembershipMgr::IsBackendInExecutorGroups(
 /// executor groups, we assume that we will read data remotely and will only send the
 /// number of executors in the largest healthy group. When expected exec group sets are
 /// specified we apply the aforementioned steps for each group set.
-void PopulateExecutorMembershipRequest(ClusterMembershipMgr::SnapshotPtr& snapshot,
+void PopulateExecutorMembershipRequest(const ClusterMembershipMgr::SnapshotPtr& snapshot,
     const vector<TExecutorGroupSet>& expected_exec_group_sets,
     TUpdateExecutorMembershipRequest& update_req) {
   vector<TExecutorGroupSet> exec_group_sets;
