@@ -26,10 +26,12 @@ namespace impala {
 // TODO: This can be tuned further with real world tests
 static const uint32_t NUM_HASH_RING_REPLICAS = 25;
 
-ExecutorGroup::ExecutorGroup(string name) : ExecutorGroup(name, 1) {}
+ExecutorGroup::ExecutorGroup(string name) : ExecutorGroup(move(name), 1) {}
 
 ExecutorGroup::ExecutorGroup(string name, int64_t min_size)
-  : name_(name), min_size_(min_size), executor_ip_hash_ring_(NUM_HASH_RING_REPLICAS),
+  : name_(move(name)),
+    min_size_(min_size),
+    executor_ip_hash_ring_(NUM_HASH_RING_REPLICAS),
     per_executor_admit_mem_limit_(0) {
   DCHECK_GT(min_size_, 0);
 }
@@ -149,14 +151,22 @@ void ExecutorGroup::RemoveExecutor(const BackendDescriptorPB& be_desc) {
                 << be_desc.krpc_address();
     return;
   }
+
+  // Copy the data necessary to update the internal state variables since the erase call
+  // has the potential to destroy the object referenced by be_desc.
+  const std::string be_hostname = be_desc.address().hostname();
+  const std::string be_ip_address = be_desc.ip_address();
+  const int64_t be_admin_mem_limit = be_desc.admit_mem_limit();
+
   be_descs.erase(remove_it);
-  if (per_executor_admit_mem_limit_ == be_desc.admit_mem_limit()) {
+
+  if (per_executor_admit_mem_limit_ == be_admin_mem_limit) {
     CalculatePerExecutorMemLimitForAdmission();
   }
   if (be_descs.empty()) {
     executor_map_.erase(be_descs_it);
-    executor_ip_map_.erase(be_desc.address().hostname());
-    executor_ip_hash_ring_.RemoveNode(be_desc.ip_address());
+    executor_ip_map_.erase(be_hostname);
+    executor_ip_hash_ring_.RemoveNode(be_ip_address);
   }
 }
 
@@ -186,6 +196,19 @@ const BackendDescriptorPB* ExecutorGroup::LookUpBackendDesc(
   return nullptr;
 }
 
+const BackendDescriptorPB* ExecutorGroup::LookUpBackendDesc(
+    const UniqueIdPB& be_id) const {
+  for (const auto& executor_list : executor_map_) {
+    for (const auto& backend : executor_list.second){
+      if (backend.backend_id().hi() == be_id.hi()
+          && backend.backend_id().lo() == be_id.lo()) {
+        return &backend;
+      }
+    }
+  }
+  return nullptr;
+}
+
 int ExecutorGroup::NumExecutors() const {
   int count = 0;
   for (const auto& executor_list : executor_map_) count += executor_list.second.size();
