@@ -259,6 +259,42 @@ bool ScheduleState::UseDedicatedCoordEstimates() const {
   return false;
 }
 
+void ScheduleState::CompareMaxBackendMemToAdmit(
+    const int64_t new_limit, const MemLimitSourcePB source) {
+  DCHECK(query_schedule_pb_->has_per_backend_mem_to_admit());
+  if (query_schedule_pb_->per_backend_mem_to_admit() < new_limit) {
+    query_schedule_pb_->set_per_backend_mem_to_admit(new_limit);
+    query_schedule_pb_->set_per_backend_mem_to_admit_source(source);
+  }
+}
+
+void ScheduleState::CompareMinBackendMemToAdmit(
+    const int64_t new_limit, const MemLimitSourcePB source) {
+  DCHECK(query_schedule_pb_->has_per_backend_mem_to_admit());
+  if (query_schedule_pb_->per_backend_mem_to_admit() > new_limit) {
+    query_schedule_pb_->set_per_backend_mem_to_admit(new_limit);
+    query_schedule_pb_->set_per_backend_mem_to_admit_source(source);
+  }
+}
+
+void ScheduleState::CompareMaxCoordinatorMemToAdmit(
+    const int64_t new_limit, const MemLimitSourcePB source) {
+  DCHECK(query_schedule_pb_->has_coord_backend_mem_to_admit());
+  if (query_schedule_pb_->coord_backend_mem_to_admit() < new_limit) {
+    query_schedule_pb_->set_coord_backend_mem_to_admit(new_limit);
+    query_schedule_pb_->set_coord_backend_mem_to_admit_source(source);
+  }
+}
+
+void ScheduleState::CompareMinCoordinatorMemToAdmit(
+    const int64_t new_limit, const MemLimitSourcePB source) {
+  DCHECK(query_schedule_pb_->has_coord_backend_mem_to_admit());
+  if (query_schedule_pb_->coord_backend_mem_to_admit() > new_limit) {
+    query_schedule_pb_->set_coord_backend_mem_to_admit(new_limit);
+    query_schedule_pb_->set_coord_backend_mem_to_admit_source(source);
+  }
+}
+
 void ScheduleState::UpdateMemoryRequirements(const TPoolConfig& pool_cfg,
     int64_t coord_mem_limit_admission, int64_t executor_mem_limit_admission) {
   // If the min_query_mem_limit and max_query_mem_limit are not set in the pool config
@@ -268,51 +304,69 @@ void ScheduleState::UpdateMemoryRequirements(const TPoolConfig& pool_cfg,
       pool_cfg.min_query_mem_limit == 0 && pool_cfg.max_query_mem_limit == 0;
   const bool use_dedicated_coord_estimates = UseDedicatedCoordEstimates();
 
-  int64_t per_backend_mem_to_admit = 0;
-  int64_t coord_backend_mem_to_admit = 0;
+  query_schedule_pb_->set_per_backend_mem_to_admit(0);
+  query_schedule_pb_->set_per_backend_mem_to_admit_source(MemLimitSourcePB::NO_LIMIT);
+  query_schedule_pb_->set_coord_backend_mem_to_admit(0);
+  query_schedule_pb_->set_coord_backend_mem_to_admit_source(MemLimitSourcePB::NO_LIMIT);
   bool is_mem_limit_set = false;
   if (query_options().__isset.mem_limit && query_options().mem_limit > 0) {
-    per_backend_mem_to_admit = query_options().mem_limit;
-    coord_backend_mem_to_admit = query_options().mem_limit;
+    query_schedule_pb_->set_per_backend_mem_to_admit(query_options().mem_limit);
+    query_schedule_pb_->set_per_backend_mem_to_admit_source(
+        MemLimitSourcePB::QUERY_OPTION_MEM_LIMIT);
+    query_schedule_pb_->set_coord_backend_mem_to_admit(query_options().mem_limit);
+    query_schedule_pb_->set_coord_backend_mem_to_admit_source(
+        MemLimitSourcePB::QUERY_OPTION_MEM_LIMIT);
     is_mem_limit_set = true;
   }
 
   if (!is_mem_limit_set) {
-    per_backend_mem_to_admit = GetPerExecutorMemoryEstimate();
-    coord_backend_mem_to_admit = use_dedicated_coord_estimates ?
-        GetDedicatedCoordMemoryEstimate() :
-        GetPerExecutorMemoryEstimate();
+    query_schedule_pb_->set_per_backend_mem_to_admit(GetPerExecutorMemoryEstimate());
+    query_schedule_pb_->set_per_backend_mem_to_admit_source(
+        MemLimitSourcePB::QUERY_PLAN_PER_HOST_MEM_ESTIMATE);
+    if (use_dedicated_coord_estimates) {
+      query_schedule_pb_->set_coord_backend_mem_to_admit(
+          GetDedicatedCoordMemoryEstimate());
+      query_schedule_pb_->set_coord_backend_mem_to_admit_source(
+          MemLimitSourcePB::QUERY_PLAN_DEDICATED_COORDINATOR_MEM_ESTIMATE);
+    } else {
+      query_schedule_pb_->set_coord_backend_mem_to_admit(GetPerExecutorMemoryEstimate());
+      query_schedule_pb_->set_coord_backend_mem_to_admit_source(
+          MemLimitSourcePB::QUERY_PLAN_PER_HOST_MEM_ESTIMATE);
+    }
     VLOG(3) << "use_dedicated_coord_estimates=" << use_dedicated_coord_estimates
-            << " coord_backend_mem_to_admit=" << coord_backend_mem_to_admit
-            << " per_backend_mem_to_admit=" << per_backend_mem_to_admit;
+            << " coord_backend_mem_to_admit="
+            << query_schedule_pb_->coord_backend_mem_to_admit()
+            << " per_backend_mem_to_admit="
+            << query_schedule_pb_->per_backend_mem_to_admit();
     if (!mimic_old_behaviour) {
       int64_t min_mem_limit_required =
           ReservationUtil::GetMinMemLimitFromReservation(largest_min_reservation());
-      per_backend_mem_to_admit = max(per_backend_mem_to_admit, min_mem_limit_required);
+      CompareMaxBackendMemToAdmit(
+          min_mem_limit_required, MemLimitSourcePB::ADJUSTED_PER_HOST_MEM_ESTIMATE);
       int64_t min_coord_mem_limit_required =
           ReservationUtil::GetMinMemLimitFromReservation(coord_min_reservation());
-      coord_backend_mem_to_admit =
-          max(coord_backend_mem_to_admit, min_coord_mem_limit_required);
+      CompareMaxCoordinatorMemToAdmit(min_coord_mem_limit_required,
+          MemLimitSourcePB::ADJUSTED_DEDICATED_COORDINATOR_MEM_ESTIMATE);
     }
   }
 
   if (!is_mem_limit_set || pool_cfg.clamp_mem_limit_query_option) {
     if (pool_cfg.min_query_mem_limit > 0) {
-      per_backend_mem_to_admit =
-          max(per_backend_mem_to_admit, pool_cfg.min_query_mem_limit);
+      CompareMaxBackendMemToAdmit(pool_cfg.min_query_mem_limit,
+          MemLimitSourcePB::POOL_CONFIG_MIN_QUERY_MEM_LIMIT);
       if (!use_dedicated_coord_estimates || is_mem_limit_set) {
         // The minimum mem limit option does not apply to dedicated coordinators -
         // this would result in over-reserving of memory. Treat coordinator and
         // executor mem limits the same if the query option was explicitly set.
-        coord_backend_mem_to_admit =
-            max(coord_backend_mem_to_admit, pool_cfg.min_query_mem_limit);
+        CompareMaxCoordinatorMemToAdmit(pool_cfg.min_query_mem_limit,
+            MemLimitSourcePB::POOL_CONFIG_MIN_QUERY_MEM_LIMIT);
       }
     }
     if (pool_cfg.max_query_mem_limit > 0) {
-      per_backend_mem_to_admit =
-          min(per_backend_mem_to_admit, pool_cfg.max_query_mem_limit);
-      coord_backend_mem_to_admit =
-          min(coord_backend_mem_to_admit, pool_cfg.max_query_mem_limit);
+      CompareMinBackendMemToAdmit(pool_cfg.max_query_mem_limit,
+          MemLimitSourcePB::POOL_CONFIG_MAX_QUERY_MEM_LIMIT);
+      CompareMinCoordinatorMemToAdmit(pool_cfg.max_query_mem_limit,
+          MemLimitSourcePB::POOL_CONFIG_MAX_QUERY_MEM_LIMIT);
     }
   }
 
@@ -321,43 +375,53 @@ void ScheduleState::UpdateMemoryRequirements(const TPoolConfig& pool_cfg,
       query_options().__isset.mem_limit_coordinators
       && query_options().mem_limit_coordinators > 0;
   if (!is_mem_limit_set && is_mem_limit_coordinators_set) {
-    coord_backend_mem_to_admit = query_options().mem_limit_coordinators;
+    query_schedule_pb_->set_coord_backend_mem_to_admit(
+        query_options().mem_limit_coordinators);
+    query_schedule_pb_->set_coord_backend_mem_to_admit_source(
+        MemLimitSourcePB::QUERY_OPTION_MEM_LIMIT_COORDINATORS);
   }
 
   // Enforce the MEM_LIMIT_EXECUTORS query option if MEM_LIMIT is not specified.
   const bool is_mem_limit_executors_set = query_options().__isset.mem_limit_executors
       && query_options().mem_limit_executors > 0;
   if (!is_mem_limit_set && is_mem_limit_executors_set) {
-    per_backend_mem_to_admit = query_options().mem_limit_executors;
+    query_schedule_pb_->set_per_backend_mem_to_admit(query_options().mem_limit_executors);
+    query_schedule_pb_->set_per_backend_mem_to_admit_source(
+        MemLimitSourcePB::QUERY_OPTION_MEM_LIMIT_EXECUTORS);
   }
 
   // Cap the memory estimate at the backend's memory limit for admission. The user's
   // provided value or the estimate from planning can each be unreasonable.
   if (FLAGS_clamp_query_mem_limit_backend_mem_limit) {
-    per_backend_mem_to_admit =
-        min(per_backend_mem_to_admit, executor_mem_limit_admission);
-    coord_backend_mem_to_admit =
-        min(coord_backend_mem_to_admit, coord_mem_limit_admission);
+    CompareMinBackendMemToAdmit(
+        executor_mem_limit_admission, MemLimitSourcePB::HOST_MEM_TRACKER_LIMIT);
+    CompareMinCoordinatorMemToAdmit(
+        coord_mem_limit_admission, MemLimitSourcePB::HOST_MEM_TRACKER_LIMIT);
   }
   // If the query is only scheduled to run on the coordinator.
   if (per_backend_schedule_states_.size() == 1 && RequiresCoordinatorFragment()) {
-    per_backend_mem_to_admit = 0;
+    query_schedule_pb_->set_per_backend_mem_to_admit(0);
+    query_schedule_pb_->set_per_backend_mem_to_admit_source(
+        MemLimitSourcePB::COORDINATOR_ONLY_OPTIMIZATION);
   }
 
-  int64_t per_backend_mem_limit;
   if (mimic_old_behaviour && !is_mem_limit_set && !is_mem_limit_executors_set
       && !is_mem_limit_coordinators_set) {
-    per_backend_mem_limit = -1;
+    query_schedule_pb_->set_per_backend_mem_limit(-1);
     query_schedule_pb_->set_coord_backend_mem_limit(-1);
   } else {
-    per_backend_mem_limit = per_backend_mem_to_admit;
-    query_schedule_pb_->set_coord_backend_mem_limit(coord_backend_mem_to_admit);
+    query_schedule_pb_->set_per_backend_mem_limit(
+        query_schedule_pb_->per_backend_mem_to_admit());
+    query_schedule_pb_->set_coord_backend_mem_limit(
+        query_schedule_pb_->coord_backend_mem_to_admit());
   }
-
-  query_schedule_pb_->set_coord_backend_mem_to_admit(coord_backend_mem_to_admit);
-  query_schedule_pb_->set_per_backend_mem_limit(per_backend_mem_limit);
-  query_schedule_pb_->set_per_backend_mem_to_admit(per_backend_mem_to_admit);
   query_schedule_pb_->set_cluster_mem_est(GetClusterMemoryToAdmit());
+
+  // Validate fields are set.
+  DCHECK(query_schedule_pb_->has_per_backend_mem_to_admit());
+  DCHECK(query_schedule_pb_->has_coord_backend_mem_to_admit());
+  DCHECK(query_schedule_pb_->has_per_backend_mem_to_admit_source());
+  DCHECK(query_schedule_pb_->has_coord_backend_mem_to_admit_source());
 }
 
 void ScheduleState::set_executor_group(string executor_group) {
