@@ -126,43 +126,9 @@ Status ChildQuery::ExecAndFetch() {
   return status;
 }
 
-template <typename T>
-void PrintQueryOptionValue (const T& option, stringstream& val) {
-  val << option;
-}
-
-void PrintQueryOptionValue(const impala::TCompressionCodec& compression_codec,
-    stringstream& val) {
-  if (compression_codec.codec != THdfsCompression::ZSTD) {
-    val << compression_codec.codec;
-  } else {
-    val << compression_codec.codec << ":" << compression_codec.compression_level;
-  }
-}
-
-void PrintQueryOptionValue(const set<impala::TRuntimeFilterType::type>& filter_types,
-    stringstream& val) {
-  val << filter_types;
-}
-
-void PrintQueryOptionValue(const std::set<int32_t>& filter_ids, stringstream& val) {
-  val << filter_ids;
-}
-
 void ChildQuery::SetQueryOptions(TExecuteStatementReq* exec_stmt_req) {
   map<string, string> conf;
-  const TQueryOptions& parent_options =
-      parent_request_state_->exec_request().query_options;
-#define QUERY_OPT_FN(NAME, ENUM, LEVEL)\
-  if (parent_options.__isset.NAME) {\
-    stringstream val;\
-    PrintQueryOptionValue(parent_options.NAME, val);\
-    conf[#ENUM] = val.str();\
-  }
-#define REMOVED_QUERY_OPT_FN(NAME, ENUM)
-  QUERY_OPTS_TABLE
-#undef QUERY_OPT_FN
-#undef REMOVED_QUERY_OPT_FN
+  TQueryOptionsToMap(parent_request_state_->exec_request().query_options, &conf);
   // Ignore debug actions on child queries because they may cause deadlock.
   map<string, string>::iterator it = conf.find("DEBUG_ACTION");
   if (it != conf.end()) conf.erase(it);
@@ -183,26 +149,28 @@ void ChildQuery::Cancel() {
     if (!is_running_) return;
     is_running_ = false;
   }
-  TUniqueId session_id;
+  TUniqueId query_id;
   TUniqueId secret_unused;
   // Ignore return statuses because they are not actionable.
-  Status status = ImpalaServer::THandleIdentifierToTUniqueId(hs2_handle_.operationId,
-      &session_id, &secret_unused);
+  Status status = ImpalaServer::THandleIdentifierToTUniqueId(
+      hs2_handle_.operationId, &query_id, &secret_unused);
   if (status.ok()) {
-    VLOG_QUERY << "Cancelling and closing child query with operation id: " <<
-        PrintId(session_id);
+    VLOG_QUERY << "Cancelling and closing child query with operation id: "
+               << PrintId(query_id);
   } else {
-    VLOG_QUERY << "Cancelling and closing child query. Failed to get query id: " <<
-        status;
+    VLOG_QUERY << "Cancelling and closing child query. Failed to get query id: "
+               << status;
+  }
+  // Bypass the HS2 layer so that the session of the child query will not be added to
+  // the connection of the caller of ChildQuery::Cancel().
+  status = parent_server_->CancelInternal(query_id);
+  if (!status.ok()) {
+    LOG(ERROR) << "Failed to cancel child query: " << status.GetDetail();
+  }
+  status = parent_server_->UnregisterQuery(query_id, true, &Status::CANCELLED);
+  if (!status.ok()) {
+    LOG(ERROR) << "Failed to unregister child query: " << status.GetDetail();
   }
-  TCancelOperationResp cancel_resp;
-  TCancelOperationReq cancel_req;
-  cancel_req.operationHandle = hs2_handle_;
-  parent_server_->CancelOperation(cancel_resp, cancel_req);
-  TCloseOperationResp close_resp;
-  TCloseOperationReq close_req;
-  close_req.operationHandle = hs2_handle_;
-  parent_server_->CloseOperation(close_resp, close_req);
 }
 
 Status ChildQuery::IsCancelled() {
