@@ -99,7 +99,9 @@ class ClientRequestState {
   void SetFrontendProfile(const TExecRequest& exec_request);
 
   /// Sets the coordinator time that the plan request was submitted at so that
-  /// the backend timeline starts where the frontend timeline ends
+  /// the backend timeline starts where the frontend timeline ends.
+  /// remote_submit_time will be ignored and replaced with internal
+  /// MonotonicStopWatch::Now() if the later is greater than the former.
   void SetRemoteSubmitTime(int64_t remote_submit_time);
 
   /// Based on query type, this either initiates execution of this ClientRequestState's
@@ -309,6 +311,11 @@ class ClientRequestState {
   /// ExecState is stored using an AtomicEnum, so reads do not require holding lock_.
   ExecState exec_state() const { return exec_state_.Load(); }
 
+  /// WaitForCompletionExecState waits until the state reaches FINISHED or ERROR
+  /// or it reaches the timeout. The timeout is specified using the long_polling_time_ms
+  /// query option.
+  void WaitForCompletionExecState();
+
   /// RetryState is stored using an AtomicEnum, so reads do not require holding lock_.
   RetryState retry_state() const { return retry_state_.Load(); }
 
@@ -338,6 +345,26 @@ class ClientRequestState {
   TUniqueId parent_query_id() const { return query_ctx_.parent_query_id; }
   const vector<TTableName>& tables() const { return exec_request().tables; }
 
+  const vector<std::string>& select_columns() const {
+    return exec_request().select_columns;
+  }
+
+  const vector<std::string>& where_columns() const {
+    return exec_request().where_columns;
+  }
+
+  const vector<std::string>& join_columns() const {
+    return exec_request().join_columns;
+  }
+
+  const vector<std::string>& aggregate_columns() const {
+    return exec_request().aggregate_columns;
+  }
+
+  const vector<std::string>& orderby_columns() const {
+    return exec_request().orderby_columns;
+  }
+
   const std::vector<std::string>& GetAnalysisWarnings() const {
     return exec_request().analysis_warnings;
   }
@@ -398,6 +425,11 @@ class ClientRequestState {
     return wait_end_time_ms - wait_start_time_ms_;
   }
 
+  /// Returns the time taken for the client to fetch all rows.
+  int64_t client_fetch_wait_time_ns() const {
+    return client_wait_timer_->value();
+  }
+
   /// Sets the RetryState to RETRYING. Updates the runtime profile with the retry status
   /// and cause. Must be called while 'lock_' is held. Sets the query_status_. Future
   /// calls to UpdateQueryStatus will not have any effect. This is necessary to prevent
@@ -565,6 +597,13 @@ class ClientRequestState {
   /// Callers are expected to use BlockOnWait() for Wait() to finish.
   ConditionVariable block_on_wait_cv_;
 
+  /// Lock used to synchronize access for exec_state_cv_. Some callers hold lock_ while
+  /// acquiring this lock, so if lock_ is needed it must be acquired before this lock.
+  std::mutex exec_state_lock_;
+
+  /// Condition variable used to signal threads that are watching the exec_state_.
+  ConditionVariable exec_state_cv_;
+
   /// Wait condition used in conjunction with block_on_wait_cv_.
   bool is_wait_done_ = false;
 
@@ -834,6 +873,9 @@ class ClientRequestState {
   /// Executes a shut down request.
   Status ExecShutdownRequest() WARN_UNUSED_RESULT;
 
+  /// Executes a command on EventProcessor.
+  Status ExecEventProcessorCmd() WARN_UNUSED_RESULT;
+
   /// Core logic of Wait(). Does not update operation_state_/query_status_.
   Status WaitInternal() WARN_UNUSED_RESULT;
 
@@ -939,6 +981,9 @@ class ClientRequestState {
   /// Core logic of executing a MIGRATE TABLE statement.
   void ExecMigrateRequestImpl();
 
+  /// The logic of executing a KILL QUERY statement.
+  Status ExecKillQueryRequest();
+
   /// Used when running into an error during table migration to extend 'status' with some
   /// hints about how to reset the original table name. 'params' holds the SQL query
   /// string the user should run.
@@ -946,5 +991,14 @@ class ClientRequestState {
 
   /// Adds the catalog execution timeline returned from catalog RPCs into the profile.
   void AddCatalogTimeline();
+
+  /// Try killing query with the given query_id locally as the requesting_user.
+  /// 'is_admin' is true if requesting_user is an admin.
+  Status TryKillQueryLocally(
+      const TUniqueId& query_id, const std::string& requesting_user, bool is_admin);
+
+  /// Try to ask other coordinators to kill query by sending the request.
+  Status TryKillQueryRemotely(
+      const TUniqueId& query_id, const KillQueryRequestPB& request);
 };
 }
