@@ -51,6 +51,9 @@ DEFINE_string(datastream_service_queue_mem_limit, "5%", QUEUE_LIMIT_MSG.c_str())
 DEFINE_int32(datastream_service_num_svc_threads, 0, "Number of threads for processing "
     "datastream services' RPCs. If left at default value 0, it will be set to number of "
     "CPU cores.  Set it to a positive value to change from the default.");
+DEFINE_int32_hidden(update_filter_min_wait_time_ms, 500,
+    "Minimum time for UpdateFilterFromRemote RPC to wait until destination QueryState is "
+    "ready.");
 DECLARE_string(debug_actions);
 
 namespace impala {
@@ -69,6 +72,7 @@ DataStreamService::DataStreamService(MetricGroup* metric_group)
   mem_tracker_.reset(new MemTracker(
       bytes_limit, "Data Stream Service Queue", process_mem_tracker));
   MemTrackerMetric::CreateMetrics(metric_group, mem_tracker_.get(), "DataStreamService");
+  DCHECK_EQ(methods_by_name_.count(string(END_DATA_STREAM)), 1);
 }
 
 Status DataStreamService::Init() {
@@ -122,6 +126,60 @@ void DataStreamService::UpdateFilter(
   RespondAndReleaseRpc(Status::OK(), resp, context, mem_tracker_.get());
 }
 
+void DataStreamService::UpdateFilterFromRemote(
+    const UpdateFilterParamsPB* req, UpdateFilterResultPB* resp, RpcContext* context) {
+  DCHECK(req->has_filter_id());
+  DCHECK(req->has_query_id());
+  DCHECK(
+      req->has_bloom_filter() || req->has_min_max_filter() || req->has_in_list_filter());
+  int64_t arrival_time = MonotonicMillis();
+
+  // Loop until destination QueryState is ready to accept filter update from remote.
+  // Sleep for few miliseconds in-between and break after 500ms grace period passed.
+  // The grace period is short so that RPC thread is not blocked for too long.
+  // This is a much simpler mechanism than KrpcDataStreamMgr::AddData.
+  // TODO: Revisit this with more sophisticated deferral mechanism if needed.
+  bool query_found = false;
+  int64_t total_wait_time = 0;
+  int32_t sleep_duration_ms = 2;
+  if (req->remaining_filter_wait_time_ms() < FLAGS_update_filter_min_wait_time_ms) {
+    LOG(INFO) << "UpdateFilterFromRemote RPC called with remaining wait time "
+              << req->remaining_filter_wait_time_ms() << " ms, less than "
+              << FLAGS_update_filter_min_wait_time_ms << " ms minimum wait time.";
+  }
+
+  do {
+    {
+      QueryState::ScopedRef qs(ProtoToQueryId(req->query_id()));
+      query_found |= (qs.get() != nullptr);
+      if (query_found) {
+        if (qs.get() == nullptr || qs->IsTerminalState()) {
+          // Query was found, but now is either missing or in terminal state.
+          // Break the loop and response with an error.
+          break;
+        } else if (qs->is_initialized()) {
+          qs->UpdateFilterFromRemote(*req, context);
+          RespondAndReleaseRpc(Status::OK(), resp, context, mem_tracker_.get());
+          return;
+        }
+      }
+    }
+    usleep(sleep_duration_ms * 1000);
+    // double sleep time for next iteration up to 128ms.
+    if (2 * sleep_duration_ms <= 128) sleep_duration_ms *= 2;
+    total_wait_time = MonotonicMillis() - arrival_time;
+  } while (total_wait_time < FLAGS_update_filter_min_wait_time_ms);
+
+  // Query state for requested query_id might have been cancelled, closed, or not ready.
+  // i.e., RUNTIME_FILTER_WAIT_TIME_MS has passed and all fragment instances of
+  // query_id has complete their execution.
+  string err_msg = Substitute("QueryState for query_id=$0 $1 after $2 ms",
+      PrintId(ProtoToQueryId(req->query_id())),
+      query_found ? "no longer running" : "not found", total_wait_time);
+  LOG(INFO) << err_msg;
+  RespondAndReleaseRpc(Status(err_msg), resp, context, mem_tracker_.get());
+}
+
 void DataStreamService::PublishFilter(
     const PublishFilterParamsPB* req, PublishFilterResultPB* resp, RpcContext* context) {
   // This failpoint is to allow jitter to be injected.
