@@ -43,6 +43,7 @@
 #include "runtime/mem-pool.h"
 #include "runtime/raw-value.h"
 #include "runtime/runtime-state.h"
+#include "scheduling/cluster-membership-mgr.h"
 #include "service/impala-server.h"
 #include "service/query-options.h"
 #include "util/bloom-filter.h"
@@ -136,7 +137,7 @@ static void SetTColumnValue(
     case TYPE_STRING:
     case TYPE_VARCHAR: {
       const StringValue* string_val = reinterpret_cast<const StringValue*>(value);
-      tmp.assign(static_cast<char*>(string_val->ptr), string_val->len);
+      tmp.assign(string_val->Ptr(), string_val->Len());
       col_val->binary_val.swap(tmp);
       col_val->__isset.binary_val = true;
       break;
@@ -256,9 +257,9 @@ Java_org_apache_impala_service_FeSupport_NativeEvalExprsWithoutRow(
     if (type.IsVarLenStringType()) {
       const StringValue* string_val = reinterpret_cast<const StringValue*>(result);
       if (string_val != nullptr) {
-        if (string_val->len > max_result_size) {
+        if (string_val->Len() > max_result_size) {
           status = Status(TErrorCode::EXPR_REWRITE_RESULT_LIMIT_EXCEEDED,
-              string_val->len, max_result_size);
+              string_val->Len(), max_result_size);
           goto error;
         }
       }
@@ -643,6 +644,35 @@ Java_org_apache_impala_service_FeSupport_NativeParseQueryOptions(
   return result_bytes;
 }
 
+// Get a list of known coordinators.
+extern "C" JNIEXPORT jbyteArray JNICALL
+Java_org_apache_impala_service_FeSupport_NativeGetCoordinators(
+    JNIEnv* env, jclass caller_class) {
+  ClusterMembershipMgr::SnapshotPtr membership_snapshot =
+      ExecEnv::GetInstance()->cluster_membership_mgr()->GetSnapshot();
+  DCHECK(membership_snapshot != nullptr);
+  vector<TNetworkAddress> coordinators = membership_snapshot->GetCoordinatorAddresses();
+
+  TAddressesList addresses_container;
+  addresses_container.__set_addresses(coordinators);
+  jbyteArray result_bytes = nullptr;
+  THROW_IF_ERROR_RET(SerializeThriftMsg(env, &addresses_container, &result_bytes), env,
+      JniUtil::internal_exc_class(), result_bytes);
+  return result_bytes;
+}
+
+// Get the number of live queries.
+extern "C" JNIEXPORT jlong JNICALL
+Java_org_apache_impala_service_FeSupport_NativeNumLiveQueries(
+    JNIEnv* env, jclass caller_class) {
+  ImpalaServer* server = ExecEnv::GetInstance()->impala_server();
+  if (LIKELY(server != nullptr)) {
+    return server->NumLiveQueries();
+  }
+  // Allow calling without an ImpalaServer, such as during PlannerTest.
+  return 0;
+}
+
 // Returns the log (base 2) of the minimum number of bytes we need for a Bloom filter
 // with 'ndv' unique elements and a false positive probability of less than 'fpp'.
 extern "C"
@@ -695,6 +725,46 @@ Java_org_apache_impala_service_FeSupport_nativeParseDateString(JNIEnv* env,
   return result_bytes;
 }
 
+// Native method to make a request to catalog server to get the null partition name.
+extern "C" JNIEXPORT jbyteArray JNICALL
+Java_org_apache_impala_service_FeSupport_NativeGetNullPartitionName(
+    JNIEnv* env, jclass fe_support_class, jbyteArray thrift_struct) {
+  TGetNullPartitionNameRequest request;
+  THROW_IF_ERROR_RET(DeserializeThriftMsg(env, thrift_struct, &request), env,
+      JniUtil::internal_exc_class(), nullptr);
+  CatalogOpExecutor catalog_op_executor(ExecEnv::GetInstance(), nullptr, nullptr);
+  TGetNullPartitionNameResponse result;
+  Status status = catalog_op_executor.GetNullPartitionName(request, &result);
+  if (!status.ok()) {
+    LOG(ERROR) << status.GetDetail();
+    status.ToThrift(&result.status);
+  }
+  jbyteArray result_bytes = nullptr;
+  THROW_IF_ERROR_RET(SerializeThriftMsg(env, &result, &result_bytes), env,
+      JniUtil::internal_exc_class(), result_bytes);
+  return result_bytes;
+}
+
+// Native method to make a request to catalog server to get the latest compactions.
+extern "C" JNIEXPORT jbyteArray JNICALL
+Java_org_apache_impala_service_FeSupport_NativeGetLatestCompactions(
+    JNIEnv* env, jclass fe_support_class, jbyteArray thrift_struct) {
+  TGetLatestCompactionsRequest request;
+  THROW_IF_ERROR_RET(DeserializeThriftMsg(env, thrift_struct, &request), env,
+      JniUtil::internal_exc_class(), nullptr);
+  CatalogOpExecutor catalog_op_executor(ExecEnv::GetInstance(), nullptr, nullptr);
+  TGetLatestCompactionsResponse result;
+  Status status = catalog_op_executor.GetLatestCompactions(request, &result);
+  if (!status.ok()) {
+    LOG(ERROR) << status.GetDetail();
+    status.ToThrift(&result.status);
+  }
+  jbyteArray result_bytes = nullptr;
+  THROW_IF_ERROR_RET(SerializeThriftMsg(env, &result, &result_bytes), env,
+      JniUtil::internal_exc_class(), result_bytes);
+  return result_bytes;
+}
+
 namespace impala {
 
 static JNINativeMethod native_methods[] = {
@@ -770,6 +840,22 @@ static JNINativeMethod native_methods[] = {
     const_cast<char*>("(Ljava/lang/String;)[B"),
     (void*)::Java_org_apache_impala_service_FeSupport_nativeParseDateString
   },
+  {
+    const_cast<char*>("NativeGetNullPartitionName"), const_cast<char*>("([B)[B"),
+    (void*) ::Java_org_apache_impala_service_FeSupport_NativeGetNullPartitionName
+  },
+  {
+    const_cast<char*>("NativeGetLatestCompactions"), const_cast<char*>("([B)[B"),
+    (void*) ::Java_org_apache_impala_service_FeSupport_NativeGetLatestCompactions
+  },
+  {
+    const_cast<char*>("NativeGetCoordinators"), const_cast<char*>("()[B"),
+    (void*)::Java_org_apache_impala_service_FeSupport_NativeGetCoordinators
+  },
+  {
+    const_cast<char*>("NativeNumLiveQueries"), const_cast<char*>("()J"),
+    (void*)::Java_org_apache_impala_service_FeSupport_NativeNumLiveQueries
+  },
 };
 
 void InitFeSupport(bool disable_codegen) {
