@@ -69,12 +69,19 @@ const TProtocolVersion::type MAX_SUPPORTED_HS2_VERSION =
     TProtocolVersion::HIVE_CLI_SERVICE_PROTOCOL_V6;
 
 // HiveServer2 error returning macro
-#define HS2_RETURN_ERROR(return_val, error_msg, error_state) \
-  do { \
-    return_val.status.__set_statusCode(thrift::TStatusCode::ERROR_STATUS); \
-    return_val.status.__set_errorMessage((error_msg)); \
-    return_val.status.__set_sqlState((error_state)); \
-    return; \
+//
+// To include query id in the error message, it is required that the query id of the
+// thread debug info is set in at least the caller's scope.
+#define HS2_RETURN_ERROR(return_val, error_msg, error_state)                \
+  do {                                                                      \
+    return_val.status.__set_statusCode(thrift::TStatusCode::ERROR_STATUS);  \
+    return_val.status.__set_errorMessage(                                   \
+        GetThreadDebugInfo()->GetQueryId() == TUniqueId() ?                 \
+            (error_msg) :                                                   \
+            Substitute(ImpalaServer::QUERY_ERROR_FORMAT,                    \
+                PrintId(GetThreadDebugInfo()->GetQueryId()), (error_msg))); \
+    return_val.status.__set_sqlState((error_state));                        \
+    return;                                                                 \
   } while (false)
 
 #define HS2_RETURN_IF_ERROR(return_val, status, error_state) \
@@ -157,7 +164,8 @@ void ImpalaServer::ExecuteMetadataOp(const THandleIdentifier& session_handle,
   Status register_status = RegisterQuery(query_ctx.query_id, session, &query_handle);
   if (!register_status.ok()) {
     status->__set_statusCode(thrift::TStatusCode::ERROR_STATUS);
-    status->__set_errorMessage(register_status.GetDetail());
+    status->__set_errorMessage(Substitute(
+        QUERY_ERROR_FORMAT, PrintId(query_ctx.query_id), register_status.GetDetail()));
     status->__set_sqlState(SQLSTATE_GENERAL_ERROR);
     return;
   }
@@ -166,7 +174,8 @@ void ImpalaServer::ExecuteMetadataOp(const THandleIdentifier& session_handle,
   if (!exec_status.ok()) {
     discard_result(UnregisterQuery(query_handle->query_id(), false, &exec_status));
     status->__set_statusCode(thrift::TStatusCode::ERROR_STATUS);
-    status->__set_errorMessage(exec_status.GetDetail());
+    status->__set_errorMessage(Substitute(
+        QUERY_ERROR_FORMAT, PrintId(query_handle->query_id()), exec_status.GetDetail()));
     status->__set_sqlState(SQLSTATE_GENERAL_ERROR);
     return;
   }
@@ -177,7 +186,8 @@ void ImpalaServer::ExecuteMetadataOp(const THandleIdentifier& session_handle,
   if (!inflight_status.ok()) {
     discard_result(UnregisterQuery(query_handle->query_id(), false, &inflight_status));
     status->__set_statusCode(thrift::TStatusCode::ERROR_STATUS);
-    status->__set_errorMessage(inflight_status.GetDetail());
+    status->__set_errorMessage(Substitute(QUERY_ERROR_FORMAT,
+        PrintId(query_handle->query_id()), inflight_status.GetDetail()));
     status->__set_sqlState(SQLSTATE_GENERAL_ERROR);
     return;
   }
@@ -188,7 +198,7 @@ void ImpalaServer::ExecuteMetadataOp(const THandleIdentifier& session_handle,
   status->__set_statusCode(thrift::TStatusCode::SUCCESS_STATUS);
 }
 
-Status ImpalaServer::FetchInternal(TUniqueId query_id, SessionState* session,
+Status ImpalaServer::FetchInternal(const TUniqueId& query_id, SessionState* session,
     int32_t fetch_size, bool fetch_first, TFetchResultsResp* fetch_results,
     int32_t* num_results) {
   bool timed_out = false;
@@ -333,6 +343,7 @@ void ImpalaServer::OpenSession(TOpenSessionResp& return_val,
     state->session_type = TSessionType::HIVESERVER2;
   }
   state->network_address = ThriftServer::GetThreadConnectionContext()->network_address;
+  state->http_origin = ThriftServer::GetThreadConnectionContext()->http_origin;
   state->last_accessed_ms = UnixMillis();
   // request.client_protocol is not guaranteed to be a valid TProtocolVersion::type, so
   // loading it can cause undefined behavior. Instead, we copy it to a value of the
@@ -357,6 +368,11 @@ void ImpalaServer::OpenSession(TOpenSessionResp& return_val,
   } else {
     state->connected_user = FLAGS_anonymous_user_name;
   }
+  // Set the 'connected_user_short' member in the SessionState. This is only used if
+  // the 'connected_user' is a kerberos principal.
+  if (!connection_context->kerberos_user_short.empty()) {
+    state->connected_user_short = connection_context->kerberos_user_short;
+  }
 
   // Process the supplied configuration map.
   state->database = "default";
@@ -462,29 +478,29 @@ void ImpalaServer::OpenSession(TOpenSessionResp& return_val,
              << "<" << TNetworkAddressToString(state->network_address) << ">.";
 }
 
-void ImpalaServer::DecrementCount(
-    std::map<std::string, int64>& loads, const std::string& key) {
+int64_t ImpalaServer::DecrementCount(
+    std::map<std::string, int64_t>& loads, const std::string& key) {
   // Check if key is present as dereferencing the map will insert it.
   // FIXME C++20: use contains().
   if (!loads.count(key)) {
     string msg = Substitute("Missing key $0 when decrementing count", key);
     LOG(WARNING) << msg;
     DCHECK(false) << msg;
-    return;
+    return 0;
   }
-  int64& current_value = loads[key];
+  int64_t& current_value = loads[key];
   if (current_value == 1) {
     // Remove the entry from the map if the current_value will go to zero.
     loads.erase(key);
-    return;
+    return 0;
   }
   if (current_value < 1) {
     // Don't allow decrement below zero.
     string msg = Substitute("Attempt to decrement below zero with key $0 ", key);
     LOG(WARNING) << msg;
-    return;
+    return 0;
   }
-  loads[key]--;
+  return --loads[key];
 }
 
 void ImpalaServer::DecrementSessionCount(const string& user_name) {
@@ -499,7 +515,7 @@ Status ImpalaServer::IncrementAndCheckSessionCount(const string& user_name) {
     lock_guard<mutex> l(per_user_session_count_lock_);
     // Only check user limit if there is already a session for the user.
     if (per_user_session_count_map_.count(user_name)) {
-      int64 load = per_user_session_count_map_[user_name];
+      int64_t load = per_user_session_count_map_[user_name];
       if (load >= FLAGS_max_hs2_sessions_per_user) {
         const string& err_msg =
             Substitute("Number of sessions for user $0 exceeds coordinator limit $1",
@@ -606,6 +622,10 @@ void ImpalaServer::ExecuteStatementCommon(TExecuteStatementResp& return_val,
 
   QueryHandle query_handle;
   status = Execute(&query_ctx, session, &query_handle, external_exec_request);
+
+  // Make query id available to the following HS2_RETURN_IF_ERROR().
+  ScopedThreadContext scoped_tdi(GetThreadDebugInfo(), query_handle->query_id());
+
   HS2_RETURN_IF_ERROR(return_val, status, SQLSTATE_GENERAL_ERROR);
 
   // Start thread to wait for results to become available.
@@ -638,7 +658,7 @@ void ImpalaServer::ExecuteStatementCommon(TExecuteStatementResp& return_val,
 }
 
 Status ImpalaServer::SetupResultsCacheing(const QueryHandle& query_handle,
-    shared_ptr<SessionState> session, int64_t cache_num_rows) {
+    const shared_ptr<SessionState>& session, int64_t cache_num_rows) {
   // Optionally enable result caching on the ClientRequestState.
   if (cache_num_rows > 0) {
     const TResultSetMetadata* result_set_md = query_handle->result_metadata();
@@ -875,6 +895,9 @@ void ImpalaServer::GetOperationStatus(TGetOperationStatusResp& return_val,
       SQLSTATE_GENERAL_ERROR);
   VLOG_ROW << "GetOperationStatus(): query_id=" << PrintId(query_id);
 
+  // Make query id available to the following HS2_RETURN_IF_ERROR().
+  ScopedThreadContext scoped_tdi(GetThreadDebugInfo(), query_id);
+
   QueryHandle query_handle;
   HS2_RETURN_IF_ERROR(
       return_val, GetActiveQueryHandle(query_id, &query_handle), SQLSTATE_GENERAL_ERROR);
@@ -887,13 +910,18 @@ void ImpalaServer::GetOperationStatus(TGetOperationStatusResp& return_val,
           session_id, SecretArg::Operation(op_secret, query_id), &session),
       SQLSTATE_GENERAL_ERROR);
 
+  // When using long polling, this waits up to long_polling_time_ms milliseconds for
+  // query completion.polling
+  query_handle->WaitForCompletionExecState();
+
   {
     lock_guard<mutex> l(*query_handle->lock());
     TOperationState::type operation_state = query_handle->TOperationState();
     return_val.__set_operationState(operation_state);
     if (operation_state == TOperationState::ERROR_STATE) {
       DCHECK(!query_handle->query_status().ok());
-      return_val.__set_errorMessage(query_handle->query_status().GetDetail());
+      return_val.__set_errorMessage(Substitute(QUERY_ERROR_FORMAT,
+          PrintId(query_id), query_handle->query_status().GetDetail()));
       return_val.__set_sqlState(SQLSTATE_GENERAL_ERROR);
     } else {
       ClientRequestState::RetryState retry_state = query_handle->retry_state();
@@ -914,6 +942,9 @@ void ImpalaServer::CancelOperation(TCancelOperationResp& return_val,
       SQLSTATE_GENERAL_ERROR);
   VLOG_QUERY << "CancelOperation(): query_id=" << PrintId(query_id);
 
+  // Make query id available to the following HS2_RETURN_IF_ERROR().
+  ScopedThreadContext scoped_tdi(GetThreadDebugInfo(), query_id);
+
   QueryHandle query_handle;
   HS2_RETURN_IF_ERROR(
       return_val, GetActiveQueryHandle(query_id, &query_handle), SQLSTATE_GENERAL_ERROR);
@@ -945,6 +976,9 @@ void ImpalaServer::CloseImpalaOperation(TCloseImpalaOperationResp& return_val,
       SQLSTATE_GENERAL_ERROR);
   VLOG_QUERY << "CloseOperation(): query_id=" << PrintId(query_id);
 
+  // Make query id available to the following HS2_RETURN_IF_ERROR().
+  ScopedThreadContext scoped_tdi(GetThreadDebugInfo(), query_id);
+
   QueryHandle query_handle;
   HS2_RETURN_IF_ERROR(
       return_val, GetActiveQueryHandle(query_id, &query_handle), SQLSTATE_GENERAL_ERROR);
@@ -977,6 +1011,9 @@ void ImpalaServer::GetResultSetMetadata(TGetResultSetMetadataResp& return_val,
       SQLSTATE_GENERAL_ERROR);
   VLOG_QUERY << "GetResultSetMetadata(): query_id=" << PrintId(query_id);
 
+  // Make query id available to the following HS2_RETURN_IF_ERROR().
+  ScopedThreadContext scoped_tdi(GetThreadDebugInfo(), query_id);
+
   QueryHandle query_handle;
   HS2_RETURN_IF_ERROR(
       return_val, GetActiveQueryHandle(query_id, &query_handle), SQLSTATE_GENERAL_ERROR);
@@ -1028,6 +1065,8 @@ void ImpalaServer::FetchResults(TFetchResultsResp& return_val,
   VLOG_ROW << "FetchResults(): query_id=" << PrintId(query_id)
            << " fetch_size=" << request.maxRows;
 
+  // Make query id available to the following HS2_RETURN_IF_ERROR().
+  ScopedThreadContext scoped_tdi(GetThreadDebugInfo(), query_id);
 
   QueryHandle query_handle;
   HS2_RETURN_IF_ERROR(
@@ -1072,6 +1111,8 @@ void ImpalaServer::GetLog(TGetLogResp& return_val, const TGetLogReq& request) {
       request.operationHandle.operationId, &query_id, &op_secret),
       SQLSTATE_GENERAL_ERROR);
 
+  // Make query id available to the following HS2_RETURN_IF_ERROR().
+  ScopedThreadContext scoped_tdi(GetThreadDebugInfo(), query_id);
 
   QueryHandle query_handle;
   HS2_RETURN_IF_ERROR(
@@ -1113,7 +1154,10 @@ void ImpalaServer::GetLog(TGetLogResp& return_val, const TGetLogReq& request) {
     DCHECK_EQ(query_handle->exec_state() == ClientRequestState::ExecState::ERROR,
         !query_status.ok());
     // If the query status is !ok, include the status error message at the top of the log.
-    if (!query_status.ok()) ss << query_status.GetDetail();
+    if (!query_status.ok()) {
+      ss << Substitute(QUERY_ERROR_FORMAT, PrintId(query_handle->query_id()),
+          query_status.GetDetail());
+    }
   }
 
   // Report analysis errors
@@ -1166,6 +1210,11 @@ void ImpalaServer::GetExecSummary(TGetExecSummaryResp& return_val,
           request.operationHandle.operationId, &query_id, &op_secret),
       SQLSTATE_GENERAL_ERROR);
 
+  VLOG_QUERY << "GetExecSummary(): query_id=" << PrintId(query_id);
+
+  // Make query id available to the following HS2_RETURN_IF_ERROR().
+  ScopedThreadContext scoped_tdi(GetThreadDebugInfo(), query_id);
+
   TExecSummary summary;
   TExecSummary original_summary;
   bool was_retried = false;
@@ -1250,7 +1299,10 @@ void ImpalaServer::GetRuntimeProfile(
       request.operationHandle.operationId, &query_id, &op_secret),
       SQLSTATE_GENERAL_ERROR);
 
-  VLOG_RPC << "GetRuntimeProfile(): query_id=" << PrintId(query_id);
+  VLOG_QUERY << "GetRuntimeProfile(): query_id=" << PrintId(query_id);
+
+  // Make query id available to the following HS2_RETURN_IF_ERROR().
+  ScopedThreadContext scoped_tdi(GetThreadDebugInfo(), query_id);
 
   // Set the RuntimeProfileOutput for the retried query (e.g. the second attempt of a
   // query). If the query has been retried this will be the retried profile. If the
