@@ -36,13 +36,15 @@
 #include "gen-cpp/ImpalaService.h"
 #include "gen-cpp/control_service.pb.h"
 #include "gen-cpp/Query_types.h"
+#include "gen-cpp/TCLIService_types.h"
 #include "kudu/util/random.h"
+#include "kudu/util/version_util.h"
 #include "rpc/thrift-server.h"
 #include "runtime/types.h"
 #include "service/internal-server.h"
 #include "service/query-options.h"
 #include "service/query-state-record.h"
-#include "service/workload-management.h"
+#include "service/workload-management-worker.h"
 #include "statestore/statestore-subscriber.h"
 #include "util/condition-variable.h"
 #include "util/container-util.h"
@@ -183,13 +185,14 @@ class TQueryExecRequest;
 /// locks are acquired, lower-numbered locks must be acquired before higher-numbered
 /// locks:
 /// 1. session_state_map_lock_
-/// 2. SessionState::lock
-/// 3. query_expiration_lock_
+/// 2. query_expiration_lock_
+/// 3. SessionState::lock
 /// 4. idle_query_statuses_lock_
 /// 5. ClientRequestState::fetch_rows_lock
 /// 6. ClientRequestState::lock
 /// 7. ClientRequestState::expiration_data_lock_
 /// 8. Coordinator::exec_summary_lock
+/// 9. ClientRequestState::exec_state_lock_
 ///
 /// The following locks are not held in conjunction with other locks:
 /// * query_log_lock_
@@ -415,16 +418,19 @@ class ImpalaServer : public ImpalaServiceIf,
   virtual Status ExecuteIgnoreResults(const std::string& user_name,
       const std::string& sql, const QueryOptionMap& query_opts = {},
       const bool persist_in_db = true, TUniqueId* query_id = nullptr);
-  virtual Status ExecuteAndFetchAllText(const std::string& user_name,
-      const std::string& sql, query_results& results, results_columns* columns = nullptr,
-      TUniqueId* query_id = nullptr);
+  virtual Status ExecuteAndFetchAllHS2(const std::string& user_name,
+      const std::string& sql,
+      std::vector<apache::hive::service::cli::thrift::TRow>& results,
+      const QueryOptionMap& query_opts = {}, const bool persist_in_db = true,
+      results_columns* columns = nullptr, TUniqueId* query_id = nullptr);
   virtual Status SubmitAndWait(const std::string& user_name, const std::string& sql,
       TUniqueId& new_session_id, TUniqueId& new_query_id,
       const QueryOptionMap& query_opts = {}, const bool persist_in_db = true);
   virtual Status WaitForResults(TUniqueId& query_id);
   virtual Status SubmitQuery(const std::string& sql, const impala::TUniqueId& session_id,
       TUniqueId& new_query_id, const bool persist_in_db = true);
-  virtual Status FetchAllRows(const TUniqueId& query_id, query_results& results,
+  virtual Status FetchAllRowsHS2(const TUniqueId& query_id,
+      std::vector<apache::hive::service::cli::thrift::TRow>& results,
       results_columns* columns = nullptr);
   virtual void CloseQuery(const TUniqueId& query_id);
   virtual void GetConnectionContextList(
@@ -508,7 +514,7 @@ class ImpalaServer : public ImpalaServiceIf,
 
   /// Return the number of live queries managed by this server. Acquires
   /// completed_queries_lock_ to check for completed queries that have not been written.
-  /// (implemented in workload-management.cc)
+  /// (implemented in workload-management-worker.cc)
   size_t NumLiveQueries();
 
   /// Returns a current snapshot of the local backend descriptor.
@@ -608,12 +614,20 @@ class ImpalaServer : public ImpalaServiceIf,
     /// Connected user for this session, i.e. the user which originated this session.
     std::string connected_user;
 
+    /// Short username for connected user. Only applies when connected_user is a kerberos
+    /// principal. Only applicable for hs2 sessions.
+    std::string connected_user_short;
+
     /// The user to delegate to. Empty for no delegation.
     std::string do_as_user;
 
     /// Client network address.
     TNetworkAddress network_address;
 
+    /// If using hs2-http protocol, this is the origin of the session
+    /// as recorded in the X-Forwarded-For http message header.
+    std::string http_origin;
+
     /// Protects all fields below. See "Locking" in the class comment for lock
     /// acquisition order.
     std::mutex lock;
@@ -702,9 +716,17 @@ class ImpalaServer : public ImpalaServiceIf,
     TQueryOptions QueryOptions();
   };
 
-  /// Helper function that decrements the value associated with the given key.
-  /// Removes the entry from the map if the value becomes zero.
-  static void DecrementCount(std::map<std::string, int64>& loads, const std::string& key);
+  /// Helper function that decrements the value associated with the given key, and
+  /// returns the new value. If the value becomes zero, then the entry is removed from
+  /// the map
+  static int64_t DecrementCount(
+      std::map<std::string, int64_t>& loads, const std::string& key);
+
+  /// Try cancelling and unregistering the query with the given query_id as the
+  /// requesting_user. 'is_admin' is true if requesting_user is an admin.
+  /// This method is used in executing a KILL QUERY statement.
+  Status KillQuery(
+      const TUniqueId& query_id, const std::string& requesting_user, bool is_admin);
 
  private:
   struct ExpirationEvent;
@@ -733,6 +755,9 @@ class ImpalaServer : public ImpalaServiceIf,
   static const char* SQLSTATE_OPTIONAL_FEATURE_NOT_IMPLEMENTED;
   /// String format of retry information returned in GetLog() RPCs.
   static const char* GET_LOG_QUERY_RETRY_INFO_FORMAT;
+  /// String format of errors related to a query. It contains a placeholder for
+  /// the query id.
+  static const char* QUERY_ERROR_FORMAT;
 
   /// Used in situations where the client provides a session ID and a query ID and the
   /// caller needs to validate that the query can be accessed from the session. The two
@@ -759,14 +784,14 @@ class ImpalaServer : public ImpalaServiceIf,
   /// have been checked out.
   /// external_exec_request is a statement that was prepared by an external frontend using
   /// Impala PlanNodes or null if the external frontend isn't being used.
-  Status Execute(TQueryCtx* query_ctx, std::shared_ptr<SessionState> session_state,
+  Status Execute(TQueryCtx* query_ctx, const std::shared_ptr<SessionState>& session_state,
       QueryHandle* query_handle, const TExecRequest* external_exec_request,
       const bool include_in_query_log = true) WARN_UNUSED_RESULT;
 
   /// Implements Execute() logic, but doesn't unregister query on error.
   Status ExecuteInternal(const TQueryCtx& query_ctx,
       const TExecRequest* external_exec_request,
-      std::shared_ptr<SessionState> session_state, bool* registered_query,
+      const std::shared_ptr<SessionState>& session_state, bool* registered_query,
       QueryHandle* query_handle);
 
   /// Common execution logic factored out from Execute to be shared with
@@ -780,7 +805,7 @@ class ImpalaServer : public ImpalaServiceIf,
   /// Registers the query with query_driver_map_ using the globally unique query_id. The
   /// caller must have checked out the session state.
   Status RegisterQuery(const TUniqueId& query_id,
-      std::shared_ptr<SessionState> session_state,
+      const std::shared_ptr<SessionState>& session_state,
       QueryHandle* query_handle) WARN_UNUSED_RESULT;
 
   /// Adds the query to the set of in-flight queries for the session. The query remains
@@ -797,7 +822,7 @@ class ImpalaServer : public ImpalaServiceIf,
   ///
   /// The query must have already been registered using RegisterQuery(). The caller
   /// must have checked out the session state.
-  Status SetQueryInflight(std::shared_ptr<SessionState> session_state,
+  Status SetQueryInflight(const std::shared_ptr<SessionState>& session_state,
       const QueryHandle& query_handle) WARN_UNUSED_RESULT;
 
   /// Starts the process of unregistering the query. The query is cancelled on the
@@ -965,9 +990,9 @@ class ImpalaServer : public ImpalaServiceIf,
   std::string ColumnTypeToBeeswaxTypeString(const TColumnType& type);
 
   /// Places a completed query into the in-memory queue of completed queries.
-  /// (implemented in workload-management.cc)
-  void EnqueueCompletedQuery(const QueryHandle& query_handle,
-      const std::shared_ptr<QueryStateRecord> qs_rec);
+  /// (implemented in workload-management-worker.cc)
+  void EnqueueCompletedQuery(
+      const QueryHandle& query_handle, std::shared_ptr<QueryStateRecord> qs_rec);
 
   /// Returns the active QueryHandle for this query id. The QueryHandle contains the
   /// active ClientRequestState. Returns an error Status if the query id cannot be found.
@@ -1010,11 +1035,14 @@ class ImpalaServer : public ImpalaServiceIf,
       const beeswax::QueryHandle& beeswax_handle, TUniqueId* query_id);
 
   /// Helper function to raise BeeswaxException
+  ///
+  /// To include query id in the error message, it is required that the query id of the
+  /// thread debug info is set in at least the caller's scope.
   [[noreturn]] void RaiseBeeswaxException(const std::string& msg, const char* sql_state);
 
   /// Executes the fetch logic. Doesn't clean up the exec state if an error occurs.
-  Status FetchInternal(TUniqueId query_id, bool start_over,
-      int32_t fetch_size, beeswax::Results* query_results) WARN_UNUSED_RESULT;
+  Status FetchInternal(const TUniqueId& query_id, bool start_over, int32_t fetch_size,
+      beeswax::Results* query_results) WARN_UNUSED_RESULT;
 
   /// Populate dml_result and clean up exec state. If the query
   /// status is an error, dml_result is not populated and the status is returned.
@@ -1042,8 +1070,8 @@ class ImpalaServer : public ImpalaServiceIf,
   /// 'num_results'. If fetch_first is true, then the query's state should be reset to
   /// fetch from the beginning of the result set. Doesn't clean up exec state if an
   /// error occurs.
-  Status FetchInternal(TUniqueId query_id, SessionState* session, int32_t fetch_size,
-      bool fetch_first,
+  Status FetchInternal(const TUniqueId& query_id, SessionState* session,
+      int32_t fetch_size, bool fetch_first,
       apache::hive::service::cli::thrift::TFetchResultsResp* fetch_results,
       int32_t* num_results) WARN_UNUSED_RESULT;
 
@@ -1055,7 +1083,8 @@ class ImpalaServer : public ImpalaServiceIf,
   /// dynamically scroll through the results, but also allow users to download a file
   /// containing all the results for a query.
   Status SetupResultsCacheing(const QueryHandle& query_handle,
-      std::shared_ptr<SessionState> session, int64_t cache_num_rows) WARN_UNUSED_RESULT;
+      const std::shared_ptr<SessionState>& session,
+      int64_t cache_num_rows) WARN_UNUSED_RESULT;
 
   /// Helper functions to translate between HiveServer2 and Impala structs
 
@@ -1133,21 +1162,22 @@ class ImpalaServer : public ImpalaServiceIf,
   /// current query ids to the admissiond.
   [[noreturn]] void AdmissionHeartbeatThread();
 
-  /// If workload management is enabled, starts workload management threads.
-  /// (implemented in workload-management.cc)
-  Status InitWorkloadManagement();
-
   /// Blocks until running workload management threads are shut down.
-  /// (implemented in workload-management.cc)
+  /// (implemented in workload-management-worker.cc)
   void ShutdownWorkloadManagement();
 
   /// Periodically writes out completed queries (if configured)
-  /// (implemented in workload-management.cc)
-  void CompletedQueriesThread();
+  /// (implemented in workload-management-worker.cc)
+  void WorkloadManagementWorker(const kudu::Version& target_schema_version);
+
+  /// Checks if the workload management state is `SHUTTING_DOWN`. If so, changes the
+  /// workload management state to `SHUTDOWN`,  and returns true. Otherwise, takes no
+  /// action and returns false.
+  bool IsWorkloadManagementShuttingDown();
 
   /// Returns a list of completed queries that have not yet been written to storage.
   /// Acquires completed_queries_lock_ to make a copy of completed_queries_ state.
-  /// (implemented in workload-management.cc)
+  /// (implemented in workload-management-worker.cc)
   std::vector<std::shared_ptr<QueryStateExpanded>> GetCompletedQueries();
 
   /// Called from ExpireQueries() to check query resource limits for 'crs'. If the query
@@ -1210,10 +1240,10 @@ class ImpalaServer : public ImpalaServiceIf,
   /// Decompresses the profile in the given QueryStateRecord into the specified format.
   /// The decompressed profile is added to the given RuntimeProfileOutput.
   Status DecompressToProfile(TRuntimeProfileFormat::type format,
-      std::shared_ptr<QueryStateRecord> query_record, RuntimeProfileOutput* profile);
+      const QueryStateRecord& query_record, RuntimeProfileOutput* profile);
 
-  void WaitForNewCatalogServiceId(TUniqueId cur_service_id,
-      std::unique_lock<std::mutex>* ver_lock);
+  void WaitForNewCatalogServiceId(
+      const TUniqueId& cur_service_id, std::unique_lock<std::mutex>* ver_lock);
 
   /// Random `impala::TUniqueID` generator. Use wherever a new `TUniqueId` is needed.
   TUniqueId RandomUniqueID();
@@ -1361,30 +1391,7 @@ class ImpalaServer : public ImpalaServiceIf,
     /// 'secret' must be provided and is validated against the stored secret for the
     /// session. Must only be called once per ScopedSessionState.
     Status WithSession(const TUniqueId& session_id, const SecretArg& secret,
-        std::shared_ptr<SessionState>* session = NULL) WARN_UNUSED_RESULT {
-      DCHECK(session_.get() == NULL);
-      RETURN_IF_ERROR(impala_->GetSessionState(
-            session_id, secret, &session_, /* mark_active= */ true));
-      if (session != NULL) (*session) = session_;
-
-      // We won't have a connection context in the case of ChildQuery, which calls into
-      // hiveserver2 functions directly without going through the Thrift stack.
-      if (ThriftServer::HasThreadConnectionContext()) {
-        // Check that the session user matches the user authenticated on the connection.
-        const ThriftServer::Username& connection_username =
-            ThriftServer::GetThreadConnectionContext()->username;
-        if (!connection_username.empty()
-            && session_->connected_user != connection_username) {
-          return Status::Expected(TErrorCode::UNAUTHORIZED_SESSION_USER,
-              connection_username, session_->connected_user);
-        }
-
-        // Try adding the session id to the connection's set of sessions in case this is
-        // the first time this session has been used on this connection.
-        impala_->AddSessionToConnection(session_id, session_.get());
-      }
-      return Status::OK();
-    }
+        std::shared_ptr<SessionState>* session = NULL) WARN_UNUSED_RESULT;
 
     /// Same as WithSession(), except:
     /// * It should only be called from beeswax with a 'connection_id' obtained from
@@ -1439,7 +1446,7 @@ class ImpalaServer : public ImpalaServiceIf,
   std::mutex connection_to_sessions_map_lock_;
 
   /// A map from user to a count of sessions created by the user.
-  typedef std::map<std::string, int64> SessionCounts;
+  typedef std::map<std::string, int64_t> SessionCounts;
   SessionCounts per_user_session_count_map_;
 
   /// Protects per_user_session_count_map_. See "Locking" in the class comment for lock
@@ -1460,9 +1467,10 @@ class ImpalaServer : public ImpalaServiceIf,
   typedef boost::unordered_map<TUniqueId, std::set<TUniqueId>> ConnectionToSessionMap;
   ConnectionToSessionMap connection_to_sessions_map_;
 
-  /// Map storing connections opened by the InternalServer functions. Key is the session
-  /// id, value is a shared pointer holding the connection's ConnectionContext. Use the
-  /// `internal_server_connections_lock_` mutex whenever accessing this map
+  /// Map storing connections opened by the methods from the InternalServer interface.
+  /// Key is the session id, value is a shared pointer holding the connection's
+  /// ConnectionContext. Use the `internal_server_connections_lock_` mutex whenever
+  /// accessing this map
   typedef std::map<TUniqueId, std::shared_ptr<ThriftServer::ConnectionContext>>
       SessionToConnectionContext;
   SessionToConnectionContext internal_server_connections_;
@@ -1479,7 +1487,7 @@ class ImpalaServer : public ImpalaServiceIf,
 
   /// Decrement the session's reference counter and mark last_accessed_ms so that state
   /// expiration can proceed.
-  inline void MarkSessionInactive(std::shared_ptr<SessionState> session) {
+  inline void MarkSessionInactive(const std::shared_ptr<SessionState>& session) {
     std::lock_guard<std::mutex> l(session->lock);
     DCHECK_GT(session->ref_count, 0);
     --session->ref_count;
@@ -1487,7 +1495,7 @@ class ImpalaServer : public ImpalaServiceIf,
   }
 
   /// Increment the session's reference counter.
-  inline void MarkSessionActive(std::shared_ptr<SessionState> session) {
+  inline void MarkSessionActive(const std::shared_ptr<SessionState>& session) {
     std::lock_guard<std::mutex> l(session->lock);
     ++session->ref_count;
   }
@@ -1501,8 +1509,9 @@ class ImpalaServer : public ImpalaServiceIf,
 
   /// Entries in the 'query_locations' map.
   struct QueryLocationInfo {
-    QueryLocationInfo(NetworkAddressPB address, TUniqueId query_id) : address(address) {
-      query_ids.insert(query_id);
+    QueryLocationInfo(NetworkAddressPB address, TUniqueId query_id)
+      : address(std::move(address)) {
+      query_ids.insert(std::move(query_id));
     }
 
     /// Used for logging and error messages so that users don't have to translate between
@@ -1650,7 +1659,6 @@ class ImpalaServer : public ImpalaServiceIf,
   boost::scoped_ptr<ThriftServer> hs2_server_;
   boost::scoped_ptr<ThriftServer> hs2_http_server_;
   boost::scoped_ptr<ThriftServer> external_fe_server_;
-  std::shared_ptr<InternalServer> internal_server_;
 
   /// Flag that records if backend and/or client services have been started. The flag is
   /// set after all services required for the server have been started.
@@ -1670,29 +1678,14 @@ class ImpalaServer : public ImpalaServiceIf,
   int64_t admission_heartbeat_version_ = 0;
 
   /// Workload Management Related Declarations.
-  /// Coordinate periodic execution of the completed queries queue processing thread.
-  std::condition_variable completed_queries_cv_;
+  /// Tracks the state of workload management processing. Access to workload_mgmt_state_
+  /// must only happen while holding the workload_mgmt_state_mu_ lock.
+  std::mutex workload_mgmt_state_mu_;
+  workloadmgmt::WorkloadManagementState workload_mgmt_state_ =
+      workloadmgmt::WorkloadManagementState::NOT_STARTED;
 
-  /// Coordinate shutdown of the completed queries queue processing thread.
-  std::condition_variable completed_queries_shutdown_cv_;
-
-  /// Tracks the state of the thread that drains the completed queries queue to the table.
-  /// The associated lock must be held before reading/modifying this variable.
-  impala::workload_management::ThreadState completed_queries_thread_state_ =
-      impala::workload_management::NOT_STARTED;
-  std::mutex completed_queries_threadstate_mu_;
-
-  /// Thread that runs CompletedQueriesThread().
-  std::unique_ptr<Thread> completed_queries_thread_;
-
-  /// Ticker that wakes up the completed_queried_thread at set intervals to process the
-  /// queued completed queries. Uses the completed_queries_lock_ to synchonize access to
-  /// the completed_queries_ list.
-  std::unique_ptr<TickerSecondsBool> completed_queries_ticker_;
-
-  /// Queue of completed queries and the lock to synchronize access to it.
-  std::list<impala::workload_management::CompletedQuery> completed_queries_;
-  std::mutex completed_queries_lock_;
+  /// Thread that runs Workload Management.
+  std::unique_ptr<Thread> workload_management_thread_;
 
 };
 }
