@@ -69,6 +69,7 @@ void QueryStateRecord::Init(const ClientRequestState& query_handle) {
   start_time_us = query_handle.start_time_us();
   end_time_us = query_handle.end_time_us();
   wait_time_ms = query_handle.wait_time_ms();
+  client_fetch_wait_time_ns = query_handle.client_fetch_wait_time_ns();
   query_handle.summary_profile()->GetTimeline(&timeline);
 
   Coordinator* coord = query_handle.GetCoordinator();
@@ -117,7 +118,11 @@ void QueryStateRecord::Init(const ClientRequestState& query_handle) {
   last_active_time_ms = query_handle.last_active_ms();
   // For statement types other than QUERY/DML, show an empty string for resource pool
   // to indicate that they are not subjected to admission control.
-  if (stmt_type == TStmtType::QUERY || stmt_type == TStmtType::DML) {
+  // CTAS query has stmt_type == DDL and query_exec_request.stmt_type == DML,
+  // so resource pool should be displayed too.
+  if (stmt_type == TStmtType::QUERY || stmt_type == TStmtType::DML
+      || (stmt_type == TStmtType::DDL
+          && request.query_exec_request.stmt_type == TStmtType::DML)) {
     resource_pool = query_handle.request_pool();
   }
   user_has_profile_access = query_handle.user_has_profile_access();
@@ -192,9 +197,9 @@ static bool find_averaged(RuntimeProfileBase* prof) {
   return boost::algorithm::istarts_with(prof->name(), "Averaged Fragment");
 }
 
-QueryStateExpanded::QueryStateExpanded(const ClientRequestState& exec_state,
-    const std::shared_ptr<QueryStateRecord> base) :
-    base_state(base ? move(base) : make_shared<QueryStateRecord>(exec_state)) {
+QueryStateExpanded::QueryStateExpanded(
+    const ClientRequestState& exec_state, std::shared_ptr<QueryStateRecord> base)
+  : base_state(base ? move(base) : make_shared<QueryStateRecord>(exec_state)) {
   if (exec_state.session()->session_type == TSessionType::HIVESERVER2){
     hiveserver2_protocol_version = exec_state.session()->hs2_version;
   }
@@ -212,6 +217,11 @@ QueryStateExpanded::QueryStateExpanded(const ClientRequestState& exec_state,
   row_materialization_rate = exec_state.row_materialization_rate();
   row_materialization_time = exec_state.row_materialization_timer();
   tables = exec_state.tables();
+  select_columns = exec_state.select_columns();
+  where_columns = exec_state.where_columns();
+  join_columns = exec_state.join_columns();
+  aggregate_columns = exec_state.aggregate_columns();
+  orderby_columns = exec_state.orderby_columns();
 
   // Update name_rows_fetched with the final count after query close.
   base_state->num_rows_fetched = exec_state.num_rows_fetched_counter();
