@@ -292,6 +292,9 @@ class Statestore : public CacheLineAligned {
     return TNetworkAddressToString(peer_statestore_ha_addr_);
   }
 
+  // Update the subscriber's catalog information.
+  void UpdateSubscriberCatalogInfo(const SubscriberId& subscriber_id);
+
  private:
   /// A TopicEntry is a single entry in a topic, and logically is a <string, byte string>
   /// pair.
@@ -508,12 +511,25 @@ class Statestore : public CacheLineAligned {
     const TNetworkAddress& network_address() const { return network_address_; }
     const SubscriberId& id() const { return subscriber_id_; }
     const RegistrationId& registration_id() const { return registration_id_; }
+    TStatestoreSubscriberType::type subscriber_type() const { return subscriber_type_; }
+    int64_t catalogd_version() const { return catalogd_version_; }
+    const TNetworkAddress& catalogd_address() const { return catalogd_address_; }
+    int64_t last_update_catalogd_time() const { return last_update_catalogd_time_; }
 
     /// Returns the time elapsed (in seconds) since the last heartbeat.
     double SecondsSinceHeartbeat() const {
       return (static_cast<double>(MonotonicMillis() - last_heartbeat_ts_.Load()))
           / 1000.0;
     }
+    /// Returns the time elapsed (in milli seconds) since the last heartbeat.
+    int64_t MilliSecondsSinceHeartbeat() const {
+      return MonotonicMillis() - last_heartbeat_ts_.Load();
+    }
+
+    /// Return true if the subscriber received heartbeat.
+    bool receivedHeartbeat() const {
+      return received_heartbeat_.Load();
+    }
 
     /// Get the Topics map that would be used to store 'topic_id'.
     const Topics& GetTopicsMapForId(const TopicId& topic_id) const {
@@ -559,7 +575,7 @@ class Statestore : public CacheLineAligned {
         TopicEntry::Version version);
 
     /// Refresh the subscriber's last heartbeat timestamp to the current monotonic time.
-    void RefreshLastHeartbeatTimestamp();
+    void RefreshLastHeartbeatTimestamp(bool received_heartbeat);
 
     /// Check if the subscriber is Catalog daemon.
     bool IsCatalogd() const {
@@ -571,6 +587,10 @@ class Statestore : public CacheLineAligned {
       return subscribe_catalogd_change_;
     }
 
+    /// The subscriber updates the catalog information.
+    void UpdateCatalogInfo(
+        int64_t catalogd_version, const TNetworkAddress& catalogd_address);
+
    private:
     /// Unique human-readable identifier for this subscriber, set by the subscriber itself
     /// on a Register call.
@@ -602,6 +622,8 @@ class Statestore : public CacheLineAligned {
     /// The timestamp of the last successful heartbeat in milliseconds. A timestamp much
     /// older than the heartbeat frequency implies an unresponsive subscriber.
     AtomicInt64 last_heartbeat_ts_{0};
+    /// True if the subscriber received heartbeat.
+    AtomicBool received_heartbeat_{false};
 
     /// Lock held when adding or deleting transient entries. See class comment for lock
     /// acquisition order.
@@ -610,6 +632,15 @@ class Statestore : public CacheLineAligned {
     /// True once DeleteAllTransientEntries() has been called during subscriber
     /// unregisteration. Protected by 'transient_entry_lock_'
     bool unregistered_ = false;
+
+    // Version of catalogd.
+    int64_t catalogd_version_ = 0L;
+
+    // Address of catalogd.
+    TNetworkAddress catalogd_address_;
+
+    // The last time to update the catalogd.
+    int64_t last_update_catalogd_time_ = 0L;
   };
 
   /// Unique identifier for this statestore instance.
@@ -657,9 +688,11 @@ class Statestore : public CacheLineAligned {
 
     ScheduledSubscriberUpdate() {}
 
-    ScheduledSubscriberUpdate(int64_t next_update_time, SubscriberId s_id,
-        RegistrationId r_id): deadline(next_update_time), subscriber_id(s_id),
-        registration_id(r_id) {}
+    ScheduledSubscriberUpdate(
+        int64_t next_update_time, SubscriberId s_id, RegistrationId r_id)
+      : deadline(next_update_time),
+        subscriber_id(std::move(s_id)),
+        registration_id(std::move(r_id)) {}
   };
 
   /// The statestore has three pools of threads that send messages to subscribers
@@ -829,6 +862,7 @@ class Statestore : public CacheLineAligned {
   /// Metric that track the registered, non-failed subscribers.
   IntGauge* num_subscribers_metric_;
   SetMetric<std::string>* subscriber_set_metric_;
+  IntGauge* num_subscribers_received_heartbeat_metric_;
 
   /// Metrics shared across all topics to sum the size in bytes of keys, values and both
   IntGauge* key_size_metric_;
@@ -1009,6 +1043,10 @@ class Statestore : public CacheLineAligned {
 
   // Return true if this statestore instance is in recovery mode.
   bool IsInRecoveryMode();
+
+  /// Json callback for /catalog_ha_info.
+  void CatalogHAInfoHandler(const Webserver::WebRequest& req,
+      rapidjson::Document* document);
 };
 
 } // namespace impala
