@@ -34,6 +34,8 @@
 #include "gen-cpp/Frontend_types.h"
 #include "util/metrics.h"
 #include "common/logging.h"
+#include "common/status.h"
+#include "rpc/authentication-util.h"
 
 DECLARE_bool(trusted_domain_use_xff_header);
 DECLARE_bool(trusted_domain_empty_xff_header_use_origin);
@@ -47,10 +49,10 @@ namespace transport {
 using namespace std;
 using strings::Substitute;
 
-THttpServerTransportFactory::THttpServerTransportFactory(const std::string server_name,
+THttpServerTransportFactory::THttpServerTransportFactory(const std::string& server_name,
     impala::MetricGroup* metrics, bool has_ldap, bool has_kerberos, bool use_cookies,
     bool check_trusted_domain, bool check_trusted_auth_header, bool has_saml,
-    bool has_jwt)
+    bool has_jwt, bool has_oauth)
   : has_ldap_(has_ldap),
     has_kerberos_(has_kerberos),
     use_cookies_(use_cookies),
@@ -58,6 +60,7 @@ THttpServerTransportFactory::THttpServerTransportFactory(const std::string serve
     check_trusted_auth_header_(check_trusted_auth_header),
     has_saml_(has_saml),
     has_jwt_(has_jwt),
+    has_oauth_(has_oauth),
     metrics_enabled_(metrics != nullptr) {
   if (metrics_enabled_) {
     if (has_ldap_) {
@@ -98,14 +101,20 @@ THttpServerTransportFactory::THttpServerTransportFactory(const std::string serve
       http_metrics_.total_jwt_token_auth_failure_ = metrics->AddCounter(
           Substitute("$0.total-jwt-token-auth-failure", server_name), 0);
     }
+    if (has_oauth_) {
+      http_metrics_.total_oauth_token_auth_success_ = metrics->AddCounter(
+          Substitute("$0.total-oauth-token-auth-success", server_name), 0);
+      http_metrics_.total_oauth_token_auth_failure_ = metrics->AddCounter(
+          Substitute("$0.total-oauth-token-auth-failure", server_name), 0);
+    }
   }
 }
 
 THttpServer::THttpServer(std::shared_ptr<TTransport> transport, bool has_ldap,
     bool has_kerberos, bool has_saml, bool use_cookies, bool check_trusted_domain,
-    bool check_trusted_auth_header, bool has_jwt, bool metrics_enabled,
+    bool check_trusted_auth_header, bool has_jwt, bool has_oauth, bool metrics_enabled,
     HttpMetrics* http_metrics)
-  : THttpTransport(transport),
+  : THttpTransport(move(transport)),
     has_ldap_(has_ldap),
     has_kerberos_(has_kerberos),
     has_saml_(has_saml),
@@ -113,6 +122,7 @@ THttpServer::THttpServer(std::shared_ptr<TTransport> transport, bool has_ldap,
     check_trusted_domain_(check_trusted_domain),
     check_trusted_auth_header_(check_trusted_auth_header),
     has_jwt_(has_jwt),
+    has_oauth_(has_oauth),
     metrics_enabled_(metrics_enabled),
     http_metrics_(http_metrics) {}
 
@@ -165,7 +175,7 @@ void THttpServer::parseHeader(char* header) {
     contentLength_ = atoi(value);
   } else if (MatchesHeader(header, HEADER_X_FORWARDED_FOR, sz)) {
     origin_ = value;
-  } else if ((has_ldap_ || has_kerberos_ || has_saml_ || has_jwt_)
+  } else if ((has_ldap_ || has_kerberos_ || has_saml_ || has_jwt_ || has_oauth_)
       && MatchesHeader(header, HEADER_AUTHORIZATION, sz)) {
     auth_value_ = string(value);
   } else if (use_cookies_ && MatchesHeader(header, HEADER_COOKIE, sz)) {
@@ -267,7 +277,16 @@ void THttpServer::headersDone() {
         << (header_x_query_id_.empty() ? "" : " x-query-id=" + header_x_query_id_);
   }
 
-  if (!has_ldap_ && !has_kerberos_ && !has_saml_ && !has_jwt_) {
+  // Trim and truncate the value of the 'X-Forwarded-For' header.
+  string origin = origin_;
+  StripWhiteSpace(&origin);
+  if (origin.length() > MAX_X_FORWARDED_HEADER_LENGTH) {
+    origin = origin.substr(0, MAX_X_FORWARDED_HEADER_LENGTH);
+  }
+  // Store the truncated value of the 'X-Forwarded-For' header in the Connection Context.
+  callbacks_.set_http_origin_fn(origin);
+
+  if (!has_ldap_ && !has_kerberos_ && !has_saml_ && !has_jwt_ && !has_oauth_) {
     // We don't need to authenticate.
     resetAuthState();
     return;
@@ -298,7 +317,7 @@ void THttpServer::headersDone() {
     }
   }
 
-  if (!authorized && has_jwt_ && !auth_value_.empty()
+  if (!authorized && (has_jwt_ || has_oauth_) && !auth_value_.empty()
       && auth_value_.find('.') != string::npos) {
     // Check Authorization header with the Bearer authentication scheme as:
     // Authorization: Bearer <token>
@@ -308,11 +327,21 @@ void THttpServer::headersDone() {
     string jwt_token;
     bool got_bearer_auth = TryStripPrefixString(auth_value_, "Bearer ", &jwt_token);
     if (got_bearer_auth) {
-      if (callbacks_.jwt_token_auth_fn(jwt_token)) {
+      if (has_jwt_ && callbacks_.jwt_token_auth_fn(jwt_token)) {
         authorized = true;
-        if (metrics_enabled_) http_metrics_->total_jwt_token_auth_success_->Increment(1);
-      } else {
-        if (metrics_enabled_) http_metrics_->total_jwt_token_auth_failure_->Increment(1);
+        if (metrics_enabled_)
+          http_metrics_->total_jwt_token_auth_success_->Increment(1);
+      }
+      if (!authorized && has_oauth_ && callbacks_.oauth_token_auth_fn(jwt_token)) {
+        authorized = true;
+        if (metrics_enabled_)
+          http_metrics_->total_oauth_token_auth_success_->Increment(1);
+      }
+      if (!authorized) {
+        if (has_jwt_ && metrics_enabled_)
+          http_metrics_->total_jwt_token_auth_failure_->Increment(1);
+        if (has_oauth_ && metrics_enabled_)
+          http_metrics_->total_oauth_token_auth_failure_->Increment(1);
       }
     }
   }
@@ -350,7 +379,7 @@ void THttpServer::headersDone() {
 
     if (!authorized && !fallback_to_other_auths) {
       // Do not fallback to other auth mechanisms, as the client probably expects
-      // only SAML related respsonses.
+      // only SAML related responses.
       if (metrics_enabled_) http_metrics_->total_saml_auth_failure_->Increment(1);
       resetAuthState();
       returnUnauthorized();
@@ -365,8 +394,13 @@ void THttpServer::headersDone() {
   // the client started the SAML workflow then it doesn't expect Impala to succeed with
   // another mechanism.
   if (!authorized && check_trusted_domain_) {
-    string origin =
-        FLAGS_trusted_domain_use_xff_header ? origin_ : transport_->getOrigin();
+    string origin;
+    if (FLAGS_trusted_domain_use_xff_header) {
+      impala::Status status = impala::GetXFFOriginClientAddress(origin_, origin);
+      if (!status.ok()) LOG(WARNING) << status.GetDetail();
+    } else {
+      origin = transport_->getOrigin();
+    }
     StripWhiteSpace(&origin);
     if (origin.empty() && FLAGS_trusted_domain_use_xff_header &&
         FLAGS_trusted_domain_empty_xff_header_use_origin) {
