@@ -61,6 +61,9 @@ struct HttpMetrics {
 
   impala::IntCounter* total_jwt_token_auth_success_ = nullptr;
   impala::IntCounter* total_jwt_token_auth_failure_ = nullptr;
+
+  impala::IntCounter* total_oauth_token_auth_success_ = nullptr;
+  impala::IntCounter* total_oauth_token_auth_failure_ = nullptr;
 };
 
 /*
@@ -103,14 +106,19 @@ public:
     // header respectively and returns true if it determines that the connection
     // originated from a trusted domain and if the basic auth header contains a valid
     // username.
-    std::function<bool(const std::string&, std::string)> trusted_domain_check_fn =
-        [&](const std::string&, std::string) { return false; };
+    std::function<bool(const std::string&, const std::string&)> trusted_domain_check_fn =
+        [&](const std::string&, const std::string&) { return false; };
+
+    // Function that stores the connection's 'X-Forwarded-For' header in the Connection
+    // Context so that it can be tracked.
+    std::function<bool(std::string)> set_http_origin_fn = [&](const std::string&) {
+      return false;
+    };
 
     // Function that takes the connection's 'Authorization' header and returns true if
     // the basic auth header contains a valid username.
-    std::function<bool(std::string)> trusted_auth_header_handle_fn = [&](std::string) {
-      return false;
-    };
+    std::function<bool(const std::string&)> trusted_auth_header_handle_fn =
+        [&](const std::string&) { return false; };
 
     // Does the first step of SAML2 SSO browser authenticaton and sets the response to
     // redirect to the SSO service.
@@ -138,11 +146,18 @@ public:
     std::function<bool(const std::string&)> jwt_token_auth_fn = [&](const std::string&) {
       return false;
     };
+
+    // Function that takes the OAuth token from the header, and returns true
+    // if verification for the token is successful.
+    std::function<bool(const std::string&)> oauth_token_auth_fn =
+        [&](const std::string&) {
+      return false;
+    };
   };
 
   THttpServer(std::shared_ptr<TTransport> transport, bool has_ldap, bool has_kerberos,
       bool has_saml, bool use_cookies, bool check_trusted_domain,
-      bool check_trusted_auth_header, bool has_jwt, bool metrics_enabled,
+      bool check_trusted_auth_header, bool has_jwt, bool has_oauth, bool metrics_enabled,
       HttpMetrics* http_metrics);
 
   virtual ~THttpServer();
@@ -183,9 +198,9 @@ protected:
   void resetAuthState();
  private:
   // If either of the following is true, a '401 - Unauthorized' will be returned to the
-  // client on requests that do not contain a valid 'Authorization' of SAML SSO or JWT
-  // related header. If 'has_ldap_' is true, 'Basic' auth headers will be processed, and
-  // if 'has_kerberos_' is true 'Negotiate' auth headers will be processed.
+  // client on requests that do not contain a valid 'Authorization' of SAML SSO, JWT or
+  // OAuth related header. If 'has_ldap_' is true, 'Basic' auth headers will be processed,
+  // and if 'has_kerberos_' is true 'Negotiate' auth headers will be processed.
   bool has_ldap_ = false;
   bool has_kerberos_ = false;
 
@@ -233,6 +248,9 @@ protected:
   // If set, support for trusting an authentication based on JWT token.
   bool has_jwt_ = false;
 
+  // If set, support for trusting an authentication based on OAuth token.
+  bool has_oauth_ = false;
+
   bool metrics_enabled_ = false;
   HttpMetrics* http_metrics_ = nullptr;
 
@@ -248,6 +266,10 @@ protected:
 
   // The value from the 'X-Impala-Query-Id' header.
   std::string header_x_query_id_ = "";
+
+  // The maximum length of the 'X-Forwarded-For' header that will be stored in the runtime
+  // Profile.
+  static const int MAX_X_FORWARDED_HEADER_LENGTH = 8096;
 };
 
 /**
@@ -257,16 +279,16 @@ class THttpServerTransportFactory : public TTransportFactory {
 public:
  THttpServerTransportFactory() {}
 
- THttpServerTransportFactory(const std::string server_name, impala::MetricGroup* metrics,
+ THttpServerTransportFactory(const std::string& server_name, impala::MetricGroup* metrics,
      bool has_ldap, bool has_kerberos, bool use_cookies, bool check_trusted_domain,
-     bool check_trusted_auth_header, bool has_saml, bool has_jwt);
+     bool check_trusted_auth_header, bool has_saml, bool has_jwt, bool has_oauth);
 
  virtual ~THttpServerTransportFactory() {}
 
  virtual std::shared_ptr<TTransport> getTransport(std::shared_ptr<TTransport> trans) {
    return std::shared_ptr<TTransport>(new THttpServer(trans, has_ldap_, has_kerberos_,
        has_saml_, use_cookies_, check_trusted_domain_, check_trusted_auth_header_,
-       has_jwt_, metrics_enabled_, &http_metrics_));
+       has_jwt_, has_oauth_, metrics_enabled_, &http_metrics_));
   }
 
  private:
@@ -277,6 +299,7 @@ public:
   bool check_trusted_auth_header_ = false;
   bool has_saml_ = false;
   bool has_jwt_ = false;
+  bool has_oauth_ = false;
 
   // Metrics for every transport produced by this factory.
   bool metrics_enabled_ = false;
