@@ -112,6 +112,8 @@ DECLARE_int32(iceberg_reload_new_files_threshold);
 DECLARE_bool(enable_skipping_older_events);
 DECLARE_bool(enable_json_scanner);
 DECLARE_bool(iceberg_allow_datafiles_in_table_location_only);
+DECLARE_bool(iceberg_always_allow_merge_on_read_operations);
+DECLARE_bool(enable_reading_puffin_stats);
 DECLARE_int32(catalog_operation_log_size);
 DECLARE_string(hostname);
 DECLARE_bool(allow_catalog_cache_op_from_masked_users);
@@ -129,6 +131,7 @@ DECLARE_int32(dbcp_max_conn_pool_size);
 DECLARE_int32(dbcp_max_wait_millis_for_conn);
 DECLARE_int32(dbcp_data_source_idle_timeout_s);
 DECLARE_bool(enable_catalogd_ha);
+DECLARE_string(injected_group_members_debug_only);
 
 // HS2 SAML2.0 configuration
 // Defined here because TAG_FLAG caused issues in global-flags.cc
@@ -273,6 +276,11 @@ DEFINE_double_hidden(query_cpu_root_factor, 1.5,
     "(Advance) The Nth root value to control sublinear scale down of unbounded "
     "cpu requirement for executor group set selection.");
 
+DEFINE_int64_hidden(data_stream_sender_buffer_size_used_by_planner, -1,
+    "Similar to data_stream_sender_buffer_size but used during planning."
+    "With default -1 the planner uses the old logic that is different"
+    "than how the backend actually works (see IMPALA-12594)");
+
 using strings::Substitute;
 
 namespace impala {
@@ -464,6 +472,10 @@ Status PopulateThriftBackendGflags(TBackendGflags& cfg) {
   cfg.__set_enable_json_scanner(FLAGS_enable_json_scanner);
   cfg.__set_iceberg_allow_datafiles_in_table_location_only(
       FLAGS_iceberg_allow_datafiles_in_table_location_only);
+  cfg.__set_iceberg_always_allow_merge_on_read_operations(
+      FLAGS_iceberg_always_allow_merge_on_read_operations);
+  cfg.__set_enable_reading_puffin_stats(
+      FLAGS_enable_reading_puffin_stats);
   cfg.__set_max_filter_error_rate_from_full_scan(
       FLAGS_max_filter_error_rate_from_full_scan);
   cfg.__set_catalog_operation_log_size(FLAGS_catalog_operation_log_size);
@@ -487,6 +499,9 @@ Status PopulateThriftBackendGflags(TBackendGflags& cfg) {
   cfg.__set_dbcp_max_conn_pool_size(FLAGS_dbcp_max_conn_pool_size);
   cfg.__set_dbcp_max_wait_millis_for_conn(FLAGS_dbcp_max_wait_millis_for_conn);
   cfg.__set_dbcp_data_source_idle_timeout(FLAGS_dbcp_data_source_idle_timeout_s);
+  cfg.__set_data_stream_sender_buffer_size_used_by_planner(
+      FLAGS_data_stream_sender_buffer_size_used_by_planner);
+  cfg.__set_injected_group_members_debug_only(FLAGS_injected_group_members_debug_only);
 #ifdef NDEBUG
   cfg.__set_is_release_build(true);
 #else
