@@ -34,6 +34,10 @@ class FileSystemUtil {
   /// Returns Status::OK if successful, or a runtime error with a message otherwise.
   static Status RemoveAndCreateDirectory(const std::string& directory) WARN_UNUSED_RESULT;
 
+  /// Create the specified directory and any ancestor directories that do not exist yet.
+  /// Returns Status::OK if successful, or a runtime error with a message otherwise.
+  static Status CreateDirectory(const string& directory) WARN_UNUSED_RESULT;
+
   /// Create a file at the specified path.
   static Status CreateFile(const std::string& file_path) WARN_UNUSED_RESULT;
 
@@ -95,7 +99,8 @@ class FileSystemUtil {
   /// any files in that directory.
   /// If 'path' ends with a wildcard - as in "/lib/*" - it will treat path as a directory
   /// excluding the wildcard.
-  static std::string FindFileInPath(string path, const std::string& regex);
+  static Status FindFileInPath(string path, const std::string& regex,
+      std::string* result);
 
   /// Ext filesystem on certain kernel versions may result in inconsistent metadata after
   /// punching holes in files. The filesystem may require fsck repair on next reboot.
