@@ -107,32 +107,59 @@ struct JniMethodDescriptor {
   jmethodID* method_id;
 };
 
-/// Helper class for lifetime management of chars from JNI, releasing JNI chars when
-/// destructed
-class JniUtfCharGuard {
+/// Helper class for the lifetime management of JNI char or byte buffers. Releases the JNI
+/// buffer when destructed. T must be either jstring or jbyteArray. If T is a jstring, the
+/// string in the buffer is in utf-8 format.
+///
+/// See also JniScopedArrayCritical, which can also be used for byte buffers but its usage
+/// is more restricted, see https://docs.oracle.com/javase/8/docs/technotes/guides/jni/spec/functions.html#GetPrimitiveArrayCritical_ReleasePrimitiveArrayCritical
+template <class T>
+class JniBufferGuard {
+  static_assert(std::is_same_v<T, jstring> || std::is_same_v<T, jbyteArray>);
  public:
-  /// Construct a JniUtfCharGuards holding nothing
-  JniUtfCharGuard() : utf_chars(nullptr) {}
+  /// Construct a JniBufferGuard holding nothing.
+  JniBufferGuard() : env(nullptr), jbuffer{}, size(0), buffer(nullptr)  {}
 
-  /// Release the held char sequence if there is one.
-  ~JniUtfCharGuard() {
-    if (utf_chars != nullptr) env->ReleaseStringUTFChars(jstr, utf_chars);
+  /// Release the held JNI buffer if there is one.
+  ~JniBufferGuard() {
+    Release(env, jbuffer, buffer);
   }
 
-  /// Try to get chars from jstr. If error is returned, utf_chars and get() remain
-  /// to be nullptr, otherwise they point to a valid char sequence. The char sequence
-  /// lives as long as this guard. jstr should not be null.
-  static Status create(JNIEnv* env, jstring jstr, JniUtfCharGuard* out);
+  /// Try to get chars/bytes from jbuffer. If an error is returned, buffer and get()
+  /// remain 'nullptr's, otherwise they point to a valid char/byte array. The char/byte
+  /// array lives as long as this guard. jbuffer should not be null.
+  static Status create(JNIEnv* env, T jbuffer, JniBufferGuard* out);
 
-  /// Get the char sequence. Returns nullptr if the guard does hold a char sequence.
-  const char* get() { return utf_chars; }
+  /// Get the size of the buffer.
+  uint32_t get_size() { return size; }
+
+  /// Get the buffer. Returns nullptr if the guard does hold a buffer.
+  const char* get() { return buffer; }
  private:
   JNIEnv* env;
-  jstring jstr;
-  const char* utf_chars;
-  DISALLOW_COPY_AND_ASSIGN(JniUtfCharGuard);
+  T jbuffer;
+  uint32_t size;
+  const char* buffer;
+  DISALLOW_COPY_AND_ASSIGN(JniBufferGuard);
+
+  static void Release(JNIEnv* env, T jbuffer, const char* buffer) {
+    if (buffer != nullptr) {
+      if constexpr (std::is_same_v<T, jstring>) {
+        env->ReleaseStringUTFChars(jbuffer, buffer);
+      } else {
+        static_assert(std::is_same_v<T, jbyteArray>);
+        /// Return buffer back. JNI_ABORT indicates to not copy contents back to java
+        /// side.
+        env->ReleaseByteArrayElements(jbuffer,
+            const_cast<jbyte*>(reinterpret_cast<const jbyte*>(buffer)), JNI_ABORT);
+      }
+    }
+  }
 };
 
+using JniUtfCharGuard = JniBufferGuard<jstring>;
+using JniByteArrayGuard = JniBufferGuard<jbyteArray>;
+
 class JniScopedArrayCritical {
  public:
   /// Construct a JniScopedArrayCritical holding nothing.
