@@ -18,13 +18,11 @@
 #pragma once
 
 #include <mutex>
-#include <boost/function.hpp>
-#include <boost/scoped_ptr.hpp>
-#include <boost/thread/thread.hpp>
-#include <boost/unordered_map.hpp>
-#include <boost/unordered_set.hpp>
+#include <unordered_map>
+#include <unordered_set>
 
 #include "util/runtime-profile.h"
+#include "util/thread.h"
 
 namespace impala {
 
@@ -116,7 +114,7 @@ class PeriodicCounterUpdater {
   [[noreturn]] void UpdateLoop(PeriodicCounterUpdater* instance);
 
   /// Thread performing asynchronous updates.
-  boost::scoped_ptr<boost::thread> update_thread_;
+  std::unique_ptr<impala::Thread> update_thread_;
 
   /// List of functions that will be called before individual counters will be sampled.
   std::vector<UpdateFn> update_fns_;
@@ -128,7 +126,7 @@ class PeriodicCounterUpdater {
   SpinLock rate_lock_;
 
   /// A map of the dst (rate) counter to the src counter and elapsed time.
-  typedef boost::unordered_map<RuntimeProfile::Counter*, RateCounterInfo> RateCounterMap;
+  typedef std::unordered_map<RuntimeProfile::Counter*, RateCounterInfo> RateCounterMap;
   RateCounterMap rate_counters_;
 
   /// Spinlock that protects the map of averages over samples of counters
@@ -136,7 +134,7 @@ class PeriodicCounterUpdater {
 
   /// A map of the dst (averages over samples) counter to the src counter (to be sampled)
   /// and number of samples taken.
-  typedef boost::unordered_map<RuntimeProfile::Counter*, SamplingCounterInfo>
+  typedef std::unordered_map<RuntimeProfile::Counter*, SamplingCounterInfo>
       SamplingCounterMap;
   SamplingCounterMap sampling_counters_;
 
@@ -144,7 +142,7 @@ class PeriodicCounterUpdater {
   SpinLock bucketing_lock_;
 
   /// Map from a bucket of counters to the src counter
-  typedef boost::unordered_map<std::vector<RuntimeProfile::Counter*>*, BucketCountersInfo>
+  typedef std::unordered_map<std::vector<RuntimeProfile::Counter*>*, BucketCountersInfo>
       BucketCountersMap;
   BucketCountersMap bucketing_counters_;
 
@@ -152,7 +150,7 @@ class PeriodicCounterUpdater {
   SpinLock time_series_lock_;
 
   /// Set of time series counters that need to be updated
-  typedef boost::unordered_set<RuntimeProfile::TimeSeriesCounter*> TimeSeriesCounters;
+  typedef std::unordered_set<RuntimeProfile::TimeSeriesCounter*> TimeSeriesCounters;
   TimeSeriesCounters time_series_counters_;
 
   /// Singleton object that keeps track of all profile rate counters and the thread
