@@ -37,7 +37,6 @@ using boost::algorithm::join;
 using boost::algorithm::split;
 using boost::algorithm::token_compress_on;
 using boost::algorithm::trim;
-using std::to_string;
 
 using namespace strings;
 
@@ -144,7 +143,7 @@ void ProcessStateInfo::ReadProcFileDescriptorCount() {
     }
     closedir(dir_stream);
   }
-  process_state_map_["fd/count"] = to_string(fd_count);
+  process_state_map_["fd/count"] = std::to_string(fd_count);
 }
 
 ProcessStateInfo::ProcessStateInfo(bool get_extended_metrics)
