@@ -168,6 +168,9 @@ DECLARE_bool(jwt_validate_signature);
 DECLARE_string(jwt_custom_claim_username);
 DECLARE_string(trusted_auth_header);
 DECLARE_string(spnego_keytab_file);
+DECLARE_bool(oauth_token_auth);
+DECLARE_bool(oauth_jwt_validate_signature);
+DECLARE_string(oauth_jwt_custom_claim_username);
 
 static const char* DOC_FOLDER = "/www/";
 static const int DOC_FOLDER_LEN = strlen(DOC_FOLDER);
@@ -320,7 +323,8 @@ Webserver::Webserver(const string& interface, const int port, MetricGroup* metri
     use_cookies_(FLAGS_max_cookie_lifetime_s > 0),
     check_trusted_domain_(!FLAGS_trusted_domain.empty()),
     check_trusted_auth_header_(!FLAGS_trusted_auth_header.empty()),
-    use_jwt_(FLAGS_jwt_token_auth) {
+    use_jwt_(FLAGS_jwt_token_auth),
+    use_oauth_(FLAGS_oauth_token_auth) {
   http_address_ = MakeNetworkAddress(interface.empty() ? "0.0.0.0" : interface, port);
   Init();
 
@@ -358,6 +362,12 @@ Webserver::Webserver(const string& interface, const int port, MetricGroup* metri
     total_jwt_token_auth_failure_ =
         metrics->AddCounter("impala.webserver.total-jwt-token-auth-failure", 0);
   }
+  if (use_oauth_) {
+    total_oauth_token_auth_success_ =
+        metrics->AddCounter("impala.webserver.total-oauth-token-auth-success", 0);
+    total_oauth_token_auth_failure_ =
+        metrics->AddCounter("impala.webserver.total-oauth-token-auth-failure", 0);
+  }
 }
 
 Webserver::~Webserver() {
@@ -673,7 +683,7 @@ sq_callback_result_t Webserver::BeginRequestCallback(struct sq_connection* conne
   bool cookie_authenticated = false;
 
   // Try authenticating with JWT token first, if enabled.
-  if (use_jwt_) {
+  if (use_jwt_ || use_oauth_) {
     const char* auth_value = nullptr;
     const char* value = sq_get_header(connection, "Authorization");
     if (value != nullptr) auth_value = StripLeadingWhiteSpace(value);
@@ -683,17 +693,34 @@ sq_callback_result_t Webserver::BeginRequestCallback(struct sq_connection* conne
     // separated by dots (.).
     if (auth_value != nullptr && strncasecmp(auth_value, "Bearer ", 7) == 0
         && strchr(auth_value, '.') != nullptr) {
-      string jwt_token = string(auth_value + 7);
-      StripWhiteSpace(&jwt_token);
-      if (!jwt_token.empty()) {
-        if (JWTTokenAuth(jwt_token, connection, request_info)) {
-          total_jwt_token_auth_success_->Increment(1);
-          authenticated = true;
-          check_csrf_protection = false;
-          // TODO: cookies are not added, but are not needed right now
-        } else {
-          LOG(INFO) << "Invalid JWT token provided: " << jwt_token;
-          total_jwt_token_auth_failure_->Increment(1);
+      string bearer_token= string(auth_value + 7);
+      StripWhiteSpace(&bearer_token);
+      if (!bearer_token.empty()) {
+        if (use_jwt_) {
+          if (JWTTokenAuth(bearer_token, connection, request_info)) {
+            total_jwt_token_auth_success_->Increment(1);
+            authenticated = true;
+            check_csrf_protection = false;
+            // TODO: cookies are not added, but are not needed right now
+          }
+        }
+        if (!authenticated && use_oauth_) {
+          if (OAuthTokenAuth(bearer_token, connection, request_info)) {
+            total_oauth_token_auth_success_->Increment(1);
+            authenticated = true;
+            check_csrf_protection = false;
+            // TODO: cookies are not added, but are not needed right now
+          }
+        }
+        if (!authenticated) {
+          if (use_jwt_) {
+            LOG(INFO) << "Invalid JWT token provided";
+            total_jwt_token_auth_failure_->Increment(1);
+          }
+          if (use_oauth_) {
+            LOG(INFO) << "Invalid OAuth token provided";
+            total_oauth_token_auth_failure_->Increment(1);
+          }
         }
       }
     }
@@ -742,10 +769,15 @@ sq_callback_result_t Webserver::BeginRequestCallback(struct sq_connection* conne
   // unpredictably costly.
   if (!authenticated && check_trusted_domain_) {
     const char* xff_origin = sq_get_header(connection, "X-Forwarded-For");
-    string xff_origin_string = !xff_origin ? "" : string(xff_origin);
-    string origin = FLAGS_trusted_domain_use_xff_header ?
-        xff_origin_string :
-        GetRemoteAddress(request_info).ToString();
+    std::string_view xff_origin_sv = !xff_origin ? "" : xff_origin;
+    string origin;
+    if (FLAGS_trusted_domain_use_xff_header) {
+      Status status = GetXFFOriginClientAddress(xff_origin_sv, origin);
+      if (!status.ok()) LOG(WARNING) << status.GetDetail();
+    } else {
+      origin = GetRemoteAddress(request_info).ToString();
+    }
+
     StripWhiteSpace(&origin);
     if (origin.empty() && FLAGS_trusted_domain_use_xff_header &&
         FLAGS_trusted_domain_empty_xff_header_use_origin) {
@@ -1044,7 +1076,7 @@ bool Webserver::JWTTokenAuth(const std::string& jwt_token,
     return false;
   }
   if (FLAGS_jwt_validate_signature) {
-    status = JWTHelper::GetInstance()->Verify(decoded_token.get());
+    status = ExecEnv::GetInstance()->GetJWTHelperInstance()->Verify(decoded_token.get());
     if (!status.ok()) {
       LOG(ERROR) << "Error verifying JWT token in Authorization header, "
                  << "Error: " << status;
@@ -1065,6 +1097,39 @@ bool Webserver::JWTTokenAuth(const std::string& jwt_token,
   return true;
 }
 
+bool Webserver::OAuthTokenAuth(const std::string& oauth_token,
+    struct sq_connection* connection, struct sq_request_info* request_info) {
+  JWTHelper::UniqueJWTDecodedToken decoded_token;
+  Status status = JWTHelper::Decode(oauth_token, decoded_token);
+  if (!status.ok()) {
+    LOG(ERROR) << "Error decoding OAuth token in Authorization header, "
+               << "Error: " << status;
+    return false;
+  }
+  if (FLAGS_oauth_jwt_validate_signature) {
+    status = ExecEnv::GetInstance()->GetOAuthHelperInstance()->Verify(
+        decoded_token.get());
+    if (!status.ok()) {
+      LOG(ERROR) << "Error verifying OAuth token in Authorization header, "
+                 << "Error: " << status;
+      return false;
+    }
+  }
+
+  DCHECK(!FLAGS_oauth_jwt_custom_claim_username.empty());
+  string username;
+  status = JWTHelper::GetCustomClaimUsername(
+      decoded_token.get(), FLAGS_oauth_jwt_custom_claim_username, username);
+  if (!status.ok()) {
+    LOG(ERROR) << "Cannot retrieve username from OAUTh token in Authorization header, "
+               << "Error: " << status;
+    return false;
+  }
+  request_info->remote_user = strdup(username.c_str());
+
+  return true;
+}
+
 Status Webserver::HandleBasic(struct sq_connection* connection,
     struct sq_request_info* request_info, vector<string>* response_headers) {
   const char* authz_header = sq_get_header(connection, "Authorization");
