@@ -39,6 +39,22 @@ sudo -E apt-get --yes install g++ gcc git libsasl2-dev libssl-dev make python-de
 
 source /etc/lsb-release
 
+# Ubuntu 20's Python 2.7.18-1~20.04.5 version has a bug in its tarfile support.
+# If we detect the affected tarfile.py, download a patched version and overwrite it.
+if [[ $DISTRIB_ID == Ubuntu && $DISTRIB_RELEASE == 20.04 ]]; then
+  if [[ -f /usr/lib/python2.7/tarfile.py ]]; then
+    TARFILE_PY_HASH=$(sha1sum /usr/lib/python2.7/tarfile.py | cut -d' ' -f1)
+    if [[ "${TARFILE_PY_HASH}" == "6e1a6d9ea2a535cbb17fe266ed9ac76eb5e27b89" ]]; then
+      TMP_DIR=$(mktemp -d)
+      pushd $TMP_DIR
+      wget -nv https://launchpadlibrarian.net/759546541/tarfile.py
+      sudo cp tarfile.py /usr/lib/python2.7/tarfile.py
+      popd
+      rm -rf $TMP_DIR
+    fi
+  fi
+fi
+
 JDK_VERSION=8
 if [[ $DISTRIB_RELEASE = 14.04 ]]
 then
@@ -48,12 +64,13 @@ sudo apt-get --yes install openjdk-${JDK_VERSION}-jdk openjdk-${JDK_VERSION}-sou
 export JAVA_HOME=/usr/lib/jvm/java-${JDK_VERSION}-openjdk-amd64
 
 # Download Maven since the packaged version is pretty old.
-if [ ! -d /usr/local/apache-maven-3.9.2 ]; then
+MVN_VERSION="3.9.8"
+if [ ! -d "/usr/local/apache-maven-${MVN_VERSION}" ]; then
   sudo wget -nv \
-    https://archive.apache.org/dist/maven/maven-3/3.9.2/binaries/apache-maven-3.9.2-bin.tar.gz
-  sha512sum -c - <<< '900bdeeeae550d2d2b3920fe0e00e41b0069f32c019d566465015bdd1b3866395cbe016e22d95d25d51d3a5e614af2c83ec9b282d73309f644859bbad08b63db  apache-maven-3.9.2-bin.tar.gz'
-  sudo tar -C /usr/local -xzf apache-maven-3.9.2-bin.tar.gz
-  sudo ln -s /usr/local/apache-maven-3.9.2/bin/mvn /usr/local/bin
+    "https://archive.apache.org/dist/maven/maven-3/${MVN_VERSION}/binaries/apache-maven-${MVN_VERSION}-bin.tar.gz"
+  sha512sum -c - <<< "7d171def9b85846bf757a2cec94b7529371068a0670df14682447224e57983528e97a6d1b850327e4ca02b139abaab7fcb93c4315119e6f0ffb3f0cbc0d0b9a2 apache-maven-${MVN_VERSION}-bin.tar.gz"
+  sudo tar -C /usr/local -xzf apache-maven-${MVN_VERSION}-bin.tar.gz
+  sudo ln -s /usr/local/apache-maven-${MVN_VERSION}/bin/mvn /usr/local/bin
 fi
 
 # Try to prepopulate the m2 directory to save time
