@@ -204,6 +204,9 @@ function notindocker {
   fi
 }
 
+# X permission on home directory is needed for some uses of postgresql (IMPALA-13693)
+chmod o+X ~
+
 # Note that yum has its own retries; see yum.conf(5).
 REAL_APT_GET=$(ubuntu which apt-get)
 function apt-get {
@@ -243,6 +246,22 @@ ubuntu16 apt-get --yes install python python-dev python-setuptools
 ubuntu18 apt-get --yes install python python-dev python-setuptools
 ubuntu20 apt-get --yes install python python-dev python-setuptools
 
+# Ubuntu 20's Python 2.7.18-1~20.04.5 version has a bug in its tarfile support.
+# If we detect the affected tarfile.py, download a patched version and overwrite it.
+if [[ $UBUNTU20 == true ]]; then
+  if [[ -f /usr/lib/python2.7/tarfile.py ]]; then
+    TARFILE_PY_HASH=$(sha1sum /usr/lib/python2.7/tarfile.py | cut -d' ' -f1)
+    if [[ "${TARFILE_PY_HASH}" == "6e1a6d9ea2a535cbb17fe266ed9ac76eb5e27b89" ]]; then
+      TMP_DIR=$(mktemp -d)
+      pushd $TMP_DIR
+      wget -nv https://launchpadlibrarian.net/759546541/tarfile.py
+      sudo cp tarfile.py /usr/lib/python2.7/tarfile.py
+      popd
+      rm -rf $TMP_DIR
+    fi
+  fi
+fi
+
 # Required by Kudu in the minicluster
 ubuntu20 apt-get --yes install libtinfo5
 ubuntu22 apt-get --yes install libtinfo5
@@ -341,18 +360,19 @@ redhat sudo yum install -y ccache
 redhat sudo yum clean all
 
 # Download Maven since the packaged version is pretty old.
-if [ ! -d /usr/local/apache-maven-3.9.2 ]; then
+MVN_VERSION="3.9.8"
+if [ ! -d "/usr/local/apache-maven-${MVN_VERSION}" ]; then
   sudo wget -nv \
-    https://archive.apache.org/dist/maven/maven-3/3.9.2/binaries/apache-maven-3.9.2-bin.tar.gz
-  sha512sum -c - <<< '900bdeeeae550d2d2b3920fe0e00e41b0069f32c019d566465015bdd1b3866395cbe016e22d95d25d51d3a5e614af2c83ec9b282d73309f644859bbad08b63db  apache-maven-3.9.2-bin.tar.gz'
-  sudo tar -C /usr/local -xzf apache-maven-3.9.2-bin.tar.gz
+    "https://archive.apache.org/dist/maven/maven-3/${MVN_VERSION}/binaries/apache-maven-${MVN_VERSION}-bin.tar.gz"
+  sha512sum -c - <<< "7d171def9b85846bf757a2cec94b7529371068a0670df14682447224e57983528e97a6d1b850327e4ca02b139abaab7fcb93c4315119e6f0ffb3f0cbc0d0b9a2 apache-maven-${MVN_VERSION}-bin.tar.gz"
+  sudo tar -C /usr/local -xzf "apache-maven-${MVN_VERSION}-bin.tar.gz"
   # Ensure that Impala's preferred version is installed locally,
   # even if a previous version exists there.
-  sudo ln -s -f /usr/local/apache-maven-3.9.2/bin/mvn /usr/local/bin
+  sudo ln -s -f "/usr/local/apache-maven-${MVN_VERSION}/bin/mvn" "/usr/local/bin"
 
   # reset permissions on redhat8
   # TODO: figure out why this is necessary for redhat8
-  MAVEN_DIRECTORY="/usr/local/apache-maven-3.9.2"
+  MAVEN_DIRECTORY="/usr/local/apache-maven-${MVN_VERSION}"
   redhat8 indocker sudo chmod 0755 ${MAVEN_DIRECTORY}
   redhat8 indocker sudo chmod 0755 ${MAVEN_DIRECTORY}/{bin,boot}
   redhat9 indocker sudo chmod 0755 ${MAVEN_DIRECTORY}
@@ -476,7 +496,8 @@ echo -e "\n* - nofile 1048576" | sudo tee -a /etc/security/limits.conf
 # uses systemd, which sets its own default. With Ubuntu 18.04 that default is 16 KB,
 # 20.04+ defaults to 64 MB. Set all to 64 MB for the current user; Impala test systems
 # require 10s of GBs of memory, so this setting should not be a problem.
-echo -e "$USER - memlock 65536" | sudo tee /etc/security/limits.d/10-memlock.conf
+USER=${USER-$(id -un)}
+echo -e "${USER} - memlock 65536" | sudo tee /etc/security/limits.d/10-memlock.conf
 
 # Default on CentOS limits a user to 1024 or 4096 processes (threads) , which isn't
 # enough for minicluster with all of its friends.
