@@ -21,10 +21,13 @@ A script for generating arbitrary junit XML reports while building Impala.
 These files will be consumed by jenkins.impala.io to generate reports for
 easier triaging of build and setup errors.
 """
+from __future__ import absolute_import, division, print_function
 import argparse
 import codecs
 import errno
 import os
+import re
+import sys
 import textwrap
 from xml.dom import minidom
 from xml.etree import ElementTree as ET
@@ -165,10 +168,29 @@ class JunitReport(object):
     junit_log_file = os.path.join(junitxml_logdir, filename)
 
     with codecs.open(junit_log_file, encoding="UTF-8", mode='w') as f:
-      f.write(unicode(self))
+      if sys.version_info.major < 3:
+        f.write(unicode(self))
+      else:
+        f.write(str(self))
 
     return junit_log_file
 
+  @staticmethod
+  def remove_ansi_escape_sequences(string):
+    """
+    Remove ANSI escape sequences from this string.
+
+    ANSI escape sequences customize terminal output by adding colors, etc.
+    Compilers use them to add color to error messages. ANSI escape
+    sequences interfere with producing the JUnitXML (and do not add any
+    value for JUnitXML), so this function strips them.
+
+    See https://stackoverflow.com/questions/14693701 for more information
+    on this solution.
+    """
+    ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
+    return ansi_escape.sub('', string)
+
   @staticmethod
   def get_xml_content(file_or_string=None):
     """
@@ -194,8 +216,11 @@ class JunitReport(object):
     else:
       # This is a string passed in on the command line. Make sure to return it as
       # a unicode string.
-      content = unicode(file_or_string, encoding="UTF-8")
-    return content
+      if sys.version_info.major < 3:
+        content = unicode(file_or_string, encoding="UTF-8")
+      else:
+        content = file_or_string
+    return JunitReport.remove_ansi_escape_sequences(content)
 
   def __unicode__(self):
     """
@@ -205,6 +230,10 @@ class JunitReport(object):
     root_node_dom = minidom.parseString(root_node_unicode)
     return root_node_dom.toprettyxml(indent=' ' * 4)
 
+  def __str__(self):
+    if sys.version_info.major < 3:
+      return unicode(self).encode('utf-8')
+    return self.__unicode__()
 
 def get_options():
   """Parse and return command line options."""
