@@ -70,11 +70,7 @@ fi
 # WARNING: If changing this value, also run these commands:
 # cd ${IMPALA_HOME}/java
 # mvn versions:set -DnewVersion=YOUR_NEW_VERSION
-export IMPALA_VERSION=4.4.1-RELEASE
-
-# Whether to build on Apache Hive (or CDP Hive). Versions of some toolchain dependencies
-# (e.g. thrift) will also depend on this.
-export USE_APACHE_HIVE=${USE_APACHE_HIVE-false}
+export IMPALA_VERSION=4.5.0-RELEASE
 
 # Whether to build the backend on Avro C++ library or C.
 # This is added temporarily to help transitioning from Avro C to C++ library.
@@ -85,13 +81,13 @@ export USE_AVRO_CPP=${USE_AVRO_CPP:=false}
 # moving to a different build of the toolchain, e.g. when a version is bumped or a
 # compile option is changed. The build id can be found in the output of the toolchain
 # build jobs, it is constructed from the build number and toolchain git hash prefix.
-export IMPALA_TOOLCHAIN_BUILD_ID_AARCH64=34-f93e2c9a86
-export IMPALA_TOOLCHAIN_BUILD_ID_X86_64=422-f93e2c9a86
+export IMPALA_TOOLCHAIN_BUILD_ID_AARCH64=63-3a0ac57d41
+export IMPALA_TOOLCHAIN_BUILD_ID_X86_64=491-3a0ac57d41
 export IMPALA_TOOLCHAIN_REPO=\
 ${IMPALA_TOOLCHAIN_REPO:-https://github.com/cloudera/native-toolchain.git}
 export IMPALA_TOOLCHAIN_BRANCH=${IMPALA_TOOLCHAIN_BRANCH:-master}
 export IMPALA_TOOLCHAIN_COMMIT_HASH=\
-${IMPALA_TOOLCHAIN_COMMIT_HASH-f93e2c9a865c80cafd76b872ad04400877766a2f}
+${IMPALA_TOOLCHAIN_COMMIT_HASH-810d0f47572d9ac4036c34eb2ad6342d51d510cd}
 # Compare the build ref in build IDs by removing everything up-to-and-including the
 # first hyphen.
 if [ "${IMPALA_TOOLCHAIN_BUILD_ID_AARCH64#*-}" \
@@ -125,7 +121,7 @@ export IMPALA_CMAKE_VERSION=3.22.2
 unset IMPALA_CMAKE_URL
 export IMPALA_CRCUTIL_VERSION=2903870057d2f1f109b245650be29e856dc8b646
 unset IMPALA_CRCUTIL_URL
-export IMPALA_CURL_VERSION=7.78.0
+export IMPALA_CURL_VERSION=8.10.1
 unset IMPALA_CURL_URL
 export IMPALA_CYRUS_SASL_VERSION=2.1.23
 unset IMPALA_CYRUS_SASL_URL
@@ -141,7 +137,7 @@ export IMPALA_GLOG_VERSION=0.3.5-p3
 unset IMPALA_GLOG_URL
 export IMPALA_GPERFTOOLS_VERSION=2.10
 unset IMPALA_GPERFTOOLS_URL
-export IMPALA_GTEST_VERSION=1.6.0
+export IMPALA_GTEST_VERSION=1.14.0
 unset IMPALA_GTEST_URL
 export IMPALA_JWT_CPP_VERSION=0.5.0
 unset IMPALA_JWT_CPP_URL
@@ -149,9 +145,9 @@ export IMPALA_LIBEV_VERSION=4.20-p1
 unset IMPALA_LIBEV_URL
 export IMPALA_LIBUNWIND_VERSION=1.7.2-p1
 unset IMPALA_LIBUNWIND_URL
-export IMPALA_LLVM_VERSION=5.0.1-p7
+export IMPALA_LLVM_VERSION=5.0.1-p8
 unset IMPALA_LLVM_URL
-export IMPALA_LLVM_ASAN_VERSION=5.0.1-p7
+export IMPALA_LLVM_ASAN_VERSION=5.0.1-p8
 unset IMPALA_LLVM_ASAN_URL
 
 # To limit maximum memory available for the mini-cluster and CDH cluster, add the
@@ -165,7 +161,7 @@ export IMPALA_LLVM_UBSAN_BASE_VERSION=5.0.1
 
 # Debug builds should use the release+asserts build to get additional coverage.
 # Don't use the LLVM debug build because the binaries are too large to distribute.
-export IMPALA_LLVM_DEBUG_VERSION=5.0.1-asserts-p7
+export IMPALA_LLVM_DEBUG_VERSION=5.0.1-asserts-p8
 unset IMPALA_LLVM_DEBUG_URL
 export IMPALA_LZ4_VERSION=1.9.3
 unset IMPALA_LZ4_URL
@@ -179,14 +175,14 @@ export IMPALA_PROTOBUF_VERSION=3.14.0
 unset IMPALA_PROTOBUF_URL
 export IMPALA_PROTOBUF_CLANG_VERSION=3.14.0-clangcompat-p2
 unset IMPALA_PROTOBUF_CLANG_URL
-export IMPALA_POSTGRES_JDBC_DRIVER_VERSION=42.5.1
+export IMPALA_POSTGRES_JDBC_DRIVER_VERSION=42.5.6
 unset IMPALA_POSTGRES_JDBC_DRIVER_URL
 export IMPALA_PYTHON_VERSION=2.7.16
 unset IMPALA_PYTHON_URL
-export IMPALA_PYTHON3_VERSION=3.7.16
-export IMPALA_RAPIDJSON_VERSION=1.1.0
+export IMPALA_PYTHON3_VERSION=3.8.18
+export IMPALA_RAPIDJSON_VERSION=1.1.0-p1
 unset IMPALA_RAPIDJSON_URL
-export IMPALA_RE2_VERSION=20190301
+export IMPALA_RE2_VERSION=2023-03-01
 unset IMPALA_RE2_URL
 export IMPALA_SNAPPY_VERSION=1.1.8
 unset IMPALA_SNAPPY_URL
@@ -213,29 +209,6 @@ unset IMPALA_MOLD_URL
 # Impala JDBC driver for testing.
 export IMPALA_SIMBA_JDBC_DRIVER_VERSION=42-2.6.32.1041
 
-# Thrift related environment variables.
-# IMPALA_THRIFT_POM_VERSION is used to populate IMPALA_THRIFT_JAVA_VERSION and
-# thrift.version in java/pom.xml.
-# IMPALA_THRIFT_PY_VERSION is used to find the thrift compiler to produce
-# the generated Python code. The code that uses the generated Python code gets
-# the corresponding Thrift runtime library by pip installing thrift (and does not
-# respect this version). If upgrading IMPALA_THRIFT_PY_VERSION, also upgrade the
-# thrift version in shell/ext-py, shell/packaging/requirements.txt, and
-# infra/python/deps/requirements.txt.
-export IMPALA_THRIFT_CPP_VERSION=0.16.0-p7
-unset IMPALA_THRIFT_CPP_URL
-if $USE_APACHE_HIVE; then
-  # Apache Hive 3 clients can't run on thrift versions >= 0.14 (IMPALA-11801)
-  export IMPALA_THRIFT_POM_VERSION=0.11.0
-  export IMPALA_THRIFT_JAVA_VERSION=${IMPALA_THRIFT_POM_VERSION}-p5
-else
-  export IMPALA_THRIFT_POM_VERSION=0.16.0
-  export IMPALA_THRIFT_JAVA_VERSION=${IMPALA_THRIFT_POM_VERSION}-p7
-fi
-unset IMPALA_THRIFT_JAVA_URL
-export IMPALA_THRIFT_PY_VERSION=0.16.0-p7
-unset IMPALA_THRIFT_PY_URL
-
 # Find system python versions for testing
 export IMPALA_SYSTEM_PYTHON2="${IMPALA_SYSTEM_PYTHON2_OVERRIDE-$(command -v python2)}"
 export IMPALA_SYSTEM_PYTHON3="${IMPALA_SYSTEM_PYTHON3_OVERRIDE-$(command -v python3)}"
@@ -255,47 +228,61 @@ fi
 : ${IMPALA_TOOLCHAIN_HOST:=native-toolchain.s3.amazonaws.com}
 export IMPALA_TOOLCHAIN_HOST
 
-export CDP_BUILD_NUMBER=45689292
+export CDP_BUILD_NUMBER=58457853
 export CDP_MAVEN_REPOSITORY=\
 "https://${IMPALA_TOOLCHAIN_HOST}/build/cdp_components/${CDP_BUILD_NUMBER}/maven"
-export CDP_AVRO_JAVA_VERSION=1.8.2.7.2.18.0-369
-export CDP_HADOOP_VERSION=3.1.1.7.2.18.0-369
-export CDP_HBASE_VERSION=2.4.17.7.2.18.0-369
-export CDP_HIVE_VERSION=3.1.3000.7.2.18.0-369
-export CDP_ICEBERG_VERSION=1.3.1.7.2.18.0-369
-export CDP_KNOX_VERSION=1.3.0.7.2.18.0-369
-export CDP_OZONE_VERSION=1.3.0.7.2.18.0-369
-export CDP_PARQUET_VERSION=1.10.99.7.2.18.0-369
-export CDP_RANGER_VERSION=2.4.0.7.2.18.0-369
-export CDP_TEZ_VERSION=0.9.1.7.2.18.0-369
+export CDP_AVRO_JAVA_VERSION=1.11.1.7.3.1.0-160
+export CDP_HADOOP_VERSION=3.1.1.7.3.1.0-160
+export CDP_HBASE_VERSION=2.4.17.7.3.1.0-160
+export CDP_HIVE_VERSION=3.1.3000.7.3.1.0-160
+export CDP_ICEBERG_VERSION=1.3.1.7.3.1.0-160
+export CDP_KNOX_VERSION=2.0.0.7.3.1.0-160
+export CDP_OZONE_VERSION=1.3.0.7.3.1.0-160
+export CDP_PARQUET_VERSION=1.12.3.7.3.1.0-160
+export CDP_RANGER_VERSION=2.4.0.7.3.1.0-160
+export CDP_TEZ_VERSION=0.9.1.7.3.1.0-160
 
 # Ref: https://infra.apache.org/release-download-pages.html#closer
 : ${APACHE_MIRROR:="https://www.apache.org/dyn/closer.cgi"}
 export APACHE_MIRROR
+export APACHE_AVRO_JAVA_VERSION=1.8.2
+export APACHE_HADOOP_VERSION=3.3.6
+export APACHE_HBASE_VERSION=2.6.0
+export APACHE_ICEBERG_VERSION=1.4.3
+export APACHE_KNOX_VERSION=1.6.1
+export APACHE_PARQUET_VERSION=1.12.3
+export APACHE_RANGER_VERSION=2.4.0
+export APACHE_TEZ_VERSION=0.10.2
 export APACHE_HIVE_VERSION=3.1.3
 export APACHE_HIVE_STORAGE_API_VERSION=2.7.0
-export APACHE_OZONE_VERSION=1.3.0
+export APACHE_OZONE_VERSION=1.4.0
 
 # Java dependencies that are not also runtime components. Declaring versions here allows
 # other branches to override them in impala-config-branch.sh for cleaner patches.
-export IMPALA_BOUNCY_CASTLE_VERSION=1.68
+export IMPALA_BOUNCY_CASTLE_VERSION=1.78
+export IMPALA_COMMONS_LANG3_VERSION=3.17.0
 export IMPALA_COS_VERSION=3.1.0-8.0.8
 export IMPALA_DERBY_VERSION=10.14.2.0
 export IMPALA_GUAVA_VERSION=32.0.1-jre
 export IMPALA_HUDI_VERSION=0.5.0-incubating
 export IMPALA_HTTP_CORE_VERSION=4.4.14
+export IMPALA_JACKSON_VERSION=2.15.3
 export IMPALA_JACKSON_DATABIND_VERSION=2.15.3
 export IMPALA_JSON_SMART_VERSION=2.4.11
 export IMPALA_JUNIT_VERSION=4.12
 export IMPALA_KITE_VERSION=1.1.0
+export IMPALA_LOG4J2_VERSION=2.18.0
 export IMPALA_ORC_JAVA_VERSION=1.7.6
 export IMPALA_PAC4J_VERSION=4.5.5
 export IMPALA_RELOAD4j_VERSION=1.2.22
 export IMPALA_SLF4J_VERSION=2.0.3
-export IMPALA_SPRINGFRAMEWORK_VERSION=5.3.27
+export IMPALA_SPRINGFRAMEWORK_VERSION=5.3.39
 export IMPALA_XMLSEC_VERSION=2.2.6
 export IMPALA_OBS_VERSION=3.1.1-hw-42
 export IMPALA_DBCP2_VERSION=2.9.0
+export IMPALA_DROPWIZARD_METRICS_VERSION=4.2.26
+export IMPALA_AIRCOMPRESSOR_VERSION=0.27
+export IMPALA_DATASKETCHES_VERSION=6.0.0
 
 # When Impala is building docker images on Redhat-based distributions,
 # it is useful to be able to customize the base image. Some users will
@@ -338,6 +325,8 @@ export IMPALA_KERBERIZE=false
 unset IMPALA_TOOLCHAIN_KUDU_MAVEN_REPOSITORY
 unset IMPALA_TOOLCHAIN_KUDU_MAVEN_REPOSITORY_ENABLED
 
+export IMPALA_USE_PYTHON3_TESTS=${IMPALA_USE_PYTHON3_TESTS:-false}
+
 # Source the branch and local config override files here to override any
 # variables above or any variables below that allow overriding via environment
 # variable.
@@ -372,22 +361,53 @@ export APACHE_OZONE_URL=${APACHE_OZONE_URL-}
 
 export CDP_COMPONENTS_HOME="$IMPALA_TOOLCHAIN/cdp_components-$CDP_BUILD_NUMBER"
 export CDH_MAJOR_VERSION=7
-export IMPALA_AVRO_JAVA_VERSION=${CDP_AVRO_JAVA_VERSION}
-export IMPALA_HADOOP_VERSION=${CDP_HADOOP_VERSION}
-export IMPALA_HADOOP_URL=${CDP_HADOOP_URL-}
-export HADOOP_HOME="$CDP_COMPONENTS_HOME/hadoop-${IMPALA_HADOOP_VERSION}/"
-export IMPALA_HBASE_VERSION=${CDP_HBASE_VERSION}
-export IMPALA_HBASE_URL=${CDP_HBASE_URL-}
-export IMPALA_ICEBERG_VERSION=${CDP_ICEBERG_VERSION}
-export IMPALA_ICEBERG_URL=${CDP_ICEBERG_URL-}
-export IMPALA_KNOX_VERSION=${CDP_KNOX_VERSION}
-export IMPALA_PARQUET_VERSION=${CDP_PARQUET_VERSION}
-export IMPALA_RANGER_VERSION=${CDP_RANGER_VERSION}
-export IMPALA_RANGER_URL=${CDP_RANGER_URL-}
-export IMPALA_TEZ_VERSION=${CDP_TEZ_VERSION}
-export IMPALA_TEZ_URL=${CDP_TEZ_URL-}
+if ${USE_APACHE_COMPONENTS:=false}; then
+  export IMPALA_AVRO_JAVA_VERSION=${APACHE_AVRO_JAVA_VERSION}
+  export IMPALA_HADOOP_VERSION=${APACHE_HADOOP_VERSION}
+  export IMPALA_HBASE_VERSION=${APACHE_HBASE_VERSION}
+  export IMPALA_ICEBERG_VERSION=${APACHE_ICEBERG_VERSION}
+  export IMPALA_KNOX_VERSION=${APACHE_KNOX_VERSION}
+  export IMPALA_OZONE_VERSION=${APACHE_OZONE_VERSION}
+  export IMPALA_PARQUET_VERSION=${APACHE_PARQUET_VERSION}
+  export IMPALA_RANGER_VERSION=${RANGER_VERSION_OVERRIDE:-"$APACHE_RANGER_VERSION"}
+  export IMPALA_TEZ_VERSION=${APACHE_TEZ_VERSION}
+  export USE_APACHE_HADOOP=true
+  export USE_APACHE_HBASE=true
+  export USE_APACHE_HIVE=true
+  export USE_APACHE_TEZ=true
+  export USE_APACHE_RANGER=true
+  export USE_APACHE_OZONE=true
+else
+  export IMPALA_AVRO_JAVA_VERSION=${CDP_AVRO_JAVA_VERSION}
+  export IMPALA_HADOOP_VERSION=${CDP_HADOOP_VERSION}
+  export IMPALA_HADOOP_URL=${CDP_HADOOP_URL-}
+  export IMPALA_HBASE_VERSION=${CDP_HBASE_VERSION}
+  export IMPALA_HBASE_URL=${CDP_HBASE_URL-}
+  export IMPALA_ICEBERG_VERSION=${CDP_ICEBERG_VERSION}
+  export IMPALA_ICEBERG_URL=${CDP_ICEBERG_URL-}
+  export IMPALA_KNOX_VERSION=${CDP_KNOX_VERSION}
+  export IMPALA_OZONE_VERSION=${CDP_OZONE_VERSION}
+  export IMPALA_PARQUET_VERSION=${CDP_PARQUET_VERSION}
+  export IMPALA_RANGER_VERSION=${RANGER_VERSION_OVERRIDE:-"$CDP_RANGER_VERSION"}
+  export IMPALA_RANGER_URL=${CDP_RANGER_URL-}
+  export IMPALA_TEZ_VERSION=${CDP_TEZ_VERSION}
+  export IMPALA_TEZ_URL=${CDP_TEZ_URL-}
+  export USE_APACHE_HADOOP=false
+  export USE_APACHE_HBASE=false
+  export USE_APACHE_HIVE=false
+  export USE_APACHE_TEZ=false
+  export USE_APACHE_RANGER=false
+  export USE_APACHE_OZONE=false
+fi
 
 export APACHE_COMPONENTS_HOME="$IMPALA_TOOLCHAIN/apache_components"
+
+if $USE_APACHE_HADOOP; then
+  export HADOOP_HOME="$APACHE_COMPONENTS_HOME/hadoop-${IMPALA_HADOOP_VERSION}"
+else
+  export HADOOP_HOME="$CDP_COMPONENTS_HOME/hadoop-${IMPALA_HADOOP_VERSION}"
+fi
+
 if $USE_APACHE_HIVE; then
   # When USE_APACHE_HIVE is set we use the apache hive version to build as well as deploy
   # in the minicluster
@@ -406,6 +426,28 @@ else
   export IMPALA_HIVE_STORAGE_API_VERSION=${HIVE_STORAGE_API_VERSION_OVERRIDE:-\
 "2.3.0.$IMPALA_HIVE_VERSION"}
 fi
+# Thrift related environment variables.
+# IMPALA_THRIFT_POM_VERSION is used to populate IMPALA_THRIFT_JAVA_VERSION and
+# thrift.version in java/pom.xml.
+# IMPALA_THRIFT_PY_VERSION is used to find the thrift compiler to produce
+# the generated Python code. The code that uses the generated Python code gets
+# the corresponding Thrift runtime library by pip installing thrift (and does not
+# respect this version). If upgrading IMPALA_THRIFT_PY_VERSION, also upgrade the
+# thrift version in shell/packaging/requirements.txt and
+# infra/python/deps/requirements.txt.
+export IMPALA_THRIFT_CPP_VERSION=0.16.0-p7
+unset IMPALA_THRIFT_CPP_URL
+if $USE_APACHE_HIVE; then
+  # Apache Hive 3 clients can't run on thrift versions >= 0.14 (IMPALA-11801)
+  export IMPALA_THRIFT_POM_VERSION=0.11.0
+  export IMPALA_THRIFT_JAVA_VERSION=${IMPALA_THRIFT_POM_VERSION}-p5
+else
+  export IMPALA_THRIFT_POM_VERSION=0.16.0
+  export IMPALA_THRIFT_JAVA_VERSION=${IMPALA_THRIFT_POM_VERSION}-p7
+fi
+unset IMPALA_THRIFT_JAVA_URL
+export IMPALA_THRIFT_PY_VERSION=0.16.0-p7
+unset IMPALA_THRIFT_PY_URL
 
 # Extract the first component of the hive version.
 # Allow overriding of Hive source location in case we want to build Impala without
@@ -420,7 +462,6 @@ if [[ "${IMPALA_HIVE_MAJOR_VERSION}" == "1" ||
   return 1
 fi
 
-export USE_APACHE_OZONE=${USE_APACHE_OZONE-false}
 if $USE_APACHE_OZONE; then
   export IMPALA_OZONE_VERSION=${APACHE_OZONE_VERSION}
   export IMPALA_OZONE_URL=${APACHE_OZONE_URL-}
@@ -556,7 +597,7 @@ export ENABLE_IMPALA_IR_DEBUG_INFO=${ENABLE_IMPALA_IR_DEBUG_INFO-false}
 # disk space for a developer environment. A large amount of the binary
 # size is due to debug information.
 #
-# These are two options for reducing the binary size and disk space
+# These are a few options for reducing the binary size and disk space
 # usage.
 # - IMPALA_MINIMAL_DEBUG_INFO=true changes the build to produce only
 #   minimal debuginfo (i.e. -g1). This has line tables and can do backtraces,
@@ -569,15 +610,25 @@ export ENABLE_IMPALA_IR_DEBUG_INFO=${ENABLE_IMPALA_IR_DEBUG_INFO-false}
 #   the Breakpad scripts have been modified to handle it, but there may
 #   be other tools that do not know how to use it. This reduces the size
 #   of binaries by 50+%.
-# Both of these are disabled by default.
-# TODO: Explore enabling IMPALA_COMPRESSED_DEBUG_INFO by default.
+# - IMPALA_SPLIT_DEBUG_INFO=true changes the build to put debug info in
+#   separate .dwo files for each C++ file. Executables contain metadata
+#   pointing to these .dwo files without needing to incorporate the debug
+#   information. This allows executables to share a single copy of
+#   the debug information. It also reduces link time, as the linker does
+#   not need to process the debug info. Tools (including gdb) mostly know
+#   how to handle this split debug information.
+#
+# Due to the major reduction in binary size and broad support in debugging
+# tools, compressed debug information is enabled by default.
 export IMPALA_MINIMAL_DEBUG_INFO=${IMPALA_MINIMAL_DEBUG_INFO-false}
-export IMPALA_COMPRESSED_DEBUG_INFO=${IMPALA_COMPRESSED_DEBUG_INFO-false}
+export IMPALA_COMPRESSED_DEBUG_INFO=${IMPALA_COMPRESSED_DEBUG_INFO-true}
+export IMPALA_SPLIT_DEBUG_INFO=${IMPALA_SPLIT_DEBUG_INFO-false}
 
 # Download and use the CDH components from S3. It can be useful to set this to false if
 # building against a custom local build using HIVE_SRC_DIR_OVERRIDE,
 # HADOOP_INCLUDE_DIR_OVERRIDE, and HADOOP_LIB_DIR_OVERRIDE.
 export DOWNLOAD_CDH_COMPONENTS=${DOWNLOAD_CDH_COMPONENTS-true}
+export DOWNLOAD_APACHE_COMPONENTS=${DOWNLOAD_APACHE_COMPONENTS-true}
 
 export IS_OSX="$(if [[ "$OSTYPE" == "darwin"* ]]; then echo true; else echo false; fi)"
 
@@ -649,8 +700,17 @@ fi
 # Set the path to the hive_metastore.thrift which is used to build thrift code
 export HIVE_METASTORE_THRIFT_DIR=${HIVE_METASTORE_THRIFT_DIR_OVERRIDE:-\
 "$HIVE_SRC_DIR/standalone-metastore/src/main/thrift"}
-export TEZ_HOME="$CDP_COMPONENTS_HOME/tez-${IMPALA_TEZ_VERSION}-minimal"
-export HBASE_HOME="$CDP_COMPONENTS_HOME/hbase-${IMPALA_HBASE_VERSION}/"
+if $USE_APACHE_TEZ; then
+  export TEZ_HOME="$APACHE_COMPONENTS_HOME/apache-tez-${IMPALA_TEZ_VERSION}-bin"
+else
+  export TEZ_HOME="$CDP_COMPONENTS_HOME/tez-${IMPALA_TEZ_VERSION}-minimal"
+fi
+
+if $USE_APACHE_HBASE; then
+  export HBASE_HOME="$APACHE_COMPONENTS_HOME/hbase-${IMPALA_HBASE_VERSION}-hadoop3/"
+else
+  export HBASE_HOME="$CDP_COMPONENTS_HOME/hbase-${IMPALA_HBASE_VERSION}/"
+fi
 if $USE_APACHE_OZONE; then
   export OZONE_HOME="$APACHE_COMPONENTS_HOME/ozone-${IMPALA_OZONE_VERSION}/"
 else
@@ -904,9 +964,22 @@ export IMPALA_ALL_LOGS_DIRS="${IMPALA_CLUSTER_LOGS_DIR}
   ${IMPALA_CUSTOM_CLUSTER_TEST_LOGS_DIR} ${IMPALA_MVN_LOGS_DIR}
   ${IMPALA_TIMEOUT_LOGS_DIR}"
 
+# Compute CPUs, using cgroup limits if present and not "max" (v2) or negative (v1)
+awk_divide_roundup='{ cores = $1/$2; print cores==int(cores) ? cores : int(cores)+1 }'
+if grep -v max /sys/fs/cgroup/cpu.max >& /dev/null; then
+  # Get CPU limits under cgroups v2
+  CORES=$(awk "$awk_divide_roundup" /sys/fs/cgroup/cpu.max)
+  echo "Detected $CORES cores from cgroups v2"
+elif grep -v '\-' /sys/fs/cgroup/cpu/cpu.cfs_quota_us >& /dev/null; then
+  # Get CPU limits under cgroups v1
+  CORES=$(paste /sys/fs/cgroup/cpu/cpu.cfs_quota_us /sys/fs/cgroup/cpu/cpu.cfs_period_us |
+          awk "$awk_divide_roundup")
+  echo "Detected $CORES cores from cgroups v1"
+else
+  CORES=$(getconf _NPROCESSORS_ONLN)
+fi
 # Reduce the concurrency for local tests to half the number of cores in the system.
-CORES=$(($(getconf _NPROCESSORS_ONLN) / 2))
-export NUM_CONCURRENT_TESTS="${NUM_CONCURRENT_TESTS-${CORES}}"
+export NUM_CONCURRENT_TESTS="${NUM_CONCURRENT_TESTS-$((CORES / 2))}"
 
 export KUDU_MASTER_HOSTS="${KUDU_MASTER_HOSTS:-${INTERNAL_LISTEN_HOST}}"
 export KUDU_MASTER_PORT="${KUDU_MASTER_PORT:-7051}"
@@ -945,7 +1018,13 @@ HADOOP_CLASSPATH="${HADOOP_CLASSPATH}:${HADOOP_HOME}/share/hadoop/tools/lib/*"
 
 export PATH="$HADOOP_HOME/bin:$PATH"
 
-export RANGER_HOME="${CDP_COMPONENTS_HOME}/ranger-${IMPALA_RANGER_VERSION}-admin"
+if $USE_APACHE_RANGER; then
+  export RANGER_HOME=${RANGER_HOME_OVERRIDE:-\
+"${APACHE_COMPONENTS_HOME}/ranger-${IMPALA_RANGER_VERSION}-admin"}
+else
+  export RANGER_HOME=\
+${RANGER_HOME_OVERRIDE:-"${CDP_COMPONENTS_HOME}/ranger-${IMPALA_RANGER_VERSION}-admin"}
+fi
 export RANGER_CONF_DIR="$IMPALA_HOME/fe/src/test/resources"
 
 # To configure Hive logging, there's a hive-log4j2.properties[.template]
@@ -971,6 +1050,13 @@ export HIVE_CONF_DIR="$IMPALA_FE_DIR/./src/test/resources"
 export POSTGRES_JDBC_DRIVER="${IMPALA_FE_DIR}/target/dependency/postgresql-${IMPALA_POSTGRES_JDBC_DRIVER_VERSION}.jar"
 
 export HIVE_AUX_JARS_PATH="$POSTGRES_JDBC_DRIVER"
+# Add the jar of iceberg-hive-runtime to have HiveIcebergStorageHandler.
+# Only needed by Apache Hive3 since CDP Hive3 has the jar of hive-iceberg-handler in its
+# lib folder.
+if $USE_APACHE_HIVE; then
+  export HIVE_AUX_JARS_PATH="$HIVE_AUX_JARS_PATH:\
+$IMPALA_HOME/fe/target/dependency/iceberg-hive-runtime-${IMPALA_ICEBERG_VERSION}.jar"
+fi
 export AUX_CLASSPATH=""
 ### Tell hive not to use jline
 export HADOOP_USER_CLASSPATH_FIRST=true
@@ -1029,8 +1115,37 @@ export ASAN_SYMBOLIZER_PATH="${IMPALA_TOOLCHAIN_PACKAGES_HOME}/llvm-${IMPALA_LLV
 
 export CLUSTER_DIR="${IMPALA_HOME}/testdata/cluster"
 
-# The number of parallel build processes we should run at a time.
-export IMPALA_BUILD_THREADS=${IMPALA_BUILD_THREADS-"$(nproc)"}
+# The number of parallel build processes we should run at a time. Require 2GB memory per
+# core as too many compilation processes can exhaust available memory and fail a build.
+if $IS_OSX; then
+  AVAILABLE_MEM=$(($(sysctl -n hw.memsize) / 1024 / 1024 / 1024))
+else
+  # MemTotal:       65550228 kB
+  AVAILABLE_MEM=$(awk '/MemTotal/{print int($2/1024/1024)}' /proc/meminfo)
+fi
+if grep -v max /sys/fs/cgroup/memory.max >& /dev/null; then
+  # Get memory limits under cgroups v2
+  CGROUP_MEM_LIMIT=$(($(cat /sys/fs/cgroup/memory.max) / 1024 / 1024 / 1024))
+  echo "Detected $CGROUP_MEM_LIMIT GB memory limit from cgroups v2"
+elif grep -v '\-' /sys/fs/cgroup/memory/memory.limit_in_bytes >& /dev/null; then
+  # Get memory limits under cgroups v1
+  CGROUP_MEM_LIMIT=$((
+      $(cat /sys/fs/cgroup/memory/memory.limit_in_bytes) / 1024 / 1024 / 1024))
+  echo "Detected $CGROUP_MEM_LIMIT GB memory limit from cgroups v1"
+else
+  CGROUP_MEM_LIMIT=8589934591 # max int64 bytes in GB
+fi
+AVAILABLE_MEM=$((AVAILABLE_MEM > $CGROUP_MEM_LIMIT ? $CGROUP_MEM_LIMIT : $AVAILABLE_MEM))
+BOUNDED_CONCURRENCY=$((AVAILABLE_MEM / 2))
+if [[ $AVAILABLE_MEM -lt 2 ]]; then
+  echo "Insufficient memory ($AVAILABLE_MEM GB) to build Impala"
+  exit 1
+elif [[ $BOUNDED_CONCURRENCY -lt $CORES ]]; then
+  echo "Bounding concurrency for available memory ($AVAILABLE_MEM GB)"
+else
+  BOUNDED_CONCURRENCY=$CORES
+fi
+export IMPALA_BUILD_THREADS=${IMPALA_BUILD_THREADS-"${BOUNDED_CONCURRENCY}"}
 
 # Additional flags to pass to make or ninja.
 export IMPALA_MAKE_FLAGS=${IMPALA_MAKE_FLAGS-}
@@ -1091,7 +1206,7 @@ echo "IMPALA_MAVEN_OPTIONS    = $IMPALA_MAVEN_OPTIONS"
 echo "IMPALA_TOOLCHAIN_HOST   = $IMPALA_TOOLCHAIN_HOST"
 echo "CDP_BUILD_NUMBER        = $CDP_BUILD_NUMBER"
 echo "CDP_COMPONENTS_HOME     = $CDP_COMPONENTS_HOME"
-if $USE_APACHE_HIVE; then
+if $USE_APACHE_COMPONENTS; then
   echo "APACHE_MIRROR           = $APACHE_MIRROR"
   echo "APACHE_COMPONENTS_HOME  = $APACHE_COMPONENTS_HOME"
 fi
@@ -1099,6 +1214,7 @@ echo "IMPALA_HADOOP_VERSION   = $IMPALA_HADOOP_VERSION"
 echo "IMPALA_AVRO_JAVA_VERSION= $IMPALA_AVRO_JAVA_VERSION"
 echo "IMPALA_PARQUET_VERSION  = $IMPALA_PARQUET_VERSION"
 echo "IMPALA_HIVE_VERSION     = $IMPALA_HIVE_VERSION"
+echo "IMPALA_TEZ_VERSION      = $IMPALA_TEZ_VERSION"
 echo "IMPALA_HBASE_VERSION    = $IMPALA_HBASE_VERSION"
 echo "IMPALA_OZONE_VERSION    = $IMPALA_OZONE_VERSION"
 echo "IMPALA_HUDI_VERSION     = $IMPALA_HUDI_VERSION"
@@ -1109,6 +1225,8 @@ echo "IMPALA_COS_VERSION      = $IMPALA_COS_VERSION"
 echo "IMPALA_OBS_VERSION      = $IMPALA_OBS_VERSION"
 echo "IMPALA_SYSTEM_PYTHON2   = $IMPALA_SYSTEM_PYTHON2"
 echo "IMPALA_SYSTEM_PYTHON3   = $IMPALA_SYSTEM_PYTHON3"
+echo "IMPALA_BUILD_THREADS    = $IMPALA_BUILD_THREADS"
+echo "NUM_CONCURRENT_TESTS    = $NUM_CONCURRENT_TESTS"
 
 # Kerberos things.  If the cluster exists and is kerberized, source
 # the required environment.  This is required for any hadoop tool to
