@@ -50,6 +50,15 @@ done
 # Note that this uses the external system python executable
 PYTHONPATH=${PYTHONPATH} python "${IMPALA_PY_DIR}/bootstrap_virtualenv.py"
 
+# Enable remote debugging if port was specified via environment variable
+if [[ ${IMPALA_SHELL_DEBUG_PORT:-0} -ne 0 ]]; then
+  echo "installing debugpy if needed"
+  ${IMPALA_PY_ENV_DIR}/bin/pip install debugpy
+  echo "impala python shell waiting for remote debugging connection on port" \
+       "${IMPALA_SHELL_DEBUG_PORT}"
+  EXTRA_ARGS=" -m debugpy --listen ${IMPALA_SHELL_DEBUG_PORT} --wait-for-client"
+fi
+
 # This uses the python executable in the impala python env
 PYTHONIOENCODING='utf-8' PYTHONPATH=${PYTHONPATH} \
-  exec "${IMPALA_PYTHON_EXECUTABLE}" ${SHELL_HOME}/impala_shell.py "$@"
+  exec "${IMPALA_PYTHON_EXECUTABLE}" ${EXTRA_ARGS:-} ${SHELL_HOME}/impala_shell.py "$@"
