@@ -81,6 +81,12 @@ fi
 : ${DATA_CACHE_EVICTION_POLICY:=}
 # Number of data cache async write threads.
 : ${DATA_CACHE_NUM_ASYNC_WRITE_THREADS:=}
+# Tuple cache root directory location.
+: ${TUPLE_CACHE_DIR:=}
+# Tuple cache capacity.
+: ${TUPLE_CACHE_CAPACITY:=}
+# Tuple cache debug dump directory location.
+: ${TUPLE_CACHE_DEBUG_DUMP_DIR:=}
 if [[ "${TARGET_FILESYSTEM}" == "local" ]]; then
   # TODO: Remove abort_on_config_error flag from here and create-load-data.sh once
   # checkConfiguration() accepts the local filesystem (see IMPALA-1850).
@@ -110,6 +116,18 @@ if [[ -n "${DATA_CACHE_DIR}" && -n "${DATA_CACHE_SIZE}" ]]; then
    fi
 fi
 
+# Enable tuple cache if configured.
+if [[ -n "${TUPLE_CACHE_DIR}" && -n "${TUPLE_CACHE_CAPACITY}" ]]; then
+   TEST_START_CLUSTER_ARGS="${TEST_START_CLUSTER_ARGS} "`
+       `"--tuple_cache_dir=${TUPLE_CACHE_DIR} "
+   TEST_START_CLUSTER_ARGS="${TEST_START_CLUSTER_ARGS} "`
+       `"--tuple_cache_capacity=${TUPLE_CACHE_CAPACITY} "
+   if [[ -n "${TUPLE_CACHE_DEBUG_DUMP_DIR}" ]]; then
+       TEST_START_CLUSTER_ARGS="${TEST_START_CLUSTER_ARGS} "`
+           `"--tuple_cache_debug_dump_dir=${TUPLE_CACHE_DEBUG_DUMP_DIR}"
+   fi
+fi
+
 if [[ "${ERASURE_CODING}" = true ]]; then
   # We do not run FE tests when erasure coding is enabled because planner tests
   # would fail.
@@ -175,7 +193,8 @@ TIMEOUT_PID=$!
 COMMON_PYTEST_ARGS="--maxfail=${MAX_PYTEST_FAILURES} --exploration_strategy=core"`
     `" --workload_exploration_strategy="`
         `"functional-query:${EXPLORATION_STRATEGY},"`
-        `"targeted-stress:${EXPLORATION_STRATEGY}"
+        `"targeted-stress:${EXPLORATION_STRATEGY},"`
+        `"tpcds-insert:${EXPLORATION_STRATEGY}"
 if [[ "${EXPLORATION_STRATEGY}" == "core" ]]; then
   # Skip the stress test in core - all stress tests are in exhaustive and
   # pytest startup takes a significant amount of time.
