@@ -1,4 +1,4 @@
-#!/usr/bin/env impala-python
+#!/usr/bin/env impala-python3
 #
 # Licensed to the Apache Software Foundation (ASF) under one
 # or more contributor license agreements.  See the NOTICE file
@@ -86,8 +86,11 @@ parser.add_option("--force_kill", dest="force_kill", action="store_true", defaul
                   help="Force kill impalad and statestore processes.")
 parser.add_option("-a", "--add_executors", dest="add_executors",
                   action="store_true", default=False,
-                  help="Start additional impalad processes. The executor group name must "
-                  "be specified using --impalad_args")
+                  help="Start additional executors. The executor group name must be"
+                  "specified using --impalad_args")
+parser.add_option("--add_impalads", dest="add_impalads",
+                  action="store_true", default=False,
+                  help="Start additional impalad processes.")
 parser.add_option("-r", "--restart_impalad_only", dest="restart_impalad_only",
                   action="store_true", default=False,
                   help="Restarts only the impalad processes")
@@ -163,7 +166,7 @@ parser.add_option("--enable_catalogd_ha", dest="enable_catalogd_ha",
                   help="If true, enables CatalogD HA - the cluster will be launched "
                   "with two catalogd instances as Active-Passive HA pair.")
 parser.add_option("--jni_frontend_class", dest="jni_frontend_class",
-                  action="store", default="org/apache/impala/service/JniFrontend",
+                  action="store", default="",
                   help="Use a custom java frontend interface.")
 parser.add_option("--enable_statestored_ha", dest="enable_statestored_ha",
                   action="store_true", default=False,
@@ -184,9 +187,17 @@ parser.add_option("--tuple_cache_capacity", dest="tuple_cache_capacity",
                   default=os.environ.get("TUPLE_CACHE_CAPACITY", "1GB"),
                   help="This specifies the maximum storage usage of the tuple cache "
                        "each Impala daemon can use.")
+parser.add_option("--tuple_cache_debug_dump_dir", dest="tuple_cache_debug_dump_dir",
+                  default=os.environ.get("TUPLE_CACHE_DEBUG_DUMP_DIR", None),
+                  help="Specifies a base directory for the dumping tuple cache files "
+                       "for debug purposes")
 parser.add_option("--tuple_cache_eviction_policy", dest="tuple_cache_eviction_policy",
                   default="LRU", help="This specifies the cache eviction policy to use "
                   "for the tuple cache.")
+parser.add_option("--use_calcite_planner", default="False", type="choice",
+                  choices=["true", "True", "false", "False"],
+                  help="If true, use the Calcite planner for query optimization "
+                  "instead of the Impala planner")
 
 # For testing: list of comma-separated delays, in milliseconds, that delay impalad catalog
 # replica initialization. The ith delay is applied to the ith impalad.
@@ -530,7 +541,7 @@ def build_impalad_arg_lists(cluster_size, num_coordinators, use_exclusive_coordi
           timeout=DISCONNECTED_SESSION_TIMEOUT,
           args=args)
 
-    if i >= num_coordinators:
+    if i - start_idx >= num_coordinators:
       args = "-is_coordinator=false {args}".format(args=args)
     elif use_exclusive_coordinators:
       # Coordinator instance that doesn't execute non-coordinator fragments
@@ -605,6 +616,23 @@ def build_impalad_arg_lists(cluster_size, num_coordinators, use_exclusive_coordi
       args = "-tuple_cache_eviction_policy={policy} {args}".format(
           policy=options.tuple_cache_eviction_policy, args=args)
 
+      if options.tuple_cache_debug_dump_dir:
+        # create the base directory
+        tuple_cache_debug_dump_path = \
+            os.path.join(options.tuple_cache_debug_dump_dir,
+                         "impala-tuplecache-debugdump-{0}".format(str(i)))
+        # Try creating the directory if it doesn't exist already. May raise exception.
+        if not os.path.exists(tuple_cache_debug_dump_path):
+          os.makedirs(tuple_cache_debug_dump_path)
+        if options.docker_network is None:
+          tuple_cache_debug_dump_path_arg = tuple_cache_debug_dump_path
+        else:
+          # The cache directory will always be mounted at the same path inside the
+          # container. Reuses the data cache dedicated mount.
+          tuple_cache_debug_dump_path_arg = DATA_CACHE_CONTAINER_PATH
+        args = "-tuple_cache_debug_dump_dir={dir} {args}".format(
+            dir=tuple_cache_debug_dump_path_arg, args=args)
+
     if options.enable_admission_service:
       args = "{args} -admission_service_host={host}".format(
           args=args, host=admissiond_host)
@@ -649,7 +677,7 @@ def build_impalad_arg_lists(cluster_size, num_coordinators, use_exclusive_coordi
       args = "{args} -geospatial_library={geospatial_library}".format(
           args=args, geospatial_library=options.geospatial_library)
 
-    if options.jni_frontend_class:
+    if options.jni_frontend_class != "":
       args = "-jni_frontend_class={jni_frontend_class} {args}".format(
           jni_frontend_class=options.jni_frontend_class, args=args)
 
@@ -658,6 +686,12 @@ def build_impalad_arg_lists(cluster_size, num_coordinators, use_exclusive_coordi
     else:
       args = "-allow_tuple_caching=true {args}".format(args=args)
 
+    if options.use_calcite_planner.lower() == 'true':
+      args = "-jni_frontend_class={jni_frontend_class} {args}".format(
+          jni_frontend_class="org/apache/impala/calcite/service/CalciteJniFrontend",
+          args=args)
+      os.environ["USE_CALCITE_PLANNER"] = "true"
+
     # Appended at the end so they can override previous args.
     if i < len(per_impalad_args):
       args = "{args} {per_impalad_args}".format(
@@ -1080,7 +1114,7 @@ if __name__ == "__main__":
     cluster_ops.kill_all_catalogds(force=options.force_kill)
   elif options.restart_statestored_only:
     cluster_ops.kill_all_statestoreds(force=options.force_kill)
-  elif options.add_executors:
+  elif options.add_executors or options.add_impalads:
     pass
   else:
     cluster_ops.kill_all_daemons(force=options.force_kill)
@@ -1112,6 +1146,11 @@ if __name__ == "__main__":
       cluster_ops.start_impalads(options.cluster_size, num_coordinators,
                                  use_exclusive_coordinators, existing_cluster_size)
       expected_cluster_size += existing_cluster_size
+    elif options.add_impalads:
+      cluster_ops.start_impalads(options.num_coordinators, options.num_coordinators,
+                                 options.use_exclusive_coordinators,
+                                 existing_cluster_size)
+      expected_cluster_size += existing_cluster_size
     else:
       cluster_ops.start_statestore()
       cluster_ops.start_catalogd()
@@ -1120,7 +1159,7 @@ if __name__ == "__main__":
       cluster_ops.start_impalads(options.cluster_size, options.num_coordinators,
                                  options.use_exclusive_coordinators)
     # Sleep briefly to reduce log spam: the cluster takes some time to start up.
-    sleep(3)
+    sleep(2)
 
     impala_cluster = cluster_ops.get_cluster()
     expected_catalog_delays = 0
@@ -1128,8 +1167,14 @@ if __name__ == "__main__":
       for delay in options.catalog_init_delays.split(","):
         if int(delay.strip()) != 0: expected_catalog_delays += 1
     # Check for the cluster to be ready.
-    impala_cluster.wait_until_ready(expected_cluster_size,
-        expected_cluster_size - expected_catalog_delays)
+    expected_num_ready_impalads = expected_cluster_size - expected_catalog_delays
+    if options.add_impalads:
+      # TODO: This is a hack to make the waiting logic work. We'd better add a dedicated
+      # option for adding a new cluster using the existing catalogd and statestore.
+      # https://issues.apache.org/jira/browse/IMPALA-13755
+      expected_num_ready_impalads = options.cluster_size
+      expected_cluster_size = options.cluster_size
+    impala_cluster.wait_until_ready(expected_cluster_size, expected_num_ready_impalads)
   except Exception as e:
     LOG.exception("Error starting cluster")
     sys.exit(1)
