@@ -33,9 +33,11 @@ import com.google.common.collect.Iterables;
 import java.net.InetAddress;
 import java.net.UnknownHostException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -107,6 +109,7 @@ import org.apache.impala.catalog.TableWriteId;
 import org.apache.impala.catalog.events.MetastoreEvents.MetastoreEvent;
 import org.apache.impala.catalog.events.MetastoreEvents.MetastoreEventType;
 import org.apache.impala.catalog.events.MetastoreEventsProcessor;
+import org.apache.impala.catalog.events.MetastoreEventsProcessor.MetaDataFilter;
 import org.apache.impala.catalog.events.MetastoreNotificationException;
 import org.apache.impala.catalog.events.MetastoreNotificationNeedsInvalidateException;
 import org.apache.impala.catalog.events.SelfEventContext;
@@ -831,6 +834,7 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
     private final CommitTxnMessage commitTxnMessage_;
     private final long txnId_;
     private Set<TableWriteId> tableWriteIds_ = Collections.emptySet();
+    private final Set<String> tableNames_ = new HashSet<>();
 
     public CommitTxnEvent(CatalogOpExecutor catalogOpExecutor, Metrics metrics,
         NotificationEvent event) {
@@ -844,6 +848,12 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
           txnId_);
     }
 
+    @Override
+    public String getTargetName() {
+      if (tableNames_.isEmpty()) return CLUSTER_WIDE_TARGET;
+      return tableNames_.stream().sorted().collect(Collectors.joining(","));
+    }
+
     @Override
     protected void process() throws MetastoreNotificationException {
       // To ensure no memory leaking in case an exception is thrown, we remove entries
@@ -905,6 +915,7 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
             getEventId(),
             Collections.singletonList(tableWriteId.getWriteId()),
             MutableValidWriteIdList.WriteIdStatus.COMMITTED);
+        tableNames_.add(tableWriteId.getDbName() + "." + tableWriteId.getTblName());
       }
     }
 
@@ -919,6 +930,7 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
         TableName tableName = new TableName(tbl.getDbName(), tbl.getTableName());
         parts.add(commitTxnMessage_.getPartitionObj(i));
         tableNameToIdxs.computeIfAbsent(tableName, k -> new ArrayList<>()).add(i);
+        tableNames_.add(tableName.toString());
       }
       for (Map.Entry<TableName, List<Integer>> entry : tableNameToIdxs.entrySet()) {
         org.apache.hadoop.hive.metastore.api.Table tbl =
@@ -1082,4 +1094,22 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
     }
     return null;
   }
+
+  public static void setNotificationEventRequestWithFilter(
+      NotificationEventRequest eventRequest, MetaDataFilter metaDataFilter) {
+    if (metaDataFilter != null) {
+      if (metaDataFilter.getCatName() != null &&
+          !metaDataFilter.getCatName().isEmpty()) {
+        eventRequest.setCatName(metaDataFilter.getCatName());
+      }
+      if (metaDataFilter.getDbName() != null &&
+          !metaDataFilter.getDbName().isEmpty()) {
+        eventRequest.setDbName(metaDataFilter.getDbName());
+      }
+      if (metaDataFilter.getTableName() != null &&
+          !metaDataFilter.getTableName().isEmpty()) {
+        eventRequest.setTableNames(Arrays.asList(metaDataFilter.getTableName()));
+      }
+    }
+  }
 }
