@@ -95,6 +95,7 @@ public class AnalysisContext {
     public boolean isQueryStmt() { return stmt_ instanceof QueryStmt; }
     public boolean isSetOperationStmt() { return stmt_ instanceof SetOperationStmt; }
     public boolean isInsertStmt() { return stmt_ instanceof InsertStmt; }
+    public boolean isMergeStmt() { return stmt_ instanceof MergeStmt; }
     public boolean isOptimizeStmt() { return stmt_ instanceof OptimizeStmt; }
     public boolean isDropDbStmt() { return stmt_ instanceof DropDbStmt; }
     public boolean isDropTableOrViewStmt() {
@@ -192,7 +193,8 @@ public class AnalysisContext {
     }
 
     public boolean isDmlStmt() {
-      return isInsertStmt() || isUpdateStmt() || isDeleteStmt() || isOptimizeStmt();
+      return isInsertStmt() || isUpdateStmt() || isDeleteStmt()
+          || isOptimizeStmt() || isMergeStmt();
     }
 
     /**
@@ -202,7 +204,7 @@ public class AnalysisContext {
     public boolean isHierarchicalAuthStmt() {
       return isQueryStmt() || isInsertStmt() || isUpdateStmt() || isDeleteStmt()
           || isCreateTableAsSelectStmt() || isCreateViewStmt() || isAlterViewStmt()
-          || isOptimizeStmt() || isTestCaseStmt();
+          || isOptimizeStmt() || isTestCaseStmt() || isMergeStmt();
     }
 
     /**
@@ -222,6 +224,10 @@ public class AnalysisContext {
       return stmt_ instanceof ConvertTableToIcebergStmt;
     }
 
+    public boolean isKillQueryStmt() {
+      return stmt_ instanceof KillQueryStmt;
+    }
+
     public AlterTableStmt getAlterTableStmt() {
       Preconditions.checkState(isAlterTableStmt());
       return (AlterTableStmt) stmt_;
@@ -307,6 +313,11 @@ public class AnalysisContext {
       return (OptimizeStmt) stmt_;
     }
 
+    public MergeStmt getMergeStmt() {
+      Preconditions.checkState(isMergeStmt());
+      return (MergeStmt) stmt_;
+    }
+
     public InsertStmt getInsertStmt() {
       if (isCreateTableAsSelectStmt()) {
         return getCreateTableAsSelectStmt().getInsertStmt();
@@ -411,6 +422,11 @@ public class AnalysisContext {
       return (ConvertTableToIcebergStmt) stmt_;
     }
 
+    public KillQueryStmt getKillQueryStmt() {
+      Preconditions.checkState(isKillQueryStmt());
+      return (KillQueryStmt) stmt_;
+    }
+
     public StatementBase getStmt() { return stmt_; }
     public Analyzer getAnalyzer() { return analyzer_; }
     public Set<TAccessEvent> getAccessEvents() { return analyzer_.getAccessEvents(); }
@@ -428,7 +444,7 @@ public class AnalysisContext {
     }
     public boolean requiresExprRewrite() {
       return isQueryStmt() || isInsertStmt() || isCreateTableAsSelectStmt()
-          || isUpdateStmt() || isDeleteStmt() || isOptimizeStmt();
+          || isUpdateStmt() || isDeleteStmt() || isOptimizeStmt() || isMergeStmt();
     }
     public boolean requiresSetOperationRewrite() {
       return analyzer_.containsSetOperation() && !isCreateViewStmt() && !isAlterViewStmt()
