@@ -433,7 +433,7 @@ public class CastExpr extends Expr {
    */
   @Override
   public Expr ignoreImplicitCast() {
-    if (isImplicit_) {
+    if (isImplicit()) {
       // we don't expect to see to consecutive implicit casts
       Preconditions.checkState(
           !(getChild(0) instanceof CastExpr) || !((CastExpr) getChild(0)).isImplicit());
@@ -472,4 +472,9 @@ public class CastExpr extends Expr {
 
   @Override
   public Expr clone() { return new CastExpr(this); }
+
+  @Override
+  public boolean recordChildrenInWorkloadManagement() {
+    return true;
+  }
 }
