@@ -24,6 +24,7 @@ import java.util.Collections;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 
 import org.apache.commons.lang3.builder.EqualsBuilder;
 import org.apache.hadoop.hive.metastore.api.FieldSchema;
@@ -383,6 +384,22 @@ public class ColumnDef {
         .isEquals();
   }
 
+  @Override
+  public int hashCode() {
+    return Objects.hash(
+        colName_,
+        comment_,
+        isPrimaryKey_,
+        isPrimaryKeyUnique_,
+        typeDef_,
+        type_,
+        isNullable_,
+        encoding_,
+        compression_,
+        defaultValue_,
+        blockSize_);
+  }
+
   public TColumn toThrift() {
     TColumn col = new TColumn(getColName(), type_.toThrift());
     Integer blockSize =
