@@ -28,6 +28,7 @@ import org.apache.hadoop.hive.metastore.api.SQLPrimaryKey;
 import org.apache.iceberg.TableProperties;
 import org.apache.iceberg.mr.Catalogs;
 import org.apache.impala.authorization.AuthorizationConfig;
+import org.apache.impala.catalog.DataSourceTable;
 import org.apache.impala.catalog.KuduTable;
 import org.apache.impala.catalog.IcebergTable;
 import org.apache.impala.catalog.RowFormat;
@@ -311,6 +312,10 @@ public class CreateTableStmt extends StatementBase {
       }
     }
 
+    if (getFileFormat() == THdfsFileFormat.JDBC) {
+      analyzeJdbcSchema(analyzer);
+    }
+
     // If lineage logging is enabled, compute minimal lineage graph.
     if (BackendConfig.INSTANCE.getComputeLineage() || RuntimeEnv.INSTANCE.isTestEnv()) {
        computeLineageGraph(analyzer);
@@ -885,6 +890,39 @@ public class CreateTableStmt extends StatementBase {
     getPartitionColumnDefs().clear();
   }
 
+  /**
+   * Analyzes the parameters of a CREATE TABLE ... STORED BY JDBC statement. Adds the
+   * table properties of DataSource so that JDBC table is stored as DataSourceTable in
+   * HMS.
+   */
+  private void analyzeJdbcSchema(Analyzer analyzer) throws AnalysisException {
+    if (!isExternal()) {
+      throw new AnalysisException("JDBC table must be created as external table");
+    }
+    for (ColumnDef col: getColumnDefs()) {
+      if (!DataSourceTable.isSupportedColumnType(col.getType())) {
+        throw new AnalysisException("Tables stored by JDBC do not support the column " +
+            "type: " + col.getType());
+      }
+    }
+
+    AnalysisUtils.throwIfNotNull(getCachingOp(),
+        "A JDBC table cannot be cached in HDFS.");
+    AnalysisUtils.throwIfNotNull(getLocation(), "LOCATION cannot be specified for a " +
+        "JDBC table.");
+    AnalysisUtils.throwIfNotEmpty(tableDef_.getPartitionColumnDefs(),
+        "PARTITIONED BY cannot be used in a JDBC table.");
+
+    // Set table properties of the DataSource to make the table saved as DataSourceTable
+    // in HMS.
+    try {
+      DataSourceTable.setJdbcDataSourceProperties(getTblProperties());
+    } catch (ImpalaRuntimeException e) {
+      throw new AnalysisException(String.format(
+          "Cannot create table '%s': %s", getTbl(), e.getMessage()));
+    }
+  }
+
   /**
    * @return true for external tables that don't have "external.table.purge" set to true.
    */
