@@ -32,10 +32,8 @@ import java.util.List;
 import java.util.Set;
 
 import org.apache.commons.collections.CollectionUtils;
-import org.apache.iceberg.types.Types;
 
 import org.apache.impala.authorization.Privilege;
-import org.apache.impala.catalog.BuiltinsDb;
 import org.apache.impala.catalog.Column;
 import org.apache.impala.catalog.FeFsPartition;
 import org.apache.impala.catalog.FeFsTable;
@@ -61,6 +59,7 @@ import org.apache.impala.planner.TableSink;
 import org.apache.impala.rewrite.ExprRewriter;
 import org.apache.impala.thrift.TIcebergPartitionTransformType;
 import org.apache.impala.thrift.TSortingOrder;
+import org.apache.impala.util.ExprUtil;
 import org.apache.impala.util.IcebergUtil;
 
 /**
@@ -464,17 +463,7 @@ public class InsertStmt extends DmlStatementBase {
 
     // We do not support (in|up)serting into tables with unsupported column types.
     for (Column c: table_.getColumns()) {
-      if (!c.getType().isSupported()) {
-        throw new AnalysisException(String.format("Unable to %s into target table " +
-            "(%s) because the column '%s' has an unsupported type '%s'.",
-            getOpName(), targetTableName_, c.getName(), c.getType().toSql()));
-      }
-      if (c.getType().isComplexType()) {
-        throw new AnalysisException(String.format("Unable to %s into target table " +
-            "(%s) because the column '%s' has a complex type '%s' and Impala doesn't " +
-            "support inserting into tables containing complex type columns",
-            getOpName(), targetTableName_, c.getName(), c.getType().toSql()));
-      }
+      checkSupportedColumn(c, getOpName(), targetTableName_);
     }
 
     // Perform operation-specific analysis.
@@ -493,6 +482,21 @@ public class InsertStmt extends DmlStatementBase {
     analyzer.getDescTbl().setTargetTable(table_);
   }
 
+  public static void checkSupportedColumn(Column c, String opName, TableName tableName)
+      throws AnalysisException {
+    if (!c.getType().isSupported()) {
+      throw new AnalysisException(String.format("Unable to %s into target table " +
+              "(%s) because the column '%s' has an unsupported type '%s'.",
+          opName, tableName, c.getName(), c.getType().toSql()));
+    }
+    if (c.getType().isComplexType()) {
+      throw new AnalysisException(String.format("Unable to %s into target table " +
+              "(%s) because the column '%s' has a complex type '%s' and Impala doesn't " +
+              "support inserting into tables containing complex type columns",
+          opName, tableName, c.getName(), c.getType().toSql()));
+    }
+  }
+
   /**
    * Performs INSERT-specific table analysis:
    * - Partition clause is invalid for unpartitioned or HBase tables
@@ -849,6 +853,9 @@ public class InsertStmt extends DmlStatementBase {
       widestTypeExprList = unionStmt.getWidestExprs();
     }
 
+    boolean convertToUtc =
+        isKuduTable && analyzer.getQueryOptions().isWrite_kudu_utc_timestamps();
+
     // Check dynamic partition columns for type compatibility.
     for (int i = 0; i < selectListExprs.size(); ++i) {
       Column targetColumn = selectExprTargetColumns.get(i);
@@ -899,7 +906,12 @@ public class InsertStmt extends DmlStatementBase {
         Column c = table_.getColumns().get(i);
         for (int j = 0; j < tmpPartitionKeyNames.size(); ++j) {
           if (c.getName().equals(tmpPartitionKeyNames.get(j))) {
-            partitionKeyExprs_.add(tmpPartitionKeyExprs.get(j));
+            Expr expr = tmpPartitionKeyExprs.get(j);
+            if (convertToUtc && expr.getType().isTimestamp()) {
+              expr = ExprUtil.toUtcTimestampExpr(
+                  analyzer, expr, true /*expectPreIfNonUnique*/);
+            }
+            partitionKeyExprs_.add(expr);
             partitionColPos_.add(i);
             break;
           }
@@ -938,7 +950,12 @@ public class InsertStmt extends DmlStatementBase {
       boolean matchFound = false;
       for (int i = 0; i < selectListExprs.size(); ++i) {
         if (selectExprTargetColumns.get(i).getName().equals(tblColumn.getName())) {
-          resultExprs_.add(selectListExprs.get(i));
+          Expr expr = selectListExprs.get(i);
+          if (convertToUtc && expr.getType().isTimestamp()) {
+            expr = ExprUtil.toUtcTimestampExpr(
+                analyzer, expr, true /*expectPreIfNonUnique*/);
+          }
+          resultExprs_.add(expr);
           if (isKuduTable) mentionedColumns_.add(col);
           matchFound = true;
           break;
