@@ -18,6 +18,8 @@
 package org.apache.impala.analysis;
 
 import org.apache.impala.catalog.Type;
+import org.apache.impala.catalog.TypeCompatibility;
+import org.apache.impala.common.AnalysisException;
 import org.apache.impala.thrift.TExprNode;
 import org.apache.impala.thrift.TExprNodeType;
 
@@ -71,6 +73,11 @@ public class NullLiteral extends LiteralExpr {
     return this;
   }
 
+  @Override
+  protected Expr uncheckedCastTo(Type targetType, TypeCompatibility compatibility) {
+    return uncheckedCastTo(targetType);
+  }
+
   @Override
   protected void toThrift(TExprNode msg) {
     msg.node_type = TExprNodeType.NULL_LITERAL;
