@@ -18,8 +18,11 @@
 package org.apache.impala.analysis;
 
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
+
+import org.apache.impala.catalog.Column;
 import org.apache.impala.catalog.ColumnStats;
 import org.apache.impala.catalog.FeView;
 import org.apache.impala.common.AnalysisException;
@@ -312,6 +315,15 @@ public class SetOperationStmt extends QueryStmt {
     if (isAnalyzed()) return;
     super.analyze(analyzer);
 
+    final org.apache.impala.thrift.TQueryOptions query_options =
+        analyzer.getQueryCtx().client_request.query_options;
+    if (query_options.values_stmt_avoid_lossy_char_padding
+        && query_options.allow_unsafe_casts) {
+      throw new AnalysisException("Query options ALLOW_UNSAFE_CASTS and " +
+          "VALUES_STMT_AVOID_LOSSY_CHAR_PADDING are not allowed to be set at the same " +
+          "time if the query contains set operation(s).");
+    }
+
     // Propagates DISTINCT from right to left.
     propagateDistinct();
 
@@ -340,7 +352,8 @@ public class SetOperationStmt extends QueryStmt {
     for (SetOperand op : operands_) {
       resultExprLists.add(op.getQueryStmt().getResultExprs());
     }
-    widestExprs_ = analyzer.castToSetOpCompatibleTypes(resultExprLists);
+    widestExprs_ = analyzer.castToSetOpCompatibleTypes(resultExprLists,
+        shouldAvoidLossyCharPadding(analyzer));
     // TODO (IMPALA-11018): Currently only UNION ALL is supported for collection types
     //       due to missing handling in BE.
     if (!hasOnlyUnionAllOps()) {
@@ -350,7 +363,6 @@ public class SetOperationStmt extends QueryStmt {
       }
     }
 
-
     // Create tuple descriptor materialized by this UnionStmt, its resultExprs, and
     // its sortInfo if necessary.
     createMetadata(analyzer);
@@ -378,6 +390,18 @@ public class SetOperationStmt extends QueryStmt {
     baseTblResultExprs_ = resultExprs_;
   }
 
+  /**
+   * If all values in a column are CHARs but they have different lengths, the common type
+   * will normally be the CHAR type of the greatest length, in which case other CHAR
+   * values are padded; this function decides whether this should be avoided by using
+   * VARCHAR as the common type. See IMPALA-10753.
+   *
+   * The default behaviour is returning false, subclasses can override it.
+   */
+  protected boolean shouldAvoidLossyCharPadding(Analyzer analyzer) {
+    return false;
+  }
+
   /**
    * Analyzes all operands and checks that they return an equal number of exprs.
    * Throws an AnalysisException if that is not the case, or if analyzing
@@ -621,15 +645,28 @@ public class SetOperationStmt extends QueryStmt {
 
     // Compute column stats for the materialized slots from the source exprs.
     List<ColumnStats> columnStats = new ArrayList<>();
+    List<Set<Column>> sourceColumns = new ArrayList<>();
     for (int i = 0; i < operands_.size(); ++i) {
       List<Expr> selectExprs = operands_.get(i).getQueryStmt().getResultExprs();
       for (int j = 0; j < selectExprs.size(); ++j) {
-        ColumnStats statsToAdd = ColumnStats.fromExpr(selectExprs.get(j));
         if (i == 0) {
+          ColumnStats statsToAdd = ColumnStats.fromExpr(selectExprs.get(j));
           columnStats.add(statsToAdd);
+          sourceColumns.add(new HashSet<>());
         } else {
+          ColumnStats statsToAdd = columnStats.get(j).hasNumDistinctValues() ?
+              ColumnStats.fromExpr(selectExprs.get(j), sourceColumns.get(j)) :
+              ColumnStats.fromExpr(selectExprs.get(j));
           columnStats.get(j).add(statsToAdd);
         }
+
+        if (columnStats.get(j).hasNumDistinctValues()) {
+          // Collect expr columns to keep ndv low in later stats addition.
+          SlotRef slotRef = selectExprs.get(j).unwrapSlotRef(false);
+          if (slotRef != null && slotRef.hasDesc()) {
+            slotRef.getDesc().collectColumns(sourceColumns.get(j));
+          }
+        }
       }
     }
 
