@@ -27,8 +27,10 @@ import java.util.Set;
 import org.apache.impala.catalog.Column;
 import org.apache.impala.catalog.ColumnStats;
 import org.apache.impala.catalog.FeKuduTable;
+import org.apache.impala.catalog.IcebergColumn;
 import org.apache.impala.catalog.KuduColumn;
 import org.apache.impala.catalog.Type;
+import org.apache.impala.common.ThriftSerializationCtx;
 import org.apache.impala.thrift.TSlotDescriptor;
 import org.apache.impala.thrift.TVirtualColumnType;
 import org.slf4j.Logger;
@@ -64,7 +66,7 @@ public class SlotDescriptor {
   private boolean isMaterialized_ = false;
 
   // If true, then computeMemLayout() should be recursively called for itemTupleDesc_.
-  private boolean materializeRecursively_ = false;
+  private boolean shouldMaterializeRecursively_ = false;
 
   // if false, this slot cannot be NULL
   // Note: it is still possible that a SlotRef pointing to this descriptor could have a
@@ -137,15 +139,39 @@ public class SlotDescriptor {
     isMaterialized_ = value;
     LOG.trace("Mark slot(sid={}) of tuple(tid={}) as {}materialized",
         id_, parent_.getId(), isMaterialized_ ? "" : "non-");
-    if (materializeRecursively_) {
+    if (shouldMaterializeRecursively_) {
       Preconditions.checkState(itemTupleDesc_ != null);
       itemTupleDesc_.materializeSlots();
     }
   }
-  public boolean isMaterializedRecursively() { return materializeRecursively_; }
-  public void setIsMaterializedRecursively(boolean value) {
-    materializeRecursively_ = value;
+  public boolean shouldMaterializeRecursively() {
+    return shouldMaterializeRecursively_;
   }
+
+  public void setShouldMaterializeRecursively(boolean value) {
+    shouldMaterializeRecursively_ = value;
+  }
+
+  /**
+   * Returns true iff this SlotDescriptor and all its children (if any) are materialized,
+   * recursively.
+   * This is relevant in case of complex types, for example when only a subset of the
+   * fields of a struct is queried - in this case the queried fields are materialized, the
+   * other fields are not materialized, but the struct itself also needs to be marked as
+   * materialized because otherwise no memory layout can be calculated for the fields.
+   */
+  public boolean isFullyMaterialized() {
+    if (!isMaterialized()) return false;
+    if (itemTupleDesc_ != null) {
+      Preconditions.checkState(type_.isComplexType());
+      for (SlotDescriptor childSlotDesc : itemTupleDesc_.getSlots()) {
+        if (!childSlotDesc.isFullyMaterialized()) return false;
+      }
+    }
+
+    return true;
+  }
+
   public boolean getIsNullable() { return isNullable_; }
   public void setIsNullable(boolean value) { isNullable_ = value; }
   public int getByteSize() { return byteSize_; }
@@ -170,10 +196,13 @@ public class SlotDescriptor {
     type_ = path_.destType();
     label_ = Joiner.on(".").join(path.getRawPath());
 
-    // Set nullability, if this refers to a KuduColumn.
+    // Set nullability based on column type.
     if (path_.destColumn() instanceof KuduColumn) {
       KuduColumn kuduColumn = (KuduColumn)path_.destColumn();
       isNullable_ = kuduColumn.isNullable();
+    } else if (path_.destColumn() instanceof IcebergColumn) {
+      IcebergColumn icebergColumn = (IcebergColumn)path_.destColumn();
+      isNullable_ = icebergColumn.isNullable();
     }
   }
 
@@ -441,11 +470,12 @@ public class SlotDescriptor {
     return true;
   }
 
-  public TSlotDescriptor toThrift() {
+  public TSlotDescriptor toThrift(ThriftSerializationCtx serialCtx) {
     Preconditions.checkState(isMaterialized_);
     List<Integer> materializedPath = getMaterializedPath();
     TSlotDescriptor result = new TSlotDescriptor(
-        id_.asInt(), parent_.getId().asInt(), type_.toThrift(),
+        serialCtx.translateSlotId(id_).asInt(),
+        serialCtx.translateTupleId(parent_.getId()).asInt(), type_.toThrift(),
         materializedPath, byteOffset_, nullIndicatorByte_, nullIndicatorBit_,
         slotIdx_, getVirtualColumnType());
     if (itemTupleDesc_ != null) {
@@ -456,7 +486,7 @@ public class SlotDescriptor {
       // have such a guarantee.
       Preconditions.checkState(!isScanSlot() ||
           itemTupleDesc_.getId().asInt() > parent_.getId().asInt());
-      result.setItemTupleId(itemTupleDesc_.getId().asInt());
+      result.setItemTupleId(serialCtx.translateTupleId(itemTupleDesc_.getId()).asInt());
     }
     return result;
   }
@@ -481,7 +511,7 @@ public class SlotDescriptor {
         .add("label", label_)
         .add("type", typeStr)
         .add("materialized", isMaterialized_)
-        .add("materializeRecursively", materializeRecursively_)
+        .add("shouldMaterializeRecursively", shouldMaterializeRecursively_)
         .add("byteSize", byteSize_)
         .add("byteOffset", byteOffset_)
         .add("nullable", isNullable_)
