@@ -214,12 +214,6 @@ public class SlotRef extends Expr {
         throw new AnalysisException("Unsupported type '"
             + fieldType.toSql() + "' in '" + toSql() + "'.");
       }
-      if (fieldType.isBinary() && !desc_.getPath().comesFromIcebergMetadataTable()) {
-        // We allow BINARY fields in collections from Iceberg metadata tables but NULL
-        // them out.
-        throw new AnalysisException("Struct containing a BINARY type is not " +
-            "allowed in the select list (IMPALA-11491).");
-      }
 
       if (fieldType.isStructType()) {
         Preconditions.checkState(child instanceof SlotRef);
@@ -334,9 +328,8 @@ public class SlotRef extends Expr {
     msg.node_type = TExprNodeType.SLOT_REF;
     msg.slot_ref = new TSlotRef(serialCtx.translateSlotId(desc_.getId()).asInt());
     // we shouldn't be sending exprs over non-materialized slots
-    Preconditions.checkState(desc_.isMaterialized(), String.format(
-        "Illegal reference to non-materialized slot: tid=%s sid=%s",
-        desc_.getParent().getId(), desc_.getId()));
+    Preconditions.checkState(desc_.isMaterialized(),
+        "Illegal reference to non-materialized slot: %s", this);
     Preconditions.checkState(desc_.getByteOffset() >= 0);
     // check that the tuples associated with this slot are executable
     desc_.getParent().checkIsExecutable();
