@@ -68,6 +68,8 @@ public class StmtRewriter {
       queryStmt = ((UpdateStmt) analysisResult.getStmt()).getQueryStmt();
     } else if (analysisResult.isDeleteStmt()) {
       queryStmt = ((DeleteStmt) analysisResult.getStmt()).getQueryStmt();
+    } else if (analysisResult.isMergeStmt()) {
+      queryStmt = ((MergeStmt) analysisResult.getStmt()).getQueryStmt();
     } else if (analysisResult.isTestCaseStmt()) {
       queryStmt = ((CopyTestCaseStmt) analysisResult.getStmt()).getQueryStmt();
     } else {
