@@ -26,6 +26,7 @@ import com.google.common.collect.Sets;
 import org.apache.impala.catalog.FeDb;
 import org.apache.impala.catalog.FeTable;
 import org.apache.impala.catalog.View;
+import org.apache.impala.catalog.local.LocalView;
 import org.apache.impala.common.RuntimeEnv;
 
 /**
@@ -66,11 +67,18 @@ public class PrivilegeRequestBuilder {
     return this;
   }
 
+  public PrivilegeRequestBuilder onStorageHandlerUri(String storageType,
+      String storageUri) {
+    Preconditions.checkState(authorizable_ == null);
+    authorizable_ = authzFactory_.newStorageHandlerUri(storageType, storageUri);
+    return this;
+  }
+
   /**
-   * Determines whether the given FeTable corresponds to a view whose creation was not
-   * authorized.
+   * Determines whether the given FeTable corresponds to a view that was created by a
+   * non-superuser in HiveMetaStore.
    */
-  public static boolean isViewCreatedWithoutAuthz(FeTable table) {
+  public static boolean isViewCreatedByNonSuperuser(FeTable table) {
     // In the FE test environment, there is no metastore.api.Table object created and
     // thus 'view' would be null. To prevent a NullPointerException from being thrown
     // due to the call to getParameters(), we only check the value of the key
@@ -80,7 +88,11 @@ public class PrivilegeRequestBuilder {
     // to a view not authorized at the creation time. 'table' could be null if it is
     // returned from a method call to Analyzer#getTableNoThrow() and the corresponding
     // FeTable is not currently cached in the local Catalog.
-    if (!(table instanceof View) || RuntimeEnv.INSTANCE.isTestEnv()) {
+    // Note that we need to consider LocalView as well because a view stored in the
+    // catalog daemon would be instantiated as a LocalView if coordinator's local catalog
+    // is enabled.
+    if (!(table instanceof View || table instanceof LocalView) ||
+        RuntimeEnv.INSTANCE.isTestEnv()) {
       return false;
     }
     org.apache.hadoop.hive.metastore.api.Table view = table.getMetaStoreTable();
@@ -97,37 +109,21 @@ public class PrivilegeRequestBuilder {
     Preconditions.checkNotNull(table);
     String dbName = Preconditions.checkNotNull(table.getTableName().getDb());
     String tblName = Preconditions.checkNotNull(table.getTableName().getTbl());
-
-    // TODO(IMPALA-10122): Remove the need for computing 'isViewCreatedWithoutAuthz' once
-    // we can properly process a PrivilegeRequest for a view whose creation was not
-    // authorized.
-    boolean isViewCreatedWithoutAuthz = isViewCreatedWithoutAuthz(table);
-    return onTable(dbName, tblName, table.getOwnerUser(), isViewCreatedWithoutAuthz);
-  }
-
-  /**
-   * Sets the authorizable object to be a table.
-   */
-  public PrivilegeRequestBuilder onTable(
-      String dbName, String tableName, String ownerUser) {
-    return onTable(dbName, tableName, ownerUser, false);
+    return onTable(dbName, tblName, table.getOwnerUser());
   }
 
   /**
    * Sets the authorizable object to be a table. This method is only called by
    * onTable(FeTable table) when 'table' corresponds to a view whose creation was not
    * authorized.
-   * TODO(IMPALA-10122): Remove the argument of 'viewCreatedWithoutAuthz' once we can
-   * properly process a PrivilegeRequest for a view whose creation was not authorized.
    */
   public PrivilegeRequestBuilder onTable(
-      String dbName, String tableName, String ownerUser,
-      boolean viewCreatedWithoutAuthz) {
+      String dbName, String tableName, String ownerUser) {
     Preconditions.checkState(authorizable_ == null);
     // Convert 'dbName' and 'tableName' to lowercase to avoid duplicate audit log entries
     // for the PrivilegeRequest of ALTER.
     authorizable_ = authzFactory_.newTable(dbName.toLowerCase(), tableName.toLowerCase(),
-        ownerUser, viewCreatedWithoutAuthz);
+        ownerUser);
     return this;
   }
 
@@ -136,20 +132,7 @@ public class PrivilegeRequestBuilder {
     // This call path is specifically meant for cases that try to mask the
     // TableNotFound AnalysisExceptions and instead propagate that as an
     // AuthorizationException.
-    return onTableUnknownOwner(dbName, tableName, false);
-  }
-
-  /**
-   * TODO(IMPALA-10122): Remove the argument of 'viewCreatedWithoutAuthz' once we can
-   * properly process a PrivilegeRequest for a view whose creation was not authorized.
-   */
-  public PrivilegeRequestBuilder onTableUnknownOwner(String dbName, String tableName,
-      boolean viewCreatedWithoutAuthz) {
-    // Useful when owner cannot be determined because the table does not exist.
-    // This call path is specifically meant for cases that try to mask the
-    // TableNotFound AnalysisExceptions and instead propagate that as an
-    // AuthorizationException.
-    return onTable(dbName, tableName, null, viewCreatedWithoutAuthz);
+    return onTable(dbName, tableName, null);
   }
 
   /**
@@ -233,6 +216,11 @@ public class PrivilegeRequestBuilder {
     return this;
   }
 
+  public PrivilegeRequestBuilder rwstorage() {
+    privilege_ = Privilege.RWSTORAGE;
+    return this;
+  }
+
   /**
    * Specifies that any privileges are sufficient.
    */
