@@ -20,7 +20,7 @@ package org.apache.impala.authorization.ranger;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.Lists;
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.impala.analysis.AnalysisContext.AnalysisResult;
 import org.apache.impala.authorization.Authorizable;
@@ -496,8 +496,11 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
         .table(tableName)
         .column(columnName)
         .build();
-    RangerAccessRequest req = new RangerAccessRequestImpl(resource,
-        SELECT_ACCESS_TYPE, user.getShortName(), getUserGroups(user));
+    RangerAccessRequestImpl req = new RangerAccessRequestImpl();
+    req.setResource(resource);
+    req.setAccessType(SELECT_ACCESS_TYPE);
+    req.setUser(user.getShortName());
+    req.setUserGroups(getUserGroups(user));
     // The method evalDataMaskPolicies() only checks whether there is a corresponding
     // column masking policy on the Ranger server and thus does not check whether the
     // requesting user/group is granted the necessary privilege on the specified
@@ -517,8 +520,11 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
         .database(dbName)
         .table(tableName)
         .build();
-    RangerAccessRequest req = new RangerAccessRequestImpl(resource,
-        SELECT_ACCESS_TYPE, user.getShortName(), getUserGroups(user));
+    RangerAccessRequestImpl req = new RangerAccessRequestImpl();
+    req.setResource(resource);
+    req.setAccessType(SELECT_ACCESS_TYPE);
+    req.setUser(user.getShortName());
+    req.setUserGroups(getUserGroups(user));
     return plugin_.evalRowFilterPolicies(req, auditHandler);
   }
 
@@ -666,8 +672,11 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
         accessType = privilege.name().toLowerCase();
       }
     }
-    RangerAccessRequestImpl request = new RangerAccessRequestImpl(resource,
-        accessType, user.getShortName(), getUserGroups(user));
+    RangerAccessRequestImpl request = new RangerAccessRequestImpl();
+    request.setResource(resource);
+    request.setAccessType(accessType);
+    request.setUser(user.getShortName());
+    request.setUserGroups(getUserGroups(user));
     request.setClusterName(plugin_.getClusterName());
     if (authzCtx.getSessionState() != null) {
       request.setClientIPAddress(
