@@ -27,10 +27,12 @@ public class RangerImpalaResourceBuilder {
   public static final String DATABASE = "database";
   public static final String TABLE = "table";
   public static final String COLUMN = "column";
+  public static final String COLUMN_SEP = ",";
   public static final String UDF = "udf";
   public static final String URL = "url";
   public static final String STORAGE_TYPE = "storage-type";
   public static final String STORAGE_URL = "storage-url";
+  public static final String WILDCARD_ASTERISK = "*";
 
   private final RangerAccessResourceImpl rangerAccessResource =
       new RangerAccessResourceImpl();
