@@ -19,13 +19,16 @@ package org.apache.impala.authorization.ranger;
 
 import com.google.common.collect.Sets;
 import org.apache.hadoop.security.UserGroupInformation;
+import org.apache.impala.authorization.AuthorizationFactory;
 import org.apache.impala.common.RuntimeEnv;
 import org.apache.impala.service.BackendConfig;
 import org.apache.impala.thrift.TPrivilege;
+import org.apache.ranger.plugin.model.RangerRole;
 
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
+import java.util.function.Predicate;
 
 /**
  * Collection of static functions to support Apache Ranger implementation
@@ -119,4 +122,10 @@ public class RangerUtil {
       throws Exception {
     plugin.getAllRoles(user, null);
   }
+
+  public static boolean roleExists(RangerImpalaPlugin plugin, String roleName) {
+    Set<RangerRole> roleSet = plugin.getRoles().getRangerRoles();
+    if (roleSet == null) return false;
+    return roleSet.stream().anyMatch(r -> r.getName().equals(roleName));
+  }
 }
