@@ -47,7 +47,7 @@ import java.util.concurrent.atomic.AtomicLong;
 import javax.annotation.Nonnull;
 import javax.annotation.Nullable;
 
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.fs.BlockLocation;
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.Path;
@@ -56,6 +56,7 @@ import org.apache.hadoop.hive.metastore.api.StorageDescriptor;
 import org.apache.impala.analysis.Expr;
 import org.apache.impala.analysis.LiteralExpr;
 import org.apache.impala.analysis.PartitionKeyValue;
+import org.apache.impala.catalog.HdfsTable.FileMetadataStats;
 import org.apache.impala.catalog.events.InFlightEvents;
 import org.apache.impala.catalog.events.MetastoreEvents.MetastoreEventPropertyKey;
 import org.apache.impala.common.FileSystemUtil;
@@ -123,7 +124,7 @@ public class HdfsPartition extends CatalogObjectImpl
       fbFileMetadata_ = null;
     }
 
-    private FileDescriptor(FbFileDesc fileDescData, FbFileMetadata fileMetadata) {
+    public FileDescriptor(FbFileDesc fileDescData, FbFileMetadata fileMetadata) {
       fbFileDescriptor_ = fileDescData;
       fbFileMetadata_ = fileMetadata;
     }
@@ -308,6 +309,10 @@ public class HdfsPartition extends CatalogObjectImpl
       return fbFileDescriptor_.fileBlocks(idx);
     }
 
+    public FbFileDesc getFbFileDescriptor() {
+      return fbFileDescriptor_;
+    }
+
     public FbFileMetadata getFbFileMetadata() {
       return fbFileMetadata_;
     }
@@ -761,9 +766,17 @@ public class HdfsPartition extends CatalogObjectImpl
   // -1 means there is no previous refresh event happened
   private final long lastRefreshEventId_;
 
+  // File statistics corresponding to the encoded file descriptors.
+  private final FileMetadataStats fileMetadataStats_;
+
   /**
-   * Constructor.  Needed for third party extensions that want to use their own builder
-   * to construct the object.
+   * Constructor. Needed for third party extensions that want to use their own builder
+   * to construct the object. A third party extension has to update the field of
+   * 'fileMetadataStats_' if it would like to use this field to get the size of the
+   * partition.
+   *
+   * Note that 'partitionIdCounter_' is not accessible to the subclasses in that its
+   * private.
    */
   protected HdfsPartition(HdfsTable table, long prevId, String partName,
       List<LiteralExpr> partitionKeyValues, HdfsStorageDescriptor fileFormatDescriptor,
@@ -781,9 +794,18 @@ public class HdfsPartition extends CatalogObjectImpl
         isMarkedCached, accessLevel, hmsParameters, cachedMsPartitionDescriptor,
         partitionStats, hasIncrementalStats, numRows, writeId,
         inFlightEvents, /*createEventId=*/-1L, /*lastCompactionId*/-1L,
-        /*lastRefreshEventId*/-1L);
+        /*lastRefreshEventId*/-1L, new HdfsTable.FileMetadataStats());
   }
 
+  /**
+   * Constructor. Needed for third party extensions that want to use their own builder
+   * to construct the object. A third party extension has to update the field of
+   * 'fileMetadataStats_' if it would like to use this field to get the size of the
+   * partition.
+   *
+   * Note that 'partitionIdCounter_' is not accessible to the subclasses in that its
+   * private.
+   */
   protected HdfsPartition(HdfsTable table, long id, long prevId, String partName,
       List<LiteralExpr> partitionKeyValues, HdfsStorageDescriptor fileFormatDescriptor,
       @Nonnull ImmutableList<byte[]> encodedFileDescriptors,
@@ -800,7 +822,35 @@ public class HdfsPartition extends CatalogObjectImpl
         isMarkedCached, accessLevel, hmsParameters, cachedMsPartitionDescriptor,
         partitionStats, hasIncrementalStats, numRows, writeId,
         inFlightEvents, /*createEventId=*/-1L, /*lastCompactionId*/-1L,
-        /*lastRefreshEventId*/-1L);
+        /*lastRefreshEventId*/-1L, new HdfsTable.FileMetadataStats());
+  }
+
+  /**
+   * Constructor. Needed for third party extensions that want to use their own builder
+   * to construct the object. It allows third party extensions to provide their own
+   * FileMetadataStats when instantiating an HdfsPartition.
+   *
+   * Note that 'partitionIdCounter_' is not accessible to the subclasses in that its
+   * private.
+   */
+  protected HdfsPartition(HdfsTable table, long id, long prevId, String partName,
+    List<LiteralExpr> partitionKeyValues, HdfsStorageDescriptor fileFormatDescriptor,
+    @Nonnull ImmutableList<byte[]> encodedFileDescriptors,
+    ImmutableList<byte[]> encodedInsertFileDescriptors,
+    ImmutableList<byte[]> encodedDeleteFileDescriptors,
+    HdfsPartitionLocationCompressor.Location location,
+    boolean isMarkedCached, TAccessLevel accessLevel, Map<String, String> hmsParameters,
+    CachedHmsPartitionDescriptor cachedMsPartitionDescriptor,
+    byte[] partitionStats, boolean hasIncrementalStats, long numRows, long writeId,
+    InFlightEvents inFlightEvents, long createEventId, long lastCompactionId,
+    HdfsTable.FileMetadataStats fileMetadataStats) {
+    this(table, partitionIdCounter_.getAndIncrement(), prevId, partName,
+        partitionKeyValues, fileFormatDescriptor, encodedFileDescriptors,
+        encodedInsertFileDescriptors, encodedDeleteFileDescriptors, location,
+        isMarkedCached, accessLevel, hmsParameters, cachedMsPartitionDescriptor,
+        partitionStats, hasIncrementalStats, numRows, writeId,
+        inFlightEvents, /*createEventId=*/-1L, /*lastCompactionId*/-1L,
+        /*lastRefreshEventId*/-1L, fileMetadataStats);
   }
 
   protected HdfsPartition(HdfsTable table, long id, long prevId, String partName,
@@ -813,7 +863,8 @@ public class HdfsPartition extends CatalogObjectImpl
       CachedHmsPartitionDescriptor cachedMsPartitionDescriptor,
       byte[] partitionStats, boolean hasIncrementalStats, long numRows, long writeId,
       InFlightEvents inFlightEvents, long createEventId, long lastCompactionId,
-      long lastRefreshEventId) {
+      long lastRefreshEventId, FileMetadataStats fileMetadataStats) {
+    Preconditions.checkArgument(fileMetadataStats != null);
     table_ = table;
     id_ = id;
     prevId_ = prevId;
@@ -840,6 +891,7 @@ public class HdfsPartition extends CatalogObjectImpl
     } else {
       partName_ = partName;
     }
+    fileMetadataStats_ = fileMetadataStats;
   }
 
   public long getCreateEventId() { return createEventId_; }
@@ -1059,6 +1111,10 @@ public class HdfsPartition extends CatalogObjectImpl
            encodedDeleteFileDescriptors_.size();
   }
 
+  public FileMetadataStats getFileMetadataStats() {
+    return fileMetadataStats_;
+  }
+
   @Override
   public boolean hasFileDescriptors() {
     return !encodedFileDescriptors_.isEmpty() ||
@@ -1071,8 +1127,11 @@ public class HdfsPartition extends CatalogObjectImpl
 
   public void setPartitionMetadata(TPartialPartitionInfo tPart) {
     // The special "prototype partition" or the only partition of an unpartitioned table
-    // have a null cachedMsPartitionDescriptor.
-    if (cachedMsPartitionDescriptor_ == null) return;
+    // don't have partition metadata.
+    if (id_ == CatalogObjectsConstants.PROTOTYPE_PARTITION_ID
+        || !table_.isPartitioned()) {
+      return;
+    }
     // Don't need to make a copy here since the caller should not modify the parameters.
     tPart.hms_parameters = getParameters();
     tPart.write_id = writeId_;
@@ -1161,6 +1220,9 @@ public class HdfsPartition extends CatalogObjectImpl
 
   @Override
   public long getSize() {
+    if (fileMetadataStats_.numFiles > 0) {
+      return fileMetadataStats_.totalFileBytes;
+    }
     long result = 0;
     for (HdfsPartition.FileDescriptor fileDescriptor: getFileDescriptors()) {
       result += fileDescriptor.getFileLength();
@@ -1294,6 +1356,9 @@ public class HdfsPartition extends CatalogObjectImpl
     private long lastRefreshEventId_ = -1L;
     private InFlightEvents inFlightEvents_ = new InFlightEvents();
 
+    // File statistics for the partition, initialized to all zeros.
+    private FileMetadataStats fileMetadataStats_ = new FileMetadataStats();
+
     @Nullable
     private HdfsPartition oldInstance_ = null;
     // True if we are generating a minimal partition instance for
@@ -1362,7 +1427,7 @@ public class HdfsPartition extends CatalogObjectImpl
           encodedDeleteFileDescriptors_, location_, isMarkedCached_, accessLevel_,
           hmsParameters_, cachedMsPartitionDescriptor_, partitionStats_,
           hasIncrementalStats_, numRows_, writeId_, inFlightEvents_, createEventId_,
-          lastCompactionId_, lastRefreshEventId_);
+          lastCompactionId_, lastRefreshEventId_, fileMetadataStats_);
     }
 
     public Builder setId(long id) {
@@ -1413,7 +1478,7 @@ public class HdfsPartition extends CatalogObjectImpl
         isMarkedCached_ = HdfsCachingUtil.getCacheDirectiveId(
             msPartition.getParameters()) != null;
         numRows_ = FeCatalogUtils.getRowCount(msPartition.getParameters());
-        if (table_.getDebugMetadataScale() > 1.0 && numRows_ > 0) {
+        if (table_.getDebugMetadataScale() > -1.0 && numRows_ > 0) {
           numRows_ *= table_.getDebugMetadataScale();
         }
         hmsParameters_ = msPartition.getParameters();
@@ -1588,6 +1653,7 @@ public class HdfsPartition extends CatalogObjectImpl
     }
 
     public Builder setFileDescriptors(HdfsPartition partition) {
+      fileMetadataStats_.set(partition.getFileMetadataStats());
       encodedFileDescriptors_ = partition.encodedFileDescriptors_;
       encodedInsertFileDescriptors_ = partition.encodedInsertFileDescriptors_;
       encodedDeleteFileDescriptors_ = partition.encodedDeleteFileDescriptors_;
@@ -1615,6 +1681,13 @@ public class HdfsPartition extends CatalogObjectImpl
       return this;
     }
 
+    public void setFileMetadataStats(FileMetadataStats fileMetadataStats) {
+      // The fileMetadataStats will not be shared by more than one HdfsPartition even if
+      // the FileMetadataLoader is reused, because a new FileMetadataStats will be
+      // created each time the file metadata of a partition gets loaded.
+      fileMetadataStats_ = fileMetadataStats;
+    }
+
     public HdfsFileFormat getFileFormat() {
       return fileFormatDescriptor_.getFileFormat();
     }
@@ -1726,13 +1799,25 @@ public class HdfsPartition extends CatalogObjectImpl
       }
 
       if (thriftPartition.isSetFile_desc()) {
-        setFileDescriptors(fdsFromThrift(thriftPartition.getFile_desc()));
+        List<FileDescriptor> fds = fdsFromThrift(thriftPartition.getFile_desc());
+        setFileDescriptors(fds);
+        for (FileDescriptor fd: fds) {
+          fileMetadataStats_.accumulate(fd);
+        }
       }
       if (thriftPartition.isSetInsert_file_desc()) {
-        setInsertFileDescriptors(fdsFromThrift(thriftPartition.getInsert_file_desc()));
+        List<FileDescriptor> fds = fdsFromThrift(thriftPartition.getInsert_file_desc());
+        setInsertFileDescriptors(fds);
+        for (FileDescriptor fd: fds) {
+          fileMetadataStats_.accumulate(fd);
+        }
       }
       if (thriftPartition.isSetDelete_file_desc()) {
-        setDeleteFileDescriptors(fdsFromThrift(thriftPartition.getDelete_file_desc()));
+        List<FileDescriptor> fds = fdsFromThrift(thriftPartition.getDelete_file_desc());
+        setDeleteFileDescriptors(fds);
+        for (FileDescriptor fd: fds) {
+          fileMetadataStats_.accumulate(fd);
+        }
       }
 
       accessLevel_ = thriftPartition.isSetAccess_level() ?
