@@ -73,7 +73,8 @@ public class HdfsStorageDescriptor {
       // TODO: Verify the following Parquet SerDe works with Impala and add
       // support for the new input/output format classes. See IMPALA-4214.
       "org.apache.hadoop.hive.ql.io.parquet.serde.ParquetHiveSerDe", // (parquet)
-      "org.apache.iceberg.mr.hive.HiveIcebergSerDe"); // (iceberg)
+      "org.apache.iceberg.mr.hive.HiveIcebergSerDe", // (iceberg)
+      "org.apache.hadoop.hive.serde2.JsonSerDe");// (json)
 
   private final static Logger LOG = LoggerFactory.getLogger(HdfsStorageDescriptor.class);
 
@@ -227,7 +228,8 @@ public class HdfsStorageDescriptor {
 
     try {
       return INTERNER.intern(new HdfsStorageDescriptor(tblName,
-          HdfsFileFormat.fromJavaClassName(sd.getInputFormat()),
+          HdfsFileFormat.fromJavaClassName(
+              sd.getInputFormat(), sd.getSerdeInfo().getSerializationLib()),
           delimMap.get(serdeConstants.LINE_DELIM),
           delimMap.get(serdeConstants.FIELD_DELIM),
           delimMap.get(serdeConstants.COLLECTION_DELIM),
