@@ -79,6 +79,7 @@ import org.apache.impala.util.HiveMetadataFormatUtils;
 import org.apache.thrift.TException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import org.apache.hadoop.hive.metastore.utils.MetaStoreUtils;
 
 /**
  * Base class for Hive 3 MetastoreShim.
@@ -266,7 +267,7 @@ public class Hive3MetastoreShimBase {
           .put("MANAGED_TABLE", TImpalaTableType.TABLE)
           .put("INDEX_TABLE", TImpalaTableType.TABLE)
           .put("VIRTUAL_VIEW", TImpalaTableType.VIEW)
-          .put("MATERIALIZED_VIEW", TImpalaTableType.VIEW).build();
+          .put("MATERIALIZED_VIEW", TImpalaTableType.MATERIALIZED_VIEW).build();
 
   /**
    * Method which maps Metastore's TableType to Impala's table type. In metastore 2
@@ -287,8 +288,9 @@ public class Hive3MetastoreShimBase {
       case MANAGED_TABLE:
         return TImpalaTableType.TABLE;
       case VIRTUAL_VIEW:
-      case MATERIALIZED_VIEW:
         return TImpalaTableType.VIEW;
+      case MATERIALIZED_VIEW:
+        return TImpalaTableType.MATERIALIZED_VIEW;
       default:
         return defaultTableType;
     }
@@ -296,14 +298,21 @@ public class Hive3MetastoreShimBase {
 
   // hive-3 introduces a catalog object in hive
   // Impala only supports the default catalog of hive
-  private static final String defaultCatalogName_ = MetaStoreUtils
+  private static final String DEFAULT_CATALOG_NAME = MetaStoreUtils
       .getDefaultCatalog(MetastoreConf.newMetastoreConf());
 
   /**
    * Gets the name of the default catalog from metastore configuration.
    */
   public static String getDefaultCatalogName() {
-    return defaultCatalogName_;
+    return DEFAULT_CATALOG_NAME;
+  }
+
+  /**
+   * Returns whether the catalog name is the default catalog.
+   */
+  public static boolean isDefaultCatalog(String catalogName) {
+    return DEFAULT_CATALOG_NAME.equalsIgnoreCase(catalogName);
   }
 
   /**
@@ -822,4 +831,11 @@ public class Hive3MetastoreShimBase {
     return wh.getDefaultTablePath(db, tbl.getTableName().toLowerCase(), isExternal)
         .toString();
   }
+
+  /**
+   * At the Metadata level there are no restrictions on column names.
+   */
+  public static boolean validateColumnName(String name) {
+    return MetaStoreUtils.validateColumnName(name);
+  }
 }
