@@ -22,13 +22,14 @@ import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import org.apache.hadoop.hive.common.StatsSetupConst;
 import org.apache.hadoop.hive.metastore.IMetaStoreClient;
 import org.apache.hadoop.hive.metastore.TableType;
 import org.apache.hadoop.hive.metastore.api.hive_metastoreConstants;
+import org.apache.iceberg.Snapshot;
 import org.apache.iceberg.mr.Catalogs;
 import org.apache.iceberg.mr.InputFormatConfig;
 import org.apache.impala.analysis.IcebergPartitionField;
@@ -36,6 +37,7 @@ import org.apache.impala.analysis.IcebergPartitionSpec;
 import org.apache.impala.analysis.IcebergPartitionTransform;
 import org.apache.impala.catalog.iceberg.GroupedContentFiles;
 import org.apache.impala.common.ImpalaRuntimeException;
+import org.apache.impala.service.BackendConfig;
 import org.apache.impala.thrift.TCatalogObjectType;
 import org.apache.impala.thrift.TCompressionCodec;
 import org.apache.impala.thrift.TGetPartialCatalogObjectRequest;
@@ -55,13 +57,31 @@ import org.apache.impala.thrift.TTableType;
 import org.apache.impala.util.EventSequence;
 import org.apache.impala.util.IcebergSchemaConverter;
 import org.apache.impala.util.IcebergUtil;
-import org.apache.thrift.TException;
 
 /**
  * Representation of an Iceberg table in the catalog cache.
+ *
+ * For an Iceberg table, stats can come from 3 places:
+ * 1. numRows: written by Iceberg
+ * 2. HMS column stats
+ * 3. NDV from Puffin
+ *
+ * If there are Puffin stats for different snapshots, the most recent one will be used for
+ * each column.
+ *
+ * If there are both HMS and Puffin stats for a column, the more recent one
+ * will be used - for HMS stats we use the 'impala.lastComputeStatsTime' table
+ * property, and for Puffin stats we use the snapshot timestamp to determine
+ * which is more recent.
+ *
+ * As Puffin only contains NDV stats, it is possible that at a given point the NDV is from
+ * Puffin but other column stats, e.g. num nulls, come from the HMS and are based on a
+ * much older state of the table.
+ * Note that reading Puffin stats may be disabled by setting the
+ * 'enable_reading_puffin_stats' startup flag or the table property
+ * 'impala.iceberg_read_puffin_stats' to false.
  */
 public class IcebergTable extends Table implements FeIcebergTable {
-
   // Alias to the string key that identifies the storage handler for Iceberg tables.
   public static final String KEY_STORAGE_HANDLER =
       hive_metastoreConstants.META_TABLE_STORAGE;
@@ -90,6 +110,9 @@ public class IcebergTable extends Table implements FeIcebergTable {
   // We use database.table instead if this property not been set in SQL
   public static final String ICEBERG_TABLE_IDENTIFIER = "iceberg.table_identifier";
 
+  public static final String ICEBERG_DISABLE_READING_PUFFIN_STATS =
+      "impala.iceberg_read_puffin_stats";
+
   // Internal Iceberg table property that specifies the absolute path of the current
   // table metadata. This property is only valid for tables in 'hive.catalog'.
   public static final String METADATA_LOCATION = "metadata_location";
@@ -208,6 +231,7 @@ public class IcebergTable extends Table implements FeIcebergTable {
   // The snapshot id cached in the CatalogD, necessary to syncronize the caches.
   private long catalogSnapshotId_ = -1;
 
+  private Map<Integer, IcebergColumn> icebergFieldIdToCol_;
   private Map<String, TIcebergPartitionStats> partitionStats_;
 
   protected IcebergTable(org.apache.hadoop.hive.metastore.api.Table msTable,
@@ -221,6 +245,7 @@ public class IcebergTable extends Table implements FeIcebergTable {
     icebergParquetPlainPageSize_ = Utils.getIcebergParquetPlainPageSize(msTable);
     icebergParquetDictPageSize_ = Utils.getIcebergParquetDictPageSize(msTable);
     hdfsTable_ = new HdfsTable(msTable, db, name, owner);
+    icebergFieldIdToCol_ = new HashMap<>();
   }
 
   /**
@@ -355,6 +380,10 @@ public class IcebergTable extends Table implements FeIcebergTable {
     return partitionStats_;
   }
 
+  public IcebergColumn getColumnByIcebergFieldId(int fieldId) {
+    return icebergFieldIdToCol_.get(fieldId);
+  }
+
   @Override
   public TTable toThrift() {
     TTable table = super.toThrift();
@@ -418,11 +447,15 @@ public class IcebergTable extends Table implements FeIcebergTable {
         hdfsTable_.setCanDataBeOutsideOfTableLocation(
             !Utils.requiresDataFilesInTableLocation(this));
         hdfsTable_.load(reuseMetadata, msClient, msTable_, reason, catalogTimeline);
-        fileStore_ = Utils.loadAllPartition(this, icebergFiles);
+        fileStore_ = new IcebergContentFileStore(this, icebergFiles);
         partitionStats_ = Utils.loadPartitionStats(this, icebergFiles);
         setIcebergTableStats();
         loadAllColumnStats(msClient, catalogTimeline);
+        applyPuffinNdvStats(catalogTimeline);
         setAvroSchema(msClient, msTbl, fileStore_, catalogTimeline);
+
+        // We no longer need to keep Iceberg's content files in memory.
+        hdfsTable_.setIcebergFiles(null);
       } catch (Exception e) {
         throw new IcebergTableLoadingException("Error loading metadata for Iceberg table "
             + icebergTableLocation_, e);
@@ -431,27 +464,100 @@ public class IcebergTable extends Table implements FeIcebergTable {
       }
 
       refreshLastUsedTime();
-
-      // Let's reset the storage descriptors, so we don't update the table unnecessarily.
-      FeIcebergTable.resetIcebergStorageDescriptor(msTable_, msTbl);
-      // Avoid updating HMS if the schema didn't change.
-      if (msTable_.equals(msTbl)) return;
-
-      // Update the table schema in HMS.
-      try {
-        updateTimestampProperty(msTable_, TBL_PROP_LAST_DDL_TIME);
-        msTable_.putToParameters(StatsSetupConst.DO_NOT_UPDATE_STATS,
-            StatsSetupConst.TRUE);
-        msClient.alter_table(msTable_.getDbName(), msTable_.getTableName(), msTable_);
-        catalogTimeline.markEvent("Updated table schema in Metastore");
-      } catch (TException e) {
-        throw new TableLoadingException(e.getMessage());
-      }
     } finally {
       context.stop();
     }
   }
 
+  // Reads NDV stats from Puffin files belonging to the table (if any).
+  //
+  // If there are Puffin stats for different snapshots, the most recent one will be used
+  // for each column.
+  //
+  // If there are both HMS and Puffin stats for a column, the more recent one will be used
+  // - for HMS stats we use the 'impala.lastComputeStatsTime' table property, and for
+  // Puffin stats we use the snapshot timestamp to determine which is more recent.
+  //
+  // Note that even if a value from HMS is overridden here, the new value will not be
+  // written back to HMS.  Other stats, e.g. number of nulls, are not modified as Puffin
+  // stats only contain NDV values.
+  private void applyPuffinNdvStats(EventSequence catalogTimeline) {
+    if (!BackendConfig.INSTANCE.enableReadingPuffinStats()) return;
+    if (!isPuffinStatsReadingEnabledForTable()) return;
+
+    long hmsStatsTimestampMs = getLastComputeStatsTimeMs();
+    Set<Integer> fieldIdsWithHmsStats = collectFieldIdsWithNdvStats();
+
+    Map<Integer, PuffinStatsLoader.PuffinStatsRecord> puffinNdvs =
+        PuffinStatsLoader.loadPuffinStats(icebergApiTable_, getFullName(),
+            hmsStatsTimestampMs, fieldIdsWithHmsStats);
+    for (Map.Entry<Integer, PuffinStatsLoader.PuffinStatsRecord> entry
+        : puffinNdvs.entrySet()) {
+      int fieldId = entry.getKey();
+      long ndv = entry.getValue().ndv;
+      long snapshotId = entry.getValue().snapshotId;
+
+      Snapshot snapshot = icebergApiTable_.snapshot(snapshotId);
+      Preconditions.checkNotNull(snapshot);
+
+      // Don't override a possibly existing HMS stat with an explicitly invalid value.
+      if (ndv >= 0) {
+        IcebergColumn col = getColumnByIcebergFieldId(fieldId);
+        Preconditions.checkNotNull(col);
+        Type colType = col.getType();
+
+        // For some types, e.g. BOOLEAN, HMS does not support NDV stats. We could still
+        // set them here, but it would cause differences between legacy and local catalog
+        // mode: in local catalog mode, the catalog sends the stats in HMS objects, so
+        // NDVs for unsupported types would be lost.
+        if (ColumnStats.supportsNdv(colType)) {
+          // Only use the value from Puffin if it is more recent than the HMS stat value
+          // or if the latter doesn't exist.
+          if (!col.getStats().hasNumDistinctValues()
+              || snapshot.timestampMillis() >= hmsStatsTimestampMs) {
+            col.getStats().setNumDistinctValues(ndv);
+
+            // In local catalog mode, the stats sent from the catalog are those of
+            // 'hdfsTable_', not those of this class.
+            Column hdfsTableCol = hdfsTable_.getColumn(col.getName());
+            Preconditions.checkNotNull(hdfsTableCol);
+            hdfsTableCol.getStats().setNumDistinctValues(ndv);
+          }
+        }
+      }
+    }
+
+    if (!puffinNdvs.isEmpty()) {
+      catalogTimeline.markEvent("Loaded Puffin stats");
+    }
+  }
+
+  private boolean isPuffinStatsReadingEnabledForTable() {
+    String val = msTable_.getParameters().get(ICEBERG_DISABLE_READING_PUFFIN_STATS);
+    if (val == null) return true;
+    return Boolean.parseBoolean(val);
+  }
+
+  private long getLastComputeStatsTimeMs() {
+    String val = msTable_.getParameters().get(Table.TBL_PROP_LAST_COMPUTE_STATS_TIME);
+    try {
+      return Long.parseLong(val) * 1000;
+    } catch (Exception e) {
+      return -1;
+    }
+  }
+
+  private Set<Integer> collectFieldIdsWithNdvStats() {
+    Set<Integer> res = new HashSet<>();
+    for (Column col : colsByPos_) {
+      if (col.getStats().hasNumDistinctValues()) {
+        IcebergColumn iCol = (IcebergColumn) col;
+        res.add(iCol.getFieldId());
+      }
+    }
+    return res;
+  }
+
   /**
    * @throws TableLoadingException when it is unsafe to load the table.
    */
@@ -519,6 +625,7 @@ public class IcebergTable extends Table implements FeIcebergTable {
   public void addColumn(Column col) {
     Preconditions.checkState(col instanceof IcebergColumn);
     IcebergColumn iCol = (IcebergColumn) col;
+    icebergFieldIdToCol_.put(iCol.getFieldId(), iCol);
     colsByPos_.add(iCol);
     colsByName_.put(iCol.getName().toLowerCase(), col);
     ((StructType) type_.getItemType()).addField(
@@ -526,6 +633,12 @@ public class IcebergTable extends Table implements FeIcebergTable {
             iCol.getFieldId()));
   }
 
+  @Override
+  public void clearColumns() {
+    super.clearColumns();
+    icebergFieldIdToCol_.clear();
+  }
+
   private void addVirtualColumns() {
     addVirtualColumn(VirtualColumn.INPUT_FILE_NAME);
     addVirtualColumn(VirtualColumn.FILE_POSITION);
