@@ -23,6 +23,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.stream.Collectors;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
@@ -218,8 +219,11 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   // Test only stats that will be injected in place of stats obtained from HMS.
   protected SideloadTableStats testStats_ = null;
 
-  // Scale factor to multiply table stats with. Only used for testing.
-  protected double testMetadataScale_ = 1.0;
+  // Scale factor to multiply table stats with.
+  // Only used for large-scale planner test simulation (see IMPALA-12726).
+  // Valid value for testing should be > 0.
+  // In non test environment, value should remain unchanged at -1.0.
+  protected double testMetadataScale_ = -1.0;
 
   protected Table(org.apache.hadoop.hive.metastore.api.Table msTable, Db db,
       String name, String owner) {
@@ -479,9 +483,10 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   // stats. This method allows each table type to volunteer the set of columns we should
   // ask the metastore for in loadAllColumnStats().
   protected List<String> getColumnNamesWithHmsStats() {
-    List<String> ret = new ArrayList<>();
-    for (String name: colsByName_.keySet()) ret.add(name);
-    return ret;
+    List<Column> columns = filterColumnsNotStoredInHms(getColumns());
+    return columns.stream()
+        .map(col->col.getName().toLowerCase())
+        .collect(Collectors.toList());
   }
 
   /**
@@ -498,15 +503,18 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
       List<ColumnStatisticsObj> colStats;
 
       // We need to only query those columns which may have stats; asking HMS for other
-      // columns causes loadAllColumnStats() to return nothing.
-      // TODO(todd): this no longer seems to be true - asking for a non-existent column
-      // is just ignored, and the columns that do exist are returned.
+      // columns can cause get_table_statistics_req to throw an exception.
+      // The exact HMS behavior depends on hive.metastore.try.direct.sql, see HIVE-28498.
       List<String> colNames = getColumnNamesWithHmsStats();
 
       try {
         colStats = MetastoreShim.getTableColumnStatistics(client, db_.getName(), name_,
             colNames);
       } catch (Exception e) {
+        // TODO: Catching all exceptions and ignoring it makes it harder to detect errors.
+        //       At least during testing it would be good to treat this as error, but
+        //       it is hard to report this well if the function is called during
+        //       HMS event processing.
         LOG.warn("Could not load column statistics for: " + getFullName(), e);
         return;
       }
@@ -561,7 +569,7 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
    * Factory method that creates a new Table from its Thrift representation.
    * Determines the type of table to create based on the Thrift table provided.
    */
-  public static Table fromThrift(Db parentDb, TTable thriftTable)
+  public static Table fromThrift(Db parentDb, TTable thriftTable, boolean loadedInImpalad)
       throws TableLoadingException {
     CatalogInterners.internFieldsInPlace(thriftTable);
     Table newTable;
@@ -582,6 +590,7 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
           IncompleteTable.createUninitializedTable(parentDb, thriftTable.getTbl_name(),
               tblType, MetadataOp.getTableComment(thriftTable.getMetastore_table()));
     }
+    newTable.storedInImpaladCatalogCache_ = loadedInImpalad;
     try {
       newTable.loadFromThrift(thriftTable);
     } catch (IcebergTableLoadingException e) {
@@ -632,8 +641,6 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
         TAccessLevel.READ_WRITE;
 
     storageMetadataLoadTime_ = thriftTable.getStorage_metadata_load_time_ns();
-
-    storedInImpaladCatalogCache_ = true;
   }
 
   /**
@@ -733,6 +740,7 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   private TCatalogObject toMinimalTCatalogObjectHelper() {
     TCatalogObject catalogObject =
         new TCatalogObject(getCatalogObjectType(), getCatalogVersion());
+    catalogObject.setLast_modified_time_ms(getLastLoadedTimeMs());
     TTable table = new TTable(getDb().getName(), getName());
     table.setTbl_comment(getTableComment());
     catalogObject.setTable(table);
@@ -792,6 +800,7 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
 
     TGetPartialCatalogObjectResponse resp = new TGetPartialCatalogObjectResponse();
     resp.setObject_version_number(getCatalogVersion());
+    resp.setObject_loaded_time_ms(getLastLoadedTimeMs());
     resp.table_info = new TPartialTableInfo();
     resp.table_info.setStorage_metadata_load_time_ns(storageMetadataLoadTime_);
     storageMetadataLoadTime_ = 0;
@@ -904,12 +913,7 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   @Override // FeTable
   public List<Column> getColumnsInHiveOrder() {
     List<Column> columns = Lists.newArrayList(getNonClusteringColumns());
-    if (getMetaStoreTable() != null &&
-        AcidUtils.isFullAcidTable(getMetaStoreTable().getParameters())) {
-      // Remove synthetic "row__id" column.
-      Preconditions.checkState(columns.get(0).getName().equals("row__id"));
-      columns.remove(0);
-    }
+    columns = filterColumnsNotStoredInHms(columns);
     columns.addAll(getClusteringColumns());
     return Collections.unmodifiableList(columns);
   }
