@@ -32,7 +32,9 @@ import java.util.concurrent.atomic.AtomicBoolean;
 
 import org.apache.impala.common.Pair;
 import org.apache.impala.thrift.TTableName;
+import org.apache.impala.util.EventSequence;
 import org.apache.impala.util.HdfsCachingUtil;
+import org.apache.impala.util.NoOpEventSequence;
 import org.apache.log4j.Logger;
 
 import com.google.common.base.Preconditions;
@@ -234,7 +236,7 @@ public class TableLoadingMgr {
    * loads of the same table.
    */
   public LoadRequest loadAsync(final TTableName tblName, final long createdEventId,
-      final String reason)
+      final String reason, final EventSequence catalogTimeline)
       throws DatabaseNotFoundException {
     final Db parentDb = catalog_.getDb(tblName.getDb_name());
     if (parentDb == null) {
@@ -245,7 +247,9 @@ public class TableLoadingMgr {
     FutureTask<Table> tableLoadTask = new FutureTask<Table>(new Callable<Table>() {
         @Override
         public Table call() throws Exception {
-          return tblLoader_.load(parentDb, tblName.table_name, createdEventId, reason);
+          catalogTimeline.markEvent("Start loading table");
+          return tblLoader_.load(parentDb, tblName.table_name, createdEventId, reason,
+              catalogTimeline);
         }});
 
     FutureTask<Table> existingValue = loadingTables_.putIfAbsent(tblName, tableLoadTask);
@@ -331,7 +335,7 @@ public class TableLoadingMgr {
     try {
       Table tbl = catalog_.getTable(tblName.getDb_name(), tblName.getTable_name());
       if (tbl == null || tbl instanceof IncompleteTable || !tbl.isLoaded()) return;
-      catalog_.reloadTable(tbl, reason);
+      catalog_.reloadTable(tbl, reason, NoOpEventSequence.INSTANCE);
     } catch (CatalogException e) {
       LOG.error("Error reloading cached table: ", e);
     }
@@ -365,4 +369,12 @@ public class TableLoadingMgr {
       }
     }
   }
+
+  public int numRemainingItems() {
+    return tableLoadingDeque_.size();
+  }
+
+  public int numLoadsInProgress() {
+    return loadingTables_.size();
+  }
 }
