@@ -23,6 +23,7 @@ import org.apache.iceberg.Schema;
 import org.apache.iceberg.Table;
 import org.apache.iceberg.catalog.TableIdentifier;
 import org.apache.impala.catalog.FeIcebergTable;
+import org.apache.impala.catalog.IcebergTableLoadingException;
 import org.apache.impala.catalog.TableLoadingException;
 import org.apache.impala.common.ImpalaRuntimeException;
 
@@ -57,7 +58,7 @@ public interface IcebergCatalog {
    *     is being used.
    */
    Table loadTable(TableIdentifier tableId, String tableLocation,
-      Map<String, String> properties) throws TableLoadingException;
+      Map<String, String> properties) throws IcebergTableLoadingException;
 
   /**
    * Drops the table from this catalog.
@@ -66,10 +67,33 @@ public interface IcebergCatalog {
    */
   boolean dropTable(FeIcebergTable feTable, boolean purge);
 
+  /**
+   * Drops the table from this catalog using database name and table name.
+   * @param dbName the database name
+   * @param tblName the table name
+   * @param purge whether to drop data/metadata files or not
+   * @return true if table was dropped, false if the table did not exist
+   */
+  boolean dropTable(String dbName, String tblName, boolean purge);
+
   /**
    * Renames Iceberg table.
    * For HadoopTables, Iceberg does not supported 'renameTable' method
    * For HadoopCatalog, Iceberg implement 'renameTable' method with Exception threw
    */
   void renameTable(FeIcebergTable feTable, TableIdentifier newTableId);
+
+  /**
+   * Some of the implemetation methods might be running on native threads as they might
+   * be invoked via JNI. In that case the context class loader for those threads are
+   * null. 'Catalogs' uses JNDI to load the catalog implementations, e.g. HadoopCatalog
+   * or HiveCatalog. JNDI uses the context class loader, but as it is null it falls back
+   * to the bootstrap class loader that doesn't have the Iceberg classes on its classpath.
+   * To avoid ClassNotFoundException we set the context class loader to the class loader
+   * that loaded this class.
+   */
+  default void setContextClassLoader() {
+    if (Thread.currentThread().getContextClassLoader() != null) return;
+    Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
+  }
 }
