@@ -17,18 +17,19 @@
 
 package org.apache.impala.catalog.iceberg;
 
+import java.io.UncheckedIOException;
 import java.lang.NullPointerException;
-import java.util.HashMap;
 import java.util.Map;
+
 import org.apache.iceberg.CatalogProperties;
 import org.apache.iceberg.PartitionSpec;
 import org.apache.iceberg.Schema;
 import org.apache.iceberg.Table;
 import org.apache.iceberg.catalog.TableIdentifier;
 import org.apache.iceberg.exceptions.NoSuchTableException;
-import org.apache.iceberg.exceptions.RuntimeIOException;
 import org.apache.iceberg.hadoop.HadoopCatalog;
 import org.apache.impala.catalog.FeIcebergTable;
+import org.apache.impala.catalog.IcebergTableLoadingException;
 import org.apache.impala.catalog.TableLoadingException;
 import org.apache.impala.common.FileSystemUtil;
 import org.apache.impala.thrift.TIcebergCatalog;
@@ -47,8 +48,9 @@ public class IcebergHadoopCatalog implements IcebergCatalog {
   private HadoopCatalog hadoopCatalog;
 
   public IcebergHadoopCatalog(String catalogLocation) {
+    setContextClassLoader();
     hadoopCatalog = new HadoopCatalog();
-    Map<String, String> props = new HashMap<>();
+    Map<String, String> props = IcebergUtil.composeCatalogProperties();
     props.put(CatalogProperties.WAREHOUSE_LOCATION, catalogLocation);
     hadoopCatalog.setConf(FileSystemUtil.getConfiguration());
     hadoopCatalog.initialize("", props);
@@ -76,7 +78,7 @@ public class IcebergHadoopCatalog implements IcebergCatalog {
 
   @Override
   public Table loadTable(TableIdentifier tableId, String tableLocation,
-      Map<String, String> properties) throws TableLoadingException {
+      Map<String, String> properties) throws IcebergTableLoadingException {
     Preconditions.checkState(tableId != null);
     final int MAX_ATTEMPTS = 5;
     final int SLEEP_MS = 500;
@@ -85,11 +87,11 @@ public class IcebergHadoopCatalog implements IcebergCatalog {
       try {
         return hadoopCatalog.loadTable(tableId);
       } catch (NoSuchTableException e) {
-        throw new TableLoadingException(e.getMessage());
-      } catch (NullPointerException | RuntimeIOException e) {
+        throw new IcebergTableLoadingException(e.getMessage());
+      } catch (NullPointerException | UncheckedIOException e) {
         if (attempt == MAX_ATTEMPTS - 1) {
           // Throw exception on last attempt.
-          throw new TableLoadingException(String.format(
+          throw new IcebergTableLoadingException(String.format(
               "Could not load Iceberg table %s", tableId), (Exception)e);
         }
         LOG.warn("Caught Exception during Iceberg table loading: {}: {}", tableId, e);
@@ -102,7 +104,7 @@ public class IcebergHadoopCatalog implements IcebergCatalog {
       }
     }
     // We shouldn't really get there, but to make the compiler happy:
-    throw new TableLoadingException(
+    throw new IcebergTableLoadingException(
         String.format("Failed to load Iceberg table with id: %s", tableId));
   }
 
@@ -114,6 +116,11 @@ public class IcebergHadoopCatalog implements IcebergCatalog {
     return hadoopCatalog.dropTable(tableId, purge);
   }
 
+  @Override
+  public boolean dropTable(String dbName, String tblName, boolean purge) {
+    return hadoopCatalog.dropTable(TableIdentifier.of(dbName, tblName), purge);
+  }
+
   @Override
   public void renameTable(FeIcebergTable feTable, TableIdentifier newTableId) {
     TableIdentifier oldTableId = IcebergUtil.getIcebergTableIdentifier(feTable);
