@@ -342,6 +342,10 @@ public class CatalogdMetaProvider implements MetaProvider {
   private TUniqueId catalogServiceId_ = Catalog.INITIAL_CATALOG_SERVICE_ID;
   private final Object catalogServiceIdLock_ = new Object();
 
+  /**
+   * Object that is used to synchronize on and signal when the catalog is ready for use.
+   */
+  private final Object catalogReadyNotifier_ = new Object();
 
   /**
    * Cache of authorization policy metadata. Populated from data pushed from the
@@ -410,6 +414,18 @@ public class CatalogdMetaProvider implements MetaProvider {
     return authPolicy_;
   }
 
+  @Override
+  public void waitForIsReady(long timeoutMs) {
+    if (isReady()) return;
+    synchronized (catalogReadyNotifier_) {
+      try {
+        catalogReadyNotifier_.wait(timeoutMs);
+      } catch (InterruptedException e) {
+        // Ignore
+      }
+    }
+  }
+
   @Override
   public boolean isReady() {
     return lastSeenCatalogVersion_.get() > Catalog.INITIAL_CATALOG_VERSION;
@@ -788,7 +804,7 @@ public class CatalogdMetaProvider implements MetaProvider {
                 new ArrayList<>() : resp.table_info.sql_constraints.getForeign_keys();
             return new TableMetaRefImpl(
                 dbName, tableName, resp.table_info.hms_table, resp.object_version_number,
-                new SqlConstraints(primaryKeys, foreignKeys),
+                resp.object_loaded_time_ms, new SqlConstraints(primaryKeys, foreignKeys),
                 resp.table_info.valid_write_ids, resp.table_info.is_marked_cached,
                 resp.table_info.partition_prefixes, resp.table_info.virtual_columns);
            }
@@ -1090,7 +1106,7 @@ public class CatalogdMetaProvider implements MetaProvider {
       } else {
         checkResponse(table.msTable_.getPartitionKeysSize() == 0, req,
             "Should not return a partition with missing partition meta unless " +
-            "the table is unpartitioned");
+            "the table is unpartitioned: %s", part);
         // For the only partition of a nonpartitioned table, reuse table-level metadata.
         try {
           hdfsStorageDescriptor = HdfsStorageDescriptor.fromStorageDescriptor(
@@ -1473,6 +1489,9 @@ public class CatalogdMetaProvider implements MetaProvider {
     // the update.
     if (nextCatalogVersion != null) {
       lastSeenCatalogVersion_.set(nextCatalogVersion);
+      synchronized (catalogReadyNotifier_) {
+        catalogReadyNotifier_.notifyAll();
+      }
     }
 
     // NOTE: the return value is ignored when this function is called by a DDL
@@ -1893,6 +1912,12 @@ public class CatalogdMetaProvider implements MetaProvider {
      */
     private final long catalogVersion_;
 
+    /**
+     * The timestamp when the table is loaded in catalogd corresponding to the catalog
+     * version.
+     */
+    private final long loadedTimeMs_;
+
     /**
      * Valid write id list of ACID tables.
      */
@@ -1907,13 +1932,14 @@ public class CatalogdMetaProvider implements MetaProvider {
     private final HdfsPartitionLocationCompressor partitionLocationCompressor_;
 
     public TableMetaRefImpl(String dbName, String tableName,
-        Table msTable, long catalogVersion, SqlConstraints sqlConstraints,
-        TValidWriteIdList validWriteIds, boolean isMarkedCached,
-        List<String> locationPrefixes, List<TColumn> tvirtCols) {
+        Table msTable, long catalogVersion, long loadedTimeMs,
+        SqlConstraints sqlConstraints, TValidWriteIdList validWriteIds,
+        boolean isMarkedCached, List<String> locationPrefixes, List<TColumn> tvirtCols) {
       this.dbName_ = dbName;
       this.tableName_ = tableName;
       this.msTable_ = msTable;
       this.catalogVersion_ = catalogVersion;
+      this.loadedTimeMs_ = loadedTimeMs;
       this.sqlConstraints_ = sqlConstraints;
       this.validWriteIds_ = validWriteIds;
       this.isMarkedCached_ = isMarkedCached;
@@ -1956,6 +1982,12 @@ public class CatalogdMetaProvider implements MetaProvider {
     public List<VirtualColumn> getVirtualColumns() {
       return virtualColumns_;
     }
+
+    @Override
+    public long getCatalogVersion() { return catalogVersion_; }
+
+    @Override
+    public long getLoadedTimeMs() { return loadedTimeMs_; }
   }
 
   /**
