@@ -24,7 +24,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import org.apache.commons.lang.NotImplementedException;
+import org.apache.commons.lang3.NotImplementedException;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.metastore.api.ColumnStatisticsObj;
 import org.apache.hadoop.hive.metastore.api.Database;
@@ -117,6 +117,11 @@ class DirectMetaProvider implements MetaProvider {
     return true;
   }
 
+  @Override
+  public void waitForIsReady(long timeoutMs) {
+    // NOOP
+  }
+
   @Override
   public ImmutableList<String> loadDbList() throws TException {
     try (MetaStoreClient c = msClientPool_.getClient()) {
@@ -543,6 +548,16 @@ class DirectMetaProvider implements MetaProvider {
       throw new UnsupportedOperationException("Virtual columns are not supported with " +
           "DirectMetaProvider implementation");
     }
+
+    @Override
+    public long getCatalogVersion() {
+      return 0;
+    }
+
+    @Override
+    public long getLoadedTimeMs() {
+      return 0;
+    }
   }
 
   @Override
