@@ -42,7 +42,6 @@ import org.slf4j.LoggerFactory;
 
 import com.google.common.base.Preconditions;
 import com.google.common.base.Predicates;
-import com.google.common.collect.Lists;
 import com.google.common.math.IntMath;
 
 /**
@@ -157,6 +156,9 @@ public class DistributedPlanner {
       Preconditions.checkState(childFragments.size() == 2);
       result = createIcebergDeleteFragment((IcebergDeleteNode) root,
           childFragments.get(0), childFragments.get(1));
+    } else if (root instanceof IcebergMergeNode) {
+      childFragments.get(0).addPlanRoot(root);
+      result = childFragments.get(0);
     } else {
       throw new InternalException("Cannot create plan fragment for this node type: "
           + root.getExplainString(ctx_.getQueryOptions()));
@@ -182,13 +184,6 @@ public class DistributedPlanner {
     return exprs.isEmpty() ? 1 : Expr.getNumDistinctValues(exprs);
   }
 
-  public PlanFragment createInsertFragment(
-    PlanFragment inputFragment, DmlStatementBase dmlStmt, Analyzer analyzer,
-      List<PlanFragment> fragments)
-      throws ImpalaException {
-    return createDmlFragment(inputFragment, dmlStmt, analyzer, fragments);
-  }
-
   /**
    * Decides whether to repartition the output of 'inputFragment' before feeding
    * its data into the table sink of the given 'insertStmt'. The decision obeys
@@ -206,12 +201,12 @@ public class DistributedPlanner {
       List<PlanFragment> fragments)
       throws ImpalaException {
     boolean isComputeCost = analyzer.getQueryOptions().isCompute_processing_cost();
-    boolean enforce_hdfs_writer_limit = dmlStmt.getTargetTable() instanceof FeFsTable
+    boolean enforceHdfsWriterLimit = dmlStmt.getTargetTable() instanceof FeFsTable
         && (analyzer.getQueryOptions().getMax_fs_writers() > 0 || isComputeCost);
 
-    if (dmlStmt.hasNoShuffleHint() && !enforce_hdfs_writer_limit) return inputFragment;
+    if (dmlStmt.hasNoShuffleHint() && !enforceHdfsWriterLimit) return inputFragment;
 
-    List<Expr> partitionExprs = Lists.newArrayList(dmlStmt.getPartitionKeyExprs());
+    List<Expr> partitionExprs = new ArrayList<>(dmlStmt.getPartitionKeyExprs());
     // Ignore constants for the sake of partitioning.
     Expr.removeConstants(partitionExprs);
 
@@ -219,49 +214,42 @@ public class DistributedPlanner {
     // Kudu tables here (IMPALA-5254).
     DataPartition inputPartition = inputFragment.getDataPartition();
     if (!partitionExprs.isEmpty()
-        && analyzer.setsHaveValueTransfer(inputPartition.getPartitionExprs(),
-            partitionExprs, true)
+        && analyzer.setsHaveValueTransfer(
+            inputPartition.getPartitionExprs(), partitionExprs, true)
         && !(dmlStmt.getTargetTable() instanceof FeKuduTable)
-        && !enforce_hdfs_writer_limit) {
+        && !enforceHdfsWriterLimit) {
       return inputFragment;
     }
 
+    long numPartitions = getNumDistinctValues(partitionExprs);
     int maxHdfsWriters = analyzer.getQueryOptions().getMax_fs_writers();
     // We also consider fragments containing union nodes along with scan fragments
     // (leaf fragments) since they are either a part of those scan fragments or are
     // co-located with them to maintain parallelism.
-    List<HdfsScanNode> hdfsScanNodes = Lists.newArrayList();
+    List<HdfsScanNode> hdfsScanNodes = new ArrayList<>();
     inputFragment.collectPlanNodes(
         Predicates.instanceOf(HdfsScanNode.class), hdfsScanNodes);
-    List<UnionNode> unionNodes = Lists.newArrayList();
+    List<UnionNode> unionNodes = new ArrayList<>();
     inputFragment.collectPlanNodes(Predicates.instanceOf(UnionNode.class), unionNodes);
     boolean hasHdfsScanORUnion = !hdfsScanNodes.isEmpty() || !unionNodes.isEmpty();
 
     int expectedNumInputInstance = inputFragment.getNumInstances();
-    if (enforce_hdfs_writer_limit && isComputeCost) {
+    if (enforceHdfsWriterLimit && isComputeCost) {
       // Default to minParallelism * numNodes if cardinality or average row size is
       // unknown.
-      int costBasedMaxWriter = IntMath.saturatedMultiply(
+      int minInstances = IntMath.saturatedMultiply(
           inputFragment.getNumNodes(), analyzer.getMinParallelismPerNode());
+      int maxInstances = IntMath.saturatedMultiply(
+          inputFragment.getNumNodes(), analyzer.getMaxParallelismPerNode());
+      int costBasedMaxWriter = minInstances;
 
+      boolean isPartitioned = !partitionExprs.isEmpty();
       PlanNode root = inputFragment.getPlanRoot();
       if (root.getCardinality() > -1 && root.getAvgRowSize() > -1) {
         // Both cardinality and avg row size is known.
-        // Estimate such that each writer will work on at least MIN_WRITE_BYTES of rows.
-        // However, if this is a partitioned insert, the output volume will be divided
-        // into several partitions. In that case, consider totalNumPartitions so that:
-        // total num writers is close to totalNumPartitions.
-        int totalNumPartitions = (int) Math.min(
-            Integer.MAX_VALUE, Math.max(1, getNumDistinctValues(partitionExprs)));
-        int minNumWriter = Math.min(totalNumPartitions, inputFragment.getNumNodes());
-        int maxNumWriter = Math.min(totalNumPartitions,
-            IntMath.saturatedMultiply(
-                inputFragment.getNumNodes(), analyzer.getMaxParallelismPerNode()));
-        costBasedMaxWriter = (int) Math.round(
-            Math.ceil((root.getAvgRowSize() / HdfsTableSink.MIN_WRITE_BYTES)
-                * root.getCardinality()));
-        costBasedMaxWriter =
-            Math.min(maxNumWriter, Math.max(minNumWriter, costBasedMaxWriter));
+        costBasedMaxWriter = HdfsTableSink.bytesBasedNumWriters(
+            inputFragment.getNumNodes(), maxInstances, isPartitioned, numPartitions,
+            root.getCardinality(), root.getAvgRowSize());
       }
 
       if (maxHdfsWriters > 0) {
@@ -271,14 +259,20 @@ public class DistributedPlanner {
         // User does not set MAX_FS_WRITER option.
         maxHdfsWriters = costBasedMaxWriter;
       }
+      LOG.trace("isPartitioned={} numDistinctPartition={} costBasedMaxWriter={} "
+              + "maxHdfsWriters={} inputCardinality={}",
+          isPartitioned, numPartitions, costBasedMaxWriter, maxHdfsWriters,
+          root.getCardinality());
       Preconditions.checkState(maxHdfsWriters > 0);
       dmlStmt.setMaxTableSinks(maxHdfsWriters);
       // At this point, parallelism of writer fragment is fixed and will not be adjusted
       // by costing phase.
 
       if (!hdfsScanNodes.isEmpty() && fragments.size() == 1) {
-        // If input fragment have HdfsScanNode and input fragment is the only fragment in
-        // the plan, check for opportunity to collocate scan nodes and table sink.
+        // If input fragment have HdfsScanNode, and input fragment is the only fragment in
+        // the plan, and the scan cost is low, expectedNumInputInstance can be lowered
+        // down. This can increase chance to  colocate scan nodes and table sinks
+        // (case 3 below).
         // Since the actual costing phase only happens later after distributed plan
         // created, this code redundantly compute the scan cost ahead of costing phase
         // to help estimate the scan parallelism.
@@ -289,9 +283,8 @@ public class DistributedPlanner {
           maxScanThread = Math.max(
               maxScanThread, scanCost.getNumInstanceMax(inputFragment.getNumNodes()));
         }
-        // Override expectedNumInputInstance so that collocation may happen
-        // (case 3 in branch below).
-        expectedNumInputInstance = maxScanThread;
+        maxScanThread = Math.min(maxInstances, maxScanThread);
+        expectedNumInputInstance = Math.min(maxScanThread, expectedNumInputInstance);
       }
     }
 
@@ -303,7 +296,7 @@ public class DistributedPlanner {
         // TODO: make a more sophisticated decision here for partitioned tables and when
         // we have info about tablet locations.
         if (partitionExprs.isEmpty()) return inputFragment;
-      } else if (!enforce_hdfs_writer_limit || !hasHdfsScanORUnion
+      } else if (!enforceHdfsWriterLimit || !hasHdfsScanORUnion
           || (expectedNumInputInstance <= maxHdfsWriters)) {
         // Only consider skipping the addition of an exchange node if
         // 1. The hdfs writer limit does not apply
@@ -314,21 +307,27 @@ public class DistributedPlanner {
         // Basically covering all cases where we don't mind restricting the parallelism
         // of their instances.
         Preconditions.checkState(
-            expectedNumInputInstance <= inputFragment.getNumInstances());
-        int input_instances = expectedNumInputInstance;
-        if (enforce_hdfs_writer_limit && !hasHdfsScanORUnion) {
+            expectedNumInputInstance <= inputFragment.getNumInstances(),
+            "expectedNumInputInstance (%s) > inputFragment.getNumInstances() (%s)",
+            expectedNumInputInstance, inputFragment.getNumInstances());
+        int inputInstances = expectedNumInputInstance;
+        if (enforceHdfsWriterLimit && !hasHdfsScanORUnion) {
           // For an internal fragment we enforce an upper limit based on the
           // resulting maxHdfsWriters.
           Preconditions.checkState(maxHdfsWriters > 0);
-          input_instances = Math.min(input_instances, maxHdfsWriters);
+          inputInstances = Math.min(inputInstances, maxHdfsWriters);
         }
         // If the existing partition exprs are a subset of the table partition exprs,
         // check if it is distributed across all nodes. If so, don't repartition.
+        // TODO: If input fragment has ScanNode leave(s), its inputPartitions can be
+        // empty and numInputPartitions=1. But the scanned table(s) might be a
+        // partitioned table(s) that superset of partitionExprs. Repartition (shuffling)
+        // might not needed in that case, but skipping repartition can risk producing
+        // small files.
         if (Expr.isSubset(inputPartition.getPartitionExprs(), partitionExprs)) {
-          long numPartitions = getNumDistinctValues(inputPartition.getPartitionExprs());
-          if (numPartitions >= input_instances) {
-            return inputFragment;
-          }
+          long numInputPartitions =
+              getNumDistinctValues(inputPartition.getPartitionExprs());
+          if (numInputPartitions >= inputInstances) { return inputFragment; }
         }
 
         // Don't repartition if we know we have fewer partitions than nodes
@@ -338,9 +337,8 @@ public class DistributedPlanner {
         // size). In order to do that, we need to come up with an estimate of the avg row
         // size in the particular file format of the output table/partition.
         // We should always know on how many nodes our input is running.
-        long numPartitions = getNumDistinctValues(partitionExprs);
         Preconditions.checkState(expectedNumInputInstance != -1);
-        if (numPartitions > 0 && numPartitions <= input_instances) {
+        if (numPartitions > 0 && numPartitions <= inputInstances) {
           return inputFragment;
         }
       }
@@ -352,7 +350,7 @@ public class DistributedPlanner {
     Preconditions.checkState(exchNode.hasValidStats());
     DataPartition partition;
     if (partitionExprs.isEmpty()) {
-      if (enforce_hdfs_writer_limit
+      if (enforceHdfsWriterLimit
           && inputFragment.getDataPartition().getType() == TPartitionType.RANDOM) {
         // This ensures the parallelism of the writers is maintained while maintaining
         // legacy behavior(when not using MAX_FS_WRITER query option).
@@ -1053,13 +1051,21 @@ public class DistributedPlanner {
       node.unsetLimit();
     }
     node.unsetNeedsFinalize();
+    // Move all HAVING predicates to be transferred into mergeAggNode.
+    List<Expr> havingPredicates = node.conjuncts_;
+    node.conjuncts_ = new ArrayList<>();
+    // Recompute stats after setIsPreagg and predicates removal.
+    // Must do this before creating mergeFragment.
+    node.computeStats(ctx_.getRootAnalyzer());
 
-    // place a merge aggregation step in a new fragment
+    // Place the merge aggregation step in a new fragment.
     PlanFragment mergeFragment = createParentFragment(childFragment, parentPartition);
     AggregationNode mergeAggNode = new AggregationNode(ctx_.getNextNodeId(),
         mergeFragment.getPlanRoot(), node.getMultiAggInfo(), AggPhase.FIRST_MERGE);
-    mergeAggNode.init(ctx_.getRootAnalyzer());
     mergeAggNode.setLimit(limit);
+    // Transfer all HAVING predicates into mergeAggNode and init().
+    // init() is called after setLimit() to avoid calling computeStats() again.
+    mergeAggNode.init(ctx_.getRootAnalyzer(), havingPredicates);
     // Carry the IsNonCorrelatedSclarSubquery_ flag to the merge node. This flag is
     // applicable regardless of the partition scheme for the children since it is a
     // logical property.
@@ -1071,12 +1077,6 @@ public class DistributedPlanner {
       mergeAggNode.setDisableCodegen(true);
     }
 
-    // HAVING predicates can only be evaluated after the merge agg step
-    node.transferConjuncts(mergeAggNode);
-    // Recompute stats after transferring the conjuncts_ (order is important).
-    node.computeStats(ctx_.getRootAnalyzer());
-    mergeFragment.getPlanRoot().computeStats(ctx_.getRootAnalyzer());
-    mergeAggNode.computeStats(ctx_.getRootAnalyzer());
     // Set new plan root after updating stats.
     mergeFragment.addPlanRoot(mergeAggNode);
 
@@ -1121,6 +1121,9 @@ public class DistributedPlanner {
     } else {
       phase1AggNode.setIntermediateTuple();
       phase1AggNode.setIsPreagg(ctx_);
+      // Recompute stats after setIsPreagg.
+      // Must do this before creating firstMergeFragment.
+      phase1AggNode.computeStats(ctx_.getRootAnalyzer());
 
       DataPartition phase1MergePartition =
           DataPartition.hashPartitioned(phase1PartitionExprs);
@@ -1157,15 +1160,23 @@ public class DistributedPlanner {
       phase2AggNode.setIsPreagg(ctx_);
       phase2MergePartition = DataPartition.hashPartitioned(phase2PartitionExprs);
     }
+
+    // Move all HAVING predicates to be transferred into phase2MergeAggNode.
+    List<Expr> havingPredicates = phase2AggNode.conjuncts_;
+    phase2AggNode.conjuncts_ = new ArrayList<>();
+    // Recompute stats after setIsPreagg and predicates removal.
+    // Must do this before creating phase2MergeAggNode.
+    phase2AggNode.computeStats(ctx_.getRootAnalyzer());
+
+    // Place the merge aggregation step in a new fragment.
     PlanFragment secondMergeFragment =
         createParentFragment(firstMergeFragment, phase2MergePartition);
-
     AggregationNode phase2MergeAggNode = new AggregationNode(ctx_.getNextNodeId(),
         phase2AggNode, phase2AggNode.getMultiAggInfo(), AggPhase.SECOND_MERGE);
-    phase2MergeAggNode.init(ctx_.getRootAnalyzer());
     phase2MergeAggNode.setLimit(limit);
-    // Transfer having predicates to final merge agg node
-    phase2AggNode.transferConjuncts(phase2MergeAggNode);
+    // Transfer all HAVING predicates into phase2MergeAggNode and init().
+    // init() is called after setLimit() to avoid calling computeStats() again.
+    phase2MergeAggNode.init(ctx_.getRootAnalyzer(), havingPredicates);
     secondMergeFragment.addPlanRoot(phase2MergeAggNode);
     return secondMergeFragment;
   }
