@@ -176,7 +176,11 @@ public class IcebergScanPlanner {
 
   private void setFileDescriptorsBasedOnFileStore() throws ImpalaException {
     IcebergContentFileStore fileStore = getIceTable().getContentFileStore();
-    dataFilesWithoutDeletes_ = fileStore.getDataFilesWithoutDeletes();
+    if (tblRef_.getSelectedDataFilesForOptimize() != null) {
+      dataFilesWithoutDeletes_ = tblRef_.getSelectedDataFilesForOptimize();
+    } else {
+      dataFilesWithoutDeletes_ = fileStore.getDataFilesWithoutDeletes();
+    }
     dataFilesWithDeletes_ = fileStore.getDataFilesWithDeletes();
     positionDeleteFiles_ = new HashSet<>(fileStore.getPositionDeleteFiles());
     initEqualityIds(fileStore.getEqualityDeleteFiles());
@@ -228,6 +232,10 @@ public class IcebergScanPlanner {
         outputExprs, false);
     unionNode.addChild(dataScanNode, outputExprs);
     unionNode.addChild(joinNode, outputExprs);
+    // One of joinNode's ScanNode child is already associated with tblRef_.getId().
+    // Unset previous mapping so that unionNode will be acknowledged as the first
+    // producer node.
+    analyzer_.unsetProducingNode(tblRef_.getId());
     unionNode.init(analyzer_);
     // Verify that the children are passed through.
     Preconditions.checkState(unionNode.getChildCount() == 2);
@@ -588,6 +596,8 @@ public class IcebergScanPlanner {
 
   private void addEqualityDeletesAndIds(FileDescriptor fd) {
     FbIcebergMetadata fileMetadata = fd.getFbFileMetadata().icebergMetadata();
+    Preconditions.checkState(fileMetadata.equalityFieldIdsLength() > 0,
+        "Equality delete file doesn't have equality field IDs: " + fd.toString());
     List<Integer> eqFieldIdList = new ArrayList<>();
     for (int i = 0; i < fileMetadata.equalityFieldIdsLength(); ++i) {
       eqFieldIdList.add(fileMetadata.equalityFieldIds(i));
