@@ -47,6 +47,7 @@ import org.apache.impala.common.PrintUtils;
 import org.apache.impala.common.ThriftSerializationCtx;
 import org.apache.impala.common.TreeNode;
 import org.apache.impala.planner.RuntimeFilterGenerator.RuntimeFilter;
+import org.apache.impala.planner.TupleCacheInfo.IneligibilityReason;
 import org.apache.impala.thrift.TExecNodePhase;
 import org.apache.impala.thrift.TExecStats;
 import org.apache.impala.thrift.TExplainLevel;
@@ -54,6 +55,7 @@ import org.apache.impala.thrift.TPlan;
 import org.apache.impala.thrift.TPlanNode;
 import org.apache.impala.thrift.TQueryOptions;
 import org.apache.impala.thrift.TSortingOrder;
+import org.apache.impala.thrift.TQueryOptionsHash;
 import org.apache.impala.util.BitUtil;
 import org.apache.impala.util.ExprUtil;
 import org.slf4j.Logger;
@@ -98,7 +100,7 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
   // unique w/in plan tree; assigned by planner, and not necessarily in c'tor
   protected PlanNodeId id_;
 
-  protected long limit_; // max. # of rows to be returned; 0: no limit_
+  protected long limit_; // max. # of rows to be returned; -1: no limit_
 
   // ids materialized by the tree rooted at this node
   protected List<TupleId> tupleIds_;
@@ -165,6 +167,9 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
   // Set in computeStats().
   protected float rowPadSize_;
 
+  // Row size without varlen data.
+  protected long getFixedLenRowSize_;
+
   // If true, disable codegen for this plan node.
   protected boolean disableCodegen_;
 
@@ -258,6 +263,8 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
   public float getAvgRowSizeWithoutPad() {
     return Math.max(0.0F, avgRowSize_ - rowPadSize_);
   }
+  public long getFixedLenRowSize() { return getFixedLenRowSize_; }
+
   public void setFragment(PlanFragment fragment) { fragment_ = fragment; }
   public PlanFragment getFragment() { return fragment_; }
   public List<Expr> getConjuncts() { return conjuncts_; }
@@ -297,6 +304,10 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
     conjuncts_.addAll(conjuncts);
   }
 
+  /**
+   * Deprecated because it may hide the need to recompute stats after conjunct transfer.
+   */
+  @Deprecated
   public void transferConjuncts(PlanNode recipient) {
     recipient.conjuncts_.addAll(conjuncts_);
     conjuncts_.clear();
@@ -512,6 +523,10 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
    */
   private void initThrift(TPlanNode msg, ThriftSerializationCtx serialCtx) {
     msg.limit = limit_;
+    if (hasLimit()) {
+      LOG.trace("{} ineligible for caching due to limit", this);
+      serialCtx.setTupleCachingIneligible(IneligibilityReason.LIMIT);
+    }
 
     if (!serialCtx.isTupleCache()) {
       msg.node_id = id_.asInt();
@@ -541,7 +556,7 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
     }
     // Serialize any runtime filters
     for (RuntimeFilter filter : runtimeFilters_) {
-      msg.addToRuntime_filters(filter.toThrift());
+      msg.addToRuntime_filters(filter.toThrift(serialCtx, this));
     }
     msg.setDisable_codegen(disableCodegen_);
     msg.pipelines = new ArrayList<>();
@@ -719,10 +734,12 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
   public void computeStats(Analyzer analyzer) {
     avgRowSize_ = 0.0F;
     rowPadSize_ = 0.0F;
+    getFixedLenRowSize_ = 0;
     for (TupleId tid: tupleIds_) {
       TupleDescriptor desc = analyzer.getTupleDesc(tid);
       avgRowSize_ += desc.getAvgSerializedSize();
       rowPadSize_ += desc.getSerializedPadSize();
+      getFixedLenRowSize_ += desc.getByteSize();
     }
     if (!children_.isEmpty()) {
       numNodes_ = getChild(0).numNodes_;
@@ -731,10 +748,7 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
   }
 
   protected long capCardinalityAtLimit(long cardinality) {
-    if (hasLimit()) {
-      return capCardinalityAtLimit(cardinality, limit_);
-    }
-    return cardinality;
+    return smallestValidCardinality(cardinality, limit_);
   }
 
   // Default implementation of computing the total data processed in bytes.
@@ -744,10 +758,6 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
         ExprUtil.computeExprsTotalCost(getConjuncts()));
   }
 
-  public static long capCardinalityAtLimit(long cardinality, long limit) {
-    return cardinality == -1 ? limit : Math.min(cardinality, limit);
-  }
-
   /**
    * Call computeMemLayout() for all materialized tuples.
    */
@@ -1316,34 +1326,76 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
    * This computes the cache key by hashing Thrift structures, but it only computes
    * the key if the node is eligible to avoid overhead.
    */
-  public void computeTupleCacheInfo(DescriptorTable descTbl) {
+  public void computeTupleCacheInfo(DescriptorTable descTbl,
+      TQueryOptionsHash queryOptsHash) {
+    if (tupleCacheInfo_ != null) {
+      // Already computed.
+      LOG.trace("Tuple cache found for {}", this);
+      return;
+    }
+
+    LOG.trace("Computing tuple cache info for {}", this);
     tupleCacheInfo_ = new TupleCacheInfo(descTbl);
     // computing the tuple cache information is a bottom-up tree traversal,
     // so visit and merge the children before processing this node's contents
-    for (int i = 0; i < getChildCount(); i++) {
-      getChild(i).computeTupleCacheInfo(descTbl);
-      tupleCacheInfo_.mergeChild(getChild(i).getTupleCacheInfo());
+    for (PlanNode child : getChildren()) {
+      child.computeTupleCacheInfo(descTbl, queryOptsHash);
+      if (!tupleCacheInfo_.mergeChild(child.getTupleCacheInfo())) {
+        LOG.trace("{} ineligible for caching due to {}", this, child);
+      }
     }
 
     if (!isTupleCachingImplemented()) {
+      LOG.trace("{} is ineligible for caching", this);
       tupleCacheInfo_.setIneligible(TupleCacheInfo.IneligibilityReason.NOT_IMPLEMENTED);
     }
 
     // If we are already ineligible, there is no need to process the Thrift
     if (!tupleCacheInfo_.isEligible()) {
-      tupleCacheInfo_.finalize();
+      tupleCacheInfo_.finalizeHash();
       return;
     }
 
+    // Include the build-side of a RuntimeFilter; look past the 1st ExchangeNode.
+    // If the build-side is hashable, merge the hash. Otherwise mark this node as
+    // ineligible because the RuntimeFilter is too complex to reason about.
+    for (RuntimeFilter filter : runtimeFilters_) {
+      // We want the build side of the join.
+      PlanNode build = filter.getSrc().getBuildNode();
+      Preconditions.checkState(!build.contains(this),
+          "Build-side contains current node, so cache info cannot be initialized");
+
+      if (build instanceof ExchangeNode && build.getChildCount() == 1) {
+        // We only look past ExchangeNodes with 1 child. IcebergDeleteNode has 2.
+        build = build.getChild(0);
+      }
+
+      // Build may not have been visited yet.
+      build.computeTupleCacheInfo(descTbl, queryOptsHash);
+      if (!tupleCacheInfo_.mergeChildWithScans(build.getTupleCacheInfo())) {
+        LOG.trace("{} on {} ineligible for caching due to {}", filter, this, build);
+        tupleCacheInfo_.finalizeHash();
+        return;
+      }
+    }
+
     // Incorporate this node's information
-    // TODO: This will also calculate eligibility via initThrift/toThrift.
-    // TODO: This will adjust the output of initThrift/toThrift to mask out items.
     TPlanNode msg = new TPlanNode();
     ThriftSerializationCtx serialCtx = new ThriftSerializationCtx(tupleCacheInfo_);
     initThrift(msg, serialCtx);
     toThrift(msg, serialCtx);
+    // Do not continue if initThrift or toThrift marked this as ineligible.
+    if (!tupleCacheInfo_.isEligible()) {
+      tupleCacheInfo_.finalizeHash();
+      return;
+    }
     tupleCacheInfo_.hashThrift(msg);
-    tupleCacheInfo_.finalize();
+    if (getChildCount() == 0 && queryOptsHash != null) {
+      // Leaf node, add query options hash.
+      tupleCacheInfo_.hashThrift(queryOptsHash);
+    }
+    tupleCacheInfo_.finalizeHash();
+    LOG.trace("Hash for {}: {}", this, tupleCacheInfo_.getHashTrace());
   }
 
   /**
@@ -1351,4 +1403,22 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
    * need to explicitly enable it.
    */
   public boolean isTupleCachingImplemented() { return false; }
+
+  /**
+   * Return the least between 'cardinality1' and 'cardinality2'
+   * that is not a negative number (unknown).
+   * Can return -1 if both number is less than 0.
+   * Both argument should not be < -1.
+   */
+  protected static long smallestValidCardinality(long cardinality1, long cardinality2) {
+    Preconditions.checkArgument(
+        cardinality1 >= -1, "cardinality1 is invalid: %s", cardinality1);
+    Preconditions.checkArgument(
+        cardinality2 >= -1, "cardinality2 is invalid: %s", cardinality2);
+    if (cardinality1 >= 0) {
+      if (cardinality2 >= 0) return Math.min(cardinality1, cardinality2);
+      return cardinality1;
+    }
+    return Math.max(-1, cardinality2);
+  }
 }
