@@ -37,6 +37,8 @@ import org.apache.impala.analysis.Expr;
 import org.apache.impala.analysis.ExprSubstitutionMap;
 import org.apache.impala.analysis.InsertStmt;
 import org.apache.impala.analysis.JoinOperator;
+import org.apache.impala.analysis.MergeCase;
+import org.apache.impala.analysis.MergeStmt;
 import org.apache.impala.analysis.QueryStmt;
 import org.apache.impala.analysis.SortInfo;
 import org.apache.impala.analysis.TupleId;
@@ -137,6 +139,10 @@ public class Planner {
     invertJoins(singleNodePlan, ctx_.isSingleNodeExec());
     singleNodePlan = useNljForSingularRowBuilds(singleNodePlan, ctx_.getRootAnalyzer());
 
+    if(ctx_.isMerge()) {
+      singleNodePlan = addMergeNode(singleNodePlan, ctx_.getRootAnalyzer());
+    }
+
     SingleNodePlanner.validatePlan(ctx_, singleNodePlan);
 
     if (ctx_.isSingleNodeExec()) {
@@ -171,12 +177,15 @@ public class Planner {
       createPreInsertSort(insertStmt, rootFragment, ctx_.getRootAnalyzer());
       // set up table sink for root fragment
       rootFragment.setSink(insertStmt.createDataSink());
-    } else if (ctx_.isUpdate() || ctx_.isDelete() || ctx_.isOptimize()) {
+    } else if (ctx_.isUpdate() || ctx_.isDelete() || ctx_.isOptimize()
+        || ctx_.isMerge()) {
       DmlStatementBase stmt;
       if (ctx_.isUpdate()) {
         stmt = ctx_.getAnalysisResult().getUpdateStmt();
       } else if (ctx_.isDelete()) {
         stmt = ctx_.getAnalysisResult().getDeleteStmt();
+      } else if (ctx_.isMerge()) {
+        stmt = ctx_.getAnalysisResult().getMergeStmt();
       } else {
         stmt = ctx_.getAnalysisResult().getOptimizeStmt();
       }
@@ -216,8 +225,9 @@ public class Planner {
 
     ColumnLineageGraph graph = ctx_.getRootAnalyzer().getColumnLineageGraph();
     if (BackendConfig.INSTANCE.getComputeLineage() || RuntimeEnv.INSTANCE.isTestEnv()) {
-      // Lineage is disabled for UPDATE AND DELETE statements
-      if (ctx_.isUpdateOrDelete()) return fragments;
+      // Lineage is disabled for UPDATE, DELETE and MERGE statements
+      if (ctx_.isUpdate() || ctx_.isDelete() || ctx_.isMerge())
+        return fragments;
       // Compute the column lineage graph
       if (ctx_.isInsertOrCtas()) {
         InsertStmt insertStmt = ctx_.getAnalysisResult().getInsertStmt();
@@ -351,7 +361,8 @@ public class Planner {
     if (ctx_.getAnalysisResult().isExplainStmt() || RuntimeEnv.INSTANCE.isTestEnv()) {
       explainLevel = ctx_.getQueryOptions().getExplain_level();
     }
-    return getExplainString(fragments, request, explainLevel);
+    return getExplainString(fragments, request, explainLevel, ctx_.getQueryOptions(),
+        ctx_.getQueryStmt());
   }
 
   /**
@@ -359,8 +370,9 @@ public class Planner {
    * explicit explain level.
    * Includes the estimated resource requirements from the request if set.
    */
-  public String getExplainString(List<PlanFragment> fragments,
-      TQueryExecRequest request, TExplainLevel explainLevel) {
+  public static String getExplainString(List<PlanFragment> fragments,
+      TQueryExecRequest request, TExplainLevel explainLevel,
+      TQueryOptions options, QueryStmt queryStmt) {
     StringBuilder str = new StringBuilder();
     boolean hasHeader = false;
 
@@ -442,9 +454,10 @@ public class Planner {
       hasHeader = true;
     }
 
-    if (explainLevel.ordinal() >= TExplainLevel.EXTENDED.ordinal()) {
+    if (explainLevel.ordinal() >= TExplainLevel.EXTENDED.ordinal() &&
+        queryStmt != null) {
       // In extended explain include the analyzed query text showing implicit casts
-      String queryText = ctx_.getQueryStmt().toSql(SHOW_IMPLICIT_CASTS);
+      String queryText = queryStmt.toSql(SHOW_IMPLICIT_CASTS);
       String wrappedText = PrintUtils.wrapString("Analyzed query: " + queryText, 80);
       str.append(wrappedText).append("\n");
       hasHeader = true;
@@ -457,12 +470,12 @@ public class Planner {
 
     if (explainLevel.ordinal() < TExplainLevel.VERBOSE.ordinal()) {
       // Print the non-fragmented parallel plan.
-      str.append(fragments.get(0).getExplainString(ctx_.getQueryOptions(), explainLevel));
+      str.append(fragments.get(0).getExplainString(options, explainLevel));
     } else {
       // Print the fragmented parallel plan.
       for (int i = 0; i < fragments.size(); ++i) {
         PlanFragment fragment = fragments.get(i);
-        str.append(fragment.getExplainString(ctx_.getQueryOptions(), explainLevel));
+        str.append(fragment.getExplainString(options, explainLevel));
         if (i < fragments.size() - 1) str.append("\n");
       }
     }
@@ -1024,4 +1037,10 @@ public class Planner {
 
     inputFragment.setPlanRoot(node);
   }
+
+  private PlanNode addMergeNode(PlanNode planNode, Analyzer analyzer)
+      throws ImpalaException {
+    MergeStmt merge = ctx_.getAnalysisResult().getMergeStmt();
+    return merge.getPlanNode(ctx_, planNode, analyzer);
+  }
 }
