@@ -96,6 +96,11 @@ public class SubplanNode extends PlanNode {
     cardinality_ = capCardinalityAtLimit(cardinality_);
   }
 
+  @Override
+  public void computeProcessingCost(TQueryOptions queryOptions) {
+    processingCost_ = ProcessingCost.basicCost(getDisplayLabel(), getCardinality(), 0);
+  }
+
   @Override
   public void computeNodeResourceProfile(TQueryOptions queryOptions) {
     // TODO: add an estimate
