@@ -440,6 +440,18 @@ public class BackendConfig {
     backendCfg_.iceberg_allow_datafiles_in_table_location_only = flag;
   }
 
+  public boolean icebergAlwaysAllowMergeOnReadOperations() {
+    return backendCfg_.iceberg_always_allow_merge_on_read_operations;
+  }
+
+  public long dataStreamSenderBufferSizeUsedByPlanner() {
+    return backendCfg_.data_stream_sender_buffer_size_used_by_planner;
+  }
+
+  public boolean enableReadingPuffinStats() {
+    return backendCfg_.enable_reading_puffin_stats;
+  }
+
   public boolean isJsonScannerEnabled() {
     return backendCfg_.enable_json_scanner;
   }
@@ -463,6 +475,10 @@ public class BackendConfig {
 
   public String debugActions() { return backendCfg_.debug_actions; }
 
+  public void setDebugActions(String debugActions) {
+    backendCfg_.debug_actions = debugActions;
+  }
+
   public boolean isInvalidateMetadataOnEventProcessFailureEnabled() {
     return backendCfg_.invalidate_metadata_on_event_processing_failure;
   }
@@ -529,4 +545,8 @@ public class BackendConfig {
   public boolean isCatalogdHAEnabled() {
     return backendCfg_.enable_catalogd_ha;
   }
+
+  public String getInjectedGroupMembersDebugOnly() {
+    return backendCfg_.injected_group_members_debug_only;
+  }
 }
