@@ -28,6 +28,7 @@ import org.apache.impala.analysis.SlotRef;
 import org.apache.impala.analysis.TableName;
 import org.apache.impala.common.InternalException;
 import org.apache.impala.common.Pair;
+import org.apache.impala.thrift.TAddressesList;
 import org.apache.impala.thrift.TCacheJarParams;
 import org.apache.impala.thrift.TCacheJarResult;
 import org.apache.impala.thrift.TCatalogObject;
@@ -36,6 +37,8 @@ import org.apache.impala.thrift.TCatalogServiceRequestHeader;
 import org.apache.impala.thrift.TColumnValue;
 import org.apache.impala.thrift.TErrorCode;
 import org.apache.impala.thrift.TExprBatch;
+import org.apache.impala.thrift.TGetNullPartitionNameRequest;
+import org.apache.impala.thrift.TGetNullPartitionNameResponse;
 import org.apache.impala.thrift.TGetPartitionStatsRequest;
 import org.apache.impala.thrift.TGetPartitionStatsResponse;
 import org.apache.impala.thrift.TParseDateStringResult;
@@ -140,6 +143,18 @@ public class FeSupport {
   // E.g.: '2011-01-01', '2011-01-1', '2011-1-01', '2011-01-01'.
   public native static byte[] nativeParseDateString(String date);
 
+  // Does an RPC to the Catalog Server to get the null partition name.
+  public native static byte[] NativeGetNullPartitionName(byte[] thriftReq);
+
+  // Does an RPC to the Catalog Server to get the latest compactions.
+  public native static byte[] NativeGetLatestCompactions(byte[] thriftReq);
+
+  // Get a list of addresses for coordinators.
+  public native static byte[] NativeGetCoordinators();
+
+  // Get the number of live queries.
+  public native static long NativeNumLiveQueries();
+
   /**
    * Locally caches the jar at the specified HDFS location.
    *
@@ -490,6 +505,57 @@ public class FeSupport {
     }
   }
 
+  private static byte[] GetNullPartitionName(byte[] thriftReq) {
+    try {
+      return NativeGetNullPartitionName(thriftReq);
+    } catch (UnsatisfiedLinkError e) { loadLibrary(); }
+    return NativeGetNullPartitionName(thriftReq);
+  }
+
+  public static String GetNullPartitionName() throws InternalException {
+    TGetNullPartitionNameRequest request = new TGetNullPartitionNameRequest();
+    TGetNullPartitionNameResponse response = new TGetNullPartitionNameResponse();
+    try {
+      byte[] result = GetNullPartitionName(
+          new TSerializer(new TBinaryProtocol.Factory()).serialize(request));
+      Preconditions.checkNotNull(result);
+      new TDeserializer(new TBinaryProtocol.Factory()).deserialize(response, result);
+      if (response.getStatus().getStatus_code() != TErrorCode.OK) {
+        throw new InternalException("Error requesting GetNullPartitionName: "
+            + Joiner.on("\n").join(response.getStatus().getError_msgs()));
+      }
+      Preconditions.checkNotNull(response.partition_value);
+      return response.partition_value;
+    } catch (TException e) {
+      // this should never happen
+      throw new InternalException("Error processing request: " + e.getMessage(), e);
+    }
+  }
+
+  public static byte[] GetLatestCompactions(byte[] thriftReq) {
+    try {
+      return NativeGetLatestCompactions(thriftReq);
+    } catch (UnsatisfiedLinkError e) { loadLibrary(); }
+    return NativeGetLatestCompactions(thriftReq);
+  }
+
+  public static TAddressesList GetCoordinators() throws InternalException {
+    try {
+      byte[] result = NativeGetCoordinators();
+      Preconditions.checkNotNull(result);
+      TDeserializer deserializer = new TDeserializer(new TBinaryProtocol.Factory());
+      TAddressesList coordinators_list = new TAddressesList();
+      deserializer.deserialize(coordinators_list, result);
+      return coordinators_list;
+    } catch (TException e) {
+      throw new InternalException("Error getting coordinators", e);
+     }
+  }
+
+  public static long NumLiveQueries() {
+    return NativeNumLiveQueries();
+  }
+
   /**
    * Calling this function before loadLibrary() causes external frontend
    * initialization to be used during NativeFeInit()
