@@ -18,7 +18,6 @@
 package org.apache.impala.service;
 
 import java.nio.ByteBuffer;
-import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -45,7 +44,6 @@ import org.apache.iceberg.UpdatePartitionSpec;
 import org.apache.iceberg.UpdateProperties;
 import org.apache.iceberg.UpdateSchema;
 import org.apache.iceberg.catalog.TableIdentifier;
-import org.apache.iceberg.exceptions.ValidationException;
 import org.apache.iceberg.expressions.Expressions;
 import org.apache.iceberg.expressions.Term;
 import org.apache.iceberg.hive.HiveCatalog;
@@ -77,6 +75,8 @@ import com.google.common.base.Preconditions;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.apache.impala.common.FileSystemUtil.deleteIfExists;
+
 /**
  * This is a helper for the CatalogOpExecutor to provide Iceberg related DDL functionality
  * such as creating and dropping tables from Iceberg.
@@ -365,8 +365,9 @@ public class IcebergCatalogOpExecutor {
     switch (icebergOp.operation) {
       case INSERT: appendFiles(feIcebergTable, txn, icebergOp); break;
       case DELETE: deleteRows(feIcebergTable, txn, icebergOp); break;
+      case MERGE:
       case UPDATE: updateRows(feIcebergTable, txn, icebergOp); break;
-      case OPTIMIZE: rewriteTable(feIcebergTable, txn, icebergOp); break;
+      case OPTIMIZE: optimizeTable(feIcebergTable, txn, icebergOp); break;
       default: throw new ImpalaRuntimeException(
           "Unknown Iceberg operation: " + icebergOp.operation);
     }
@@ -380,19 +381,15 @@ public class IcebergCatalogOpExecutor {
       DeleteFile deleteFile = createDeleteFile(feIcebergTable, buf);
       rowDelta.addDeletes(deleteFile);
     }
-    try {
-      // Validate that there are no conflicting data files, because if data files are
-      // added in the meantime, they potentially contain records that should have been
-      // affected by this DELETE operation.
-      rowDelta.validateFromSnapshot(icebergOp.getInitial_snapshot_id());
-      rowDelta.validateNoConflictingDataFiles();
-      rowDelta.validateDataFilesExist(
-          icebergOp.getData_files_referenced_by_position_deletes());
-      rowDelta.validateDeletedFiles();
-      rowDelta.commit();
-    } catch (ValidationException e) {
-      throw new ImpalaRuntimeException(e.getMessage(), e);
-    }
+    // Validate that there are no conflicting data files, because if data files are
+    // added in the meantime, they potentially contain records that should have been
+    // affected by this DELETE operation.
+    rowDelta.validateFromSnapshot(icebergOp.getInitial_snapshot_id());
+    rowDelta.validateNoConflictingDataFiles();
+    rowDelta.validateDataFilesExist(
+        icebergOp.getData_files_referenced_by_position_deletes());
+    rowDelta.validateDeletedFiles();
+    rowDelta.commit();
   }
 
   private static void updateRows(FeIcebergTable feIcebergTable, Transaction txn,
@@ -408,22 +405,18 @@ public class IcebergCatalogOpExecutor {
       DataFile dataFile = createDataFile(feIcebergTable, buf);
       rowDelta.addRows(dataFile);
     }
-    try {
-      // Validate that there are no conflicting data files, because if data files are
-      // added in the meantime, they potentially contain records that should have been
-      // affected by this UPDATE operation. Also validate that there are no conflicting
-      // delete files, because we don't want to revive records that have been deleted
-      // in the meantime.
-      rowDelta.validateFromSnapshot(icebergOp.getInitial_snapshot_id());
-      rowDelta.validateNoConflictingDataFiles();
-      rowDelta.validateNoConflictingDeleteFiles();
-      rowDelta.validateDataFilesExist(
-          icebergOp.getData_files_referenced_by_position_deletes());
-      rowDelta.validateDeletedFiles();
-      rowDelta.commit();
-    } catch (ValidationException e) {
-      throw new ImpalaRuntimeException(e.getMessage(), e);
-    }
+    // Validate that there are no conflicting data files, because if data files are
+    // added in the meantime, they potentially contain records that should have been
+    // affected by this UPDATE operation. Also validate that there are no conflicting
+    // delete files, because we don't want to revive records that have been deleted
+    // in the meantime.
+    rowDelta.validateFromSnapshot(icebergOp.getInitial_snapshot_id());
+    rowDelta.validateNoConflictingDataFiles();
+    rowDelta.validateNoConflictingDeleteFiles();
+    rowDelta.validateDataFilesExist(
+        icebergOp.getData_files_referenced_by_position_deletes());
+    rowDelta.validateDeletedFiles();
+    rowDelta.commit();
   }
 
   private static DataFile createDataFile(FeIcebergTable feIcebergTable, ByteBuffer buf)
@@ -487,11 +480,7 @@ public class IcebergCatalogOpExecutor {
       DataFile dataFile = createDataFile(feIcebergTable, buf);
       batchWrite.addFile(dataFile);
     }
-    try {
-      batchWrite.commit();
-    } catch (ValidationException e) {
-      throw new ImpalaRuntimeException(e.getMessage(), e);
-    }
+    batchWrite.commit();
   }
 
   private static Metrics buildDataFileMetrics(FbIcebergDataFile dataFile) {
@@ -521,7 +510,7 @@ public class IcebergCatalogOpExecutor {
         nullValueCounts, null, lowerBounds, upperBounds);
   }
 
-  private static void rewriteTable(FeIcebergTable feIcebergTable, Transaction txn,
+  private static void optimizeTable(FeIcebergTable feIcebergTable, Transaction txn,
       TIcebergOperationParam icebergOp) throws ImpalaRuntimeException {
     GroupedContentFiles contentFiles;
     try {
@@ -533,11 +522,21 @@ public class IcebergCatalogOpExecutor {
       throw new ImpalaRuntimeException(e.getMessage(), e);
     }
     RewriteFiles rewrite = txn.newRewrite();
-    // Delete current data files from table.
-    for (DataFile dataFile : contentFiles.dataFilesWithDeletes) {
-      rewrite.deleteFile(dataFile);
+    // Delete current data files from table if the operation is a full table rewrite.
+    // If there was file filtering, keep the data files without deletes that were not
+    // selected, and delete only the selected, rewritten files.
+    if (icebergOp.isSetReplaced_data_files_without_deletes()) {
+      for (DataFile dataFile : contentFiles.dataFilesWithoutDeletes) {
+        if (icebergOp.replaced_data_files_without_deletes.contains(dataFile.path())) {
+          rewrite.deleteFile(dataFile);
+        }
+      }
+    } else {
+      for (DataFile dataFile : contentFiles.dataFilesWithoutDeletes) {
+        rewrite.deleteFile(dataFile);
+      }
     }
-    for (DataFile dataFile : contentFiles.dataFilesWithoutDeletes) {
+    for (DataFile dataFile : contentFiles.dataFilesWithDeletes) {
       rewrite.deleteFile(dataFile);
     }
     // Delete current delete files from table.
@@ -553,12 +552,8 @@ public class IcebergCatalogOpExecutor {
       DataFile dataFile = createDataFile(feIcebergTable, buf);
       rewrite.addFile(dataFile);
     }
-    try {
-      rewrite.validateFromSnapshot(icebergOp.getInitial_snapshot_id());
-      rewrite.commit();
-    } catch (ValidationException e) {
-      throw new ImpalaRuntimeException(e.getMessage(), e);
-    }
+    rewrite.validateFromSnapshot(icebergOp.getInitial_snapshot_id());
+    rewrite.commit();
   }
 
   /**
@@ -583,4 +578,29 @@ public class IcebergCatalogOpExecutor {
                     String.valueOf(version));
     updateProps.commit();
   }
+
+  /**
+   * When a write operation fails the validation check due to conflicts, the data/delete
+   * files created by it cannot be committed to the table: they would become orphan files.
+   * This method deletes these uncommitted data/delete files from the file system.
+   * @param icebergOp the failed DML operation that contains the list of newly written
+   *                  data and delete files which have to be cleaned up.
+   */
+  protected static void cleanupUncommittedFiles(TIcebergOperationParam icebergOp) {
+    if (icebergOp.isSetIceberg_data_files_fb()) {
+      deleteIcebergFiles(icebergOp.getIceberg_data_files_fb());
+    }
+    if (icebergOp.isSetIceberg_delete_files_fb()) {
+      deleteIcebergFiles(icebergOp.getIceberg_delete_files_fb());
+    }
+  }
+
+  private static void deleteIcebergFiles(Iterable<ByteBuffer> icebergFiles) {
+    for (ByteBuffer buf : icebergFiles) {
+      String pathString = FbIcebergDataFile.getRootAsFbIcebergDataFile(buf).path();
+      if (pathString != null) {
+        deleteIfExists(new org.apache.hadoop.fs.Path((pathString)));
+      }
+    }
+  }
 }
