@@ -39,6 +39,7 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 
 import org.apache.hadoop.conf.Configuration;
@@ -654,7 +655,7 @@ public class IcebergUtil {
 
   private static TableScan createScanAsOf(FeIcebergTable table,
       TimeTravelSpec timeTravelSpec) {
-    TableScan scan = table.getIcebergApiTable().newScan();
+    TableScan scan = newScan(table);
     if (timeTravelSpec == null) {
       scan = scan.useSnapshot(table.snapshotId());
     } else {
@@ -686,7 +687,7 @@ public class IcebergUtil {
     if (table.snapshotId() == -1) {
       return new GroupedContentFiles(CloseableIterable.empty());
     }
-    TableScan scan = table.getIcebergApiTable().newScan();
+    TableScan scan = newScan(table);
     scan = scan.useSnapshot(snapshotId);
     for (Expression predicate : predicates) {
       scan = scan.filter(predicate);
@@ -698,6 +699,11 @@ public class IcebergUtil {
     }
   }
 
+  private static TableScan newScan(FeIcebergTable table) {
+    TableScan scan = table.getIcebergApiTable().newScan();
+    return scan.caseSensitive(false);
+  }
+
   /**
    * Use ContentFile path to generate 128-bit Murmur3 hash as map key, cached in memory
    */
