@@ -18,6 +18,7 @@
 package org.apache.impala.util;
 
 import static java.lang.String.format;
+import static org.apache.impala.service.KuduCatalogOpExecutor.GOT_KUDU_CLIENT;
 
 import java.math.BigDecimal;
 import java.math.BigInteger;
@@ -98,6 +99,16 @@ public class KuduUtil {
     return client;
   }
 
+  /**
+   * Wrapper to get kudu client and mark the given 'catalogTimeline' when it finishes.
+   */
+  public static KuduClient getKuduClient(String masterHosts,
+      EventSequence catalogTimeline) {
+    KuduClient kudu = KuduUtil.getKuduClient(masterHosts);
+    catalogTimeline.markEvent(GOT_KUDU_CLIENT);
+    return kudu;
+  }
+
   /**
    * Creates a PartialRow from a list of range partition boundary values.
    * 'rangePartitionColumns' must be specified in Kudu case.
@@ -431,10 +442,10 @@ public class KuduUtil {
       case DECIMAL: return org.apache.kudu.Type.DECIMAL;
       case DATE: return org.apache.kudu.Type.DATE;
       case VARCHAR: return org.apache.kudu.Type.VARCHAR;
+      case BINARY: return org.apache.kudu.Type.BINARY;
       /* Fall through below */
       case INVALID_TYPE:
       case NULL_TYPE:
-      case BINARY:
       case DATETIME:
       case CHAR:
       default:
@@ -460,6 +471,7 @@ public class KuduUtil {
         return ScalarType.createDecimalType(
             typeAttributes.getPrecision(), typeAttributes.getScale());
       case VARCHAR: return ScalarType.createVarcharType(typeAttributes.getLength());
+      case BINARY: return Type.BINARY;
       default:
         throw new ImpalaRuntimeException(String.format(
             "Kudu type '%s' is not supported in Impala", t.getName()));
