@@ -21,6 +21,7 @@ import java.io.File;
 import java.io.IOException;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.util.Map;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicReference;
 
@@ -33,12 +34,9 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.impala.authorization.User;
 import org.apache.impala.common.ByteUnits;
-import org.apache.impala.common.ImpalaException;
 import org.apache.impala.common.InternalException;
-import org.apache.impala.common.JniUtil;
 import org.apache.impala.common.RuntimeEnv;
 import org.apache.impala.thrift.TErrorCode;
-import org.apache.impala.thrift.TPoolConfigParams;
 import org.apache.impala.thrift.TPoolConfig;
 import org.apache.impala.thrift.TResolveRequestPoolParams;
 import org.apache.impala.thrift.TResolveRequestPoolResult;
@@ -68,7 +66,7 @@ import com.google.common.collect.Lists;
  * Yarn {@link AllocationFileLoaderService} and the Llama configuration uses a subclass of
  * the {@link FileWatchService}. There are two different mechanisms because there is
  * different parsing/configuration code for the allocation file and the Llama
- * configuration (which is a regular Hadoop conf file so it can use the
+ * configuration (which is a regular Hadoop conf file, so it can use the
  * {@link Configuration} class). start() and stop() will start/stop watching and reloading
  * both of these files.
  *
@@ -138,7 +136,7 @@ public class RequestPoolService {
   @VisibleForTesting
   final AllocationFileLoaderService allocLoader_;
 
-  // Provides access to the fair scheduler allocation file. An AtomicReference becaus it
+  // Provides access to the fair scheduler allocation file. An AtomicReference because it
   // is reset when the allocation configuration file changes and other threads access it.
   private final AtomicReference<AllocationConfiguration> allocationConf_;
 
@@ -292,6 +290,8 @@ public class RequestPoolService {
     stopInternal();
   }
 
+  public boolean isRunning() { return running_.get(); }
+
   /**
    * Stops the RequestPoolService instance without checking the running state. Only
    * called by stop() (which is only used in tests) or by start() if a failure occurs.
@@ -369,10 +369,16 @@ public class RequestPoolService {
     long maxMemoryMb = allocationConf_.get().getMaxResources(pool).getMemory();
     result.setMax_mem_resources(
         maxMemoryMb == Integer.MAX_VALUE ? -1 : maxMemoryMb * ByteUnits.MEGABYTE);
+    Map<String, Integer> userQueryLimits = allocationConf_.get().getUserQueryLimits(pool);
+    result.setUser_query_limits(userQueryLimits);
+    Map<String, Integer> groupQueryLimits =
+        allocationConf_.get().getGroupQueryLimits(pool);
+    result.setGroup_query_limits(groupQueryLimits);
     if (conf_ == null) {
       result.setMax_requests(MAX_PLACED_RESERVATIONS_DEFAULT);
       result.setMax_queued(MAX_QUEUED_RESERVATIONS_DEFAULT);
       result.setDefault_query_options("");
+      result.setOnly_coordinators(false);
     } else {
       // Capture the current conf_ in case it changes while we're using it.
       Configuration currentConf = conf_;
@@ -398,18 +404,22 @@ public class RequestPoolService {
           getPoolConfigValue(currentConf, pool, MAX_QUERY_CPU_CORE_PER_NODE_LIMIT, 0L));
       result.setMax_query_cpu_core_coordinator_limit(getPoolConfigValue(
           currentConf, pool, MAX_QUERY_CPU_CORE_COORDINATOR_LIMIT, 0L));
+      result.setOnly_coordinators(allocationConf_.get().isOnlyCoordinators(pool));
     }
     if (LOG.isTraceEnabled()) {
       LOG.debug("getPoolConfig(pool={}): max_mem_resources={}, max_requests={},"
               + " max_queued={},  queue_timeout_ms={}, default_query_options={},"
               + " max_query_mem_limit={}, min_query_mem_limit={},"
               + " clamp_mem_limit_query_option={}, max_query_cpu_core_per_node_limit={},"
-              + " max_query_cpu_core_coordinator_limit={}",
+              + " max_query_cpu_core_coordinator_limit={}"
+              + " user_query_limits={}"
+              + " group_query_limits={}",
           pool, result.max_mem_resources, result.max_requests, result.max_queued,
           result.queue_timeout_ms, result.default_query_options,
           result.max_query_mem_limit, result.min_query_mem_limit,
           result.clamp_mem_limit_query_option, result.max_query_cpu_core_per_node_limit,
-          result.max_query_cpu_core_coordinator_limit);
+          result.max_query_cpu_core_coordinator_limit, result.user_query_limits,
+          result.group_query_limits);
     }
     return result;
   }
@@ -502,4 +512,4 @@ public class RequestPoolService {
     Preconditions.checkState(RuntimeEnv.INSTANCE.isTestEnv());
     return allocationConf_.get();
   }
-}
+}
\ No newline at end of file
