@@ -27,17 +27,18 @@ import org.apache.impala.authorization.ranger.RangerAuthorizationFactory;
 import org.apache.impala.authorization.ranger.RangerCatalogdAuthorizationManager;
 import org.apache.impala.authorization.ranger.RangerImpalaPlugin;
 import org.apache.impala.authorization.ranger.RangerImpalaResourceBuilder;
-import org.apache.impala.catalog.Role;
 import org.apache.impala.catalog.ScalarFunction;
 import org.apache.impala.catalog.Type;
 import org.apache.impala.common.AnalysisException;
 import org.apache.impala.common.FrontendTestBase;
 import org.apache.impala.common.ImpalaException;
 import org.apache.impala.service.Frontend;
+import org.apache.impala.testutil.CatalogServiceTestCatalog;
 import org.apache.impala.testutil.ImpaladTestCatalog;
 import org.apache.impala.thrift.TColumnValue;
 import org.apache.impala.thrift.TDescribeOutputStyle;
 import org.apache.impala.thrift.TDescribeResult;
+import org.apache.impala.thrift.TDescribeTableParams;
 import org.apache.impala.thrift.TFunctionBinaryType;
 import org.apache.impala.thrift.TPrivilege;
 import org.apache.impala.thrift.TPrivilegeLevel;
@@ -112,6 +113,8 @@ public abstract class AuthorizationTestBase extends FrontendTestBase {
   protected final RangerImpalaPlugin rangerImpalaPlugin_;
   protected final RangerRESTClient rangerRestClient_;
 
+  protected final CatalogServiceTestCatalog testCatalog_;
+
   public AuthorizationTestBase(AuthorizationProvider authzProvider)
       throws ImpalaException {
     authzProvider_ = authzProvider;
@@ -122,7 +125,8 @@ public abstract class AuthorizationTestBase extends FrontendTestBase {
             SERVER_NAME, null, null, null);
         authzFactory_ = createAuthorizationFactory(authzProvider);
         authzCtx_ = createAnalysisCtx(authzFactory_, user_.getName());
-        authzCatalog_ = new ImpaladTestCatalog(authzFactory_);
+        testCatalog_ = CatalogServiceTestCatalogWithRanger.createWithAuth(authzFactory_);
+        authzCatalog_ = new ImpaladTestCatalog(testCatalog_);
         authzFrontend_ = new Frontend(authzFactory_, authzCatalog_);
         rangerImpalaPlugin_ =
             ((RangerAuthorizationChecker) authzFrontend_.getAuthzChecker())
@@ -203,7 +207,8 @@ public abstract class AuthorizationTestBase extends FrontendTestBase {
           // owner of the resource.
           getName(),
           Collections.emptyList(), Collections.emptyList(),
-          rangerImpalaPlugin_.getClusterName(), "127.0.0.1", privileges);
+          rangerImpalaPlugin_.getClusterName(), "127.0.0.1", privileges,
+          /*resourceOwner*/ null);
     }
   }
 
@@ -217,7 +222,8 @@ public abstract class AuthorizationTestBase extends FrontendTestBase {
           // owner of the resource.
           (as_owner_ ? OWNER_GROUPS : GROUPS), Collections.emptyList(),
           // groups,
-          rangerImpalaPlugin_.getClusterName(), "127.0.0.1", privileges);
+          rangerImpalaPlugin_.getClusterName(), "127.0.0.1", privileges,
+          /*resourceOwner*/ null);
     }
   }
 
@@ -269,8 +275,11 @@ public abstract class AuthorizationTestBase extends FrontendTestBase {
       Preconditions.checkArgument(includedStrings_.length != 0 ||
               excludedStrings_.length != 0,
           "One or both of included or excluded strings must be defined.");
-      List<String> result = resultToStringList(authzFrontend_.describeTable(table,
-          outputStyle_, user_));
+      TDescribeTableParams testParams = new TDescribeTableParams();
+      testParams.setTable_name(table);
+      testParams.setOutput_style(outputStyle_);
+      List<String> result = resultToStringList(authzFrontend_.describeTable(testParams,
+          user_));
       for (String str: includedStrings_) {
         assertTrue(String.format("\"%s\" is not in the describe output.\n" +
                 "Expected : %s\n Actual   : %s", str, Arrays.toString(includedStrings_),
