@@ -21,6 +21,7 @@ import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.Set;
@@ -32,6 +33,7 @@ import org.apache.impala.testutil.TestUtils;
 import org.apache.impala.thrift.QueryConstants;
 import org.apache.impala.thrift.TQueryCtx;
 import org.apache.impala.thrift.TQueryOptions;
+import org.apache.impala.util.MathUtil;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
 import org.junit.Test;
@@ -201,8 +203,10 @@ public class CardinalityTest extends PlannerTestBase {
     // NDV(id) = 26 * ndv(null_str) = 1 ; 26 * 1 = 26
     verifyCardinality(baseStmt + "nullrows.id, null_str", 26);
     // NDV(id) = 26 * ndv(group_str) = 156
-    // Planner does not know that id determines group_str
-    verifyCardinality(baseStmt + "nullrows.id, group_str", 156);
+    // However, id and group_str belong to the same nullrows table that has only 26 rows.
+    verifyCardinality(baseStmt + "nullrows.id, group_str", 26);
+    // NDV(nullrows.id) * NDV(alltypes.month) = 26 * 12 = 312
+    verifyCardinality(baseStmt + "nullrows.id, alltypes.month", 312);
   }
 
   /**
@@ -217,7 +221,7 @@ public class CardinalityTest extends PlannerTestBase {
     verifyCardinality(
         "SELECT COUNT(*)" + joinClause + "GROUP BY t1.id", 7300);
     verifyCardinality(
-        "SELECT COUNT(*)" + joinClause + "GROUP BY t1.id, t1.int_col", 7300 * 10);
+        "SELECT COUNT(*)" + joinClause + "GROUP BY t1.id, t1.int_col", 7300);
   }
 
   @Test
@@ -1030,6 +1034,54 @@ public class CardinalityTest extends PlannerTestBase {
         ImmutableSet.of(), pathToKuduScanNodeSg, KuduScanNode.class);
   }
 
+  @Test
+  public void testByteBasedNumWriters() {
+    int min_bytes = HdfsTableSink.MIN_WRITE_BYTES;
+
+    // Test unpartitioned insert cases.
+    assertEquals(1, HdfsTableSink.bytesBasedNumWriters(10, 50, false, 1, min_bytes, 1));
+    assertEquals(25, HdfsTableSink.bytesBasedNumWriters(10, 50, false, 1, min_bytes, 25));
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 10, false, 1, min_bytes, 25));
+
+    // Test partitioned insert cases with unknown totalNumPartitions.
+    assertEquals(1, HdfsTableSink.bytesBasedNumWriters(10, 50, true, -1, min_bytes, 1));
+    assertEquals(25, HdfsTableSink.bytesBasedNumWriters(10, 50, true, -1, min_bytes, 25));
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 10, true, -1, min_bytes, 25));
+
+    // Test partitioned insert cases with totalNumPartitions == 1.
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 1, min_bytes, 1));
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 1, min_bytes, 25));
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 10, true, 1, min_bytes, 25));
+
+    // Test partitioned insert cases with totalNumPartitions == maxNumWriters.
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 50, min_bytes, 1));
+    assertEquals(25, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 50, min_bytes, 25));
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 10, true, 10, min_bytes, 25));
+
+    // Test partitioned insert cases with totalNumPartitions > maxNumWriters.
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 60, min_bytes, 1));
+    assertEquals(25, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 60, min_bytes, 25));
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 10, true, 11, min_bytes, 25));
+
+    // Test partitioned insert cases with totalNumPartitions < maxNumWriters.
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 40, min_bytes, 1));
+    assertEquals(25, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 40, min_bytes, 25));
+    assertEquals(9, HdfsTableSink.bytesBasedNumWriters(10, 10, true, 9, min_bytes, 25));
+
+    // Test partitioned insert cases with totalNumPartitions < numInputNodes.
+    assertEquals(8, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 8, min_bytes, 1));
+    assertEquals(8, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 8, min_bytes, 25));
+    assertEquals(8, HdfsTableSink.bytesBasedNumWriters(10, 10, true, 8, min_bytes, 25));
+
+    // Test 0 cardinality.
+    assertEquals(1, HdfsTableSink.bytesBasedNumWriters(10, 50, false, 1, 0, 25));
+    assertEquals(1, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 50, 0, 25));
+
+    // Test 0 avgRowSize (this is hypothetical and defensive).
+    assertEquals(1, HdfsTableSink.bytesBasedNumWriters(10, 50, false, 1, min_bytes, 0));
+    assertEquals(10, HdfsTableSink.bytesBasedNumWriters(10, 50, true, 50, min_bytes, 0));
+  }
+
   /**
    * Given a query and an expected cardinality, checks that the root
    * node of the single-fragment plan has the expected cardinality.
@@ -1261,4 +1313,91 @@ public class CardinalityTest extends PlannerTestBase {
         expected * CARDINALITY_TOLERANCE);
   }
 
+  @Test
+  public void testSmallestValidCardinality() {
+    long[] validCard = {0, 1, Long.MAX_VALUE};
+    long unknown = -1;
+
+    // Case 1: both argument is valid.
+    for (long c1 : validCard) {
+      for (long c2 : validCard) {
+        assertEquals(c1 + " vs " + c2, Math.min(c1, c2),
+            PlanNode.smallestValidCardinality(c1, c2));
+      }
+    }
+    // Case 2: One argument is valid, the other is unknown.
+    for (long c : validCard) {
+      assertEquals(
+          c + " vs " + unknown, c, PlanNode.smallestValidCardinality(c, unknown));
+      assertEquals(
+          unknown + " vs " + c, c, PlanNode.smallestValidCardinality(unknown, c));
+    }
+    // Case 3: both argument is unknown.
+    assertEquals(unknown + " vs " + unknown, unknown,
+        PlanNode.smallestValidCardinality(unknown, unknown));
+  }
+
+  @Test
+  public void testEstimatePreaggCardinality() {
+    List<Long> positiveLong = Arrays.asList(1L, 2L, 5L, 10L, 100L, 1000L, Long.MAX_VALUE);
+    List<Long> validCard = new ArrayList<>(positiveLong);
+    validCard.add(-1L);
+    validCard.add(0L);
+
+    for (Long totalInstances : positiveLong) {
+      for (Long globalNdv : positiveLong) {
+        for (Long inputCard : validCard) {
+          String pattern = "totalInstances=%d, globalNdv=%d, inputCard=%d,"
+          + " isNonGroupingAggregation=%b, canCompteleEarly=%b, limit=%d, ouputCard=%d";
+
+          // Test regular preaggregation
+          long outputCard = AggregationNode.estimatePreaggCardinality(
+              totalInstances, globalNdv, inputCard, false, false, -1);
+          String message = String.format(pattern, totalInstances, globalNdv, inputCard,
+              false, false, -1, outputCard);
+          assertTrue(message + ", expect>=0", outputCard >= 0);
+          if (inputCard == 0) {
+            assertTrue(message + ", expect=0", outputCard == 0);
+          } else if (inputCard == -1 || totalInstances == 1) {
+            long leastInputVsNdv =
+                PlanNode.smallestValidCardinality(inputCard, globalNdv);
+            assertTrue(
+                message + ", expect=" + leastInputVsNdv, outputCard == leastInputVsNdv);
+          } else {
+            assertTrue(message + ", expect<=" + inputCard, outputCard <= inputCard);
+          }
+
+          // Test non-grouping preaggregation.
+          outputCard = AggregationNode.estimatePreaggCardinality(
+              totalInstances, globalNdv, inputCard, true, false, -1);
+          message = String.format(
+              pattern, totalInstances, globalNdv, inputCard, true, false, -1, outputCard);
+          assertTrue(message + ", expect>=0", outputCard >= 0);
+          long allDuplicate =
+              MathUtil.saturatingMultiplyCardinalities(globalNdv, totalInstances);
+          long leastInputVsAllDuplicate =
+              PlanNode.smallestValidCardinality(inputCard, allDuplicate);
+          if (allDuplicate < inputCard) {
+            assertTrue(message + ", expect=" + leastInputVsAllDuplicate,
+                outputCard == leastInputVsAllDuplicate);
+          }
+
+          // Test preaggregation with limit.
+          for (Long limit : validCard) {
+            if (limit < 0) continue;
+            outputCard = AggregationNode.estimatePreaggCardinality(
+                totalInstances, globalNdv, inputCard, false, true, limit);
+            message = String.format(pattern, totalInstances, globalNdv, inputCard, false,
+                true, limit, outputCard);
+            assertTrue(message + ", expect>=0", outputCard >= 0);
+            long allAtLimit =
+                MathUtil.saturatingMultiplyCardinalities(totalInstances, limit);
+            long leastOfAll =
+                PlanNode.smallestValidCardinality(allAtLimit, leastInputVsAllDuplicate);
+            assertTrue(message + ", expect=" + leastOfAll, outputCard == leastOfAll);
+          }
+        }
+      }
+    }
+  }
 }
