@@ -33,7 +33,6 @@ import org.apache.impala.catalog.HBaseColumn;
 import org.apache.impala.catalog.Type;
 import org.apache.impala.common.ImpalaException;
 import org.apache.impala.datagenerator.HBaseTestDataRegionAssignment;
-import org.apache.impala.service.BackendConfig;
 import org.apache.impala.service.Frontend.PlanCtx;
 import org.apache.impala.testutil.TestUtils;
 import org.apache.impala.testutil.TestUtils.IgnoreValueFilter;
@@ -70,6 +69,13 @@ public class PlannerTest extends PlannerTestBase {
     assignment.close();
   }
 
+  private void runCardinalityVerifyingTest(String test_file) {
+    TQueryOptions options = defaultQueryOptions();
+    options.setDisable_hdfs_num_rows_estimate(true); // Needed for deterministic estimate.
+    runPlannerTestFile(
+        test_file, options, ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
+  }
+
   /**
    * Scan node cardinality test
    */
@@ -158,32 +164,40 @@ public class PlannerTest extends PlannerTestBase {
     // TODO: Multiple distinct in subqueries.
     // TODO: Multiple distinct lineage tests.
     // TODO: Multiple distinct and SHUFFLE_DISTINCT_EXPRS tests
-    runPlannerTestFile("multiple-distinct");
+    runCardinalityVerifyingTest("multiple-distinct");
   }
 
   @Test
   public void testMultipleDistinctMaterialization() {
-    runPlannerTestFile("multiple-distinct-materialization");
+    runCardinalityVerifyingTest("multiple-distinct-materialization");
   }
 
   @Test
   public void testMultipleDistinctPredicates() {
-    runPlannerTestFile("multiple-distinct-predicates");
+    runCardinalityVerifyingTest("multiple-distinct-predicates");
   }
 
   @Test
   public void testMultipleDistinctLimit() {
-    runPlannerTestFile("multiple-distinct-limit");
+    runCardinalityVerifyingTest("multiple-distinct-limit");
   }
 
   @Test
   public void testShuffleByDistinctExprs() {
-    runPlannerTestFile("shuffle-by-distinct-exprs");
+    runCardinalityVerifyingTest("shuffle-by-distinct-exprs");
   }
 
   @Test
   public void testAggregation() {
-    runPlannerTestFile("aggregation");
+    runCardinalityVerifyingTest("aggregation");
+  }
+
+  @Test
+  public void testAggregationNoTupleAnalysis() {
+    Set<PlannerTestOption> testOptions = tpcdsParquetTestOptions();
+    TQueryOptions options = tpcdsParquetQueryOptions();
+    runPlannerTestFile(
+        "aggregation-no-tuple-analysis", "tpcds_parquet", options, testOptions);
   }
 
   @Test
@@ -377,10 +391,7 @@ public class PlannerTest extends PlannerTestBase {
 
   @Test
   public void testJoinsWithHDFSNumRowsEstDisabled() {
-    TQueryOptions options = defaultQueryOptions();
-    options.setDisable_hdfs_num_rows_estimate(true);
-    runPlannerTestFile("joins", options,
-        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
+    runCardinalityVerifyingTest("joins");
   }
 
   @Test
@@ -491,6 +502,11 @@ public class PlannerTest extends PlannerTestBase {
     runPlannerTestFile("setoperation-rewrite", options);
   }
 
+  @Test
+  public void testRewrite() {
+    runPlannerTestFile("rewrite");
+  }
+
   @Test
   public void testValues() {
     runPlannerTestFile("values");
@@ -742,6 +758,17 @@ public class PlannerTest extends PlannerTestBase {
     runPlannerTestFile("kudu-stats-agg");
   }
 
+  @Test
+  public void testKuduDmlWithUtcConversion() {
+    TQueryOptions options = defaultQueryOptions();
+    options.setExplain_level(TExplainLevel.VERBOSE);
+    options.setWrite_kudu_utc_timestamps(true);
+    // convert_kudu_utc_timestamps is not really needed for the planner test, but it would
+    // be critical for update/delete if the queries were actually executed.
+    options.setConvert_kudu_utc_timestamps(true);
+    runPlannerTestFile("kudu-dml-with-utc-conversion", options);
+  }
+
   @Test
   public void testMtDopValidation() {
     // Tests that queries planned with mt_dop > 0 produce a parallel plan.
@@ -1249,7 +1276,9 @@ public class PlannerTest extends PlannerTestBase {
     TQueryOptions options = defaultQueryOptions();
     options.setPreagg_bytes_limit(64 * 1024 * 1024);
     options.setExplain_level(TExplainLevel.EXTENDED);
-    runPlannerTestFile("preagg-bytes-limit", "tpch_parquet", options);
+    runPlannerTestFile("preagg-bytes-limit", "tpch_parquet", options,
+        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY,
+            PlannerTestOption.VALIDATE_RESOURCES));
   }
 
   /**
@@ -1281,6 +1310,17 @@ public class PlannerTest extends PlannerTestBase {
         ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
   }
 
+  /**
+   * Checks file format listing for Iceberg tables.
+   */
+  @Test
+  public void testIcebergMixedFileFormat() {
+    TQueryOptions options = defaultQueryOptions();
+    options.setExplain_level(TExplainLevel.VERBOSE);
+    runPlannerTestFile("iceberg-mixed-file-format", "functional_parquet", options,
+        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
+  }
+
   /**
    * Checks exercising predicate pushdown with Iceberg tables, without predicate
    * subsetting.
@@ -1306,6 +1346,21 @@ public class PlannerTest extends PlannerTestBase {
             PlannerTestOption.VALIDATE_ICEBERG_SNAPSHOT_IDS));
   }
 
+  /**
+   * Check that Iceberg V2 table scans work as expected. Also check resource estimates.
+   */
+  @Test
+  public void testIcebergV2TableScansWithResources() {
+    TQueryOptions options = defaultQueryOptions();
+    options.setTimezone("UTC");
+    runPlannerTestFile("iceberg-v2-tables-resources", "functional_parquet", options,
+        ImmutableSet.of(
+            PlannerTestOption.EXTENDED_EXPLAIN,
+            PlannerTestOption.VALIDATE_CARDINALITY,
+            PlannerTestOption.VALIDATE_ICEBERG_SNAPSHOT_IDS,
+            PlannerTestOption.VALIDATE_RESOURCES));
+  }
+
   /**
    * Check that Iceberg V2 table scans work as expected with hash join
    */
@@ -1578,4 +1633,13 @@ public class PlannerTest extends PlannerTestBase {
   public void testRuntimeFilterRepeatedExpr() {
     runPlannerTestFile("runtime-filter-repeated-expr", "functional");
   }
+
+  @Test
+  public void testIcebergMerge() {
+    runPlannerTestFile("iceberg-merge");
+    TQueryOptions options = new TQueryOptions();
+    runPlannerTestFile("iceberg-merge-insert-only", options,
+        ImmutableSet.of(PlannerTestOption.EXTENDED_EXPLAIN,
+            PlannerTestOption.INCLUDE_EXPLAIN_HEADER));
+  }
 }
