@@ -32,7 +32,7 @@ import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import org.apache.commons.lang.exception.ExceptionUtils;
+import org.apache.commons.lang3.exception.ExceptionUtils;
 import org.apache.hadoop.fs.Path;
 import org.apache.impala.analysis.ColumnLineageGraph;
 import org.apache.impala.analysis.DescriptorTable;
@@ -428,7 +428,10 @@ public class PlannerTestBase extends FrontendTestBase {
         .setMax_fragment_instances_per_node(12)
         .setReplica_preference(TReplicaPreference.REMOTE)
         .setSlot_count_strategy(TSlotCountStrategy.PLANNER_CPU_ASK)
-        .setPlanner_testcase_mode(true);
+        .setPlanner_testcase_mode(true)
+        // Required so that output doesn't vary by whether scanned tables have stats &
+        // numRows property or not.
+        .setDisable_hdfs_num_rows_estimate(true);
   }
 
   protected static Set<PlannerTestOption> tpcdsParquetTestOptions() {
@@ -451,8 +454,8 @@ public class PlannerTestBase extends FrontendTestBase {
     String query = testCase.getQuery();
     LOG.info("running query " + query);
     if (query.isEmpty()) {
-      throw new IllegalStateException("Cannot plan empty query in line: " +
-          testCase.getStartingLineNum());
+      throw new IllegalStateException("Cannot plan empty query in " +
+          testCase.getFileNameAndLineNum());
     }
     // Set up the query context. Note that we need to deep copy it before planning each
     // time since planning modifies it.
@@ -513,7 +516,8 @@ public class PlannerTestBase extends FrontendTestBase {
           && firstPlanFragment.output_sink.isSetTable_sink()) {
         TTableSink tableSink = firstPlanFragment.output_sink.table_sink;
         if (!seenTableIds.contains(tableSink.target_table_id)
-            || tableSink.target_table_id != DescriptorTable.TABLE_SINK_ID) {
+            || (tableSink.target_table_id != DescriptorTable.TABLE_SINK_ID
+            && !tableSink.isSetIceberg_delete_sink())) {
           throw new IllegalStateException("Table sink id error for target table:\n" +
               tableSink.toString());
         }
@@ -600,12 +604,14 @@ public class PlannerTestBase extends FrontendTestBase {
       String planDiff = TestUtils.compareOutput(
           Lists.newArrayList(explainStr.split("\n")), expectedPlan, true, resultFilters);
       if (!planDiff.isEmpty()) {
-        errorLog.append(String.format("\nSection %s of query at line %d:\n%s\n\n%s",
-            section, testCase.getStartingLineNum(), query, planDiff));
-        // Append the VERBOSE explain plan because it contains details about
-        // tuples/sizes/cardinality for easier debugging.
-        String verbosePlan = getVerboseExplainPlan(queryCtx);
-        errorLog.append("\nVerbose plan:\n" + verbosePlan);
+        errorLog.append(String.format("\nSection %s of query at %s:\n%s\n\n%s",
+            section, testCase.getFileNameAndLineNum(), query, planDiff));
+        if (!testOptions.contains(PlannerTestOption.EXTENDED_EXPLAIN)) {
+          // Append the VERBOSE explain plan because it contains details about
+          // tuples/sizes/cardinality for easier debugging.
+          String verbosePlan = getVerboseExplainPlan(queryCtx);
+          errorLog.append("\nVerbose plan:\n" + verbosePlan);
+        }
       }
     }
     return execRequest;
