@@ -98,9 +98,10 @@ def utf8_encode_if_needed(val):
     val = val.encode('utf-8', errors='replace')
   return val
 
+
 # Regular expression that matches the progress line added to HS2 logs by
 # the Impala server.
-HS2_LOG_PROGRESS_REGEX = re.compile("Query.*Complete \([0-9]* out of [0-9]*\)\n")
+HS2_LOG_PROGRESS_REGEX = re.compile(r"Query.*Complete \([0-9]* out of [0-9]*\)\n")
 
 # Exception types to differentiate between the different RPCExceptions.
 # RPCException raised when TApplicationException is caught.
@@ -108,6 +109,158 @@ RPC_EXCEPTION_TAPPLICATION = "TAPPLICATION_EXCEPTION"
 # RPCException raised when impala server sends a TStatusCode.ERROR_STATUS status code.
 RPC_EXCEPTION_SERVER = "SERVER_ERROR"
 
+
+def build_exec_summary_table(summary, idx, indent_level, new_indent_level, output,
+                             is_prettyprint=True, separate_prefix_column=False):
+  """Direct translation of Coordinator::PrintExecSummary() to recursively build a list
+  of rows of summary statistics, one per exec node
+
+  summary: the TExecSummary object that contains all the summary data
+
+  idx: the index of the node to print
+
+  indent_level: the number of spaces to print before writing the node's label, to give
+  the appearance of a tree. The 0th child of a node has the same indent_level as its
+  parent. All other children have an indent_level of one greater than their parent.
+
+  new_indent_level: If true, this indent level is different from the previous row's.
+
+  output: the list of rows into which to append the rows produced for this node and its
+  children.
+
+  is_prettyprint: Optional. If True, print time, units, and bytes columns in pretty
+  printed format.
+
+  separate_prefix_column: Optional. If True, the prefix and operator name will be
+  returned as separate column. Otherwise, prefix and operater name will be concatenated
+  into single column.
+
+  Returns the index of the next exec node in summary.exec_nodes that should be
+  processed, used internally to this method only.
+  """
+  attrs = ["latency_ns", "cpu_time_ns", "cardinality", "memory_used"]
+
+  # Initialise aggregate and maximum stats
+  agg_stats, max_stats = TExecStats(), TExecStats()
+  for attr in attrs:
+    setattr(agg_stats, attr, 0)
+    setattr(max_stats, attr, 0)
+
+  node = summary.nodes[idx]
+  if node.exec_stats is not None:
+    for stats in node.exec_stats:
+      for attr in attrs:
+        val = getattr(stats, attr)
+        if val is not None:
+          setattr(agg_stats, attr, getattr(agg_stats, attr) + val)
+          setattr(max_stats, attr, max(getattr(max_stats, attr), val))
+
+  if node.exec_stats is not None and node.exec_stats:
+    avg_time = agg_stats.latency_ns / len(node.exec_stats)
+  else:
+    avg_time = 0
+
+  is_sink = node.node_id == -1
+  # If the node is a broadcast-receiving exchange node, the cardinality of rows produced
+  # is the max over all instances (which should all have received the same number of
+  # rows). Otherwise, the cardinality is the sum over all instances which process
+  # disjoint partitions.
+  if is_sink:
+    cardinality = -1
+  elif node.is_broadcast:
+    cardinality = max_stats.cardinality
+  else:
+    cardinality = agg_stats.cardinality
+
+  est_stats = node.estimated_stats
+  label_prefix = ""
+  if indent_level > 0:
+    label_prefix = "|"
+    label_prefix += "  |" * (indent_level - 1)
+    if new_indent_level:
+      label_prefix += "--"
+    else:
+      label_prefix += "  "
+
+  def prettyprint(val, units, divisor):
+    for unit in units:
+      if val < divisor:
+        if unit == units[0]:
+          return "%d%s" % (val, unit)
+        else:
+          return "%3.2f%s" % (val, unit)
+      val /= divisor
+
+  def prettyprint_bytes(byte_val):
+    return prettyprint(byte_val, [' B', ' KB', ' MB', ' GB', ' TB'], 1024.0)
+
+  def prettyprint_units(unit_val):
+    return prettyprint(unit_val, ["", "K", "M", "B"], 1000.0)
+
+  def prettyprint_time(time_val):
+    return prettyprint(time_val, ["ns", "us", "ms", "s"], 1000.0)
+
+  instances = 0
+  if node.exec_stats is not None:
+    instances = len(node.exec_stats)
+  latency = max_stats.latency_ns
+  cardinality_est = est_stats.cardinality
+  memory_used = max_stats.memory_used
+  memory_est = est_stats.memory_used
+  if (is_prettyprint):
+    avg_time = prettyprint_time(avg_time)
+    latency = prettyprint_time(latency)
+    cardinality = "" if is_sink else prettyprint_units(cardinality)
+    cardinality_est = "" if is_sink else prettyprint_units(cardinality_est)
+    memory_used = prettyprint_bytes(memory_used)
+    memory_est = prettyprint_bytes(memory_est)
+
+  row = list()
+  if separate_prefix_column:
+    row.append(label_prefix)
+    row.append(node.label)
+  else:
+    row.append(label_prefix + node.label)
+
+  row.extend([
+    node.num_hosts,
+    instances,
+    avg_time,
+    latency,
+    cardinality,
+    cardinality_est,
+    memory_used,
+    memory_est,
+    node.label_detail])
+
+  output.append(row)
+  try:
+    sender_idx = summary.exch_to_sender_map[idx]
+    # This is an exchange node or a join node with a separate builder, so the source
+    # is a fragment root, and should be printed next.
+    sender_indent_level = indent_level + node.num_children
+    sender_new_indent_level = node.num_children > 0
+    build_exec_summary_table(summary, sender_idx, sender_indent_level,
+                             sender_new_indent_level, output, is_prettyprint,
+                             separate_prefix_column)
+  except (KeyError, TypeError):
+    # Fall through if idx not in map, or if exch_to_sender_map itself is not set
+    pass
+
+  idx += 1
+  if node.num_children > 0:
+    first_child_output = []
+    idx = build_exec_summary_table(summary, idx, indent_level, False, first_child_output,
+                                   is_prettyprint, separate_prefix_column)
+    for child_idx in xrange(1, node.num_children):
+      # All other children are indented (we only have 0, 1 or 2 children for every exec
+      # node at the moment)
+      idx = build_exec_summary_table(summary, idx, indent_level + 1, True, output,
+                                     is_prettyprint, separate_prefix_column)
+    output += first_child_output
+  return idx
+
+
 class QueryOptionLevels:
   """These are the levels used when displaying query options.
   The values correspond to the ones in TQueryOptionLevel"""
@@ -137,7 +290,7 @@ class ImpalaClient(object):
                verbose=True, use_http_base_transport=False, http_path=None,
                http_cookie_names=None, http_socket_timeout_s=None, value_converter=None,
                connect_max_tries=4, rpc_stdout=False, rpc_file=None, http_tracing=True,
-               jwt=None):
+               jwt=None, oauth=None, hs2_x_forward=None):
     self.connected = False
     self.impalad_host = impalad[0]
     self.impalad_port = int(impalad[1])
@@ -161,6 +314,7 @@ class ImpalaClient(object):
     self.http_cookie_names = http_cookie_names
     self.http_tracing = http_tracing
     self.jwt = jwt
+    self.oauth = oauth
     # This is set from ImpalaShell's signal handler when a query is cancelled
     # from command line via CTRL+C. It is used to suppress error messages of
     # query cancellation.
@@ -172,6 +326,8 @@ class ImpalaClient(object):
     self.value_converter = value_converter
     self.rpc_stdout = rpc_stdout
     self.rpc_file = rpc_file
+    # In h2s-http clients only, the value of the X-Forwarded-For http header.
+    self.hs2_x_forward = hs2_x_forward
 
   def connect(self):
     """Creates a connection to an Impalad instance. Returns a tuple with the impala
@@ -194,7 +350,7 @@ class ImpalaClient(object):
     try:
       self._open_session()
       return self._ping_impala_service()
-    except:
+    except Exception:
       # Ensure we are in a disconnected state if we failed above.
       self.close_connection()
       raise
@@ -269,7 +425,9 @@ class ImpalaClient(object):
     If 'periodic_callback' is provided, it is called periodically with no arguments."""
     loop_start = time.time()
     while True:
+      start_rpc_time = time.time()
       query_state = self.get_query_state(last_query_handle)
+      rpc_time = time.time() - start_rpc_time
       if query_state == self.FINISHED_STATE:
         break
       elif query_state in (self.ERROR_STATE, self.CANCELED_STATE):
@@ -280,7 +438,9 @@ class ImpalaClient(object):
           raise DisconnectedException("Not connected to impalad.")
 
       if periodic_callback is not None: periodic_callback()
-      time.sleep(self._get_sleep_interval(loop_start))
+      sleep_time = self._get_sleep_interval(loop_start)
+      if rpc_time < sleep_time:
+        time.sleep(sleep_time - rpc_time)
 
   def get_query_state(self, last_query_handle):
     """Return the query state string for 'last_query_handle'. Returns self.ERROR_STATE
@@ -414,11 +574,13 @@ class ImpalaClient(object):
       url = "https://{0}/{1}".format(host_and_port, self.http_path)
       transport = ImpalaHttpClient(url, ssl_context=ssl_ctx,
                                    http_cookie_names=self.http_cookie_names,
-                                   socket_timeout_s=self.http_socket_timeout_s)
+                                   socket_timeout_s=self.http_socket_timeout_s,
+                                   verbose=self.verbose)
     else:
       url = "http://{0}/{1}".format(host_and_port, self.http_path)
       transport = ImpalaHttpClient(url, http_cookie_names=self.http_cookie_names,
-                                   socket_timeout_s=self.http_socket_timeout_s)
+                                   socket_timeout_s=self.http_socket_timeout_s,
+                                   verbose=self.verbose)
 
     if self.use_ldap:
       # Set the BASIC authorization
@@ -431,6 +593,8 @@ class ImpalaClient(object):
       transport.setLdapAuth(auth)
     elif self.jwt is not None:
       transport.setJwtAuth(self.jwt)
+    elif self.oauth is not None:
+      transport.setOAuthAuth(self.oauth)
     elif self.use_kerberos or self.kerberos_host_fqdn:
       # Set the Kerberos service
       if self.kerberos_host_fqdn is not None:
@@ -517,130 +681,9 @@ class ImpalaClient(object):
     sock.setTimeout(None)
     return transport
 
-  def build_summary_table(self, summary, idx, is_fragment_root, indent_level,
-      new_indent_level, output):
-    """Direct translation of Coordinator::PrintExecSummary() to recursively build a list
-    of rows of summary statistics, one per exec node
-
-    summary: the TExecSummary object that contains all the summary data
-
-    idx: the index of the node to print
-
-    is_fragment_root: true if the node to print is the root of a fragment (and therefore
-    feeds into an exchange)
-
-    indent_level: the number of spaces to print before writing the node's label, to give
-    the appearance of a tree. The 0th child of a node has the same indent_level as its
-    parent. All other children have an indent_level of one greater than their parent.
-
-    output: the list of rows into which to append the rows produced for this node and its
-    children.
-
-    Returns the index of the next exec node in summary.exec_nodes that should be
-    processed, used internally to this method only.
-
-    NOTE: This is duplicated in impala_beeswax.py, and changes made here should also be
-    made there. TODO: refactor into a shared library. (IMPALA-5792)
-    """
-    attrs = ["latency_ns", "cpu_time_ns", "cardinality", "memory_used"]
-
-    # Initialise aggregate and maximum stats
-    agg_stats, max_stats = TExecStats(), TExecStats()
-    for attr in attrs:
-      setattr(agg_stats, attr, 0)
-      setattr(max_stats, attr, 0)
-
-    node = summary.nodes[idx]
-    if node.exec_stats is not None:
-      for stats in node.exec_stats:
-        for attr in attrs:
-          val = getattr(stats, attr)
-          if val is not None:
-            setattr(agg_stats, attr, getattr(agg_stats, attr) + val)
-            setattr(max_stats, attr, max(getattr(max_stats, attr), val))
-
-    if node.exec_stats is not None and node.exec_stats:
-      avg_time = agg_stats.latency_ns / len(node.exec_stats)
-    else:
-      avg_time = 0
-
-    # If the node is a broadcast-receiving exchange node, the cardinality of rows produced
-    # is the max over all instances (which should all have received the same number of
-    # rows). Otherwise, the cardinality is the sum over all instances which process
-    # disjoint partitions.
-    if node.is_broadcast:
-      cardinality = max_stats.cardinality
-    else:
-      cardinality = agg_stats.cardinality
-
-    est_stats = node.estimated_stats
-    label_prefix = ""
-    if indent_level > 0:
-      label_prefix = "|"
-      label_prefix += "  |" * (indent_level - 1)
-      if new_indent_level:
-        label_prefix += "--"
-      else:
-        label_prefix += "  "
-
-    def prettyprint(val, units, divisor):
-      for unit in units:
-        if val < divisor:
-          if unit == units[0]:
-            return "%d%s" % (val, unit)
-          else:
-            return "%3.2f%s" % (val, unit)
-        val /= divisor
-
-    def prettyprint_bytes(byte_val):
-      return prettyprint(byte_val, [' B', ' KB', ' MB', ' GB', ' TB'], 1024.0)
-
-    def prettyprint_units(unit_val):
-      return prettyprint(unit_val, ["", "K", "M", "B"], 1000.0)
-
-    def prettyprint_time(time_val):
-      return prettyprint(time_val, ["ns", "us", "ms", "s"], 1000.0)
-
-    instances = 0
-    if node.exec_stats is not None:
-      instances = len(node.exec_stats)
-    is_sink = node.node_id == -1
-    row = [ label_prefix + node.label,
-            node.num_hosts, instances,
-            prettyprint_time(avg_time),
-            prettyprint_time(max_stats.latency_ns),
-            "" if is_sink else prettyprint_units(cardinality),
-            "" if is_sink else prettyprint_units(est_stats.cardinality),
-            prettyprint_bytes(max_stats.memory_used),
-            prettyprint_bytes(est_stats.memory_used),
-            node.label_detail ]
-
-    output.append(row)
-    try:
-      sender_idx = summary.exch_to_sender_map[idx]
-      # This is an exchange node or a join node with a separate builder, so the source
-      # is a fragment root, and should be printed next.
-      sender_indent_level = indent_level + node.num_children
-      sender_new_indent_level = node.num_children > 0
-      self.build_summary_table(
-          summary, sender_idx, True, sender_indent_level, sender_new_indent_level, output)
-    except (KeyError, TypeError):
-      # Fall through if idx not in map, or if exch_to_sender_map itself is not set
-      pass
-
-    idx += 1
-    if node.num_children > 0:
-      first_child_output = []
-      idx = \
-        self.build_summary_table(
-            summary, idx, False, indent_level, False, first_child_output)
-      for child_idx in xrange(1, node.num_children):
-        # All other children are indented (we only have 0, 1 or 2 children for every exec
-        # node at the moment)
-        idx = self.build_summary_table(
-            summary, idx, False, indent_level + 1, True, output)
-      output += first_child_output
-    return idx
+  def build_summary_table(self, summary, output):
+    build_exec_summary_table(summary, 0, 0, False, output, is_prettyprint=True,
+                             separate_prefix_column=False)
 
   def _get_sleep_interval(self, start_time):
     """Returns a step function of time to sleep in seconds before polling
@@ -720,8 +763,8 @@ class ImpalaHS2Client(ImpalaClient):
                           username=self.user)
     resp = self._do_hs2_rpc(OpenSession, req, retry_on_error=True)
     self._check_hs2_rpc_status(resp.status)
-    assert (resp.serverProtocolVersion ==
-            TProtocolVersion.HIVE_CLI_SERVICE_PROTOCOL_V6), resp.serverProtocolVersion
+    assert (resp.serverProtocolVersion
+            == TProtocolVersion.HIVE_CLI_SERVICE_PROTOCOL_V6), resp.serverProtocolVersion
     # TODO: ensure it's closed if needed
     self.session_handle = resp.sessionHandle
 
@@ -742,10 +785,11 @@ class ImpalaHS2Client(ImpalaClient):
       assert getattr(self, "_current_request_id", None) is not None, \
         "request id was not set"
       headers["X-Request-Id"] = self._current_request_id
+    if self.hs2_x_forward:
+      headers["X-Forwarded-For"] = self.hs2_x_forward
 
     return headers
 
-
   def close_connection(self):
     if self.session_handle is not None:
       # Attempt to close session explicitly. Do not fail if there is an error
@@ -760,7 +804,8 @@ class ImpalaHS2Client(ImpalaClient):
         resp = self._do_hs2_rpc(CloseSession, req)
         self._check_hs2_rpc_status(resp.status)
       except Exception as e:
-        print("Warning: close session RPC failed: {0}, {1}".format(str(e), type(e)))
+        log_exception_with_timestamp(e, "Warning",
+           "close session RPC failed: {0}".format(type(e)), stderr_flag=False)
       self.session_handle = None
     self._close_transport()
 
@@ -788,16 +833,16 @@ class ImpalaHS2Client(ImpalaClient):
         QueryStateException):
         raise
       except RPCException as r:
-        if (r.exception_type == RPC_EXCEPTION_TAPPLICATION or
-          r.exception_type == RPC_EXCEPTION_SERVER):
+        if (r.exception_type == RPC_EXCEPTION_TAPPLICATION
+            or r.exception_type == RPC_EXCEPTION_SERVER):
           raise
-        print('Caught exception {0}, type={1} when listing query options. {2}'
-          .format(str(r), type(r), retry_msg), file=sys.stderr)
+        log_exception_with_timestamp(r, "Exception",
+           "type={0} when listing query options. {1}".format(type(r), retry_msg))
         if raise_error:
           raise
       except Exception as e:
-        print('Caught exception {0}, type={1} when listing query options. {2}'
-          .format(str(e), type(e), retry_msg), file=sys.stderr)
+        log_exception_with_timestamp(e, "Exception",
+           "type={0} when listing query options. {1}".format(type(e), retry_msg))
         if raise_error:
           raise
       finally:
@@ -1124,7 +1169,6 @@ class ImpalaHS2Client(ImpalaClient):
 
     self._request_num += 1
     self._current_request_id = "{0}-{1}".format(self._base_request_id, self._request_num)
-
     self._check_connected()
     num_tries = 1
     max_tries = num_tries
@@ -1151,8 +1195,8 @@ class ImpalaHS2Client(ImpalaClient):
         if isinstance(e.inner, socket.error):
           e = e.inner
         # issue with the connection with the impalad
-        print('Caught exception {0}, type={1} in {2}. {3}'
-          .format(str(e), type(e), rpc.__name__, retry_msg), file=sys.stderr)
+        log_exception_with_timestamp(e, "Exception",
+           "type={0} in {1}. {2}".format(type(e), rpc.__name__, retry_msg))
         self._print_rpc_end(rpc, None, start_time, "Error - TTransportException")
         if raise_error:
           if isinstance(e, TTransportException):
@@ -1176,18 +1220,18 @@ class ImpalaHS2Client(ImpalaClient):
             except ValueError:
               retry_secs = None
         if retry_secs:
-          print('Caught exception {0}, type={1} in {2}. {3}, retry after {4} secs'
-                .format(str(h), type(h), rpc.__name__, retry_msg, retry_secs),
-                file=sys.stderr)
+          log_exception_with_timestamp(h, "Exception",
+              "type={0} in {1}. {2}, retry after {3} secs"
+              .format(type(h), rpc.__name__, retry_msg, retry_secs))
         else:
-          print('Caught exception {0}, type={1} in {2}. {3}'
-                .format(str(h), type(h), rpc.__name__, retry_msg), file=sys.stderr)
+          log_exception_with_timestamp(h, "Exception", "type={0} in {1}. {2}"
+                .format(type(h), rpc.__name__, retry_msg))
         self._print_rpc_end(rpc, None, start_time, "Error - HttpError")
         if raise_error:
           raise
       except Exception as e:
-        print('Caught exception {0}, type={1} in {2}. {3}'
-          .format(str(e), type(e), rpc.__name__, retry_msg), file=sys.stderr)
+        log_exception_with_timestamp(e, "Exception", "type={0} in {1}. {2}"
+          .format(type(e), rpc.__name__, retry_msg))
         self._print_rpc_end(rpc, None, start_time, "Error")
         if raise_error:
           raise
@@ -1454,7 +1498,7 @@ class ImpalaBeeswaxClient(ImpalaClient):
     dml_result, rpc_status = self._do_beeswax_rpc(
         lambda: self.imp_service.CloseInsert(last_query_handle))
     if rpc_status != RpcStatus.OK:
-       raise RPCException()
+      raise RPCException()
     last_query_handle.is_closed = True
     return self._process_dml_result(dml_result)
 
@@ -1603,6 +1647,24 @@ class ImpalaBeeswaxClient(ImpalaClient):
         if "QueryNotFoundException" in str(e):
           raise QueryStateException('Error: Stale query handle')
         # Print more details for other kinds of exceptions
-        print('Caught exception {0}, type={1}'.format(str(e), type(e)), file=sys.stderr)
+        log_exception_with_timestamp(e, "Exception", "type={0}".format(type(e)))
         traceback.print_exc()
         raise Exception("Encountered unknown exception")
+
+
+def log_exception_with_timestamp(e, type="Exception", msg="", stderr_flag=True):
+  # method log_exception_with_timestamp prints timestamp with exception trace
+  # and accepts custom message before timestamp. stderr_flag controls print statement
+  # to be logged in stderr, by default it is true.
+  if(stderr_flag):
+      print("%s [%s] %s" % (time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()), type,
+      msg), e, file=sys.stderr)
+  else:
+      print("%s [%s] %s" % (time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()), type,
+       msg), e)
+
+
+def log_timestamp(type="Exception", msg=""):
+  # method log_timestamp prints timestamp with custom message
+  print("%s [%s] %s" % (time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()), type, msg),
+      file=sys.stderr)
