@@ -232,6 +232,10 @@ def get_option_parser(defaults):
                     action="store_true",
                     help="Use JWT to authenticate with Impala. Impala must be configured"
                     " to allow JWT authentication. \t\t")
+  parser.add_option("-a", "--oauth", dest="use_oauth",
+                    action="store_true",
+                    help="Use OAuth to authenticate with Impala. Impala must be"
+                    "configured to allow Oauth authentication. \t\t")
   parser.add_option("-u", "--user", dest="user",
                     help="User to authenticate with.")
   parser.add_option("--ssl", dest="ssl",
@@ -273,6 +277,8 @@ def get_option_parser(defaults):
                     help="Shell command to run to retrieve the LDAP password")
   parser.add_option("--jwt_cmd", dest="jwt_cmd",
                     help="Shell command to run to retrieve the JWT")
+  parser.add_option("--oauth_cmd", dest="oauth_cmd",
+                    help="Shell command to run to retrieve the Oauth Token")
   parser.add_option("--var", dest="keyval", action="append",
                     help="Defines a variable to be used within the Impala session."
                          " Can be used multiple times to set different variables."
@@ -333,12 +339,14 @@ def get_option_parser(defaults):
                     "spooling is disabled only a single row batch can be fetched at a "
                     "time regardless of the specified fetch_size.")
   parser.add_option("--http_cookie_names", dest="http_cookie_names",
-                    default="impala.auth,impala.session.id",
+                    default="*",
                     help="A comma-separated list of HTTP cookie names that are supported "
                     "by the impala-shell. If a cookie with one of these names is "
                     "returned in an http response by the server or an intermediate proxy "
                     "then it will be included in each subsequent request for the same "
-                    "connection.")
+                    "connection. If set to wildcard (*), all cookies in an http response "
+                    "will be preserved. The name of an authentication cookie must end "
+                    "with '.auth', for example 'impala.auth'.")
   parser.add_option("--no_http_tracing", dest="no_http_tracing",
                     action="store_true",
                     help="Tracing http headers 'X-Request-Id', 'X-Impala-Session-Id', "
@@ -350,6 +358,10 @@ def get_option_parser(defaults):
                     "values when using the HS2 protocol. The default behaviour makes the "
                     "values handled by Python's str() built-in method. Use '16G' to "
                     "match the Beeswax protocol's floating-point output format.")
+  parser.add_option("--hs2_x_forward", type="str",
+                    dest="hs2_x_forward", default=None,
+                    help="When using the hs2-http protocol, set this value in the "
+                    "X-Forwarded-For header. This is primarily for testing purposes.")
 
   # add default values to the help text
   for option in parser.option_list:
