@@ -26,21 +26,17 @@
 #   result = client.execute(query_string)
 #   where result is an object of the class ImpalaBeeswaxResult.
 from __future__ import absolute_import, division, print_function
-from builtins import filter, map, range
+from builtins import filter, map
 import logging
 import time
 import shlex
 import getpass
 import re
+import sys
 
 from beeswaxd import BeeswaxService
 from beeswaxd.BeeswaxService import QueryState
 from datetime import datetime
-try:
-  # If Exec Summary is not implemented in Impala, this cannot be imported
-  from ExecStats.ttypes import TExecStats
-except ImportError:
-  pass
 from ImpalaService import ImpalaService
 from tests.util.thrift_util import create_transport
 from thrift.transport.TTransport import TTransportException
@@ -48,19 +44,25 @@ from thrift.protocol import TBinaryProtocol
 from thrift.Thrift import TApplicationException
 
 LOG = logging.getLogger('impala_beeswax')
+# time to sleep in seconds before polling again. This uses a fixed
+# 50 millisecond sleep that doesn't vary by elapsed time. This is only used for
+# testing, and there is no reason to sleep longer for test environments.
+DEFAULT_SLEEP_INTERVAL = 0.05
+
 
 # Custom exception wrapper.
 # All exceptions coming from thrift/beeswax etc. go through this wrapper.
-# __str__ preserves the exception type.
 # TODO: Add the ability to print some of the stack.
 class ImpalaBeeswaxException(Exception):
   __name__ = "ImpalaBeeswaxException"
+
   def __init__(self, message, inner_exception):
     self.__message = message
     self.inner_exception = inner_exception
 
   def __str__(self):
-    return "%s:\n %s" % (self.__name__, self.__message)
+    return self.__message
+
 
 class ImpalaBeeswaxResult(object):
   def __init__(self, **kwargs):
@@ -100,10 +102,10 @@ class ImpalaBeeswaxResult(object):
                'Took: %s(s)\n'
                'Data:\n%s\n'
                % (self.summary, self.success, self.time_taken,
-                  self.__format_data())
-              )
+                  self.__format_data()))
     return message
 
+
 # Interface to beeswax. Responsible for executing queries, fetching results.
 class ImpalaBeeswaxClient(object):
   # Regex applied to all tokens of a query to detect the query type.
@@ -134,7 +136,7 @@ class ImpalaBeeswaxClient(object):
     return self.__query_options
 
   def set_query_option(self, name, value):
-    self.__query_options[name.upper()] = value
+    self.__query_options[name.lower()] = value
 
   def set_query_options(self, query_option_dict):
     if query_option_dict is None:
@@ -144,7 +146,7 @@ class ImpalaBeeswaxClient(object):
       self.set_query_option(name, value)
 
   def get_query_option(self, name):
-    return self.__query_options.get(name.upper())
+    return self.__query_options.get(name.lower())
 
   def clear_query_options(self):
     self.__query_options.clear()
@@ -244,121 +246,24 @@ class ImpalaBeeswaxClient(object):
     if summary is None or summary.nodes is None:
       return None
       # If exec summary is not implemented in Impala, this function returns, so we do not
-      # get the function __build_summary_table which requires TExecStats to be imported.
+      # get the function build_exec_summary_table which requires TExecStats to be
+      # imported.
 
-    output = []
-    self.__build_summary_table(summary, 0, False, 0, False, output)
+    output = list()
+    self.__build_summary_table(summary, output)
     return output
 
-  def __build_summary_table(self, summary, idx, is_fragment_root, indent_level,
-      new_indent_level, output):
-    """NOTE: This was taken from impala_shell.py. Changes made here must be made there as
-    well. TODO: This method will be a placed in a library that is shared between
-    impala_shell and this file. (IMPALA-5792)
-
-    Direct translation of Coordinator::PrintExecSummary() to recursively build a list
-    of rows of summary statistics, one per exec node
-
-    summary: the TExecSummary object that contains all the summary data
-
-    idx: the index of the node to print
-
-    is_fragment_root: true if the node to print is the root of a fragment (and therefore
-    feeds into an exchange)
-
-    indent_level: the number of spaces to print before writing the node's label, to give
-    the appearance of a tree. The 0th child of a node has the same indent_level as its
-    parent. All other children have an indent_level of one greater than their parent.
-
-    new_indent_level: If true, this indent level is different from the previous row's.
-
-    output: the list of rows into which to append the rows produced for this node and its
-    children.
-
-    Returns the index of the next exec node in summary.exec_nodes that should be
-    processed, used internally to this method only.
-    """
-    attrs = ["latency_ns", "cpu_time_ns", "cardinality", "memory_used"]
-
-    # Initialise aggregate and maximum stats
-    agg_stats, max_stats = TExecStats(), TExecStats()
-    for attr in attrs:
-      setattr(agg_stats, attr, 0)
-      setattr(max_stats, attr, 0)
-
-    row = {}
-    node = summary.nodes[idx]
-    # exec_stats may not be set even if the query is FINISHED if there are fragments that
-    # are still executing or that were cancelled before sending a status report.
-    if node.exec_stats is not None:
-      for stats in node.exec_stats:
-        for attr in attrs:
-          val = getattr(stats, attr)
-          if val is not None:
-            setattr(agg_stats, attr, getattr(agg_stats, attr) + val)
-            setattr(max_stats, attr, max(getattr(max_stats, attr), val))
-
-      if len(node.exec_stats) > 0:
-        avg_time = agg_stats.latency_ns // len(node.exec_stats)
-      else:
-        avg_time = 0
-
-      row["num_instances"] = len(node.exec_stats)
-      row["num_hosts"] = node.num_hosts
-      row["avg_time"] = avg_time
-
-    is_sink = node.node_id == -1
-    # If the node is a broadcast-receiving exchange node, the cardinality of rows produced
-    # is the max over all instances (which should all have received the same number of
-    # rows). Otherwise, the cardinality is the sum over all instances which process
-    # disjoint partitions.
-    if is_sink:
-      cardinality = -1
-    elif node.is_broadcast:
-      cardinality = max_stats.cardinality
-    else:
-      cardinality = agg_stats.cardinality
-
-    est_stats = node.estimated_stats
-
-    label_prefix = ""
-    if indent_level > 0:
-      label_prefix = "|"
-      label_prefix += "  |" * (indent_level - 1)
-      if new_indent_level:
-        label_prefix += "--"
-      else:
-        label_prefix += "  "
-
-    row["prefix"] = label_prefix
-    row["operator"] = node.label
-    row["max_time"] = max_stats.latency_ns
-    row["num_rows"] = cardinality
-    row["est_num_rows"] = est_stats.cardinality
-    row["peak_mem"] = max_stats.memory_used
-    row["est_peak_mem"] = est_stats.memory_used
-    row["detail"] = node.label_detail
-    output.append(row)
-
-    if summary.exch_to_sender_map is not None and idx in summary.exch_to_sender_map:
-      sender_idx = summary.exch_to_sender_map[idx]
-      # This is an exchange node, so the sender is a fragment root, and should be printed
-      # next.
-      self.__build_summary_table(summary, sender_idx, True, indent_level, False, output)
-
-    idx += 1
-    if node.num_children > 0:
-      first_child_output = []
-      idx = \
-        self.__build_summary_table(
-            summary, idx, False, indent_level, False, first_child_output)
-      for child_idx in range(1, node.num_children):
-        # All other children are indented (we only have 0, 1 or 2 children for every exec
-        # node at the moment)
-        idx = self.__build_summary_table(
-            summary, idx, False, indent_level + 1, True, output)
-      output += first_child_output
-    return idx
+  def __build_summary_table(self, summary, output):
+    from shell.impala_client import build_exec_summary_table
+    result = list()
+    build_exec_summary_table(summary, 0, 0, False, result, is_prettyprint=False,
+                             separate_prefix_column=True)
+    keys = ['prefix', 'operator', 'num_hosts', 'num_instances', 'avg_time', 'max_time',
+            'num_rows', 'est_num_rows', 'peak_mem', 'est_peak_mem', 'detail']
+    for row in result:
+      assert len(keys) == len(row)
+      summ_map = dict(zip(keys, row))
+      output.append(summ_map)
 
   def get_runtime_profile(self, handle):
     return self.__do_rpc(lambda: self.imp_service.GetRuntimeProfile(handle))
@@ -394,7 +299,9 @@ class ImpalaBeeswaxClient(object):
     """Given a query handle, polls the coordinator waiting for the query to transition to
        'FINISHED' state"""
     while True:
+      start_rpc_time = time.time()
       query_state = self.get_state(query_handle)
+      rpc_time = time.time() - start_rpc_time
       # if the rpc succeeded, the output is the query state
       if query_state == self.query_states["FINISHED"]:
         break
@@ -402,17 +309,20 @@ class ImpalaBeeswaxClient(object):
         try:
           error_log = self.__do_rpc(
             lambda: self.imp_service.get_log(query_handle.log_context))
-          raise ImpalaBeeswaxException("Query aborted:" + error_log, None)
+          raise ImpalaBeeswaxException(error_log, None)
         finally:
           self.close_query(query_handle)
-      time.sleep(0.05)
+      if rpc_time < DEFAULT_SLEEP_INTERVAL:
+        time.sleep(DEFAULT_SLEEP_INTERVAL - rpc_time)
 
   def wait_for_finished_timeout(self, query_handle, timeout=10):
     """Given a query handle and a timeout, polls the coordinator waiting for the query to
        transition to 'FINISHED' state till 'timeout' seconds"""
     start_time = time.time()
     while (time.time() - start_time < timeout):
+      start_rpc_time = time.time()
       query_state = self.get_state(query_handle)
+      rpc_time = time.time() - start_rpc_time
       # if the rpc succeeded, the output is the query state
       if query_state == self.query_states["FINISHED"]:
         return True
@@ -420,20 +330,27 @@ class ImpalaBeeswaxClient(object):
         try:
           error_log = self.__do_rpc(
             lambda: self.imp_service.get_log(query_handle.log_context))
-          raise ImpalaBeeswaxException("Query aborted:" + error_log, None)
+          raise ImpalaBeeswaxException(error_log, None)
         finally:
           self.close_query(query_handle)
-      time.sleep(0.05)
+      if rpc_time < DEFAULT_SLEEP_INTERVAL:
+        time.sleep(DEFAULT_SLEEP_INTERVAL - rpc_time)
     return False
 
-  def wait_for_admission_control(self, query_handle):
+  def wait_for_admission_control(self, query_handle, timeout_s=60):
     """Given a query handle, polls the coordinator waiting for it to complete
-      admission control processing of the query"""
-    while True:
+      admission control processing of the query.
+      Return True if query pass admission control after given timeout_s."""
+    start_time = time.time()
+    while (time.time() - start_time < timeout_s):
+      start_rpc_time = time.time()
       query_state = self.get_state(query_handle)
+      rpc_time = time.time() - start_rpc_time
       if query_state > self.query_states["COMPILED"]:
-        break
-      time.sleep(0.05)
+        return True
+      if rpc_time < DEFAULT_SLEEP_INTERVAL:
+        time.sleep(DEFAULT_SLEEP_INTERVAL - rpc_time)
+    return False
 
   def get_admission_result(self, query_handle):
     """Given a query handle, returns the admission result from the query profile"""
@@ -454,7 +371,7 @@ class ImpalaBeeswaxClient(object):
   def get_log(self, query_handle):
     return self.__do_rpc(lambda: self.imp_service.get_log(query_handle))
 
-  def fetch_results(self, query_string, query_handle, max_rows = -1):
+  def fetch_results(self, query_string, query_handle, max_rows=-1):
     """Fetches query results given a handle and query type (insert, use, other)"""
     query_type = self.__get_query_type(query_string)
     if query_type == 'use':
@@ -472,7 +389,7 @@ class ImpalaBeeswaxClient(object):
     exec_result.query = query_string
     return exec_result
 
-  def __fetch_results(self, handle, max_rows = -1):
+  def __fetch_results(self, handle, max_rows=-1):
     """Handles query results, returns a ImpalaBeeswaxResult object"""
     schema = self.__do_rpc(lambda: self.imp_service.get_results_metadata(handle)).schema
     # The query has finished, we can fetch the results
@@ -504,6 +421,10 @@ class ImpalaBeeswaxClient(object):
     return exec_result
 
   def __get_query_type(self, query_string):
+    # Python 2's shlex does not work if the query string contains Unicode characters.
+    # Convert to bytes.
+    if sys.version_info.major == 2:
+      query_string = query_string.encode('utf-8')
     # Set posix=True and add "'" to escaped quotes
     # to deal with escaped quotes in string literals
     lexer = shlex.shlex(query_string.lstrip(), posix=True)
@@ -523,7 +444,7 @@ class ImpalaBeeswaxClient(object):
     message = str(exception)
     if isinstance(exception, BeeswaxService.BeeswaxException):
       message = exception.message
-    return 'INNER EXCEPTION: %s\n MESSAGE: %s' % (exception.__class__, message)
+    return message
 
   def __do_rpc(self, rpc):
     """Executes the RPC lambda provided with some error checking.
