@@ -32,7 +32,7 @@ from RuntimeProfile.ttypes import TRuntimeProfileFormat
 from tests.beeswax.impala_beeswax import ImpalaBeeswaxClient
 
 
-LOG = logging.getLogger('impala_connection')
+LOG = logging.getLogger(__name__)
 console_handler = logging.StreamHandler()
 console_handler.setLevel(logging.INFO)
 # All logging needs to be either executable SQL or a SQL comment (prefix with --).
@@ -54,17 +54,30 @@ MAX_SQL_LOGGING_LENGTH = 128 * 1024
 def log_sql_stmt(sql_stmt):
   """If the 'sql_stmt' is shorter than MAX_SQL_LOGGING_LENGTH, log it unchanged. If
      it is larger than MAX_SQL_LOGGING_LENGTH, truncate it and comment it out."""
+  # sql_stmt could contain Unicode characters, so explicitly use unicode literals
+  # so that Python 2 works.
   if (len(sql_stmt) <= MAX_SQL_LOGGING_LENGTH):
-    LOG.info("{0};\n".format(sql_stmt))
+    LOG.info(u"{0};\n".format(sql_stmt))
   else:
     # The logging output should be valid SQL, so the truncated SQL is commented out.
     LOG.info("-- Skip logging full SQL statement of length {0}".format(len(sql_stmt)))
     LOG.info("-- Logging a truncated version, commented out:")
     for line in sql_stmt[0:MAX_SQL_LOGGING_LENGTH].split("\n"):
-      LOG.info("-- {0}".format(line))
+      LOG.info(u"-- {0}".format(line))
     LOG.info("-- [...]")
 
 
+def collect_default_query_options(options, name, val, kind):
+  if kind == 'REMOVED':
+    return
+  name = name.lower()
+  val = str(val).strip('"')
+  if ',' in val:
+    # Value is a list. Wrap it with double quote.
+    val = '"{}"'.format(val)
+  options[name] = val
+
+
 # Common wrapper around the internal types of HS2/Beeswax operation/query handles.
 class OperationHandle(object):
   def __init__(self, handle, sql_stmt):
@@ -87,6 +100,17 @@ class ImpalaConnection(with_metaclass(abc.ABCMeta, object)):
   def __exit__(self, exc_type, exc_value, traceback):
     self.close()
 
+  @abc.abstractmethod
+  def get_test_protocol(self):
+    """Return client protocol name that is specific to Impala test framework.
+    Possible return value are either of 'beeswax', 'hs2', or 'hs2-http'."""
+    pass
+
+  @abc.abstractmethod
+  def get_host_port(self):
+    """Return the 'host:port' string of impala server that this object connecting to."""
+    pass
+
   @abc.abstractmethod
   def set_configuration_option(self, name, value):
     """Sets a configuration option name to the given value"""
@@ -164,6 +188,18 @@ class ImpalaConnection(with_metaclass(abc.ABCMeta, object)):
     rows returned is less than max_rows, all the rows have been fetched."""
     pass
 
+  @abc.abstractmethod
+  def handle_id(self, operation_handle):
+    """Return a string id for given operation_handle.
+    Most implementations will return an Impala query id for given handle.
+    Otherwise, return str(operation_handle)."""
+    pass
+
+  @abc.abstractmethod
+  def log_handle(self, operation_handle, message):
+    """Log 'message' at INFO level, along with id of 'operation_handle'."""
+    pass
+
 
 # Represents a connection to Impala using the Beeswax API.
 class BeeswaxConnection(ImpalaConnection):
@@ -172,19 +208,37 @@ class BeeswaxConnection(ImpalaConnection):
     self.__beeswax_client = ImpalaBeeswaxClient(host_port, use_kerberos, user=user,
                                                 password=password, use_ssl=use_ssl)
     self.__host_port = host_port
+    # Default query option, obtained at first call to get_default_configuration().
+    # Query option names are in lower case for consistency.
+    self.__default_query_options = None
     self.QUERY_STATES = self.__beeswax_client.query_states
 
+  def get_test_protocol(self):
+    return 'beeswax'
+
+  def get_host_port(self):
+    return self.__host_port
+
   def set_configuration_option(self, name, value):
     # Only set the option if it's not already set to the same value.
+    name = name.lower()
+    value = str(value)
     if self.__beeswax_client.get_query_option(name) != value:
-      LOG.info('SET %s=%s;' % (name, value))
+      LOG.info("set_option('{}', '{}')".format(name, value))
       self.__beeswax_client.set_query_option(name, value)
 
-  def get_default_configuration(self):
-    result = {}
+  def __collect_default_options(self):
+    options = {}
     for item in self.__beeswax_client.get_default_configuration():
-      result[item.key] = item.value
-    return result
+      collect_default_query_options(
+        options, item.key, item.value, str(item.level))
+    LOG.debug("Default query options: {0}".format(options))
+    return options
+
+  def get_default_configuration(self):
+    if self.__default_query_options is None:
+      self.__default_query_options = self.__collect_default_options()
+    return self.__default_query_options.copy()
 
   def clear_configuration(self):
     self.__beeswax_client.clear_query_options()
@@ -202,11 +256,11 @@ class BeeswaxConnection(ImpalaConnection):
     self.__beeswax_client.close_connection()
 
   def close_query(self, operation_handle):
-    LOG.info("-- closing query for operation handle: %s" % operation_handle)
+    self.log_handle(operation_handle, 'closing query for operation')
     self.__beeswax_client.close_query(operation_handle.get_handle())
 
   def close_dml(self, operation_handle):
-    LOG.info("-- closing DML query for operation handle: %s" % operation_handle)
+    self.log_handle(operation_handle, 'closing DML query')
     self.__beeswax_client.close_dml(operation_handle.get_handle())
 
   def execute(self, sql_stmt, user=None, fetch_profile_after_close=False):
@@ -222,48 +276,60 @@ class BeeswaxConnection(ImpalaConnection):
     return OperationHandle(beeswax_handle, sql_stmt)
 
   def cancel(self, operation_handle):
-    LOG.info("-- canceling operation: %s" % operation_handle)
+    self.log_handle(operation_handle, 'canceling operation')
     return self.__beeswax_client.cancel_query(operation_handle.get_handle())
 
   def get_state(self, operation_handle):
-    LOG.info("-- getting state for operation: %s" % operation_handle)
+    self.log_handle(operation_handle, 'getting state')
     return self.__beeswax_client.get_state(operation_handle.get_handle())
 
   def state_is_finished(self, operation_handle):
-    LOG.info("-- checking finished state for operation: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'checking finished state for operation')
     return self.get_state(operation_handle) == self.QUERY_STATES["FINISHED"]
 
   def get_exec_summary(self, operation_handle):
-    LOG.info("-- getting exec summary operation: %s" % operation_handle)
+    self.log_handle(operation_handle, 'getting exec summary operation')
     return self.__beeswax_client.get_exec_summary(operation_handle.get_handle())
 
   def get_runtime_profile(self, operation_handle):
-    LOG.info("-- getting runtime profile operation: %s" % operation_handle)
+    self.log_handle(operation_handle, 'getting runtime profile operation')
     return self.__beeswax_client.get_runtime_profile(operation_handle.get_handle())
 
   def wait_for_finished_timeout(self, operation_handle, timeout):
-    LOG.info("-- waiting for query to reach FINISHED state: %s" % operation_handle)
+    self.log_handle(operation_handle, 'waiting for query to reach FINISHED state')
     return self.__beeswax_client.wait_for_finished_timeout(
       operation_handle.get_handle(), timeout)
 
-  def wait_for_admission_control(self, operation_handle):
-    LOG.info("-- waiting for completion of the admission control processing of the "
-        "query: %s" % operation_handle)
-    return self.__beeswax_client.wait_for_admission_control(operation_handle.get_handle())
+  def wait_for_admission_control(self, operation_handle, timeout_s=60):
+    self.log_handle(operation_handle, 'waiting for completion of the admission control')
+    return self.__beeswax_client.wait_for_admission_control(
+      operation_handle.get_handle(), timeout_s=timeout_s)
 
   def get_admission_result(self, operation_handle):
-    LOG.info("-- getting the admission result: %s" % operation_handle)
+    self.log_handle(operation_handle, 'getting the admission result')
     return self.__beeswax_client.get_admission_result(operation_handle.get_handle())
 
   def get_log(self, operation_handle):
-    LOG.info("-- getting log for operation: %s" % operation_handle)
+    self.log_handle(operation_handle, 'getting log for operation')
     return self.__beeswax_client.get_log(operation_handle.get_handle().log_context)
 
   def fetch(self, sql_stmt, operation_handle, max_rows=-1):
-    LOG.info("-- fetching results from: %s" % operation_handle)
+    self.log_handle(operation_handle, 'fetching {} rows'.format(
+      'all' if max_rows < 0 else max_rows))
     return self.__beeswax_client.fetch_results(
         sql_stmt, operation_handle.get_handle(), max_rows)
 
+  def handle_id(self, operation_handle):
+    query_id = operation_handle.get_handle().id
+    return query_id if query_id else str(operation_handle)
+
+  def log_handle(self, operation_handle, message):
+    handle_id = self.handle_id(operation_handle)
+    LOG.info("-- {0}: {1}".format(handle_id, message))
+
+  def get_query_id(self, operation_handle):
+    return operation_handle.get_handle().id
+
 
 class ImpylaHS2Connection(ImpalaConnection):
   """Connection to Impala using the impyla client connecting to HS2 endpoint.
@@ -272,10 +338,12 @@ class ImpylaHS2Connection(ImpalaConnection):
   TODO: implement support for kerberos, SSL, etc.
   """
   def __init__(self, host_port, use_kerberos=False, is_hive=False,
-               use_http_transport=False, http_path=""):
+               use_http_transport=False, http_path="", use_ssl=False,
+               collect_profile_and_log=True):
     self.__host_port = host_port
     self.__use_http_transport = use_http_transport
     self.__http_path = http_path
+    self.__use_ssl = use_ssl
     if use_kerberos:
       raise NotImplementedError("Kerberos support not yet implemented")
     # Impyla connection and cursor is initialised in connect(). We need to reuse the same
@@ -284,12 +352,31 @@ class ImpylaHS2Connection(ImpalaConnection):
     # at a time per connection, which is a limitation also imposed by the Beeswax API.
     self.__impyla_conn = None
     self.__cursor = None
+    # Default query option obtained from initial connect.
+    # Query option names are in lower case for consistency.
+    self.__default_query_options = {}
     # Query options to send along with each query.
     self.__query_options = {}
     self._is_hive = is_hive
+    # Some Hive HS2 protocol, such as custom Calcite planner, may be able to collect
+    # profile and log from Impala.
+    self._collect_profile_and_log = collect_profile_and_log
+
+  def get_test_protocol(self):
+    if self.__http_path:
+      return 'hs2-http'
+    else:
+      return 'hs2'
+
+  def get_host_port(self):
+    return self.__host_port
 
   def set_configuration_option(self, name, value):
-    self.__query_options[name] = str(value)
+    name = name.lower()
+    value = str(value)
+    if self.__query_options.get(name) != value:
+      LOG.info("set_option('{}', '{}')".format(name, value))
+      self.__query_options[name] = value
 
   def get_default_configuration(self):
     return self.__default_query_options.copy()
@@ -307,14 +394,15 @@ class ImpylaHS2Connection(ImpalaConnection):
       conn_kwargs['auth_mechanism'] = 'PLAIN'
     self.__impyla_conn = impyla.connect(host=host, port=int(port),
                                         use_http_transport=self.__use_http_transport,
-                                        http_path=self.__http_path, **conn_kwargs)
+                                        http_path=self.__http_path,
+                                        use_ssl=self.__use_ssl, **conn_kwargs)
     # Get the default query options for the session before any modifications are made.
     self.__cursor = self.__impyla_conn.cursor(convert_types=False)
     self.__default_query_options = {}
     if not self._is_hive:
       self.__cursor.execute("set all")
-      for name, val, _ in self.__cursor:
-        self.__default_query_options[name] = val
+      for name, val, kind in self.__cursor:
+        collect_default_query_options(self.__default_query_options, name, val, kind)
       self.__cursor.close_operation()
       LOG.debug("Default query options: {0}".format(self.__default_query_options))
 
@@ -323,7 +411,7 @@ class ImpylaHS2Connection(ImpalaConnection):
     try:
       # Explicitly close the cursor so that it will close the session.
       self.__cursor.close()
-    except Exception as e:
+    except Exception:
       # The session may no longer be valid if the impalad was restarted during the test.
       pass
     try:
@@ -343,11 +431,14 @@ class ImpylaHS2Connection(ImpalaConnection):
     return self.__cursor.fetchall()
 
   def close_query(self, operation_handle):
-    LOG.info("-- closing query for operation handle: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'closing query for operation')
     operation_handle.get_handle().close_operation()
 
   def execute(self, sql_stmt, user=None, profile_format=TRuntimeProfileFormat.STRING,
       fetch_profile_after_close=False):
+    LOG.info("-- executing against {0} at {1}\n".format(
+      self._is_hive and 'Hive' or 'Impala', self.__host_port))
+    log_sql_stmt(sql_stmt)
     self.__cursor.execute(sql_stmt, configuration=self.__query_options)
     handle = OperationHandle(self.__cursor, sql_stmt)
 
@@ -390,64 +481,77 @@ class ImpylaHS2Connection(ImpalaConnection):
       raise
 
   def cancel(self, operation_handle):
-    LOG.info("-- canceling operation: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'canceling operation')
     cursor = operation_handle.get_handle()
     return cursor.cancel_operation(reset_state=False)
 
   def get_query_id(self, operation_handle):
-    """Return the string representation of the query id."""
-    guid_bytes = \
-        operation_handle.get_handle()._last_operation.handle.operationId.guid
-    return "{0}:{1}".format(codecs.encode(guid_bytes[7::-1], 'hex_codec'),
-                            codecs.encode(guid_bytes[16:7:-1], 'hex_codec'))
+    """Return the string representation of the query id.
+    Return empty string if handle is already canceled or closed."""
+    last_op = operation_handle.get_handle()._last_operation
+    if last_op is None:
+      return ""
+    guid_bytes = last_op.handle.operationId.guid
+    # hex_codec works on bytes, so this needs to a decode() to get back to a string
+    hi_str = codecs.encode(guid_bytes[7::-1], 'hex_codec').decode()
+    lo_str = codecs.encode(guid_bytes[16:7:-1], 'hex_codec').decode()
+    return "{0}:{1}".format(hi_str, lo_str)
+
+  def handle_id(self, operation_handle):
+    query_id = self.get_query_id(operation_handle)
+    return query_id if query_id else str(operation_handle)
+
+  def log_handle(self, operation_handle, message):
+    handle_id = self.handle_id(operation_handle)
+    LOG.info("-- {0}: {1}".format(handle_id, message))
 
   def get_state(self, operation_handle):
-    LOG.info("-- getting state for operation: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'getting state')
     cursor = operation_handle.get_handle()
     return cursor.status()
 
   def state_is_finished(self, operation_handle):
-    LOG.info("-- checking finished state for operation: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'checking finished state for operation')
     cursor = operation_handle.get_handle()
     # cursor.status contains a string representation of one of
     # TCLIService.TOperationState.
     return cursor.status() == "FINISHED_STATE"
 
   def get_exec_summary(self, operation_handle):
-    LOG.info("-- getting exec summary operation: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'getting exec summary operation')
     cursor = operation_handle.get_handle()
     # summary returned is thrift, not string.
     return cursor.get_summary()
 
   def get_runtime_profile(self, operation_handle, profile_format):
-    LOG.info("-- getting runtime profile operation: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'getting runtime profile operation')
     cursor = operation_handle.get_handle()
     return cursor.get_profile(profile_format=profile_format)
 
   def wait_for_finished_timeout(self, operation_handle, timeout):
-    LOG.info("-- waiting for query to reach FINISHED state: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'waiting for query to reach FINISHED state')
     raise NotImplementedError("Not yet implemented for HS2 - states differ from beeswax")
 
   def wait_for_admission_control(self, operation_handle):
-    LOG.info("-- waiting for completion of the admission control processing of the "
-        "query: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'waiting for completion of the admission control')
     raise NotImplementedError("Not yet implemented for HS2 - states differ from beeswax")
 
   def get_admission_result(self, operation_handle):
-    LOG.info("-- getting the admission result: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'getting the admission result')
     raise NotImplementedError("Not yet implemented for HS2 - states differ from beeswax")
 
   def get_log(self, operation_handle):
-    LOG.info("-- getting log for operation: {0}".format(operation_handle))
+    self.log_handle(operation_handle, 'getting log for operation')
     # HS2 includes non-error log messages that we need to filter out.
     cursor = operation_handle.get_handle()
     lines = [line for line in cursor.get_log().split('\n')
              if not PROGRESS_LOG_RE.match(line)]
     return '\n'.join(lines)
 
-  def fetch(self, sql_stmt, handle, max_rows=-1):
-    LOG.info("-- fetching results from: {0}".format(handle))
-    return self.__fetch_results(handle, max_rows)
+  def fetch(self, sql_stmt, operation_handle, max_rows=-1):
+    self.log_handle(operation_handle, 'fetching {} rows'.format(
+      'all' if max_rows < 0 else max_rows))
+    return self.__fetch_results(operation_handle, max_rows)
 
   def __fetch_results(self, handle, max_rows=-1,
                       profile_format=TRuntimeProfileFormat.STRING):
@@ -467,7 +571,7 @@ class ImpylaHS2Connection(ImpalaConnection):
       else:
         result_tuples = cursor.fetchmany(max_rows)
 
-    if not self._is_hive:
+    if not self._is_hive and self._collect_profile_and_log:
       log = self.get_log(handle)
       profile = self.get_runtime_profile(handle, profile_format=profile_format)
     else:
@@ -489,7 +593,8 @@ class ImpylaHS2ResultSet(object):
     self.column_types = column_types
     self.query = query
     self.log = log
-    self.profile = profile
+    # ImpalaBeeswaxResult store profile at runtime_profile field
+    self.runtime_profile = profile
     self.query_id = query_id
     self.__result_tuples = result_tuples
     # self.data is the data in the ImpalaBeeswaxResult format: a list of rows with each
@@ -498,6 +603,10 @@ class ImpylaHS2ResultSet(object):
     if result_tuples is not None:
       self.data = [self.__convert_result_row(tuple) for tuple in result_tuples]
 
+  def tuples(self):
+    """Return the raw HS2 result set, which is a list of tuples."""
+    return self.__result_tuples
+
   def __convert_result_row(self, result_tuple):
     """Take primitive values from a result tuple and construct the tab-separated string
     that would have been returned via beeswax."""
@@ -516,16 +625,19 @@ class ImpylaHS2ResultSet(object):
 
 
 def create_connection(host_port, use_kerberos=False, protocol='beeswax',
-    is_hive=False):
+    is_hive=False, use_ssl=False, collect_profile_and_log=True):
   if protocol == 'beeswax':
-    c = BeeswaxConnection(host_port=host_port, use_kerberos=use_kerberos)
+    c = BeeswaxConnection(host_port=host_port, use_kerberos=use_kerberos,
+                          use_ssl=use_ssl)
   elif protocol == 'hs2':
     c = ImpylaHS2Connection(host_port=host_port, use_kerberos=use_kerberos,
-        is_hive=is_hive)
+        is_hive=is_hive, use_ssl=use_ssl,
+        collect_profile_and_log=collect_profile_and_log)
   else:
     assert protocol == 'hs2-http'
     c = ImpylaHS2Connection(host_port=host_port, use_kerberos=use_kerberos,
-        is_hive=is_hive, use_http_transport=True, http_path='cliservice')
+        is_hive=is_hive, use_http_transport=True, http_path='cliservice',
+        use_ssl=use_ssl, collect_profile_and_log=collect_profile_and_log)
 
   # A hook in conftest sets tests.common.current_node. Skip for Hive connections since
   # Hive cannot modify client_identifier at runtime.
