@@ -19,9 +19,15 @@
 
 from __future__ import absolute_import, division, print_function
 from builtins import map, range
+# Python 3 doesn't have the "unicode" type, as its regular string is Unicode. This
+# replaces Python 2's unicode with future's str. On Python 3, it is the builtin string.
+# On Python 2, it uses future's str implementation, which is similar to Python 3's string
+# but subclasses "unicode". See https://python-future.org/compatible_idioms.html#unicode
+from builtins import str as unicode_compat
 import logging
 import math
 import re
+import sys
 
 from functools import wraps
 from tests.util.test_file_parser import (join_section_lines, remove_comments,
@@ -143,7 +149,10 @@ class ResultRow(object):
     """Allows accessing a column value using the column alias or the position of the
     column in the result set. All values are returned as strings and an exception is
     thrown if the column label or column position does not exist."""
-    if isinstance(key, basestring):
+    # Python 2's str type won't match unicode type. This is ok, because currently the
+    # key is never unicode. On Python 3, str is unicode, and this would not have that
+    # limitation.
+    if isinstance(key, str):
       for col in self.columns:
         if col.column_label == key.lower(): return col.value
       raise IndexError('No column with label: ' + key)
@@ -258,8 +267,8 @@ def verify_query_result_is_subset(expected_results, actual_results):
   """Check whether the results in expected_results are a subset of the results in
   actual_results. This uses set semantics, i.e. any duplicates are ignored."""
   expected_literals, expected_non_literals = expected_results.separate_rows()
-  expected_literal_strings = set([unicode(row) for row in expected_literals])
-  actual_literal_strings = set([unicode(row) for row in actual_results.rows])
+  expected_literal_strings = set([unicode_compat(row) for row in expected_literals])
+  actual_literal_strings = set([unicode_compat(row) for row in actual_results.rows])
   # Expected literal strings must all be present in the actual strings.
   assert expected_literal_strings <= actual_literal_strings
   # Expected patterns must be present in the actual strings.
@@ -270,17 +279,17 @@ def verify_query_result_is_subset(expected_results, actual_results):
         matched = True
         break
     assert matched, u"Could not find expected row {0} in actual rows:\n{1}".format(
-        unicode(expected_row), unicode(actual_results))
+        unicode_compat(expected_row), unicode_compat(actual_results))
 
 def verify_query_result_is_superset(expected_results, actual_results):
   """Check whether the results in expected_results are a superset of the results in
   actual_results. This uses set semantics, i.e. any duplicates are ignored."""
   expected_literals, expected_non_literals = expected_results.separate_rows()
-  expected_literal_strings = set([unicode(row) for row in expected_literals])
+  expected_literal_strings = set([unicode_compat(row) for row in expected_literals])
   # Check that all actual rows are present in either expected_literal_strings or
   # expected_non_literals.
   for actual_row in actual_results.rows:
-    if unicode(actual_row) in expected_literal_strings:
+    if unicode_compat(actual_row) in expected_literal_strings:
       # Matched to a literal string
       continue
     matched = False
@@ -289,7 +298,7 @@ def verify_query_result_is_superset(expected_results, actual_results):
         matched = True
         break
     assert matched, u"Could not find actual row {0} in expected rows:\n{1}".format(
-        unicode(actual_row), unicode(expected_results))
+        unicode_compat(actual_row), unicode_compat(expected_results))
 
 def verify_query_result_is_equal(expected_results, actual_results):
   assert_args_not_none(expected_results, actual_results)
@@ -301,8 +310,8 @@ def verify_query_result_is_not_in(banned_results, actual_results):
   banned_literals, banned_non_literals = banned_results.separate_rows()
 
   # Part 1: No intersection with the banned literals
-  banned_literals_set = set([unicode(row) for row in banned_literals])
-  actual_set = set(map(unicode, actual_results.rows))
+  banned_literals_set = set([unicode_compat(row) for row in banned_literals])
+  actual_set = set(map(unicode_compat, actual_results.rows))
   assert banned_literals_set.isdisjoint(actual_set)
 
   # Part 2: Walk through each banned non-literal / regex and make sure that no row
@@ -315,7 +324,7 @@ def verify_query_result_is_not_in(banned_results, actual_results):
         matched = True
         break
     assert not matched, u"Found banned row {0} in actual rows:\n{1}".format(
-      unicode(banned_row), unicode(actual_results))
+      unicode_compat(banned_row), unicode_compat(actual_results))
 
 # Global dictionary that maps the verification type to appropriate verifier.
 # The RESULTS section of a .test file is tagged with the verifier type. We may
@@ -367,7 +376,7 @@ def apply_error_match_filter(error_list, replace_filenames=True):
   return [replace_fn(row) for row in error_list]
 
 
-def verify_raw_results(test_section, exec_result, file_format, result_section,
+def verify_raw_results(test_section, exec_result, vector, result_section,
                        type_section='TYPES', update_section=False,
                        replace_filenames=True):
   """
@@ -388,10 +397,12 @@ def verify_raw_results(test_section, exec_result, file_format, result_section,
   TODO: separate out the handling of sections like ERRORS from checking of query results
   to allow regular RESULTS/ERRORS sections in tests with DML_RESULTS (IMPALA-4471).
   """
+  file_format = vector.get_value('table_format').file_format
+  protocol = vector.get_value('protocol')
   expected_results = None
   if result_section in test_section:
     expected_results = remove_comments(test_section[result_section])
-    if isinstance(expected_results, str):
+    if sys.version_info.major == 2 and isinstance(expected_results, str):
       # Always convert 'str' to 'unicode' since pytest will fail to report assertion
       # failures when any 'str' values contain non-ascii bytes (IMPALA-10419).
       try:
@@ -416,15 +427,24 @@ def verify_raw_results(test_section, exec_result, file_format, result_section,
       else:
         raise
 
+  if type_section == 'TYPES' and protocol.startswith('hs2'):
+    # hs2 or hs2-http
+    if 'HS2_TYPES' in test_section:
+      assert 'TYPES' in test_section, \
+        "Base TYPES section must always be included alongside HS2_TYPES"
+      # In some cases HS2 types are expected differ from Beeswax types (e.g. see
+      # IMPALA-914), so use the HS2-specific section if present.
+      type_section = 'HS2_TYPES'
+
   if type_section in test_section:
     # Distinguish between an empty list and a list with an empty string.
     section = test_section[type_section]
     expected_types = [c.strip().upper()
                       for c in remove_comments(section).rstrip('\n').split(',')]
 
-    # Avro and Kudu represent TIMESTAMP columns as strings, so tests using TIMESTAMP are
+    # Avro represents TIMESTAMP columns as strings, so tests using TIMESTAMP are
     # skipped because results will be wrong.
-    if file_format in ('avro', 'kudu') and 'TIMESTAMP' in expected_types:
+    if file_format == 'avro' and 'TIMESTAMP' in expected_types:
         LOG.info("TIMESTAMP columns unsupported in %s, skipping verification." %\
             file_format)
         return
@@ -539,7 +559,7 @@ def parse_result_rows(exec_result, escape_strings=True):
     for i in range(len(cols)):
       if col_types[i] in ['STRING', 'CHAR', 'VARCHAR', 'BINARY']:
         col = cols[i]
-        if isinstance(col, str):
+        if sys.version_info.major == 2 and isinstance(col, str):
           try:
             col = col.decode('utf-8')
           except UnicodeDecodeError as e:
@@ -797,3 +817,43 @@ def assert_codegen_cache_hit(profile_string, expect_hit):
     assert "NumCachedFunctions: 0 " not in profile_string
   else:
     assert "NumCachedFunctions: 0 " in profile_string
+
+
+# A query id consists of two 64-bit non-zero hex numbers connected with a ":".
+QUERY_ID_REGEX = r"(?!0{16})[0-9a-z]{16}:(?!0{16})[0-9a-z]{16}"
+
+
+def error_msg_expected(actual_msg, expected_msg="", query_id=None):
+  """
+  Check if the actual error message is expected.
+
+  As defined in `ImpalaServer::QUERY_ERROR_FORMAT`, an error message is expected to
+  has the following form:
+
+    Query <query_id> failed:\n<error_detail>\n
+
+  - For `query_id`,
+      - If a query id is specified in the parameter, it checks if the actual error
+        message contains exactly the query id.
+      - Otherwise, it checks whether `query_id` match the format using a
+        regular expresssion.
+  - For `error_detail`, it checks whether this part starts with the `expected_msg` if it
+    is specified in the parameter. This is sufficient to distinguish one kind of error
+    from another.
+  """
+  if query_id is None:
+    ERROR_REGEX = "^Query " + QUERY_ID_REGEX + " failed:\n"
+    m = re.search(ERROR_REGEX, actual_msg)
+    if m is None:
+      return False
+    return actual_msg.find(expected_msg, m.end()) != -1
+
+  # The beginning of `ImpalaServer::QUERY_ERROR_FORMAT`
+  ERROR_PROMPT = "Query {} failed:\n{}"
+  return actual_msg.startswith(ERROR_PROMPT.format(query_id, expected_msg))
+
+
+def error_msg_equal(msg1, msg2):
+  """Check if two error messages are equal ignoring the query ids."""
+  return re.sub(QUERY_ID_REGEX, "<query_id>", msg1) == \
+         re.sub(QUERY_ID_REGEX, "<query_id>", msg2)
