@@ -59,8 +59,22 @@
 from __future__ import absolute_import, division, print_function
 from itertools import product
 from copy import deepcopy
+import logging
 
-EXEC_OPTION_KEY = 'exec_option'
+
+LOG = logging.getLogger(__name__)
+# Literal constants to refer to some standard dimension names.
+EXEC_OPTION = 'exec_option'
+PROTOCOL = 'protocol'
+TABLE_FORMAT = 'table_format'
+# Literal constants to refer protocol names.
+BEESWAX = 'beeswax'
+HS2 = 'hs2'
+HS2_HTTP = 'hs2-http'
+
+
+def assert_exec_option_key(key):
+  assert key.islower(), "Exec option " + key + " is not in lower case"
 
 
 # A list of test dimension values.
@@ -88,17 +102,25 @@ class ImpalaTestVector(object):
         return vector_value.value
     raise ValueError("Test vector does not contain value '%s'" % name)
 
+  def get_protocol(self):
+    return self.get_value(PROTOCOL)
+
+  def get_table_format(self):
+    return self.get_value(TABLE_FORMAT)
+
   def get_exec_option_dict(self):
-    return self.get_value(EXEC_OPTION_KEY)
+    return self.get_value(EXEC_OPTION)
 
   def get_exec_option(self, option_name):
-    return self.get_value(EXEC_OPTION_KEY)[option_name]
+    value = self.get_value(EXEC_OPTION).get(option_name.lower())
+    assert value is not None
+    return value
 
   def set_exec_option(self, option_name, option_value):
-    self.get_value(EXEC_OPTION_KEY)[option_name] = option_value
+    self.get_value(EXEC_OPTION)[option_name.lower()] = str(option_value)
 
   def unset_exec_option(self, option_name):
-    del self.get_value(EXEC_OPTION_KEY)[option_name]
+    del self.get_value(EXEC_OPTION)[option_name.lower()]
 
   def __str__(self):
       return ' | '.join(['%s' % vector_value for vector_value in self.vector_values])
@@ -124,12 +146,35 @@ class ImpalaTestMatrix(object):
 
   def add_dimension(self, dimension):
     self.dimensions[dimension.name] = dimension
+    if dimension.name == EXEC_OPTION:
+      for name in list(self.independent_exec_option_names):
+        LOG.warn("Reassigning {} dimension will remove exec option {}={} that was "
+            "independently declared through add_exec_option_dimension.".format(
+              EXEC_OPTION, name, [v.value for v in self.dimensions[name]]))
+        self.clear_dimension(name)
 
-  def add_mandatory_exec_option(self, exec_option_key, exec_option_value):
-    assert EXEC_OPTION_KEY in self.dimensions, (
-      "Must have '" + EXEC_OPTION_KEY + "' dimension previously declared!")
-    for vector in self.dimensions[EXEC_OPTION_KEY]:
-      vector.value[exec_option_key] = exec_option_value
+  def assert_unique_exec_option_key(self, key):
+    """Assert that 'exec_option' dimension exist and 'key' is not exist yet
+    in self.dimension['exec_option']."""
+    assert_exec_option_key(key)
+    assert EXEC_OPTION in self.dimensions, (
+        "Must have '" + EXEC_OPTION + "' dimension previously declared!")
+
+    for vector in self.dimensions[EXEC_OPTION]:
+      assert key not in vector.value, (
+          "'{}' is already exist in '{}' dimension!".format(key, EXEC_OPTION))
+
+    # 'key' must not previously declared with add_exec_option_dimension().
+    assert key not in self.independent_exec_option_names, (
+        "['{}']['{}'] was previously declared with non-unique value: {}".format(
+          EXEC_OPTION, key, [dim.value for dim in self.dimensions[key]]))
+
+  def add_mandatory_exec_option(self, key, value):
+    """Append key=value pair into 'exec_option' dimension."""
+    self.assert_unique_exec_option_key(key)
+
+    for vector in self.dimensions[EXEC_OPTION]:
+      vector.value[key] = value
 
   def add_exec_option_dimension(self, dimension):
     """
@@ -139,11 +184,11 @@ class ImpalaTestMatrix(object):
     exhaustive combination correct while eliminating the need for individual test method
     to append the custom exec options into 'exec_option' dimension themself.
     """
-    assert EXEC_OPTION_KEY in self.dimensions, (
-      "Must have '" + EXEC_OPTION_KEY + "' dimension previously declared!")
+    self.assert_unique_exec_option_key(dimension.name)
     assert len(dimension) > 1, (
-      "Dimension " + dimension.name + " must have more than 1 possible value! "
-      "Use add_mandatory_exec_option() instead.")
+        "Dimension " + dimension.name + " must have more than 1 possible value! "
+        "Use add_mandatory_exec_option() instead.")
+
     self.add_dimension(dimension)
     self.independent_exec_option_names.add(dimension.name)
 
@@ -169,28 +214,32 @@ class ImpalaTestMatrix(object):
     else:
       raise ValueError('Unknown exploration strategy: %s' % exploration_strategy)
 
-  def embed_independent_exec_options(self, vector_values):
-    if not self.independent_exec_option_names:
-      return vector_values
+  def __deepcopy_vector_values(self, vector_values):
+    """Return a deepcopy of vector_values and merge exec options declared through
+    add_exec_option_dimension() into 'exec_option' dimension."""
     values = []
     exec_values = []
     exec_option = None
     for val in vector_values:
-      if val.name == EXEC_OPTION_KEY:
+      if val.name == EXEC_OPTION:
+        # 'exec_option' is a map. We need to deepcopy the value for each vector.
         exec_option = deepcopy(val.value)
       elif val.name in self.independent_exec_option_names:
+        # save this to merge into exec_option later.
         exec_values.append(val)
       else:
         values.append(val)
-    assert exec_option is not None, (
-      "Must have '" + EXEC_OPTION_KEY + "' dimension previously declared!")
-    for val in exec_values:
-      exec_option[val.name] = val.value
-    values.append(ImpalaTestVector.Value(EXEC_OPTION_KEY, exec_option))
+    if self.independent_exec_option_names:
+      assert exec_option is not None, (
+        "Must have '" + EXEC_OPTION + "' dimension previously declared!")
+      for val in exec_values:
+        exec_option[val.name] = val.value
+    if exec_option:
+      values.append(ImpalaTestVector.Value(EXEC_OPTION, exec_option))
     return values
 
   def __generate_exhaustive_combinations(self):
-    return [ImpalaTestVector(self.embed_independent_exec_options(vec))
+    return [ImpalaTestVector(self.__deepcopy_vector_values(vec))
       for vec in product(*self.__extract_vector_values()) if self.is_valid(vec)]
 
   def __generate_pairwise_combinations(self):
@@ -201,7 +250,7 @@ class ImpalaTestMatrix(object):
     # results will be the same.
     if len(self.dimensions) == 1:
       return self.__generate_exhaustive_combinations()
-    return [ImpalaTestVector(self.embed_independent_exec_options(vec))
+    return [ImpalaTestVector(self.__deepcopy_vector_values(vec))
       for vec in all_pairs(self.__extract_vector_values(), filter_func=self.is_valid)]
 
   def add_constraint(self, constraint_func):
