@@ -201,7 +201,7 @@ class MiniCluster(Cluster):
     return local_shell(cmd, timeout_secs=timeout_secs)
 
   def _init_local_hadoop_conf_dir(self):
-    self._local_hadoop_conf_dir = mkdtemp()
+    self._local_hadoop_conf_dir = mkdtemp(prefix='impala_mini_cluster_')
 
     node_conf_dir = self._get_node_conf_dir()
     for file_name in os.listdir(node_conf_dir):
@@ -232,6 +232,7 @@ class MiniCluster(Cluster):
                 for p in range(self.num_impalads)]
     self._impala = Impala(self, impalads)
 
+
 class MiniHiveCluster(MiniCluster):
   """
   A MiniCluster useful for running against Hive. It allows Hadoop configuration files
@@ -324,7 +325,7 @@ class CmCluster(Cluster):
           self._ssh_clients_by_host_name[host_name].append(client)
 
   def _init_local_hadoop_conf_dir(self):
-    self._local_hadoop_conf_dir = mkdtemp()
+    self._local_hadoop_conf_dir = mkdtemp(prefix='impala_mini_hive_cluster_')
     data = BytesIO(self.cm.get("/clusters/%s/services/%s/clientConfig"
       % (self.cm_cluster.name, self._find_service("HIVE").name)))
     zip_file = ZipFile(data)
@@ -430,7 +431,7 @@ class HdfsClient(object):
     s.verify = False
     if use_kerberos:
       try:
-        from hdfs.ext.kerberos import KerberosClient
+        self.init_kerberos_client(url, s)
       except ImportError as e:
         if "No module named requests_kerberos" not in str(e):
           raise e
@@ -444,15 +445,18 @@ class HdfsClient(object):
                       stdout=subprocess.PIPE,
                       stderr=subprocess.STDOUT)
           LOG.info("kerberos installation complete.")
+          self.init_kerberos_client(url, s)
         except Exception as e:
           LOG.error("kerberos installation failed. Try installing libkrb5-dev and"
               " then try again.")
           raise e
-      from hdfs.ext.kerberos import KerberosClient
-      self._client = KerberosClient(url, session=s)
     else:
       self._client = hdfs.client.InsecureClient(url, user=user_name, session=s)
 
+  def init_kerberos_client(self, url, session):
+    from hdfs.ext.kerberos import KerberosClient
+    self._client = KerberosClient(url, session=session)
+
   def __getattr__(self, name):
     return getattr(self._client, name)
 
@@ -487,8 +491,8 @@ class Yarn(Service):
     """
     env = dict(os.environ)
     env['HADOOP_CONF_DIR'] = self.cluster.local_hadoop_conf_dir
-    env['CDH_MR2_HOME'] =  os.environ['HADOOP_HOME']
-    env['HADOOP_USER_NAME'] =  self.cluster.hadoop_user_name
+    env['CDH_MR2_HOME'] = os.environ['HADOOP_HOME']
+    env['HADOOP_USER_NAME'] = self.cluster.hadoop_user_name
     local_shell('hadoop jar %s %s' % (jar_path, job_args), stdout=subprocess.PIPE,
         stderr=subprocess.STDOUT, env=env)
 
@@ -835,7 +839,7 @@ class Impalad(with_metaclass(ABCMeta, object)):
     for metric in self.get_metrics():
       if metric["name"] == name:
         return metric
-    raise Exception("Metric '%s' not found" % name);
+    raise Exception("Metric '%s' not found" % name)
 
   def __repr__(self):
     return "<%s host: %s>" % (type(self).__name__, self.label)
