@@ -15,6 +15,9 @@
 # specific language governing permissions and limitations
 # under the License.
 
+from __future__ import absolute_import, division, print_function
+from builtins import object, range
+from future.utils import with_metaclass
 from abc import ABCMeta, abstractproperty
 from copy import deepcopy
 from logging import getLogger
@@ -42,16 +45,14 @@ class StatementExecutionMode(object):
       DML_SETUP,
       # a DML statement that's actually a test
       DML_TEST,
-  ) = xrange(5)
+  ) = range(5)
 
 
-class AbstractStatement(object):
+class AbstractStatement(with_metaclass(ABCMeta, object)):
   """
   Abstract query representation
   """
 
-  __metaclass__ = ABCMeta
-
   def __init__(self):
     # reference to statement's parent. For example the right side of a UNION clause
     # SELECT will have a parent as the SELECT on the left, which for the query
@@ -234,9 +235,9 @@ class SelectItemSubList(object):
   def __len__(self):
     return sum(1 for _ in self)
 
-  def __nonzero__(self):
+  def __bool__(self):
     try:
-      iter(self).next()
+      next(iter(self))
       return True
     except StopIteration:
       return False
@@ -258,7 +259,7 @@ class SelectItemSubList(object):
       items = list()
       while start < stop:
         try:
-          idx, item = self_iter.next()
+          idx, item = next(self_iter)
         except StopIteration:
           break
         if idx < start:
@@ -296,7 +297,7 @@ class SelectItemSubList(object):
       filtered_idx = 0
       while start < stop:
         try:
-          idx, item = self_iter.next()
+          idx, item = next(self_iter)
         except StopIteration:
           break
         if not self.filter(item):
