@@ -18,7 +18,7 @@
 # py.test configuration module
 #
 from __future__ import absolute_import, division, print_function
-from builtins import map, range
+from builtins import map, range, zip
 from impala.dbapi import connect as impala_connect
 from kudu import connect as kudu_connect
 from random import choice, sample
@@ -32,14 +32,17 @@ import sys
 
 import tests.common
 from impala_py_lib.helpers import find_all_files, is_core_dump
+import tests.common.base_test_suite
 from tests.common.environ import build_flavor_timeout
 from tests.common.test_result_verifier import QueryTestResult
 from tests.common.patterns import is_valid_impala_identifier
+from tests.common.test_vector import BEESWAX, HS2, HS2_HTTP
 from tests.comparison.db_connection import ImpalaConnection
 from tests.util.filesystem_utils import FILESYSTEM, ISILON_WEBHDFS_PORT, WAREHOUSE
 
 LOG = logging.getLogger('test_configuration')
 LOG_FORMAT = "-- %(asctime)s %(levelname)-8s %(threadName)s: %(message)s"
+VALID_TEST_PROTOCOLS = [BEESWAX, HS2, HS2_HTTP]
 
 DEFAULT_CONN_TIMEOUT = 45
 DEFAULT_EXPLORATION_STRATEGY = 'core'
@@ -54,6 +57,7 @@ DEFAULT_KUDU_MASTER_HOSTS = os.getenv('KUDU_MASTER_HOSTS', '127.0.0.1')
 DEFAULT_KUDU_MASTER_PORT = os.getenv('KUDU_MASTER_PORT', '7051')
 DEFAULT_METASTORE_SERVER = 'localhost:9083'
 DEFAULT_NAMENODE_ADDR = None
+DEFAULT_TEST_PROTOCOL = os.getenv('DEFAULT_TEST_PROTOCOL', BEESWAX)
 if FILESYSTEM == 'isilon':
   DEFAULT_NAMENODE_ADDR = "{node}:{port}".format(node=os.getenv("ISILON_NAMENODE"),
                                                  port=ISILON_WEBHDFS_PORT)
@@ -62,6 +66,7 @@ if FILESYSTEM == 'isilon':
 PYTEST_TIMEOUT_S = \
     build_flavor_timeout(2 * 60 * 60, slow_build_timeout=4 * 60 * 60)
 
+
 def pytest_configure(config):
   """ Hook startup of pytest. Sets up log format and per-test timeout. """
   configure_logging()
@@ -81,7 +86,7 @@ def configure_logging():
   # Verify that the logging level is set to the correct value.
   rootLoggerLevel = logging.getLogger().getEffectiveLevel()
   print("rootLoggerLevel = {0}".format(logging.getLevelName(rootLoggerLevel)))
-  assert(rootLoggerLevel == logging.INFO)
+  assert (rootLoggerLevel == logging.INFO)
 
 
 def pytest_addoption(parser):
@@ -175,6 +180,26 @@ def pytest_addoption(parser):
                    "version used to execute the tests. "
                    "(See $IMPALA_HOME/bin/set-pythonpath.sh.)")
 
+  parser.addoption("--calcite_report_mode", action="store_true", default=False,
+                   help="Mode designed to provide coverage for the Calcite planner. "
+                   "Produces a JSON file for each run_test_case() invocation and "
+                   "continues past errors. These JSON files can be processed to produce "
+                   "a report to detect improvements and protect against regressions.")
+
+  parser.addoption("--calcite_report_output_dir", default=None,
+                   help="Location to store the output JSON files for "
+                   "calcite_report_mode. Defaults to ${IMPALA_LOGS_DIR}/calcite_report.")
+
+  parser.addoption("--default_test_protocol", default=DEFAULT_TEST_PROTOCOL,
+                   choices=VALID_TEST_PROTOCOLS,
+                   help="Impala protocol to run test if test does not specify any.")
+
+
+def assert_impala_test_suite(request):
+  assert issubclass(request.cls, tests.common.impala_test_suite.ImpalaTestSuite), (
+    "This fixture is only applicable to ImpalaTestSuite subclasses."
+  )
+
 
 def pytest_assertrepr_compare(op, left, right):
   """
@@ -186,7 +211,7 @@ def pytest_assertrepr_compare(op, left, right):
   if isinstance(left, QueryTestResult) and isinstance(right, QueryTestResult) and \
      op == "==":
     result = ['Comparing QueryTestResults (expected vs actual):']
-    for l, r in map(None, left.rows, right.rows):
+    for l, r in zip(left.rows, right.rows):
       result.append("%s == %s" % (l, r) if l == r else "%s != %s" % (l, r))
     if len(left.rows) != len(right.rows):
       result.append('Number of rows returned (expected vs actual): '
@@ -223,6 +248,9 @@ def pytest_generate_tests(metafunc):
   If a test has the 'vector' fixture specified, this code is invoked and it will build
   a set of test vectors to parameterize the test with.
   """
+  # All Impala tests classes must inherit BaseTestSuite.
+  assert issubclass(metafunc.cls, tests.common.base_test_suite.BaseTestSuite)
+  assert metafunc.cls.default_test_protocol() in VALID_TEST_PROTOCOLS
   if 'vector' in metafunc.fixturenames:
     metafunc.cls.add_test_dimensions()
     vectors = metafunc.cls.ImpalaTestMatrix.generate_test_vectors(
@@ -281,7 +309,7 @@ def testid_checksum(request):
   #  "'abort_on_error': 1, 'exec_single_node_rows_threshold': 0, 'batch_size': 0, "
   #  "'num_nodes': 0} | query_type: SELECT | cancel_delay: 3 | action: WAIT | "
   #  "query: select l_returnflag from lineitem]")
-  return '{0:x}'.format(crc32(request.node.nodeid) & 0xffffffff)
+  return '{0:x}'.format(crc32(request.node.nodeid.encode('utf-8')) & 0xffffffff)
 
 
 @pytest.fixture
@@ -337,6 +365,7 @@ def unique_database(request, testid_checksum):
   assert 'function' == request.scope, ('This fixture must have scope "function" since '
                                        'the fixture must guarantee unique per-test '
                                        'databases.')
+  assert_impala_test_suite(request)
 
   db_name_prefix = request.function.__name__
   sync_ddl = False
@@ -360,11 +389,10 @@ def unique_database(request, testid_checksum):
                        ' test function name or any prefixes for long length or invalid '
                        'characters.'.format(db_name))
 
-  def cleanup_database(db_name, must_exist):
-    request.instance.execute_query_expect_success(request.instance.client,
-        'DROP DATABASE {0} `{1}` CASCADE'.format(
-            "" if must_exist else "IF EXISTS", db_name),
-        {'sync_ddl': sync_ddl})
+  def cleanup_database(client, db_name, must_exist):
+    result = client.execute('DROP DATABASE {0} `{1}` CASCADE'.format(
+      "" if must_exist else "IF EXISTS", db_name))
+    assert result.success
     # The database directory may not be removed if there are external tables in the
     # database when it is dropped. The external locations are not removed by cascade.
     # These preexisting files/directories can cause errors when tests run repeatedly or
@@ -375,21 +403,25 @@ def unique_database(request, testid_checksum):
 
   def cleanup():
     # Make sure we don't try to drop the current session database
+    # TODO: clean this up via IMPALA-13758.
     request.instance.execute_query_expect_success(request.instance.client, "use default")
-    for db_name in db_names:
-      cleanup_database(db_name, True)
-      LOG.info('Dropped database "{0}" for test ID "{1}"'.format(
+    with request.cls.create_impala_client(protocol=HS2) as client:
+      client.set_configuration({'sync_ddl': sync_ddl})
+      for db_name in db_names:
+        cleanup_database(client, db_name, True)
+        LOG.info('Dropped database "{0}" for test ID "{1}"'.format(
           db_name, str(request.node.nodeid)))
 
   request.addfinalizer(cleanup)
 
-  for db_name in db_names:
-    cleanup_database(db_name, False)
-    request.instance.execute_query_expect_success(
-        request.instance.client, 'CREATE DATABASE `{0}`'.format(db_name),
-        {'sync_ddl': sync_ddl})
-    LOG.info('Created database "{0}" for test ID "{1}"'.format(db_name,
-                                                               str(request.node.nodeid)))
+  with request.cls.create_impala_client(protocol=HS2) as client:
+    client.set_configuration({'sync_ddl': sync_ddl})
+    for db_name in db_names:
+      cleanup_database(client, db_name, False)
+      result = client.execute('CREATE DATABASE `{0}`'.format(db_name))
+      assert result.success
+      LOG.info('Created database "{0}" for test ID "{1}"'.format(
+        db_name, str(request.node.nodeid)))
   return first_db_name
 
 
@@ -682,8 +714,9 @@ def pytest_collection_modifyitems(items, config, session):
   # We must modify the items list in place for it to take effect.
   items[:] = items_selected
 
-  logging.info("pytest shard selection enabled %s. Of %d items, selected %d items by hash.",
-      config.option.shard_tests, num_items, len(items))
+  logging.info(
+    "pytest shard selection enabled %s. Of %d items, selected %d items by hash.",
+    config.option.shard_tests, num_items, len(items))
 
 
 @pytest.hookimpl(trylast=True)
@@ -694,3 +727,6 @@ def pytest_runtest_logstart(nodeid, location):
   # than being elided.
   tests.common.current_node = \
       nodeid.replace(",", ";").replace(" ", "").replace("=", "-")[0:255]
+
+  # Store the unaltered nodeid as well
+  tests.common.nodeid = nodeid
