@@ -1276,6 +1276,147 @@ class TestEventProcessingCustomConfigs(TestEventProcessingCustomConfigsBase):
     tables_removed_after = EventProcessorUtils.get_int_metric("tables-removed")
     assert tables_removed_after == tables_removed_before
 
+  @CustomClusterTestSuite.with_args(
+    catalogd_args="--hms_event_polling_interval_s=0")
+  def test_ep_delay_metadata_reload_for_insert(self, unique_database):
+    """IMPALA-12277: This test verifies that insert operation on a partitioned table
+    succeeds if the partition is dropped previously externally with the event processor
+    being lagging or not active. Impala should create a new partition in this case."""
+
+    def verify_partition(is_transactional, test_table="insert_part_reload"):
+      query = " ".join(["create table {}.{} (i int)", " partitioned by (year int) ",
+          self._get_transactional_tblproperties(is_transactional)])
+      self.client.execute(query.format(unique_database, test_table))
+      self.client.execute("insert into {}.{} partition(year) values (0,2024), (0,2022)"
+          .format(unique_database, test_table))
+      self.run_stmt_in_hive("alter table {}.{} drop partition(year=2024)"
+          .format(unique_database, test_table))
+      self.run_stmt_in_hive("alter table {}.{} add partition(year=2023)"
+          .format(unique_database, test_table))
+      self.run_stmt_in_hive("insert into {}.{} partition(year=2021) values (1)"
+          .format(unique_database, test_table))
+      self.client.execute(
+        "insert into {}.{} partition(year) values (0,2023), (0,2024), (0,2022)"
+        .format(unique_database, test_table))
+      EventProcessorUtils.wait_for_event_processing(self)
+      results = self.client.execute(
+        "select DISTINCT(year) from {}.{} order by year desc"
+        .format(unique_database, test_table))
+      assert results.data == ["2024", "2023", "2022", "2021"]
+      self.client.execute("drop table {}.{}".format(unique_database, test_table))
+    verify_partition(True)
+    verify_partition(False)
+
+  @pytest.mark.execute_serially
+  @CustomClusterTestSuite.with_args(
+    impalad_args="--use_local_catalog=true",
+    catalogd_args="--catalog_topic_mode=minimal --hms_event_polling_interval_s=5",
+    disable_log_buffering=True, cluster_size=1)
+  def test_invalidate_stale_partition_on_reload(self, unique_database):
+    test_tbl = unique_database + ".test_invalidate_table"
+    self.client.execute("create table {} (id int) partitioned by (p int)"
+        .format(test_tbl))
+    self.client.execute("alter table {} add partition (p=0)".format(test_tbl))
+    self.client.execute("alter table {} add partition (p=1)".format(test_tbl))
+    self.client.execute("alter table {} add partition (p=2)".format(test_tbl))
+    self.run_stmt_in_hive("SET hive.exec.dynamic.partition.mode=nonstrict; "
+        "insert into {} partition(p) values (0,0),(1,1),(2,2)".format(test_tbl))
+    self.client.execute("select * from {}".format(test_tbl))
+    EventProcessorUtils.wait_for_event_processing(self)
+    log_regex = r"Invalidated objects in cache: \[partition {}:p={}"\
+      .format(test_tbl, '%d')
+    self.assert_impalad_log_contains('INFO', log_regex % 0, expected_count=1,
+        timeout_s=20)
+    self.assert_impalad_log_contains('INFO', log_regex % 1, expected_count=1,
+        timeout_s=20)
+    self.assert_impalad_log_contains('INFO', log_regex % 2, expected_count=1,
+        timeout_s=20)
+
+  @pytest.mark.execute_serially
+  @CustomClusterTestSuite.with_args(
+    impalad_args="--use_local_catalog=true",
+    catalogd_args="--catalog_topic_mode=minimal "
+                  "--hms_event_polling_interval_s=1 "
+                  "--invalidate_metadata_on_event_processing_failure=false "
+                  "--debug_actions=mock_write_lock_failure:true",
+    disable_log_buffering=True, cluster_size=1)
+  def test_write_lock_on_partitioned_events(self, unique_database):
+    """IMPALA-12277: This test verifies that CommitCompactionEvent on a partitioned table
+    succeeds if the write lock is not held by the table while processing the event by the
+    event processor. 'mock_write_lock_failure' mocks that there is a failure while
+    acquiring write lock for CommitCompactionEvent"""
+    test_tbl = unique_database + ".test_invalidate_table"
+    acid_props = self._get_transactional_tblproperties(True)
+    self.client.execute("create table {} (id int) partitioned by (p int) {}"
+                        .format(test_tbl, acid_props))
+    for _ in range(10):
+      self.client.execute(
+        "insert into {} partition(p=0) values (1),(2),(3)".format(test_tbl))
+    self.run_stmt_in_hive(
+      "alter table {} partition(p=0) compact 'major' and wait".format(test_tbl))
+    EventProcessorUtils.wait_for_event_processing(self)
+    assert EventProcessorUtils.get_event_processor_status() == "ACTIVE"
+
+  @pytest.mark.execute_serially
+  @CustomClusterTestSuite.with_args(
+    catalogd_args="--enable_reload_events=true "
+                  "--invalidate_metadata_on_event_processing_failure=false")
+  def test_reload_events_modified_concurrently(self):
+    """IMPALA-13126: This test verifies that the event processor successfully consumes or
+    ignores the RELOAD event triggered by refresh operation on a partitioned table if the
+    partitions are modified concurrently on the table."""
+    tbl = "scale_db.num_partitions_1234_blocks_per_partition_1"
+    refresh_stmt = "refresh {} partition(j=0)".format(tbl)
+    for _ in range(32):
+      self.client.execute_async(refresh_stmt)
+    for _ in range(100):
+      self.client.execute(
+        "alter table {} add if not exists partition(j=-1)".format(tbl))
+      self.client.execute(
+        "alter table {} drop partition(j=-1)".format(tbl))
+
+    try:
+      EventProcessorUtils.wait_for_event_processing(self, 1000)  # bigger timeout required
+      assert EventProcessorUtils.get_event_processor_status() == "ACTIVE"
+    finally:
+      # Make sure the table doesn't change after this test
+      self.execute_query("alter table {} drop if exists partition(j=-1)".format(tbl))
+
+  @CustomClusterTestSuite.with_args(
+    catalogd_args="--invalidate_metadata_on_event_processing_failure=false")
+  def test_encoded_partition_values(self, unique_database):
+    # Create a table with URL encoded partition values
+    tbl = unique_database + ".tbl"
+    self.execute_query("create table {}(i int) partitioned by(s string)".format(tbl))
+    part_values = [
+      "00:00:00",
+      "00%3A00%3A00",
+      "00%253A00%253A00",
+    ]
+    for p in part_values:
+      self.execute_query("alter table {} add partition (s='{}')".format(tbl, p))
+
+    # Generate INSERT event on a URL encoded partition which could be incorrectly decoded
+    # to another partition, e.g. '00%3A00%3A00' -> '00:00:00'
+    self.run_stmt_in_hive(
+        "insert into table {} partition(s='00%3A00%3A00') values (0)".format(tbl))
+    EventProcessorUtils.wait_for_event_processing(self)
+    res = self.execute_query("select * from " + tbl)
+    assert len(res.data) == 1
+    assert res.data[0] == '0\t00%3A00%3A00'
+
+    # Test on processing batch of such partition events
+    self.run_stmt_in_hive(
+        "set hive.exec.dynamic.partition.mode=nonstrict; "
+        "insert into table {} values (1, '00%3A00%3A00'), (2, '00%253A00%253A00')"
+        .format(tbl))
+    EventProcessorUtils.wait_for_event_processing(self)
+    res = self.execute_query("select * from " + tbl)
+    assert len(res.data) == 3
+    assert '0\t00%3A00%3A00' in res.data
+    assert '1\t00%3A00%3A00' in res.data
+    assert '2\t00%253A00%253A00' in res.data
+
 
 @SkipIfFS.hive
 class TestEventProcessingWithImpala(TestEventProcessingCustomConfigsBase):
