@@ -19,15 +19,15 @@ from __future__ import absolute_import, division, print_function
 import logging
 import os
 import pytest
-import tempfile
+from copy import deepcopy
 from kudu.schema import INT32
-from time import sleep
 
 from tests.beeswax.impala_beeswax import ImpalaBeeswaxException
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
 from tests.common.kudu_test_suite import KuduTestSuite
 from tests.common.skip import SkipIfKudu, SkipIfBuildType, SkipIf
-from tests.common.test_dimensions import add_mandatory_exec_option
+from tests.common.test_dimensions import BEESWAX, add_mandatory_exec_option
+from tests.common.test_result_verifier import error_msg_expected
 from tests.util.event_processor_utils import EventProcessorUtils
 
 KUDU_MASTER_HOSTS = pytest.config.option.kudu_master_hosts
@@ -40,6 +40,13 @@ class CustomKuduTest(CustomClusterTestSuite, KuduTestSuite):
   def get_workload(cls):
     return 'functional-query'
 
+  @classmethod
+  def default_test_protocol(cls):
+    # run_test_case() can produce different result types between beeswax vs hs2 protocol
+    # in some tests. This fix the test to use beeswax protocol until we can migrate
+    # to hs2.
+    return BEESWAX
+
   @classmethod
   def add_custom_cluster_constraints(cls):
     # Override this method to relax the set of constraints added in
@@ -47,9 +54,9 @@ class CustomKuduTest(CustomClusterTestSuite, KuduTestSuite):
     # 'file_format' and 'compression_codec' being "kudu" and "none" respectively will not
     # be skipped.
     cls.ImpalaTestMatrix.add_constraint(lambda v:
-        v.get_value('exec_option')['batch_size'] == 0 and
-        v.get_value('exec_option')['disable_codegen'] is False and
-        v.get_value('exec_option')['num_nodes'] == 0)
+        v.get_value('exec_option')['batch_size'] == 0
+        and v.get_value('exec_option')['disable_codegen'] is False
+        and v.get_value('exec_option')['num_nodes'] == 0)
 
 
 class TestKuduOperations(CustomKuduTest):
@@ -66,13 +73,16 @@ class TestKuduOperations(CustomKuduTest):
     add_mandatory_exec_option(cls, "kudu_read_mode", "READ_AT_SNAPSHOT")
 
   @pytest.mark.execute_serially
-  @CustomClusterTestSuite.with_args(impalad_args=\
-      "--use_local_tz_for_unix_timestamp_conversions=true")
+  @CustomClusterTestSuite.with_args(
+      impalad_args="--use_local_tz_for_unix_timestamp_conversions=true")
   @SkipIfKudu.no_hybrid_clock()
   @SkipIfKudu.hms_integration_enabled()
   def test_local_tz_conversion_ops(self, vector, unique_database):
     """IMPALA-5539: Test Kudu timestamp reads/writes are correct with the
        use_local_tz_for_unix_timestamp_conversions flag."""
+    # Remove 'abort_on_error' option so we can set it at .test file.
+    # Revisit this if 'abort_on_error' dimension size increase.
+    vector.unset_exec_option('abort_on_error')
     # These tests provide enough coverage of queries with timestamps.
     self.run_test_case('QueryTest/kudu-scan-node', vector, use_db=unique_database)
     self.run_test_case('QueryTest/kudu_insert', vector, use_db=unique_database)
@@ -141,8 +151,6 @@ class TestKuduClientTimeout(CustomKuduTest):
 
 @SkipIf.is_test_jdk
 class TestKuduHMSIntegration(CustomKuduTest):
-  START_END_TIME_LINEAGE_LOG_DIR = tempfile.mkdtemp(prefix="start_end_time")
-
   # TODO(IMPALA-8614): parameterize the common tests in query_test/test_kudu.py
   # to run with HMS integration enabled. Also avoid restarting Impala to reduce
   # tests time.
@@ -174,18 +182,21 @@ class TestKuduHMSIntegration(CustomKuduTest):
        with the Hive Metastore for managed tables. Increase timeout of individual Kudu
        client rpcs to avoid requests fail due to operation delay in the Hive Metastore
        for managed tables (IMPALA-8856)."""
-    vector.get_value('exec_option')['kudu_read_mode'] = "READ_AT_SNAPSHOT"
-    self.run_test_case('QueryTest/kudu_create', vector, use_db=unique_database)
+    new_vector = deepcopy(vector)
+    new_vector.get_value('exec_option')['kudu_read_mode'] = "READ_AT_SNAPSHOT"
+    self.run_test_case('QueryTest/kudu_create', new_vector, use_db=unique_database)
 
   @pytest.mark.execute_serially
   @SkipIfKudu.no_hybrid_clock()
-  @CustomClusterTestSuite.with_args(impalad_args="-kudu_client_rpc_timeout_ms=30000 "
-                                    "--lineage_event_log_dir={0}"
-                                    .format(START_END_TIME_LINEAGE_LOG_DIR))
+  @CustomClusterTestSuite.with_args(
+      impalad_args=("-kudu_client_rpc_timeout_ms=30000 "
+        "--lineage_event_log_dir={start_end_time}"),
+      tmp_dir_placeholders=['start_end_time'])
   def test_create_kudu_tables_with_lineage_enabled(self, vector, unique_database):
     """Same as above test_create_managed_kudu_tables, but with lineage enabled."""
-    vector.get_value('exec_option')['kudu_read_mode'] = "READ_AT_SNAPSHOT"
-    self.run_test_case('QueryTest/kudu_create', vector, use_db=unique_database)
+    new_vector = deepcopy(vector)
+    new_vector.get_value('exec_option')['kudu_read_mode'] = "READ_AT_SNAPSHOT"
+    self.run_test_case('QueryTest/kudu_create', new_vector, use_db=unique_database)
 
   @pytest.mark.execute_serially
   def test_implicit_external_table_props(self, cursor, kudu_client):
@@ -358,6 +369,7 @@ class TestKuduHMSIntegration(CustomKuduTest):
       vector,
       use_db=unique_database)
 
+
 class TestKuduTransactionBase(CustomClusterTestSuite):
   """
   This is a base class of other TestKuduTransaction classes.
@@ -432,19 +444,19 @@ class TestKuduTransactionBase(CustomClusterTestSuite):
       self.execute_query(self._update_query.format(table_name))
       assert False, "query was expected to fail"
     except ImpalaBeeswaxException as e:
-      assert "Query aborted:Kudu reported error: Not implemented" in str(e)
+      assert error_msg_expected(str(e), "Kudu reported error: Not implemented")
 
     try:
       self.execute_query(self._upsert_query.format(table_name))
       assert False, "query was expected to fail"
     except ImpalaBeeswaxException as e:
-      assert "Query aborted:Kudu reported error: Not implemented" in str(e)
+      assert error_msg_expected(str(e), "Kudu reported error: Not implemented")
 
     try:
       self.execute_query(self._delete_query.format(table_name))
       assert False, "query was expected to fail"
     except ImpalaBeeswaxException as e:
-      assert "Query aborted:Kudu reported error: Not implemented" in str(e)
+      assert error_msg_expected(str(e), "Kudu reported error: Not implemented")
 
     # Verify that number of rows has not been changed.
     cursor.execute(self._row_num_query.format(table_name))
