@@ -15,6 +15,7 @@
 # specific language governing permissions and limitations
 # under the License.
 
+from __future__ import absolute_import, division, print_function
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
 from tests.common.skip import SkipIfFS
 
@@ -295,3 +296,33 @@ class TestMetadataNoEventsProcessing(CustomClusterTestSuite):
     result = self.client.execute("show partitions %s" % tbl)
     assert result.get_data().startswith("1\t1\t2"),\
         "Incorrect partition stats %s" % result.get_data()
+
+  @CustomClusterTestSuite.with_args(catalogd_args="--hms_event_polling_interval_s=0")
+  def test_invalidate_metadata(self, unique_name):
+    """Verify invalidate metadata on tables under unloaded db won't fail"""
+    db = unique_name + "_db"
+    tbl = db + "." + unique_name + "_tbl"
+    try:
+      self.run_stmt_in_hive("create database " + db)
+      self.run_stmt_in_hive("create table %s (i int)" % tbl)
+      self.client.execute("invalidate metadata %s" % tbl)
+      res = self.client.execute("describe %s" % tbl)
+      assert res.data == ["i\tint\t"]
+    finally:
+      self.run_stmt_in_hive("drop database %s cascade" % db)
+
+  @CustomClusterTestSuite.with_args(
+    catalogd_args="--catalog_topic_mode=minimal --hms_event_polling_interval_s=0",
+    impalad_args="--use_local_catalog=true")
+  def test_invalidate_metadata_v2(self, unique_name):
+    """Verify invalidate metadata on tables under unloaded db won't fail"""
+    db = unique_name + "_db"
+    tbl = db + "." + unique_name + "_tbl"
+    try:
+      self.run_stmt_in_hive("create database " + db)
+      self.run_stmt_in_hive("create table %s (i int)" % tbl)
+      self.client.execute("invalidate metadata %s" % tbl)
+      res = self.client.execute("describe %s" % tbl)
+      assert res.data == ["i\tint\t"]
+    finally:
+      self.run_stmt_in_hive("drop database %s cascade" % db)
