@@ -60,13 +60,12 @@ class TestParquetMaxPageHeader(CustomClusterTestSuite):
 
   def setup_method(self, method):
     super(TestParquetMaxPageHeader, self).setup_method(method)
-    impalad = self.cluster.impalads[0]
-    client = impalad.service.create_beeswax_client()
-    self.client = client
+    self.create_impala_clients()
     self.__create_test_tbls()
 
   def teardown_method(self, method):
     self.__drop_test_tbls()
+    self.close_impala_clients()
 
   def __drop_test_tbls(self):
     self.client.execute("DROP TABLE IF EXISTS %s PURGE" % self.TEXT_TABLE_NAME)
@@ -96,7 +95,7 @@ class TestParquetMaxPageHeader(CustomClusterTestSuite):
     random_text2 = "".join([random.choice(string.ascii_letters)
         for i in range(self.MAX_STRING_LENGTH)])
     put = subprocess.Popen(["hdfs", "dfs", "-put", "-d", "-f", "-", file_name],
-        stdin=subprocess.PIPE, bufsize=-1)
+        stdin=subprocess.PIPE, bufsize=-1, universal_newlines=True)
     put.stdin.write(random_text1 + "\n")
     put.stdin.write(random_text2)
     put.stdin.close()
