@@ -101,28 +101,32 @@ class TestPartitionDeletion(CustomClusterTestSuite):
   @CustomClusterTestSuite.with_args(
     statestored_args="--statestore_update_frequency_ms=2000",
     impalad_args="--use_local_catalog=false",
-    catalogd_args="--catalog_topic_mode=full --hms_event_polling_interval_s=0")
+    catalogd_args="--catalog_topic_mode=full --hms_event_polling_interval_s=0",
+    disable_log_buffering=True)
   def test_legacy_catalog_no_event_processing(self, unique_database):
     self._test_partition_deletion(unique_database)
 
   @CustomClusterTestSuite.with_args(
     statestored_args="--statestore_update_frequency_ms=2000",
     impalad_args="--use_local_catalog=false",
-    catalogd_args="--catalog_topic_mode=full --hms_event_polling_interval_s=1")
+    catalogd_args="--catalog_topic_mode=full --hms_event_polling_interval_s=1",
+    disable_log_buffering=True)
   def test_legacy_catalog_with_event_processing(self, unique_database):
     self._test_partition_deletion(unique_database)
 
   @CustomClusterTestSuite.with_args(
     statestored_args="--statestore_update_frequency_ms=2000",
     impalad_args="--use_local_catalog=true",
-    catalogd_args="--catalog_topic_mode=minimal --hms_event_polling_interval_s=0")
+    catalogd_args="--catalog_topic_mode=minimal --hms_event_polling_interval_s=0",
+    disable_log_buffering=True)
   def test_local_catalog_no_event_processing(self, unique_database):
     self._test_partition_deletion(unique_database)
 
   @CustomClusterTestSuite.with_args(
     statestored_args="--statestore_update_frequency_ms=2000",
     impalad_args="--use_local_catalog=true",
-    catalogd_args="--catalog_topic_mode=minimal --hms_event_polling_interval_s=1")
+    catalogd_args="--catalog_topic_mode=minimal --hms_event_polling_interval_s=1",
+    disable_log_buffering=True)
   def test_local_catalog_with_event_processing(self, unique_database):
     self._test_partition_deletion(unique_database)
 
@@ -162,8 +166,9 @@ class TestPartitionDeletion(CustomClusterTestSuite):
         self.assert_catalogd_log_contains("INFO", deletion_log_regex.format(tbl, i))
 
     # Restart impalad and check the partitions on it
+    self.close_impala_clients()
     self.cluster.impalads[0].restart()
-    self.client = self.create_impala_client()
+    self.create_impala_clients()
     new_res = self.client.execute("show partitions {}".format(tbl))
     assert new_res.data == res.data
     self.assert_impalad_log_contains("WARNING", "stale partition", expected_count=0)
@@ -186,8 +191,9 @@ class TestPartitionDeletion(CustomClusterTestSuite):
       assert len(res.data) == 3
 
     # Restart impalad and check the partitions on it
+    self.close_impala_clients()
     self.cluster.impalads[0].restart()
-    self.client = self.create_impala_client()
+    self.create_impala_clients()
     new_res = self.client.execute("show partitions {}".format(tbl))
     assert new_res.data == res.data
     self.assert_impalad_log_contains("WARNING", "stale partition", expected_count=0)
@@ -205,8 +211,9 @@ class TestPartitionDeletion(CustomClusterTestSuite):
     self.client.execute("create table {}(i int) partitioned by (p int)".format(tbl))
     self.client.execute("describe " + tbl)
     # Restart impalad and check the partitions on it
+    self.close_impala_clients()
     self.cluster.impalads[0].restart()
-    self.client = self.create_impala_client()
+    self.create_impala_clients()
     res = self.client.execute("show partitions {}".format(tbl))
     assert len(res.data) == 1
     self.assert_impalad_log_contains("WARNING", "stale partition", expected_count=0)
