@@ -24,6 +24,7 @@ from getpass import getuser
 from signal import SIGRTMIN
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
 from tests.common.impala_cluster import DEFAULT_KRPC_PORT
+from tests.util.retry import retry
 from tests.util.workload_management import assert_query
 from time import sleep
 
@@ -33,22 +34,28 @@ class TestQueryLive(CustomClusterTestSuite):
 
   def setup_method(self, method):
     super(TestQueryLive, self).setup_method(method)
-    create_match = self.assert_impalad_log_contains("INFO", r'\]\s+(\w+:\w+)\]\s+'
-        r'Analyzing query: CREATE EXTERNAL TABLE IF NOT EXISTS sys.impala_query_live',
-        timeout_s=60)
-    self.assert_impalad_log_contains("INFO", r'Query successfully unregistered: '
-        r'query_id={}'.format(create_match.group(1)),
-        timeout_s=60)
+    self.wait_for_wm_init_complete()
+
+    # Wait until sys.impala_query_live is available in the coordinator's catalog cache.
+    def table_exists():
+      catalog_objs = self.cluster.get_first_impalad() \
+          .service.read_debug_webpage("catalog?json")
+      return "impala_query_live" in catalog_objs
+
+    assert retry(func=table_exists, max_attempts=5, sleep_time_s=3, backoff=1)
 
   def assert_describe_extended(self):
     describe_ext_result = self.execute_query('describe extended sys.impala_query_live')
-    assert len(describe_ext_result.data) == 80
+    # Alter can add additional event fields. Filter them out.
+    describe_ext_data = [
+        line for line in describe_ext_result.data if 'impala.events.catalog' not in line]
+    assert len(describe_ext_data) == 86
     system_table_re = re.compile(r'__IMPALA_SYSTEM_TABLE\s+true')
-    assert list(filter(system_table_re.search, describe_ext_result.data))
+    assert list(filter(system_table_re.search, describe_ext_data))
     external_re = re.compile(r'EXTERNAL\s+TRUE')
-    assert list(filter(external_re.search, describe_ext_result.data))
+    assert list(filter(external_re.search, describe_ext_data))
     external_table_re = re.compile(r'Table Type:\s+EXTERNAL_TABLE')
-    assert list(filter(external_table_re.search, describe_ext_result.data))
+    assert list(filter(external_table_re.search, describe_ext_data))
 
   def assert_impalads(self, profile, present=[0, 1, 2], absent=[]):
     for port_idx in present:
@@ -78,7 +85,8 @@ class TestQueryLive(CustomClusterTestSuite):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--cluster_id=test_query_live",
-                                    catalogd_args="--enable_workload_mgmt")
+                                    catalogd_args="--enable_workload_mgmt",
+                                    disable_log_buffering=True)
   def test_query_live(self):
     """Asserts the query live table shows and allows filtering queries."""
     # Use a query that reads data from disk for the 1st one, as more representative and a
@@ -131,7 +139,7 @@ class TestQueryLive(CustomClusterTestSuite):
 
     # describe query
     describe_result = self.execute_query('describe sys.impala_query_live')
-    assert len(describe_result.data) == 49
+    assert len(describe_result.data) == 54
     self.assert_describe_extended()
 
     # show create table
@@ -178,7 +186,8 @@ class TestQueryLive(CustomClusterTestSuite):
                                     catalogd_args="--enable_workload_mgmt "
                                                   "--catalog_topic_mode=minimal",
                                     default_query_options=[
-                                      ('default_transactional_type', 'insert_only')])
+                                      ('default_transactional_type', 'insert_only')],
+                                    disable_log_buffering=True)
   def test_default_transactional(self):
     """Asserts the query live table works when impala is started with
     default_transactional_type=insert_only."""
@@ -192,7 +201,8 @@ class TestQueryLive(CustomClusterTestSuite):
                                                  "--cluster_id=test_query_live "
                                                  "--use_local_catalog=true",
                                     catalogd_args="--enable_workload_mgmt "
-                                                  "--catalog_topic_mode=minimal")
+                                                  "--catalog_topic_mode=minimal",
+                                    disable_log_buffering=True)
   def test_local_catalog(self):
     """Asserts the query live table works with local catalog mode."""
     result = self.client.execute("select * from functional.alltypes",
@@ -200,11 +210,48 @@ class TestQueryLive(CustomClusterTestSuite):
     assert_query('sys.impala_query_live', self.client, 'test_query_live',
                  result.runtime_profile)
 
+  @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
+                                                 "--cluster_id=test_query_live",
+                                    catalogd_args="--enable_workload_mgmt",
+                                    disable_log_buffering=True)
+  def test_alter(self):
+    """Asserts alter works on query live table."""
+    column_desc = 'test_alter\tstring\t'
+
+    add_column = self.execute_query(
+        'alter table sys.impala_query_live add columns(test_alter string)')
+    assert add_column.data == ['New column(s) have been added to the table.']
+
+    try:
+      describe_column = self.execute_query('describe sys.impala_query_live')
+      assert len(describe_column.data) == 55
+      assert column_desc in describe_column.data
+
+      select_column = self.execute_query(
+          'select test_alter from sys.impala_query_live limit 1')
+      assert select_column.data == ['NULL']
+      self.assert_impalad_log_contains('WARNING', r'Unknown column \(position 54\)'
+          + ' added to table IMPALA_QUERY_LIVE; check if a coordinator was upgraded')
+    finally:
+      # Ensure new column is dropped in case of test failure
+      drop_column = self.execute_query(
+          'alter table sys.impala_query_live drop test_alter')
+
+    assert drop_column.data == ['Column has been dropped.']
+
+    describe_column2 = self.execute_query('describe sys.impala_query_live')
+    assert len(describe_column2.data) == 54
+    assert column_desc not in describe_column2.data
+
+    select_column2 = self.execute_query('select * from sys.impala_query_live')
+    assert len(select_column2.data) > 1
+
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--cluster_id=test_query_live",
                                     catalogd_args="--enable_workload_mgmt",
                                     cluster_size=3,
-                                    num_exclusive_coordinators=2)
+                                    num_exclusive_coordinators=2,
+                                    disable_log_buffering=True)
   def test_dedicated_coordinators(self):
     """Asserts scans are performed only on coordinators."""
     # Use a query that reads data from disk for the 1st one, as more representative and a
@@ -237,11 +284,13 @@ class TestQueryLive(CustomClusterTestSuite):
                                                  "--cluster_id=test_query_live",
                                     catalogd_args="--enable_workload_mgmt",
                                     cluster_size=3,
-                                    num_exclusive_coordinators=2)
+                                    num_exclusive_coordinators=2,
+                                    disable_log_buffering=True)
   def test_executor_groups(self):
     """Asserts scans are performed only on coordinators with multiple executor groups."""
     # Add a (non-dedicated) coordinator and executor in a different executor group.
-    self._start_impala_cluster(options=['--impalad_args=--executor_groups=extra'],
+    self._start_impala_cluster(options=['--impalad_args=--executor_groups=extra',
+                                        '--impalad_args=--cluster_id=test_query_live'],
                                cluster_size=1,
                                add_executors=True,
                                expected_num_impalads=4)
@@ -254,7 +303,8 @@ class TestQueryLive(CustomClusterTestSuite):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--cluster_id=test_query_live",
-                                    catalogd_args="--enable_workload_mgmt")
+                                    catalogd_args="--enable_workload_mgmt",
+                                    disable_log_buffering=True)
   def test_query_entries_are_unique(self):
     """Asserts queries in the query live table are unique."""
     # Start a query and close it with a delay between CloseClientRequestState and
@@ -297,7 +347,8 @@ class TestQueryLive(CustomClusterTestSuite):
                                                  "--cluster_id=test_query_live",
                                     catalogd_args="--enable_workload_mgmt",
                                     cluster_size=3,
-                                    num_exclusive_coordinators=2)
+                                    num_exclusive_coordinators=2,
+                                    disable_log_buffering=True)
   def test_missing_coordinator(self):
     """Asserts scans finish if a coordinator disappears mid-schedule. Depends on
     test config of statestore_heartbeat_frequency_ms=50."""
@@ -319,7 +370,8 @@ class TestQueryLive(CustomClusterTestSuite):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--cluster_id=test_query_live",
-                                    catalogd_args="--enable_workload_mgmt")
+                                    catalogd_args="--enable_workload_mgmt",
+                                    disable_log_buffering=True)
   def test_shutdown_coordinator(self):
     """Asserts query fails if a coordinator disappears after scheduling. Depends on
     test config of statestore_heartbeat_frequency_ms=50."""
@@ -343,7 +395,8 @@ class TestQueryLive(CustomClusterTestSuite):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--cluster_id=test_query_live",
-                                    catalogd_args="--enable_workload_mgmt")
+                                    catalogd_args="--enable_workload_mgmt",
+                                    disable_log_buffering=True)
   def test_graceful_shutdown_coordinator(self):
     """Asserts query succeeds if another coordinator is shutdown gracefully after
     scheduling. Depends on test config of statestore_heartbeat_frequency_ms=50."""
@@ -375,8 +428,10 @@ class TestQueryLive(CustomClusterTestSuite):
                                                  "--query_log_write_interval_s=1 "
                                                  "--cluster_id=test_query_live",
                                     catalogd_args="--enable_workload_mgmt",
+                                    impalad_graceful_shutdown=True,
                                     cluster_size=3,
-                                    num_exclusive_coordinators=2)
+                                    num_exclusive_coordinators=2,
+                                    disable_log_buffering=True)
   def test_multi_table_union(self):
     """Asserts only system table scan fragments are scheduled to coordinators."""
     utc_timestamp = self.execute_query('select utc_timestamp()')
@@ -410,7 +465,8 @@ class TestQueryLive(CustomClusterTestSuite):
                                                  "--cluster_id=test_query_live",
                                     catalogd_args="--enable_workload_mgmt",
                                     cluster_size=3,
-                                    num_exclusive_coordinators=2)
+                                    num_exclusive_coordinators=2,
+                                    disable_log_buffering=True)
   def test_multi_table_join(self, unique_database):
     """Asserts only system table scan fragments are scheduled to coordinators."""
     self.execute_query('create table {}.users (user string)'.format(unique_database))
