@@ -19,7 +19,6 @@ from __future__ import absolute_import, division, print_function
 
 import os
 import string
-import tempfile
 
 from getpass import getuser
 from ImpalaService import ImpalaHiveServer2Service
@@ -31,11 +30,9 @@ from thrift.transport.TTransport import TBufferedTransport
 from thrift.protocol import TBinaryProtocol
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
 from tests.common.impala_test_suite import IMPALAD_HS2_HOST_PORT
-from tests.common.test_dimensions import create_single_exec_option_dimension
 from tests.common.test_vector import ImpalaTestDimension
 from tests.util.retry import retry
-from tests.util.workload_management import assert_query, COMPRESSED_BYTES_SPILLED, \
-    BYTES_READ_CACHE_TOTAL
+from tests.util.workload_management import assert_query
 from time import sleep, time
 
 
@@ -51,27 +48,16 @@ class TestQueryLogTableBase(CustomClusterTestSuite):
   @classmethod
   def add_test_dimensions(cls):
     super(TestQueryLogTableBase, cls).add_test_dimensions()
-    cls.ImpalaTestMatrix.add_dimension(create_single_exec_option_dimension())
     cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('protocol',
         cls.PROTOCOL_BEESWAX, cls.PROTOCOL_HS2))
 
-  def get_client(self, protocol, query_table_name=QUERY_TBL):
+  def setup_method(self, method):
+    super(TestQueryLogTableBase, self).setup_method(method)
+    self.wait_for_wm_init_complete()
+
+  def get_client(self, protocol):
     """Retrieves the default Impala client for the specified protocol. This client is
-       automatically closed after the test completes. Also ensures the completed queries
-       table has been successfully created by checking the logs to verify the create
-       table sql has finished."""
-
-    # These tests run very quickly and can actually complete before Impala has finished
-    # creating the completed queries table. Thus, to make these tests more robust, this
-    # code checks to make sure the table create has finished before returning.
-    create_re = r'\]\s+(\w+:\w+)\]\s+Analyzing query: CREATE TABLE IF NOT EXISTS {}' \
-        .format(query_table_name)
-    create_match = self.assert_impalad_log_contains("INFO", create_re, timeout_s=60)
-
-    finish_re = r'Query successfully unregistered: query_id={}' \
-        .format(create_match.group(1))
-    self.assert_impalad_log_contains("INFO", finish_re, timeout_s=60)
-
+       automatically closed after the test completes."""
     if protocol == self.PROTOCOL_BEESWAX:
       return self.client
     elif protocol == self.PROTOCOL_HS2:
@@ -89,25 +75,26 @@ class TestQueryLogTableBeeswax(TestQueryLogTableBase):
     cls.ImpalaTestMatrix.add_constraint(lambda v:
         v.get_value('protocol') == 'beeswax')
 
-  CACHE_DIR = tempfile.mkdtemp(prefix="cache_dir")
+  @classmethod
+  def get_workload(self):
+    return 'functional-query'
+
   MAX_SQL_PLAN_LEN = 2000
-  LOG_DIR_MAX_WRITES = tempfile.mkdtemp(prefix="max_writes")
+  LOG_DIR_MAX_WRITES = 'max_attempts_exceeded'
   FLUSH_MAX_RECORDS_CLUSTER_ID = "test_query_log_max_records_" + str(int(time()))
   FLUSH_MAX_RECORDS_QUERY_COUNT = 30
-  OTHER_TBL = "completed_queries_table_{0}".format(int(time()))
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
                                                  "--cluster_id=test_max_select "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60 "
                                                  "--query_log_max_sql_length={0} "
                                                  "--query_log_max_plan_length={0}"
                                                  .format(MAX_SQL_PLAN_LEN),
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_lower_max_sql_plan(self, vector):
-    """Asserts that lower limits on the sql and plan columns in the completed queries
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_lower_max_sql_plan(self, vector):
+    """Asserts that length limits on the sql and plan columns in the completed queries
        table are respected."""
     client = self.get_client(vector.get_value('protocol'))
     rand_long_str = "".join(choice(string.ascii_letters) for _ in
@@ -139,12 +126,11 @@ class TestQueryLogTableBeeswax(TestQueryLogTableBase):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
-                                                 "--cluster_id=test_max_select "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60",
+                                                 "--cluster_id=test_max_select",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_over_max_sql_plan(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_sql_plan_too_long(self, vector):
     """Asserts that very long queries have their corresponding plan and sql columns
        shortened in the completed queries table."""
     client = self.get_client(vector.get_value('protocol'))
@@ -177,15 +163,14 @@ class TestQueryLogTableBeeswax(TestQueryLogTableBase):
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
                                                  "--cluster_id=test_query_hist_1 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60 "
                                                  "--query_log_size=0 "
                                                  "--query_log_size_in_bytes=0",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_no_query_log_select(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_no_query_log(self, vector):
     """Asserts queries are written to the completed queries table when the in-memory
-    query log is turned off."""
+       query log queue is turned off."""
     client = self.get_client(vector.get_value('protocol'))
 
     # Run a select query.
@@ -208,14 +193,14 @@ class TestQueryLogTableBeeswax(TestQueryLogTableBase):
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
                                                  "--cluster_id=test_query_hist_2 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60 "
                                                  "--always_use_data_cache "
-                                                 "--data_cache={0}:5GB".format(CACHE_DIR),
+                                                 "--data_cache={query_data_cache}:5GB",
                                     catalogd_args="--enable_workload_mgmt",
                                     impalad_graceful_shutdown=True,
-                                    cluster_size=1)
-  def test_query_log_table_query_cache(self, vector):
+                                    cluster_size=1,
+                                    tmp_dir_placeholders=['query_data_cache'],
+                                    disable_log_buffering=True)
+  def test_query_data_cache(self, vector):
     """Asserts the values written to the query log table match the values from the
        query profile. Specifically focuses on the data cache metrics."""
     client = self.get_client(vector.get_value('protocol'))
@@ -248,23 +233,24 @@ class TestQueryLogTableBeeswax(TestQueryLogTableBase):
     # for this test to not actually assert anything different than other tests. Thus, an
     # additional assert is needed to ensure that there actually was data read from the
     # cache.
-    assert data[BYTES_READ_CACHE_TOTAL] != "0", "bytes read from cache total was " \
+    assert data["BYTES_READ_CACHE_TOTAL"] != "0", "bytes read from cache total was " \
         "zero, test did not assert anything"
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
-                                                 "--query_log_write_interval_s=5 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60 ",
-                                    impala_log_dir=LOG_DIR_MAX_WRITES,
+                                                 "--query_log_write_interval_s=5",
+                                    impala_log_dir=("{" + LOG_DIR_MAX_WRITES + "}"),
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_max_attempts_exceeded(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    tmp_dir_placeholders=[LOG_DIR_MAX_WRITES],
+                                    disable_log_buffering=True)
+  def test_max_attempts_exceeded(self, vector):
     """Asserts that completed queries are only attempted 3 times to be inserted into the
        completed queries table. This test deletes the completed queries table thus it must
        not come last otherwise the table stays deleted. Subsequent tests will re-create
        the table."""
 
-    print("USING LOG DIRECTORY: {0}".format(self.LOG_DIR_MAX_WRITES))
+    log_dir = self.get_tmp_dir(self.LOG_DIR_MAX_WRITES)
+    print("USING LOG DIRECTORY: {0}".format(log_dir))
 
     impalad = self.cluster.get_first_impalad()
     client = self.get_client(vector.get_value('protocol'))
@@ -281,7 +267,7 @@ class TestQueryLogTableBeeswax(TestQueryLogTableBase):
     # Allow time for logs to be written to disk.
     sleep(5)
 
-    with open(os.path.join(self.LOG_DIR_MAX_WRITES, "impalad.ERROR")) as file:
+    with open(os.path.join(log_dir, "impalad.ERROR")) as file:
       for line in file:
         if line.find('could not write completed query table="{0}" query_id="{1}"'
                           .format(self.QUERY_TBL, res.query_id)) >= 0:
@@ -303,18 +289,17 @@ class TestQueryLogTableBeeswax(TestQueryLogTableBase):
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_max_queued={0} "
                                                  "--query_log_write_interval_s=9999 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60 "
                                                  "--cluster_id={1}"
                                                  .format(FLUSH_MAX_RECORDS_QUERY_COUNT,
                                                  FLUSH_MAX_RECORDS_CLUSTER_ID),
                                     catalogd_args="--enable_workload_mgmt",
                                     default_query_options=[
                                       ('statement_expression_limit', 1024)],
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_flush_max_records(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_flush_on_queued_count_exceeded(self, vector):
     """Asserts that queries that have completed are written to the query log table when
-       the maximum number of queued records it reached. Also verifies that writing
+       the maximum number of queued records is reached. Also verifies that writing
        completed queries is not limited by default statement_expression_limit."""
 
     impalad = self.cluster.get_first_impalad()
@@ -369,46 +354,13 @@ class TestQueryLogTableBeeswax(TestQueryLogTableBase):
         "impala-server.completed-queries.queued") == 2
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
-                                                 "--query_log_write_interval_s=1 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=30 "
-                                                 "--blacklisted_dbs=information_schema "
-                                                 "--query_log_table_name={0}"
-                                                 .format(OTHER_TBL),
-                                    catalogd_args="--enable_workload_mgmt "
-                                                  "--blacklisted_dbs=information_schema",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_different_table(self, vector):
-    """Asserts that the completed queries table can be renamed."""
-
-    client = self.get_client(vector.get_value('protocol'),
-        "{}.{}".format(self.WM_DB, self.OTHER_TBL))
-
-    try:
-      res = client.execute("show tables in {0}".format(self.WM_DB))
-      assert res.success
-      assert len(res.data) > 0, "could not find any tables in database {0}" \
-          .format(self.WM_DB)
-
-      tbl_found = False
-      for tbl in res.data:
-        if tbl.startswith(self.OTHER_TBL):
-          tbl_found = True
-          break
-      assert tbl_found, "could not find table '{0}' in database '{1}'" \
-          .format(self.OTHER_TBL, self.WM_DB)
-    finally:
-      client.execute("drop table {0}.{1} purge".format(self.WM_DB, self.OTHER_TBL))
-
-  @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
-                                                 "--query_log_write_interval_s=1 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60",
+                                                 "--query_log_write_interval_s=1",
                                     cluster_size=3,
                                     num_exclusive_coordinators=2,
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=False)
-  def test_query_log_table_query_select_dedicate_coordinator(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_dedicated_coordinator_no_mt_dop(self, vector):
     """Asserts the values written to the query log table match the values from the
        query profile when dedicated coordinators are used."""
     client = self.get_client(vector.get_value('protocol'))
@@ -430,14 +382,13 @@ class TestQueryLogTableBeeswax(TestQueryLogTableBase):
       client2.close()
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
-                                                 "--query_log_write_interval_s=1 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60",
+                                                 "--query_log_write_interval_s=1",
                                     cluster_size=3,
                                     num_exclusive_coordinators=2,
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=False)
-  def test_query_log_table_query_select_mt_dop(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_dedicated_coordinator_with_mt_dop(self, vector):
     """Asserts the values written to the query log table match the values from the
        query profile when dedicated coordinators are used along with an MT_DOP setting
        greater than 0."""
@@ -461,6 +412,52 @@ class TestQueryLogTableBeeswax(TestQueryLogTableBase):
       client2.close()
 
 
+class TestQueryLogOtherTable(TestQueryLogTableBase):
+  """Tests to assert that query_log_table_name works with non-default value."""
+
+  OTHER_TBL = "completed_queries_table_{0}".format(int(time()))
+  # Used in TestQueryLogTableBase.setup_method
+  QUERY_TBL = "{0}.{1}".format(TestQueryLogTableBase.WM_DB, OTHER_TBL)
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestQueryLogOtherTable, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_constraint(lambda v:
+        v.get_value('protocol') == 'beeswax')
+
+  @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
+                                                 "--query_log_write_interval_s=1 "
+                                                 "--blacklisted_dbs=information_schema "
+                                                 "--query_log_table_name={0}"
+                                                 .format(OTHER_TBL),
+                                    catalogd_args="--enable_workload_mgmt "
+                                                  "--blacklisted_dbs=information_schema "
+                                                  "--query_log_table_name={0}"
+                                                  .format(OTHER_TBL),
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_renamed_log_table(self, vector):
+    """Asserts that the completed queries table can be renamed."""
+
+    client = self.get_client(vector.get_value('protocol'))
+
+    try:
+      res = client.execute("show tables in {0}".format(self.WM_DB))
+      assert res.success
+      assert len(res.data) > 0, "could not find any tables in database {0}" \
+          .format(self.WM_DB)
+
+      tbl_found = False
+      for tbl in res.data:
+        if tbl.startswith(self.OTHER_TBL):
+          tbl_found = True
+          break
+      assert tbl_found, "could not find table '{0}' in database '{1}'" \
+          .format(self.OTHER_TBL, self.WM_DB)
+    finally:
+      client.execute("drop table {0}.{1} purge".format(self.WM_DB, self.OTHER_TBL))
+
+
 class TestQueryLogTableHS2(TestQueryLogTableBase):
   """Tests to assert the query log table is correctly populated when using the HS2
      client protocol."""
@@ -475,14 +472,13 @@ class TestQueryLogTableHS2(TestQueryLogTableBase):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
-                                                 "--cluster_id={} "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60"
+                                                 "--cluster_id={}"
                                                  .format(HS2_OPERATIONS_CLUSTER_ID),
                                     catalogd_args="--enable_workload_mgmt",
                                     cluster_size=2,
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_hs2_operations(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_hs2_metadata_operations(self, vector):
     """Certain HS2 operations appear to Impala as a special kind of query. Specifically,
        these operations have a type of unknown and a normally invalid sql syntax. This
        test asserts those queries are not written to the completed queries table since
@@ -605,13 +601,12 @@ class TestQueryLogTableHS2(TestQueryLogTableBase):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
-                                                 "--cluster_id=test_query_hist_mult "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60",
+                                                 "--cluster_id=test_query_hist_mult",
                                     catalogd_args="--enable_workload_mgmt",
                                     cluster_size=2,
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_query_multiple(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_query_multiple_tables(self, vector):
     """Asserts the values written to the query log table match the values from the
        query profile for a query that reads from multiple tables."""
     client = self.get_client(vector.get_value('protocol'))
@@ -636,12 +631,11 @@ class TestQueryLogTableHS2(TestQueryLogTableBase):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
-                                                 "--cluster_id=test_query_hist_3 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60",
+                                                 "--cluster_id=test_query_hist_3",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_query_insert_select(self, vector, unique_database,
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_insert_select(self, vector, unique_database,
       unique_name):
     """Asserts the values written to the query log table match the values from the
        query profile for a query that insert selects."""
@@ -670,12 +664,11 @@ class TestQueryLogTableHS2(TestQueryLogTableBase):
       client2.close()
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
-                                                 "--query_log_write_interval_s=15 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60",
+                                                 "--query_log_write_interval_s=15",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_flush_interval(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_flush_on_interval(self, vector):
     """Asserts that queries that have completed are written to the query log table
        after the specified write interval elapses."""
 
@@ -687,20 +680,23 @@ class TestQueryLogTableHS2(TestQueryLogTableBase):
       res = client.execute("select sleep(1000)")
       assert res.success
 
-    # At least 10 seconds have already elapsed, wait up to 10 more seconds for the
-    # queries to be written to the completed queries table.
+    # Wait for at least one iteration of the workload management processing loop to write
+    # to the completed queries table.
     self.cluster.get_first_impalad().service.wait_for_metric_value(
-      "impala-server.completed-queries.written", query_count, 10)
+      "impala-server.completed-queries.written", query_count, 20)
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=9999 "
-                                                 "--shutdown_grace_period_s=30 "
-                                                 "--shutdown_deadline_s=30",
+                                                 "--shutdown_grace_period_s=0 "
+                                                 "--shutdown_deadline_s=15 "
+                                                 "--debug_actions="
+                                                 "WM_SHUTDOWN_DELAY:SLEEP@5000",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=False)
-  def test_query_log_table_flush_on_shutdown(self, vector):
+                                    disable_log_buffering=True)
+  def test_flush_on_shutdown(self, vector):
     """Asserts that queries that have completed but are not yet written to the query
-       log table are flushed to the table before the coordinator exits."""
+       log table are flushed to the table before the coordinator exits. Graceful shutdown
+       for 2nd coordinator not needed because query_log_write_interval_s is very long."""
 
     impalad = self.cluster.get_first_impalad()
     client = self.get_client(vector.get_value('protocol'))
@@ -719,25 +715,58 @@ class TestQueryLogTableHS2(TestQueryLogTableBase):
         60)
 
     impalad.kill_and_wait_for_exit(SIGRTMIN)
+    self.assert_impalad_log_contains("INFO", r'Workload management shutdown successful',
+      timeout_s=60)
 
     client2 = self.create_client_for_nth_impalad(1, vector.get_value('protocol'))
 
     try:
-      def assert_func(last_iteration):
+      def assert_func():
         results = client2.execute("select query_id,sql from {0} where query_id in "
                                   "('{1}','{2}','{3}')".format(self.QUERY_TBL,
                                   sql1.query_id, sql2.query_id, sql3.query_id))
 
-        success = len(results.data) == 3
-        if last_iteration:
-          assert len(results.data) == 3
+        return len(results.data) == 3
 
-        return success
-
-      assert retry(func=assert_func, max_attempts=5, sleep_time_s=5)
+      assert retry(func=assert_func, max_attempts=5, sleep_time_s=3)
     finally:
       client2.close()
 
+  @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
+                                                 "--query_log_write_interval_s=9999 "
+                                                 "--shutdown_grace_period_s=0 "
+                                                 "--query_log_shutdown_timeout_s=3 "
+                                                 "--shutdown_deadline_s=15 "
+                                                 "--debug_actions="
+                                                 "WM_SHUTDOWN_DELAY:SLEEP@10000",
+                                    catalogd_args="--enable_workload_mgmt",
+                                    disable_log_buffering=True)
+  def test_shutdown_flush_timed_out(self, vector):
+    """Asserts that queries that have completed but are not yet written to the query
+       log table are lost if the completed queries queue drain takes too long and that
+       the coordinator logs the estimated number of queries lost."""
+
+    impalad = self.cluster.get_first_impalad()
+    client = self.get_client(vector.get_value('protocol'))
+
+    # Execute sql statements to ensure all get written to the query log table.
+    sql1 = client.execute("select 1")
+    assert sql1.success
+
+    sql2 = client.execute("select 2")
+    assert sql2.success
+
+    sql3 = client.execute("select 3")
+    assert sql3.success
+
+    impalad.service.wait_for_metric_value("impala-server.completed-queries.queued", 3,
+        60)
+
+    impalad.kill_and_wait_for_exit(SIGRTMIN)
+    self.assert_impalad_log_contains("INFO", r"Workload management shutdown timed out. "
+      r"Up to '3' queries may have been lost",
+      timeout_s=60)
+
 
 class TestQueryLogTableAll(TestQueryLogTableBase):
   """Tests to assert the query log table is correctly populated when using all the
@@ -745,12 +774,11 @@ class TestQueryLogTableAll(TestQueryLogTableBase):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
-                                                 "--cluster_id=test_query_hist_2 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60",
+                                                 "--cluster_id=test_query_hist_2",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_ddl(self, vector, unique_database, unique_name):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_ddl(self, vector, unique_database, unique_name):
     """Asserts the values written to the query log table match the values from the
        query profile for a DDL query."""
     create_tbl_sql = "create table {0}.{1} (id INT, product_name STRING) " \
@@ -773,12 +801,11 @@ class TestQueryLogTableAll(TestQueryLogTableBase):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
-                                                 "--cluster_id=test_query_hist_3 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60",
+                                                 "--cluster_id=test_query_hist_3",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_dml(self, vector, unique_database, unique_name):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_dml(self, vector, unique_database, unique_name):
     """Asserts the values written to the query log table match the values from the
        query profile for a DML query."""
     tbl_name = "{0}.{1}".format(unique_database, unique_name)
@@ -808,12 +835,11 @@ class TestQueryLogTableAll(TestQueryLogTableBase):
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
-                                                 "--cluster_id=test_query_hist_2 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60 ",
+                                                 "--cluster_id=test_query_hist_2",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_invalid_query(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_invalid_query(self, vector):
     """Asserts correct values are written to the completed queries table for a failed
        query. The query profile is used as the source of expected values."""
     client = self.get_client(vector.get_value('protocol'))
@@ -841,12 +867,11 @@ class TestQueryLogTableAll(TestQueryLogTableBase):
         expected_cluster_id="test_query_hist_2", impalad=impalad, query_id=result.data[0])
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
-                                                 "--query_log_write_interval_s=1 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60",
+                                                 "--query_log_write_interval_s=1",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_ignored_sqls(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_ignored_sqls_not_written(self, vector):
     """Asserts that expected queries are not written to the query log table."""
     client = self.get_client(vector.get_value('protocol'))
 
@@ -892,7 +917,9 @@ class TestQueryLogTableAll(TestQueryLogTableBase):
     sqls["select 1"] = True
 
     control_queries_count = 0
-    for sql, experiment_control in sqls.items():
+    # Note: This needs to iterate over a copy of sqls.items(), because it modifies
+    # sqls as it iterates.
+    for sql, experiment_control in list(sqls.items()):
       results = client.execute(sql)
       assert results.success, "could not execute query '{0}'".format(sql)
       sqls[sql] = results.query_id
@@ -935,12 +962,11 @@ class TestQueryLogTableAll(TestQueryLogTableBase):
         "impala-server.completed-queries.failure") == 0
 
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
-                                                 "--query_log_write_interval_s=1 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60",
+                                                 "--query_log_write_interval_s=1",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_sql_injection(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    disable_log_buffering=True)
+  def test_sql_injection_attempts(self, vector):
     client = self.get_client(vector.get_value('protocol'))
     impalad = self.cluster.get_first_impalad()
 
@@ -958,14 +984,21 @@ class TestQueryLogTableAll(TestQueryLogTableBase):
 
     # Attempt to cause an error using multiline comments.
     sql3_str = "select 1' /* foo"
-    self.__run_sql_inject(impalad, client, sql3_str, "multiline comments", 10, False)
+    self.__run_sql_inject(impalad, client, sql3_str, "multiline comments", 11, False)
 
     # Attempt to cause an error using single line comments.
     sql4_str = "select 1' -- foo"
-    self.__run_sql_inject(impalad, client, sql4_str, "single line comments", 13, False)
+    self.__run_sql_inject(impalad, client, sql4_str, "single line comments", 15, False)
 
   def __run_sql_inject(self, impalad, client, sql, test_case, expected_writes,
                        expect_success=True):
+    # Capture coordinators "now" so we match only queries in this test case.
+    start_time = None
+    if not expect_success:
+      utc_timestamp = self.execute_query('select utc_timestamp()')
+      assert len(utc_timestamp.data) == 1
+      start_time = utc_timestamp.data[0]
+
     sql_result = None
     try:
       sql_result = client.execute(sql)
@@ -993,11 +1026,11 @@ class TestQueryLogTableAll(TestQueryLogTableBase):
                                         .format(sql_result.query_id, test_case)
       assert sql_verify.data[0] == sql, test_case
     else:
+      assert start_time is not None
       esc_sql = sql.replace("'", "\\'")
       sql_verify = client.execute("select sql from {0} where sql='{1}' "
-                                  "and start_time_utc > "
-                                  "date_sub(utc_timestamp(), interval 25 seconds);"
-                                  .format(self.QUERY_TBL, esc_sql))
+                                  "and start_time_utc > '{2}'"
+                                  .format(self.QUERY_TBL, esc_sql, start_time))
       assert sql_verify.success, test_case
       assert len(sql_verify.data) == 1, "did not find query '{0}' in query log " \
                                         "table for test case '{1}" \
@@ -1007,8 +1040,6 @@ class TestQueryLogTableAll(TestQueryLogTableBase):
 class TestQueryLogTableBufferPool(TestQueryLogTableBase):
   """Base class for all query log tests that set the buffer pool query option."""
 
-  SCRATCH_DIR = tempfile.mkdtemp(prefix="scratch_dir")
-
   @classmethod
   def add_test_dimensions(cls):
     super(TestQueryLogTableBufferPool, cls).add_test_dimensions()
@@ -1018,13 +1049,12 @@ class TestQueryLogTableBufferPool(TestQueryLogTableBase):
   @CustomClusterTestSuite.with_args(impalad_args="--enable_workload_mgmt "
                                                  "--query_log_write_interval_s=1 "
                                                  "--cluster_id=test_query_hist_1 "
-                                                 "--shutdown_grace_period_s=10 "
-                                                 "--shutdown_deadline_s=60 "
-                                                 "--scratch_dirs={0}:5G"
-                                                 .format(SCRATCH_DIR),
+                                                 "--scratch_dirs={scratch_dir}:5G",
                                     catalogd_args="--enable_workload_mgmt",
-                                    impalad_graceful_shutdown=True)
-  def test_query_log_table_query_select(self, vector):
+                                    impalad_graceful_shutdown=True,
+                                    tmp_dir_placeholders=['scratch_dir'],
+                                    disable_log_buffering=True)
+  def test_select(self, vector):
     """Asserts the values written to the query log table match the values from the
        query profile. If the buffer_pool_limit parameter is not None, then this test
        requires that the query spills to disk to assert that the spill metrics are correct
@@ -1066,5 +1096,5 @@ class TestQueryLogTableBufferPool(TestQueryLogTableBase):
       # potential for this test to not actually assert anything different than other
       # tests. Thus, an additional assert is needed to ensure that there actually was
       # data that was spilled.
-      assert data[COMPRESSED_BYTES_SPILLED] != "0", "compressed bytes spilled total " \
+      assert data["COMPRESSED_BYTES_SPILLED"] != "0", "compressed bytes spilled total " \
           "was zero, test did not assert anything"
