@@ -166,7 +166,9 @@ class TestRestart(CustomClusterTestSuite):
 
     self.cluster.catalogd.start()
     thread.join()
-    self.wait_for_state(query_handle[0], QueryState.FINISHED, 30000)
+    max_wait_time = 300
+    finished = self.client.wait_for_finished_timeout(query_handle[0], max_wait_time)
+    assert finished, "Statement did not finish after {0} seconds".format(max_wait_time)
 
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
@@ -201,7 +203,8 @@ class TestRestart(CustomClusterTestSuite):
     statestored_args="--statestore_update_frequency_ms=2000",
     impalad_args=("--wait_for_new_catalog_service_id_timeout_sec={} \
                   --wait_for_new_catalog_service_id_max_iterations=-1"
-                  .format(WAIT_FOR_CATALOG_UPDATE_TIMEOUT_SEC)))
+                  .format(WAIT_FOR_CATALOG_UPDATE_TIMEOUT_SEC)),
+    disable_log_buffering=True)
   def test_restart_catalogd_while_handling_rpc_response_with_timeout(self,
       unique_database):
     """Regression test for IMPALA-12267. We'd like to cause a situation where
@@ -219,28 +222,38 @@ class TestRestart(CustomClusterTestSuite):
     # Make the catalog object version grow large enough
     self.execute_query_expect_success(self.client, "invalidate metadata")
 
-    debug_action_sleep_time_sec = 10
+    # IMPALA-12616: If this sleep is not long enough, the alter table could wake up
+    # before the new catalog service ID is finalized, and the query can fail due to the
+    # difference in the service ID. This was a particular problem on s3, which runs a
+    # bit slower.
+    debug_action_sleep_time_sec = 30
     DEBUG_ACTION = ("WAIT_BEFORE_PROCESSING_CATALOG_UPDATE:SLEEP@{}"
                     .format(debug_action_sleep_time_sec * 1000))
 
     query = "alter table {} add columns (age int)".format(tbl_name)
     handle = self.execute_query_async(query, query_options={"debug_action": DEBUG_ACTION})
 
-    # Wait a bit so the RPC from the catalogd arrives to the coordinator.
-    time.sleep(0.5)
+    # Wait a bit so the RPC from the catalogd arrives to the coordinator. Using a generous
+    # value here gives the catalogd plenty of time to respond.
+    time.sleep(5)
 
     self.cluster.catalogd.restart()
 
     # Wait for the query to finish.
     max_wait_time = (debug_action_sleep_time_sec
         + self.WAIT_FOR_CATALOG_UPDATE_TIMEOUT_SEC + 10)
-    self.wait_for_state(handle, self.client.QUERY_STATES["FINISHED"], max_wait_time)
+    finished = self.client.wait_for_finished_timeout(handle, max_wait_time)
+    assert finished, "Statement did not finish after {0} seconds".format(max_wait_time)
 
     self.assert_impalad_log_contains("WARNING",
         "Waiting for catalog update with a new catalog service ID timed out.")
     self.assert_impalad_log_contains("WARNING",
         "Ignoring catalog update result of catalog service ID")
 
+    # Clear the query options so the following statements don't use the debug_action
+    # set above.
+    self.client.clear_configuration()
+
     self.execute_query_expect_success(self.client, "select age from {}".format(tbl_name))
 
     self.execute_query_expect_success(self.client,
@@ -256,7 +269,8 @@ class TestRestart(CustomClusterTestSuite):
         STATESTORE_UPDATE_FREQ_SEC * 1000),
     impalad_args=("--wait_for_new_catalog_service_id_timeout_sec=-1 \
                   --wait_for_new_catalog_service_id_max_iterations={}"
-                  .format(WAIT_FOR_CATALOG_UPDATE_MAX_ITERATIONS)))
+                  .format(WAIT_FOR_CATALOG_UPDATE_MAX_ITERATIONS)),
+    disable_log_buffering=True)
   def test_restart_catalogd_while_handling_rpc_response_with_max_iters(self,
       unique_database):
     """We create the same situation as described in
@@ -270,15 +284,20 @@ class TestRestart(CustomClusterTestSuite):
     # Make the catalog object version grow large enough
     self.execute_query_expect_success(self.client, "invalidate metadata")
 
-    debug_action_sleep_time_sec = 10
+    # IMPALA-12616: If this sleep is not long enough, the alter table could wake up
+    # before the new catalog service ID is finalized, and the query can fail due to the
+    # difference in the service ID. This was a particular problem on s3, which runs a
+    # bit slower.
+    debug_action_sleep_time_sec = 30
     DEBUG_ACTION = ("WAIT_BEFORE_PROCESSING_CATALOG_UPDATE:SLEEP@{}"
                     .format(debug_action_sleep_time_sec * 1000))
 
     query = "alter table {} add columns (age int)".format(tbl_name)
     handle = self.execute_query_async(query, query_options={"debug_action": DEBUG_ACTION})
 
-    # Wait a bit so the RPC from the catalogd arrives to the coordinator.
-    time.sleep(0.5)
+    # Wait a bit so the RPC from the catalogd arrives to the coordinator. Using a generous
+    # value here gives the catalogd plenty of time to respond.
+    time.sleep(5)
 
     self.cluster.catalogd.restart()
 
@@ -286,6 +305,10 @@ class TestRestart(CustomClusterTestSuite):
     # waiting for catalog updates.
     time.sleep(debug_action_sleep_time_sec + 0.5)
 
+    # Clear the query options so the following statements don't use the debug_action
+    # set above.
+    self.client.clear_configuration()
+
     # Issue DML queries so that the coordinator receives catalog updates.
     for i in range(self.WAIT_FOR_CATALOG_UPDATE_MAX_ITERATIONS):
       try:
@@ -297,7 +320,8 @@ class TestRestart(CustomClusterTestSuite):
 
     # Wait for the query to finish.
     max_wait_time = 10
-    self.wait_for_state(handle, self.client.QUERY_STATES["FINISHED"], max_wait_time)
+    finished = self.client.wait_for_finished_timeout(handle, max_wait_time)
+    assert finished, "Statement did not finish after {0} seconds".format(max_wait_time)
 
     expected_log_msg = "Received {} non-empty catalog updates from the statestore " \
         "while waiting for an update with a new catalog service ID but the catalog " \
@@ -875,7 +899,8 @@ class TestGracefulShutdown(CustomClusterTestSuite, HS2TestSuite):
           --shutdown_deadline_s={deadline} \
           --hostname={hostname}".format(grace_period=IDLE_SHUTDOWN_GRACE_PERIOD_S,
             deadline=EXEC_SHUTDOWN_DEADLINE_S, hostname=socket.gethostname()),
-      cluster_size=1)
+      cluster_size=1,
+      disable_log_buffering=True)
   def test_shutdown_signal(self):
     """Test that an idle impalad shuts down in a timely manner after the shutdown grace
     period elapses."""
@@ -898,7 +923,7 @@ class TestGracefulShutdown(CustomClusterTestSuite, HS2TestSuite):
         self.EXEC_SHUTDOWN_DEADLINE_S))
 
   @pytest.mark.execute_serially
-  @CustomClusterTestSuite.with_args(cluster_size=1)
+  @CustomClusterTestSuite.with_args(cluster_size=1, disable_log_buffering=True)
   def test_sending_multiple_shutdown_signals(self):
     """Test that multiple IMPALA_SHUTDOWN_SIGNAL signals are all handeled without
     crashing the process."""
