@@ -22,10 +22,8 @@ from builtins import range
 import os
 import pytest
 import re
-import shutil
 import stat
 import subprocess
-import tempfile
 import time
 
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
@@ -33,6 +31,7 @@ from tests.common.skip import SkipIf
 from tests.util.hdfs_util import NAMENODE
 from tests.verifiers.metric_verifier import MetricVerifier
 
+
 class TestScratchDir(CustomClusterTestSuite):
   @classmethod
   def get_workload(cls):
@@ -75,29 +74,30 @@ class TestScratchDir(CustomClusterTestSuite):
   def get_dirs(dirs):
     return ','.join(dirs)
 
-  def generate_dirs(self, num, writable=True, non_existing=False):
+  def get_name_for_dir(self, prefix, num):
+    return '{}_{}'.format(prefix, num)
+
+  def generate_dirs(self, prefix, num, writable=True, non_existing=False):
     result = []
     for i in range(num):
-      dir_path = tempfile.mkdtemp()
+      name = self.get_name_for_dir(prefix, i)
+      dir_path = self.make_tmp_dir(name)
+      result.append(dir_path)
+      print("Generated dir " + dir_path)
       if non_existing:
-        shutil.rmtree(dir_path)
+        self.clear_tmp_dir(name)
       elif not writable:
         os.chmod(dir_path, stat.S_IREAD)
-      if not non_existing:
-        self.created_dirs.append(dir_path)
-      result.append(dir_path)
-      print("Generated dir" + dir_path)
     return result
 
   def setup_method(self, method):
     # Don't call the superclass method to prevent starting Impala before each test. In
     # this file, each test is responsible for doing that because we want to generate
     # the parameter string to start-impala-cluster in each test method.
-    self.created_dirs = []
+    pass
 
   def teardown_method(self, method):
-    for dir_path in self.created_dirs:
-      shutil.rmtree(dir_path, ignore_errors=True)
+    self.clear_tmp_dirs()
     self.check_deleted_file_fd()
 
   @pytest.mark.execute_serially
@@ -105,7 +105,7 @@ class TestScratchDir(CustomClusterTestSuite):
     """ 5 empty directories are created in the /tmp directory and we verify that only
         one of those directories is used as scratch disk. Only one should be used as
         scratch because all directories are on same disk."""
-    normal_dirs = self.generate_dirs(5)
+    normal_dirs = self.generate_dirs('normal_dirs', 5)
     self._start_impala_cluster([
       '--impalad_args=-logbuflevel=-1 -scratch_dirs={0}'.format(','.join(normal_dirs)),
       '--impalad_args=--allow_multiple_scratch_dirs_per_device=false',
@@ -125,7 +125,7 @@ class TestScratchDir(CustomClusterTestSuite):
     """ Test we can execute a query with no scratch dirs """
     self._start_impala_cluster(['--impalad_args=-logbuflevel=-1 -scratch_dirs='])
     self.assert_impalad_log_contains("WARNING",
-        "Running without spill to disk: no scratch directories provided\.")
+        r"Running without spill to disk: no scratch directories provided\.")
     exec_option = vector.get_value('exec_option')
     exec_option['buffer_pool_limit'] = self.buffer_pool_limit
     impalad = self.cluster.get_any_impalad()
@@ -143,15 +143,15 @@ class TestScratchDir(CustomClusterTestSuite):
   @pytest.mark.execute_serially
   def test_non_writable_dirs(self, vector):
     """ Test we can execute a query with only bad non-writable scratch """
-    non_writable_dirs = self.generate_dirs(5, writable=False)
-    self._start_impala_cluster([
-      '--impalad_args=-logbuflevel=-1 -scratch_dirs={0}'.format(
-      ','.join(non_writable_dirs))])
+    non_writable_dirs = self.generate_dirs('non_writable_dirs', 5, writable=False)
+    self._start_impala_cluster(
+      ['--impalad_args=-logbuflevel=-1 -scratch_dirs={0}'.format(
+       ','.join(non_writable_dirs))])
     self.assert_impalad_log_contains("ERROR", "Running without spill to disk: could "
         + "not use any scratch directories in list:.*. See previous "
         + "warnings for information on causes.")
     self.assert_impalad_log_contains("WARNING", "Could not remove and recreate directory "
-            + ".*: cannot use it for scratch\. Error was: .*", expected_count=5)
+            + r".*: cannot use it for scratch\. Error was: .*", expected_count=5)
     exec_option = vector.get_value('exec_option')
     exec_option['buffer_pool_limit'] = self.buffer_pool_limit
     # IMPALA-9856: Disable query result spooling so that in_mem_query does not spill to
@@ -168,10 +168,10 @@ class TestScratchDir(CustomClusterTestSuite):
   @pytest.mark.execute_serially
   def test_non_existing_dirs(self, vector):
     """ Test that non-existing directories are not created or used """
-    non_existing_dirs = self.generate_dirs(5, non_existing=True)
-    self._start_impala_cluster([
-      '--impalad_args=-logbuflevel=-1 -scratch_dirs={0}'.format(
-      ','.join(non_existing_dirs))])
+    non_existing_dirs = self.generate_dirs('non_existing_dirs', 5, non_existing=True)
+    self._start_impala_cluster(
+      ['--impalad_args=-logbuflevel=-1 -scratch_dirs={0}'.format(
+       ','.join(non_existing_dirs))])
     self.assert_impalad_log_contains("ERROR", "Running without spill to disk: could "
         + "not use any scratch directories in list:.*. See previous "
         + "warnings for information on causes.")
@@ -195,7 +195,7 @@ class TestScratchDir(CustomClusterTestSuite):
   def test_write_error_failover(self, vector):
     """ Test that we can fail-over to writable directories if other directories
         have permissions changed or are removed after impalad startup."""
-    dirs = self.generate_dirs(3)
+    dirs = self.generate_dirs('write_error_failover', 3)
     self._start_impala_cluster([
       '--impalad_args=-logbuflevel=-1 -scratch_dirs={0}'.format(','.join(dirs)),
       '--impalad_args=--allow_multiple_scratch_dirs_per_device=true',
@@ -206,7 +206,8 @@ class TestScratchDir(CustomClusterTestSuite):
     exec_option = vector.get_value('exec_option')
     exec_option['buffer_pool_limit'] = self.buffer_pool_limit
     # Trigger errors when writing the first two directories.
-    shutil.rmtree(dirs[0]) # Remove the first directory.
+    # Remove the first directory.
+    self.clear_tmp_dir(self.get_name_for_dir('write_error_failover', 0))
     # Make all subdirectories in the second directory non-writable.
     for dirpath, dirnames, filenames in os.walk(dirs[1]):
       os.chmod(dirpath, stat.S_IREAD)
@@ -215,6 +216,9 @@ class TestScratchDir(CustomClusterTestSuite):
     impalad = self.cluster.get_any_impalad()
     client = impalad.service.create_beeswax_client()
     self.execute_query_expect_success(client, self.spill_query, exec_option)
+    # Restore second directory mod for cleanup later.
+    for dirpath, dirnames, filenames in os.walk(dirs[1]):
+      os.chmod(dirpath, stat.S_IRWXU)
 
   @pytest.mark.execute_serially
   def test_scratch_dirs_default_priority(self, vector):
@@ -222,7 +226,7 @@ class TestScratchDir(CustomClusterTestSuite):
         of those directories are used as scratch disk. By default, all directories
         should have the same priority and so all should be used in a round robin
         manner."""
-    normal_dirs = self.generate_dirs(5)
+    normal_dirs = self.generate_dirs('scratch_dirs_default_priority', 5)
     self._start_impala_cluster([
       '--impalad_args=-logbuflevel=-1 -scratch_dirs={0}'.format(','.join(normal_dirs)),
       '--impalad_args=--allow_multiple_scratch_dirs_per_device=true'], cluster_size=1,
@@ -247,7 +251,7 @@ class TestScratchDir(CustomClusterTestSuite):
   def test_scratch_dirs_prioritized_spill(self, vector):
     """ 5 empty directories are created in the /tmp directory and we verify that only
         the directories with highest priority are used as scratch disk."""
-    normal_dirs = self.generate_dirs(5)
+    normal_dirs = self.generate_dirs('scratch_dirs_prioritized_spill', 5)
     normal_dirs[0] = '{0}::{1}'.format(normal_dirs[0], 1)
     normal_dirs[1] = '{0}::{1}'.format(normal_dirs[1], 0)
     normal_dirs[2] = '{0}::{1}'.format(normal_dirs[2], 1)
@@ -318,7 +322,7 @@ class TestScratchDir(CustomClusterTestSuite):
   @SkipIf.not_scratch_fs
   def test_scratch_dirs_remote_spill(self, vector):
     # Test one remote directory with one its local buffer directory.
-    normal_dirs = self.generate_dirs(1)
+    normal_dirs = self.generate_dirs('scratch_dirs_remote_spill', 1)
     # Use dfs for testing.
     normal_dirs.append(self.dfs_tmp_path())
     self._start_impala_cluster([
@@ -348,7 +352,7 @@ class TestScratchDir(CustomClusterTestSuite):
     '''Two local directories, the first one is always used as local buffer for
        remote directories. Set the second directory big enough so that only spills
        to local in the test'''
-    normal_dirs = self.generate_dirs(2)
+    normal_dirs = self.generate_dirs('scratch_dirs_mix_spill_local', 2)
     normal_dirs[0] = '{0}::{1}'.format(normal_dirs[0], 1)
     normal_dirs[1] = '{0}:2GB:{1}'.format(normal_dirs[1], 0)
     normal_dirs.append(self.dfs_tmp_path())
@@ -382,7 +386,7 @@ class TestScratchDir(CustomClusterTestSuite):
     '''Two local directories, the first one is always used as local buffer for
        remote directories. Set the second directory small enough so that it spills
        to both directories in the test'''
-    normal_dirs = self.generate_dirs(2)
+    normal_dirs = self.generate_dirs('scratch_dirs_mix_spill_both', 2)
     normal_dirs[0] = '{0}:32MB:{1}'.format(normal_dirs[0], 0)
     normal_dirs[1] = '{0}:4MB:{1}'.format(normal_dirs[1], 1)
     normal_dirs.append(self.dfs_tmp_path())
@@ -415,7 +419,7 @@ class TestScratchDir(CustomClusterTestSuite):
   @SkipIf.not_scratch_fs
   def test_scratch_dirs_remote_spill_with_options(self, vector):
     # One local buffer directory and one remote directory.
-    normal_dirs = self.generate_dirs(1)
+    normal_dirs = self.generate_dirs('scratch_dirs_remote_spill_with_option', 1)
     normal_dirs[0] = '{0}:16MB:{1}'.format(normal_dirs[0], 0)
     normal_dirs.append(self.dfs_tmp_path())
     self._start_impala_cluster([
@@ -449,7 +453,7 @@ class TestScratchDir(CustomClusterTestSuite):
   def test_scratch_dirs_remote_spill_concurrent(self, vector):
     '''Concurrently execute multiple queries that trigger the spilling to the remote
     directory to test if there is a deadlock issue.'''
-    normal_dirs = self.generate_dirs(1)
+    normal_dirs = self.generate_dirs('scratch_dirs_remote_spill_concurrent', 1)
     normal_dirs[0] = '{0}:16MB:{1}'.format(normal_dirs[0], 0)
     normal_dirs.append(self.dfs_tmp_path())
     num = 5
@@ -495,7 +499,7 @@ class TestScratchDir(CustomClusterTestSuite):
   @SkipIf.not_scratch_fs
   def test_scratch_dirs_batch_reading(self, vector):
     # Set the buffer directory small enough to spill to the remote one.
-    normal_dirs = self.generate_dirs(1)
+    normal_dirs = self.generate_dirs('scratch_dirs_batch_reading', 1)
     normal_dirs[0] = '{0}:2MB:{1}'.format(normal_dirs[0], 1)
     normal_dirs.append(self.dfs_tmp_path())
     self._start_impala_cluster([
