@@ -32,6 +32,7 @@ from tests.common.skip import SkipIfFS
 from tests.shell.util import run_impala_shell_cmd
 
 SMALL_QUERY_LOG_SIZE_IN_BYTES = 40 * 1024
+CATALOG_URL = "http://localhost:25020/catalog"
 
 
 class TestWebPage(CustomClusterTestSuite):
@@ -297,7 +298,8 @@ class TestWebPage(CustomClusterTestSuite):
     assert response.status_code == requests.codes.ok
     operations = json.loads(response.text)
     assert "inflight_catalog_operations" in operations
-    return operations["inflight_catalog_operations"]
+    assert "num_inflight_catalog_ops" in operations
+    return operations
 
   @staticmethod
   def _get_finished_catalog_operations():
@@ -377,7 +379,9 @@ class TestWebPage(CustomClusterTestSuite):
       assert "RPC recv timed out" in str(e)
     # In impalad side, the query fails by the above error. However, in catalogd side,
     # the RPCs are still running. Check the in-flight operations.
-    inflight_operations = self._get_inflight_catalog_operations()
+    operations = self._get_inflight_catalog_operations()
+    assert operations["num_inflight_catalog_ops"] == 2
+    inflight_operations = operations["inflight_catalog_operations"]
     assert len(inflight_operations) == 2
     for op in inflight_operations:
       assert op["status"] == "STARTED"
@@ -387,7 +391,7 @@ class TestWebPage(CustomClusterTestSuite):
     assert inflight_operations[0]["thread_id"] != inflight_operations[1]["thread_id"]
 
     # Wait until the catalog operations finish
-    while len(self._get_inflight_catalog_operations()) != 0:
+    while self._get_inflight_catalog_operations()["num_inflight_catalog_ops"] != 0:
       time.sleep(1)
 
     # Verify both RPC attempts are shown as finished operations.
@@ -502,3 +506,43 @@ class TestWebPage(CustomClusterTestSuite):
     self.execute_query("invalidate metadata")
     page = requests.get("http://localhost:25020/events").text
     assert "Unexpected exception" not in page, "Still see error message:\n" + page
+
+  def _test_catalog_tables_stats_after_describe(self, table_full_name, num_files):
+    """Test the lists of tables with Most Number of Files and Highest Memory Requirements
+    in the catalog page. Start a new cluster to make sure the table is not loaded before
+    DESCRIBE."""
+
+    def get_table_metric(content, list_name, key):
+      table_list = content[list_name]
+      for table in table_list:
+        if table["name"] == table_full_name:
+          return table[key]
+      return None
+
+    # The table is not in the lists after the cluster starts
+    content = self.get_debug_page(CATALOG_URL + "?json")
+    assert get_table_metric(content, "large_tables", "mem_estimate") is None
+    assert get_table_metric(content, "high_file_count_tables", "num_files") is None
+
+    self.client.execute("DESCRIBE {0}".format(table_full_name))
+    content = self.get_debug_page(CATALOG_URL + "?json")
+    assert get_table_metric(content, "large_tables", "mem_estimate") > 0
+    assert get_table_metric(content, "high_file_count_tables", "num_files") == num_files
+
+  @pytest.mark.execute_serially
+  @CustomClusterTestSuite.with_args(
+    catalogd_args="--catalog_topic_mode=full",
+    impalad_args="--use_local_catalog=false")
+  def test_catalog_tables_stats_legacy_catalog(self):
+    self._test_catalog_tables_stats_after_describe("functional.alltypes", 24)
+    self._test_catalog_tables_stats_after_describe(
+        "functional_parquet.iceberg_lineitem_sixblocks", 4)
+
+  @pytest.mark.execute_serially
+  @CustomClusterTestSuite.with_args(
+    catalogd_args="--catalog_topic_mode=minimal",
+    impalad_args="--use_local_catalog=true")
+  def test_catalog_tables_stats_local_catalog(self):
+    self._test_catalog_tables_stats_after_describe("functional.alltypes", 24)
+    self._test_catalog_tables_stats_after_describe(
+        "functional_parquet.iceberg_lineitem_sixblocks", 4)
