@@ -23,7 +23,8 @@
 # TODO: For each test, verify all the metadata available via Hive and
 # Impala, in all the possible ways of validating that metadata.
 
-
+from __future__ import absolute_import, division, print_function
+from builtins import range
 import pytest
 import random
 import string
@@ -120,29 +121,16 @@ class TestHmsIntegrationSanity(ImpalaTestSuite):
     self.client.execute("DESCRIBE {0}.json_tbl"
                         .format(unique_database))
 
-  def test_json_file_unsupported(self, unique_database):
-    """
-    Since JSON file format is not yet supported,this function tests
-    the blocking logic of reading JSON tables.
-    """
-    self.client.execute("create table {0}.json_tbl(id int, name string, age int)"
-                        " stored as jsonfile".format(unique_database))
-    self.run_stmt_in_hive("insert  into {0}.json_tbl values(0,'Alice',10)"
-                          .format(unique_database))
-    self.run_stmt_in_hive("insert  into {0}.json_tbl values(1,'Bob',20)"
-                          .format(unique_database))
-    self.run_stmt_in_hive("insert  into {0}.json_tbl values(2,'Oracle',16)"
-                          .format(unique_database))
-    self.client.execute("refresh {0}.json_tbl".format(unique_database))
-    self.client.execute("show files in {0}.json_tbl".format(unique_database))
+  def test_invalidate_metadata(self, unique_name):
+    """Verify invalidate metadata on tables under unloaded db won't fail"""
+    db = unique_name + "_db"
+    tbl = db + "." + unique_name + "_tbl"
     try:
-      self.client.execute("select * from {0}.json_tbl".format(unique_database))
-    except Exception as e:
-      assert 'Scan of table {0}.json_tbl in format \'JSON\' is not supported.'\
-        .format(unique_database) in str(e)
-    else:
-      assert False
-
+      self.run_stmt_in_hive("create database " + db)
+      self.run_stmt_in_hive("create table %s (i int)" % tbl)
+      self.client.execute("invalidate metadata %s" % tbl)
+    finally:
+      self.run_stmt_in_hive("drop database %s cascade" % db)
 
 @SkipIfFS.hive
 class TestHmsIntegration(ImpalaTestSuite):
@@ -206,18 +194,12 @@ class TestHmsIntegration(ImpalaTestSuite):
     output = self.client.execute('show table stats %s' % table).get_data()
     output_lines = output.split('\n')
     result = {}
+    headers = ['#rows', '#files', 'size', 'bytes cached', 'cache replication',
+        'format', 'incremental stats', 'location', 'ec policy']
     for line in output_lines:
       parts = line.split('\t')
-      stats = {}
-      stats['location'] = parts[-1]
-      stats['incremental stats'] = parts[-2]
-      stats['format'] = parts[-3]
-      stats['cache replication'] = parts[-4]
-      stats['bytes cached'] = parts[-5]
-      stats['size'] = parts[-6]
-      stats['#files'] = parts[-7]
-      stats['#rows'] = parts[-8]
-      result[tuple(parts[:-8])] = stats
+      id_parts, stat_parts = parts[:-len(headers)], parts[-len(headers):]
+      result[tuple(id_parts)] = dict(zip(headers, stat_parts))
     return result
 
   def impala_all_column_stats(self, table):
@@ -241,7 +223,7 @@ class TestHmsIntegration(ImpalaTestSuite):
     dictionary that holds the parsed attributes."""
     result = {}
     output_lines = output.split('\n')
-    stat_names = map(string.strip, output_lines[0].split(','))
+    stat_names = [s.strip() for s in output_lines[0].split(',')]
     stat_values = output_lines[3].split(',')
     assert len(stat_names) == len(stat_values)
     for i in range(0, len(stat_names)):
@@ -253,7 +235,7 @@ class TestHmsIntegration(ImpalaTestSuite):
     dictionary that holds the parsed attributes."""
     result = {}
     for line in output.split('\n'):
-      line_elements = map(string.strip, line.split(','))
+      line_elements = [s.strip() for s in line.split(',')]
       if len(line_elements) >= 2:
         result[line_elements[0]] = line_elements[1]
     return result
@@ -741,7 +723,7 @@ class TestHmsIntegration(ImpalaTestSuite):
                             '(x int, y int) stored as parquet') as table_name:
         self.client.execute('invalidate metadata')
         self.client.execute('invalidate metadata %s' % table_name)
-        print self.impala_table_stats(table_name)
+        print(self.impala_table_stats(table_name))
         assert 'PARQUET' == self.impala_table_stats(table_name)[()]['format']
         self.run_stmt_in_hive(
             'alter table %s set fileformat avro' % table_name)
