@@ -16,7 +16,7 @@
 # under the License.
 
 from __future__ import absolute_import, division, print_function
-from test_ddl_base import TestDdlBase
+from tests.metadata.test_ddl_base import TestDdlBase
 
 
 class TestResetMetadata(TestDdlBase):
