@@ -47,7 +47,8 @@ class TestCodegen(ImpalaTestSuite):
   def test_select_node_codegen(self, vector):
     """Test that select node is codegened"""
     result = self.execute_query('select * from (select * from functional.alltypes '
-        'limit 1000000) t1 where int_col > 10 limit 10')
+        'limit 1000000) t1 where int_col > 10 limit 10',
+        {'disable_codegen_rows_threshold': 7000})
     exec_options = get_node_exec_options(result.runtime_profile, 1)
     # Make sure test fails if there are no exec options in the profile for the node
     assert len(exec_options) > 0
