@@ -22,14 +22,16 @@ from __future__ import absolute_import, division, print_function
 from builtins import range
 import decimal
 import math
-import pytest
 import random
 
 from tests.beeswax.impala_beeswax import ImpalaBeeswaxException
 from tests.common.impala_test_suite import ImpalaTestSuite
-from tests.common.test_dimensions import create_single_exec_option_dimension
+from tests.common.test_dimensions import (
+  add_mandatory_exec_option,
+  create_single_exec_option_dimension)
 from tests.common.test_vector import ImpalaTestDimension, ImpalaTestMatrix
 
+
 class TestDecimalFuzz(ImpalaTestSuite):
 
   # Impala's max precision for decimals is 38, so we should have the same in the tests
@@ -43,7 +45,13 @@ class TestDecimalFuzz(ImpalaTestSuite):
   def add_test_dimensions(cls):
     cls.ImpalaTestMatrix = ImpalaTestMatrix()
     cls.ImpalaTestMatrix.add_dimension(create_single_exec_option_dimension())
-    cls.iterations = 10000
+
+    total_iterations = 10000
+    batches = list(range(0, 10))
+    cls.iterations = total_iterations / len(batches)
+    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension("test_batch", *batches))
+    add_mandatory_exec_option(cls, 'decimal_v2', 'true')
+    add_mandatory_exec_option(cls, 'long_polling_time_ms', 100)
 
   def weighted_choice(self, options):
     total_weight = sum(options.values())
@@ -202,7 +210,7 @@ class TestDecimalFuzz(ImpalaTestSuite):
         return True
     return False
 
-  def execute_one_decimal_op(self):
+  def execute_one_decimal_op(self, query_options):
     '''Executes a single query and compares the result to a result that we computed in
     Python.'''
     op = random.choice(['+', '-', '*', '/', '%'])
@@ -215,8 +223,8 @@ class TestDecimalFuzz(ImpalaTestSuite):
         value2=value2, precision2=precision2, scale2=scale2)
 
     try:
-      result = self.execute_scalar(query, query_options={'decimal_v2': 'true'})
-    except ImpalaBeeswaxException as e:
+      result = self.execute_scalar(query, query_options)
+    except ImpalaBeeswaxException:
       result = None
     if result is not None:
       result = decimal.Decimal(result)
@@ -239,15 +247,15 @@ class TestDecimalFuzz(ImpalaTestSuite):
           expected_result = decimal.Decimal(value1) % decimal.Decimal(value2)
         else:
           assert False
-      except decimal.InvalidOperation as e:
+      except decimal.InvalidOperation:
         expected_result = None
-      except decimal.DivisionByZero as e:
+      except decimal.DivisionByZero:
         expected_result = None
       assert self.result_equals(expected_result, result)
 
   def test_decimal_ops(self, vector):
     for _ in range(self.iterations):
-      self.execute_one_decimal_op()
+      self.execute_one_decimal_op(vector.get_exec_option_dict())
 
   def width_bucket(self, val, min_range, max_range, num_buckets):
     # Multiplying the values by 10**40 guarantees that the numbers can be converted
@@ -267,7 +275,7 @@ class TestDecimalFuzz(ImpalaTestSuite):
     dist_from_min = val_int - min_range_int
     return (num_buckets * dist_from_min) // range_size + 1
 
-  def execute_one_width_bucket(self):
+  def execute_one_width_bucket(self, query_options):
     val, val_prec, val_scale = self.get_decimal()
     min_range, min_range_prec, min_range_scale = self.get_decimal()
     max_range, max_range_prec, max_range_scale = self.get_decimal()
@@ -291,7 +299,7 @@ class TestDecimalFuzz(ImpalaTestSuite):
       return
 
     try:
-      result = self.execute_scalar(query, query_options={'decimal_v2': 'true'})
+      result = self.execute_scalar(query, query_options)
       assert int(result) == expected_result
     except ImpalaBeeswaxException as e:
       if "You need to wrap the arguments in a CAST" not in str(e):
@@ -301,4 +309,4 @@ class TestDecimalFuzz(ImpalaTestSuite):
 
   def test_width_bucket(self, vector):
     for _ in range(self.iterations):
-      self.execute_one_width_bucket()
+      self.execute_one_width_bucket(vector.get_exec_option_dict())
