@@ -17,13 +17,13 @@
 
 from __future__ import absolute_import, division, print_function
 from builtins import range
+from collections import defaultdict, namedtuple
 import datetime
 import logging
 import os
 import pytest
 import pytz
 import random
-
 import re
 import time
 
@@ -39,11 +39,12 @@ from tests.beeswax.impala_beeswax import ImpalaBeeswaxException
 from tests.common.iceberg_test_suite import IcebergTestSuite
 from tests.common.skip import SkipIf, SkipIfFS, SkipIfDockerizedCluster
 from tests.common.test_dimensions import add_exec_option_dimension
+from tests.common.test_result_verifier import error_msg_expected
 from tests.common.file_utils import (
   create_iceberg_table_from_directory,
   create_table_from_parquet)
 from tests.shell.util import run_impala_shell_cmd
-from tests.util.filesystem_utils import get_fs_path, IS_HDFS
+from tests.util.filesystem_utils import get_fs_path, IS_HDFS, WAREHOUSE, FILESYSTEM_PREFIX
 from tests.util.get_parquet_metadata import get_parquet_metadata
 from tests.util.iceberg_util import cast_ts, quote, get_snapshots, IcebergCatalogs
 
@@ -269,6 +270,69 @@ class TestIcebergTable(IcebergTestSuite):
     self.run_test_case('QueryTest/iceberg-migrated-table-field-id-resolution',
                        vector, unique_database)
 
+  def test_column_case_sensitivity(self, vector, unique_database):
+    create_iceberg_table_from_directory(self.client, unique_database,
+        "iceberg_column_case_sensitivity_issue", "parquet")
+    self.run_test_case('QueryTest/iceberg-column-case-sensitivity-issue',
+                       vector, unique_database)
+
+  @SkipIfFS.hive
+  def test_migrated_table_field_id_resolution_complex(self, vector, unique_database):
+    def get_table_loc(tbl_name):
+      return '%s/%s.db/%s/' % (WAREHOUSE, unique_database, tbl_name)
+
+    def create_table(tbl_name, file_format, partition_cols):
+      self.execute_query("""CREATE TABLE IF NOT EXISTS {}.{} (
+            id INT,
+            name STRING,
+            teststeps array<struct<step_number:int,step_description:string>>)
+          PARTITIONED BY ({})
+          STORED AS {}
+          """.format(unique_database, tbl_name, partition_cols, file_format))
+
+    def add_file_to_table_partition(tbl_name, part_dir, local_filename):
+      tbl_loc = get_table_loc(tbl_name)
+      part_dir = os.path.join(tbl_loc, part_dir)
+      self.filesystem_client.make_dir(part_dir)
+      data_file_path = os.path.join(os.environ['IMPALA_HOME'], "testdata",
+          "migrated_iceberg", local_filename)
+      self.filesystem_client.copy_from_local(data_file_path, part_dir)
+
+    def finalize_table(tbl_name):
+      self.execute_query("ALTER TABLE {}.{} RECOVER PARTITIONS".format(
+          unique_database, tbl_name))
+      self.execute_query("ALTER TABLE {}.{} CONVERT TO ICEBERG".format(
+          unique_database, tbl_name))
+
+    def prepare_test_table(tbl_name, file_format, partition_cols, part_dir, datafile):
+      create_table(tbl_name, file_format, partition_cols)
+      add_file_to_table_partition(tbl_name, part_dir, datafile)
+      finalize_table(tbl_name)
+
+    prepare_test_table('all_part_cols_stored_parquet',
+        "PARQUET",
+        "result_date STRING",
+        "result_date=2024-08-26",
+        "complextypes_and_partition_columns_in_data_files.parquet")
+    prepare_test_table('not_all_part_cols_stored_parquet',
+        "PARQUET",
+        "result_date STRING, p INT",
+        "result_date=2024-08-26/p=3",
+        "complextypes_and_partition_columns_in_data_files.parquet")
+    prepare_test_table('all_part_cols_stored_orc',
+        "ORC",
+        "result_date STRING",
+        "result_date=2024-08-26",
+        "complextypes_and_partition_columns_in_data_files.orc")
+    prepare_test_table('not_all_part_cols_stored_orc',
+        "ORC",
+        "result_date STRING, p INT",
+        "result_date=2024-08-26/p=3",
+        "complextypes_and_partition_columns_in_data_files.orc")
+
+    self.run_test_case('QueryTest/iceberg-migrated-table-field-id-resolution-complex',
+                       vector, unique_database)
+
   def test_describe_history(self, vector, unique_database):
     self.run_test_case('QueryTest/iceberg-table-history', vector, use_db=unique_database)
 
@@ -1378,7 +1442,7 @@ class TestIcebergTable(IcebergTestSuite):
         query_options=abort_ice_transaction_options)
     # Check that the error message looks reasonable.
     result = str(err)
-    assert "Query aborted:CommitFailedException: simulated commit failure" in result
+    assert error_msg_expected(result, "CommitFailedException: simulated commit failure")
     # Check that no data was inserted.
     data = self.execute_query_expect_success(self.client,
         "select * from {0}".format(tbl_name))
@@ -1393,7 +1457,8 @@ class TestIcebergTable(IcebergTestSuite):
         .format(tbl_name, "j"), query_options=abort_ice_transaction_options)
     ddl_result = str(ddl_err)
     # Check that the error message looks reasonable.
-    assert "Query aborted:CommitFailedException: simulated commit failure" in ddl_result
+    assert error_msg_expected(ddl_result,
+                              "CommitFailedException: simulated commit failure")
     # Check that no column was added.
     data = self.execute_query_expect_success(self.client,
         "select * from {0}".format(tbl_name))
@@ -1461,8 +1526,26 @@ class TestIcebergV2Table(IcebergTestSuite):
   @SkipIfDockerizedCluster.internal_hostname
   @SkipIf.hardcoded_uris
   def test_read_position_deletes(self, vector):
+    # Remove 'batch_size' option so we can set it at .test file.
+    # Revisit this if 'batch_size' dimension size increase.
+    vector.unset_exec_option('batch_size')
     self.run_test_case('QueryTest/iceberg-v2-read-position-deletes', vector)
 
+  @SkipIfDockerizedCluster.internal_hostname
+  @SkipIf.hardcoded_uris
+  def test_read_position_deletes_orc(self, vector):
+    self.run_test_case('QueryTest/iceberg-v2-read-position-deletes-orc', vector)
+
+  @SkipIfDockerizedCluster.internal_hostname
+  @SkipIf.hardcoded_uris
+  @pytest.mark.execute_serially
+  def test_read_position_deletes_compute_stats(self, vector):
+    """Tests COMPUTE STATS on Iceberg V2 tables. Need to be executed serially
+    because it modifies tables that are used by other tests (e.g. multiple
+    instances of this test)."""
+    self.run_test_case('QueryTest/iceberg-v2-read-position-deletes-stats', vector)
+    self.run_test_case('QueryTest/iceberg-v2-read-position-deletes-orc-stats', vector)
+
   @SkipIfFS.hive
   def test_read_mixed_format_position_deletes(self, vector, unique_database):
     self.run_test_case('QueryTest/iceberg-mixed-format-position-deletes',
@@ -1498,11 +1581,6 @@ class TestIcebergV2Table(IcebergTestSuite):
   def test_read_equality_deletes(self, vector):
     self.run_test_case('QueryTest/iceberg-v2-read-equality-deletes', vector)
 
-  @SkipIfDockerizedCluster.internal_hostname
-  @SkipIf.hardcoded_uris
-  def test_read_position_deletes_orc(self, vector):
-    self.run_test_case('QueryTest/iceberg-v2-read-position-deletes-orc', vector)
-
   @SkipIfDockerizedCluster.internal_hostname
   @SkipIf.hardcoded_uris
   def test_table_sampling_v2(self, vector):
@@ -1511,6 +1589,9 @@ class TestIcebergV2Table(IcebergTestSuite):
 
   @SkipIf.hardcoded_uris
   def test_metadata_tables(self, vector, unique_database):
+    # Remove 'batch_size' option so we can set it at .test file.
+    # Revisit this if 'batch_size' dimension size increase.
+    vector.unset_exec_option('batch_size')
     with self.create_impala_client() as impalad_client:
       overwrite_snapshot_id = impalad_client.execute("select snapshot_id from "
                              "functional_parquet.iceberg_query_metadata.snapshots "
@@ -1613,8 +1694,9 @@ class TestIcebergV2Table(IcebergTestSuite):
     assert hive_output == "id\n4\n5\n6\n7\n8\n"
 
   def test_update_basic(self, vector, unique_database):
+    udf_location = get_fs_path('/test-warehouse/libTestUdfs.so')
     self.run_test_case('QueryTest/iceberg-update-basic', vector,
-        unique_database)
+        unique_database, test_file_vars={'UDF_LOCATION': udf_location})
     self._test_update_basic_snapshots(unique_database)
     if IS_HDFS and self.should_run_for_hive(vector):
       self._update_basic_hive_tests(unique_database)
@@ -1788,6 +1870,184 @@ class TestIcebergV2Table(IcebergTestSuite):
             tbl_name, snapshot_before_last.get_snapshot_id()))
     assert result_after_opt.data.sort() == result_time_travel.data.sort()
 
+  def _check_file_filtering(self, tbl_name, threshold_mb, mode, had_partition_evolution):
+    threshold_bytes = threshold_mb * 1024 * 1024
+    DATA_FILE = "0"
+    DELETE_FILE = "1"
+    FileMetadata = namedtuple('FileMetadata', 'content, path, partition, size')
+    metadata_query = """select content, file_path, `partition`, file_size_in_bytes
+                    from {0}.`files`;""".format(tbl_name)
+    result = self.execute_query(metadata_query)
+    files_before = set()
+    files_per_partition = defaultdict(set)
+    for line in result.data:
+      file = FileMetadata._make(line.split("\t"))
+      partition = file.partition
+      files_per_partition[partition].add(file)
+      files_before.add(file)
+
+    selected_files = set()
+    partitions_with_removed_files = set()
+    for partition, files in files_per_partition.items():
+        if len(files) > 1:
+          num_small_files = 0
+          # count small datafiles
+          for file in files:
+            if file.content == DATA_FILE and int(file.size) < threshold_bytes:
+              num_small_files += 1
+          for file in files:
+            # We assume that a delete file in a partition references all data files in
+            # that partition, because we cannot differentiate between data files
+            # with/without deletes.
+            if file.content == DELETE_FILE:
+              selected_files.update(files)
+              partitions_with_removed_files.add(partition)
+              break
+            # Only merge small files if there are at least 2 of them.
+            elif num_small_files > 1 and int(file.size) < threshold_bytes:
+              selected_files.add(file)
+              partitions_with_removed_files.add(partition)
+
+    self.execute_query(
+        "OPTIMIZE TABLE {0} (file_size_threshold_mb={1});".format(tbl_name, threshold_mb))
+    optimized_result = self.execute_query(metadata_query)
+    files_after = set()
+    for line in optimized_result.data:
+      file = FileMetadata._make(line.split("\t"))
+      files_after.add(file)
+
+    # Check the resulting files and the modified partitions after the OPTIMIZE operation.
+    # files_after = files_before - selected_files + 1 new file per partition
+    # The result should not contain the files that were selected and should contain 1 new
+    # file per written partition.
+    unchanged_files = files_before - selected_files
+    # Check that files that were not selected are still present in the result.
+    assert unchanged_files.issubset(files_after)
+    # Check that selected files are rewritten and not present in the result.
+    assert selected_files.isdisjoint(files_after)
+    new_files = files_after - unchanged_files
+    assert new_files == files_after - files_before
+
+    if mode == "NOOP":
+      assert selected_files == set([])
+      assert files_after == files_before
+    elif mode == "REWRITE_ALL":
+      assert selected_files == files_before
+      assert files_after.isdisjoint(files_before)
+    elif mode == "PARTIAL":
+      assert selected_files < files_before and selected_files != set([])
+      assert unchanged_files < files_after and unchanged_files != set([])
+      assert unchanged_files == files_after.intersection(files_before)
+
+    # Check that all delete files were merged.
+    for file in files_after:
+      assert file.content == DATA_FILE
+    # Check that there's only one new file in every partition.
+    partitions_with_new_files = set()
+    for file in new_files:
+      assert file.partition not in partitions_with_new_files
+      partitions_with_new_files.add(file.partition)
+    assert len(new_files) == len(partitions_with_new_files)
+
+    # WITH PARTITION EVOLUTION
+    # Only new partitions are written to.
+    # WITHOUT PARTITION EVOLUTION
+    if not had_partition_evolution:
+      # Check that 1 new content file is written in every updated partition.
+      assert len(new_files) == len(partitions_with_removed_files)
+      assert partitions_with_new_files == partitions_with_removed_files
+
+  def test_optimize_file_filtering(self, unique_database):
+    tbl_name = unique_database + ".ice_optimize_filter"
+    self.execute_query("""CREATE TABLE {0} partitioned by spec (l_linenumber)
+                       STORED BY ICEBERG TBLPROPERTIES ('format-version'='2')
+                       AS SELECT * FROM tpch_parquet.lineitem
+                       WHERE l_quantity < 10;""".format(tbl_name))
+    self.execute_query("""INSERT INTO {0} SELECT * FROM tpch_parquet.lineitem
+                       WHERE l_quantity>=10 AND l_quantity<=12;""".format(tbl_name))
+    # There are no delete files in the table, so this should be a no-op. Check that no new
+    # snapshot was created.
+    self._check_file_filtering(tbl_name, 0, "NOOP", False)
+    assert len(get_snapshots(self.client, tbl_name)) == 2
+    self._check_file_filtering(tbl_name, 5, "PARTIAL", False)
+    self._check_file_filtering(tbl_name, 50, "PARTIAL", False)
+    # Check that the following is a no-op, since the table is already in a compact form.
+    self._check_file_filtering(tbl_name, 100, "NOOP", False)
+    self.execute_query("""UPDATE {0} SET l_linenumber=7 WHERE l_linenumber>4 AND
+                       l_linestatus='F';""".format(tbl_name))
+    self._check_file_filtering(tbl_name, 6, "PARTIAL", False)
+    self.execute_query("""ALTER TABLE {0} SET PARTITION SPEC(l_linestatus);"""
+                       .format(tbl_name))
+    self.execute_query("""UPDATE {0} SET l_shipmode='AIR' WHERE l_shipmode='MAIL'
+                       AND l_linenumber<4;""".format(tbl_name))
+    self.execute_query("""INSERT INTO {0} SELECT * FROM tpch_parquet.lineitem
+                       WHERE l_quantity=13 AND l_linenumber<3;""".format(tbl_name))
+    self.execute_query("""INSERT INTO {0} SELECT * FROM tpch_parquet.lineitem
+                       WHERE l_quantity=14 AND l_linenumber<3;""".format(tbl_name))
+    # Merges the delete files and rewrites the small files.
+    self._check_file_filtering(tbl_name, 2, "PARTIAL", True)
+    # Rewrites the remaining small files (2MB <= file_size < 100MB).
+    self._check_file_filtering(tbl_name, 100, "PARTIAL", True)
+    self.execute_query("""UPDATE {0} SET l_shipmode='AIR' WHERE l_shipmode='MAIL';"""
+                       .format(tbl_name))
+    # All partitions have delete files, therefore the entire table is rewritten.
+    self._check_file_filtering(tbl_name, 100, "REWRITE_ALL", True)
+
+  def test_merge(self, vector, unique_database):
+    udf_location = get_fs_path('/test-warehouse/libTestUdfs.so')
+    self.run_test_case('QueryTest/iceberg-merge', vector, unique_database,
+        test_file_vars={'UDF_LOCATION': udf_location})
+
+  def test_merge_partition(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-merge-partition', vector, unique_database)
+
+  def test_merge_partition_sort(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-merge-partition-sort', vector, unique_database)
+
+  def test_merge_long(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-merge-long', vector, unique_database)
+
+  def test_merge_star(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-merge-star', vector, unique_database)
+
+  def test_cleanup(self, unique_database):
+      """Test that all uncommitted files written by Impala are removed from the file
+      system when a DML commit to an Iceberg table fails, and that the effects of the
+      failed operation are not visible."""
+      table_name = "iceberg_cleanup_failure"
+      fq_tbl_name = unique_database + "." + table_name
+      # The query options that inject an iceberg validation check failure.
+      fail_ice_commit_options = {'debug_action':
+                        'CATALOGD_ICEBERG_CONFLICT:EXCEPTION@'
+                        'ValidationException@'
+                        'simulated validation check failure'}
+      # Create an iceberg table and insert a row.
+      self.execute_query_expect_success(self.client, """CREATE TABLE {0} (i int)
+          STORED BY ICEBERG TBLPROPERTIES ('format-version'='2')""".format(fq_tbl_name))
+      self.execute_query_expect_success(self.client,
+          "insert into {0} values (1)".format(fq_tbl_name))
+
+      # Run a query that would update a row, but pass the query options that
+      # will cause the iceberg validation check to fail.
+      err = self.execute_query_expect_failure(self.client,
+          "update {0} set i=2 where i=1".format(fq_tbl_name),
+          query_options=fail_ice_commit_options)
+      # Check that we get the error message.
+      assert error_msg_expected(
+          str(err), "ValidationException: simulated validation check failure")
+      # Check that the table content was not updated.
+      data = self.execute_query_expect_success(self.client,
+          "select * from {0}".format(fq_tbl_name))
+      assert len(data.data) == 1
+      assert data.data[0] == '1'
+
+      # Check that the uncommitted data and delete files are removed from the file system
+      # and only the first data file remains.
+      table_location = "{0}/test-warehouse/{1}.db/{2}/data".format(
+          FILESYSTEM_PREFIX, unique_database, table_name)
+      files_result = check_output(["hdfs", "dfs", "-ls", table_location])
+      assert "Found 1 items" in files_result
+
 
 # Tests to exercise the DIRECTED distribution mode for V2 Iceberg tables. Note, that most
 # of the test coverage is in TestIcebergV2Table.test_read_position_deletes but since it
