@@ -191,8 +191,9 @@ class TestObservability(ImpalaTestSuite):
 
   def test_profile(self):
     """Test that expected fields are populated in the profile."""
-    query = """select count(distinct a.int_col) from functional.alltypes a
-        inner join functional.alltypessmall b on (a.id = b.id + cast(sleep(15) as INT))"""
+    query = """select a.month, sum(a.int_col) from functional.alltypes a join
+        functional.alltypestiny b using (id) where a.year > 2000 and b.year > 2000
+        group by a.month order by a.month"""
     result = self.execute_query(query)
 
     assert "Query Type: QUERY" in result.runtime_profile
@@ -201,7 +202,36 @@ class TestObservability(ImpalaTestSuite):
     tables = re.search(r'\n\s+Tables Queried:\s+(.*?)\n', result.runtime_profile)
     assert tables is not None
     assert sorted(tables.group(1).split(",")) \
-        == ["functional.alltypes", "functional.alltypessmall"]
+        == ["functional.alltypes", "functional.alltypestiny"]
+
+    select_columns = re.search(r'\n\s+Select Columns:\s+(.*?)\n', result.runtime_profile)
+    assert select_columns is not None and select_columns.group(1) \
+        == 'functional.alltypes.int_col,functional.alltypes.month'
+    where_columns = re.search(r'\n\s+Where Columns:\s+(.*?)\n', result.runtime_profile)
+    assert where_columns is not None and where_columns.group(1) \
+        == 'functional.alltypes.year,functional.alltypestiny.year'
+    join_columns = re.search(r'\n\s+Join Columns:\s+(.*?)\n', result.runtime_profile)
+    assert join_columns is not None
+    assert join_columns.group(1) == 'functional.alltypes.id,functional.alltypestiny.id'
+    agg_columns = re.search(r'\n\s+Aggregate Columns:\s+(.*?)\n', result.runtime_profile)
+    assert agg_columns is not None
+    assert agg_columns.group(1) == 'functional.alltypes.month'
+    ordby_columns = re.search(r'\n\s+OrderBy Columns:\s+(.*?)\n', result.runtime_profile)
+    assert ordby_columns is not None
+    assert ordby_columns.group(1) == 'functional.alltypes.month'
+
+  def test_minimal_profile(self):
+    """Test that empty fields are omitted from the profile."""
+    result = self.execute_query('select 1')
+    assert "Query Type: QUERY" in result.runtime_profile
+    assert "Query State: " in result.runtime_profile
+    assert "Default Db: default" in result.runtime_profile
+    assert re.search(r'\n\s+Tables Queried:\s+(.*?)\n', result.runtime_profile) is None
+    assert re.search(r'\n\s+Select Columns:\s+(.*?)\n', result.runtime_profile) is None
+    assert re.search(r'\n\s+Where Columns:\s+(.*?)\n', result.runtime_profile) is None
+    assert re.search(r'\n\s+Join Columns:\s+(.*?)\n', result.runtime_profile) is None
+    assert re.search(r'\n\s+Aggregate Columns:\s+(.*?)\n', result.runtime_profile) is None
+    assert re.search(r'\n\s+OrderBy Columns:\s+(.*?)\n', result.runtime_profile) is None
 
   def test_exec_summary(self):
     """Test that the exec summary is populated correctly in every query state"""
@@ -309,6 +339,7 @@ class TestObservability(ImpalaTestSuite):
       load_event_regexes = [
         r'Frontend:',
         r'Referenced Tables:',
+        r'Original Table Versions:',
         r'CatalogFetch.ColumnStats.Hits',
         r'CatalogFetch.ColumnStats.Misses',
         r'CatalogFetch.ColumnStats.Requests',
@@ -721,7 +752,7 @@ class TestObservability(ImpalaTestSuite):
     self.hs2_client.set_configuration(query_opts)
     result = self.hs2_client.execute("select sleep(2000)",
                                      profile_format=TRuntimeProfileFormat.THRIFT)
-    thrift_profile = result.profile
+    thrift_profile = result.runtime_profile
 
     expected_keys = ["HostCpuUserPercentage", "HostNetworkRx", "HostDiskReadThroughput"]
     for key in expected_keys:
@@ -738,7 +769,7 @@ class TestObservability(ImpalaTestSuite):
     that consume Impala debug webpages."""
     query = "select sleep(5)"
     result = self.hs2_client.execute(query, profile_format=TRuntimeProfileFormat.THRIFT)
-    tree = result.profile
+    tree = result.runtime_profile
 
     # tree.nodes[1] corresponds to ClientRequestState::summary_profile_
     # See be/src/service/client-request-state.[h|cc].
@@ -885,9 +916,11 @@ class TestObservability(ImpalaTestSuite):
     assert "Resizes:" in runtime_profile
     nprobes = re.search('Probes:.*\((\d+)\)', runtime_profile)
     # Probes and travel can be 0. The number can be an integer or float with K.
-    assert nprobes and len(nprobes.groups()) == 1 and nprobes.group(1) >= 0
+    # The number extracted is the number inside parenthesis, which is always
+    # an integer.
+    assert nprobes and len(nprobes.groups()) == 1 and int(nprobes.group(1)) >= 0
     ntravel = re.search('Travel:.*\((\d+)\)', runtime_profile)
-    assert ntravel and len(ntravel.groups()) == 1 and ntravel.group(1) >= 0
+    assert ntravel and len(ntravel.groups()) == 1 and int(ntravel.group(1)) >= 0
 
   def test_query_profle_hashtable(self):
     """Test that the profile for join/aggregate contains hash table related
