@@ -39,10 +39,12 @@ from tests.common.skip import SkipIf
 from tests.common.test_dimensions import (
   create_client_protocol_dimension, create_client_protocol_strict_dimension,
   create_uncompressed_text_dimension, create_single_exec_option_dimension)
+from tests.common.test_result_verifier import error_msg_expected
 from time import sleep, time
 from tests.shell.util import (get_impalad_host_port, assert_var_substitution,
   run_impala_shell_cmd, ImpalaShell, build_shell_env, wait_for_query_state,
-  create_impala_shell_executable_dimension, get_impala_shell_executable)
+  create_impala_shell_executable_dimension, get_impala_shell_executable,
+  stderr_get_first_error_msg)
 from contextlib import closing
 
 
@@ -52,6 +54,19 @@ QUERY_FILE_PATH = os.path.join(os.environ['IMPALA_HOME'], 'tests', 'shell')
 RUSSIAN_CHARS = (u"А, Б, В, Г, Д, Е, Ё, Ж, З, И, Й, К, Л, М, Н, О, П, Р,"
                  u"С, Т, У, Ф, Х, Ц,Ч, Ш, Щ, Ъ, Ы, Ь, Э, Ю, Я")
 
+"""IMPALA-12216 implemented timestamp to be printed in case of any error/warning
+  during query execution, below is an example :
+
+   2024-07-15 12:49:27 [Exception] type=<class 'socket.error'> in FetchResults.
+   2024-07-15 12:49:27 [Warning]  Cancelling Query
+   2024-07-15 12:49:27 [Warning] close session RPC failed: <class 'shell_exceptions.
+   QueryCancelledByShellException'>
+
+  To avoid test flakiness due to timestamp, we would be ignoring timestamp in actual
+  result before asserting with expected result, (YYYY-MM-DD hh:mm:ss ) is of length 20
+"""
+TS_LEN = 20
+
 
 def find_query_option(key, string, strip_brackets=True):
   """
@@ -183,11 +198,11 @@ class TestImpalaShell(ImpalaTestSuite):
   def test_unsecure_message(self, vector):
     if vector.get_value('strict_hs2_protocol'):
       pytest.skip("Strict protocol always runs with LDAP.")
-    results = run_impala_shell_cmd(vector, [], wait_until_connected=False)
+    results = run_impala_shell_cmd(vector, [])
     assert "with no authentication" in results.stderr
 
   def test_fastbinary_warning_message(self, vector):
-    results = run_impala_shell_cmd(vector, [], wait_until_connected=False)
+    results = run_impala_shell_cmd(vector, [])
     # Verify that we don't print any message about fastbinary
     # This doesn't check the full error string, because that could change.
     assert "fastbinary" not in results.stderr
@@ -274,8 +289,11 @@ class TestImpalaShell(ImpalaTestSuite):
     args = ['-q', 'set abort_on_error=true;'
             'select id from functional_parquet.bad_column_metadata t']
     result = run_impala_shell_cmd(vector, args, expect_success=False)
-    assert 'ERROR: Column metadata states there are 11 values, ' in result.stderr
-    assert 'but read 10 values from column id.' in result.stderr
+    assert error_msg_expected(
+      stderr_get_first_error_msg(result.stderr),
+      "Column metadata states there are 11 values, but read 10 values from column id."
+    )
+
 
   def test_completed_query_errors_2(self, vector):
     if vector.get_value('strict_hs2_protocol'):
@@ -284,9 +302,10 @@ class TestImpalaShell(ImpalaTestSuite):
             'select id, cnt from functional_parquet.bad_column_metadata t, '
             '(select 1 cnt) u']
     result = run_impala_shell_cmd(vector, args, expect_success=False)
-    assert 'ERROR: Column metadata states there are 11 values, ' in result.stderr,\
-        result.stderr
-    assert 'but read 10 values from column id.' in result.stderr, result.stderr
+    assert error_msg_expected(
+      stderr_get_first_error_msg(result.stderr),
+      "Column metadata states there are 11 values, but read 10 values from column id."
+    )
 
   def test_no_warnings_in_log_with_quiet_mode(self, vector):
     if vector.get_value('strict_hs2_protocol'):
@@ -408,11 +427,20 @@ class TestImpalaShell(ImpalaTestSuite):
     args = base_args + ['create table %s.shell_profile_test (id int)' % db]
     create = run_impala_shell_cmd(vector, args)
     assert "Referenced Tables: \n" in create.stdout
+    assert "Original Table Versions: \n" in create.stdout
 
+    TABLE_VERSION = re.compile(
+        r"Original Table Versions: (\w+\.\w+), (\d+), (\d+), ([^\n]*)\n")
     for statement in statements:
       args = base_args + [statement]
       result = run_impala_shell_cmd(vector, args)
       assert "Referenced Tables: %s.shell_profile_test" % unique_database in result.stdout
+      m = TABLE_VERSION.search(result.stdout)
+      assert m, "Original Table Versions not found in profile:\n" + result.stdout
+      assert m.group(1) == unique_database + ".shell_profile_test"
+      assert int(m.group(2)) > 0, "Invalid catalog version in " + m.group(0) + statement
+      assert int(m.group(3)) > 0, "Invalid loaded timestamp in " + m.group(0) + statement
+      assert len(m.group(4)) > 0, "Invalid timestamp string in " + m.group(0) + statement
 
   def test_runtime_profile_multiple_referenced_tables(self, vector, unique_database):
     if vector.get_value('strict_hs2_protocol'):
@@ -448,6 +476,16 @@ class TestImpalaShell(ImpalaTestSuite):
     for i in range(0, 2):
       assert "{db}.shell_profile_test{index}".format(db=db, index=i) in referenced_tables
 
+    TABLE_VERSIONS = re.compile(r"Original Table Versions: (\w+\.\w+), (\d+), (\d+), "
+                                r"([^\n]*)\n(\w+\.\w+), (\d+), (\d+), ([^\n]*)\n")
+    m = TABLE_VERSIONS.search(result.stdout)
+    assert m, "Original Table Versions not found in profile:\n" + result.stdout
+    for i in (0, 4):
+      assert db + ".shell_profile_test" in m.group(i + 1), "missing tables:" + m.group(0)
+      assert int(m.group(i + 2)) > 0, "Invalid catalog version: " + m.group(0)
+      assert int(m.group(i + 3)) > 0, "Invalid timestamp: " + m.group(0)
+      assert len(m.group(i + 4)) > 0, "Invalid timestamp string: " + m.group(0)
+
   def test_summary(self, vector):
     if vector.get_value('strict_hs2_protocol'):
       pytest.skip("Summary not supported in strict hs2 mode.")
@@ -729,8 +767,7 @@ class TestImpalaShell(ImpalaTestSuite):
 
     # Testing config file related warning and error messages
     args = ['--config_file=%s/impalarc_with_warnings' % QUERY_FILE_PATH]
-    result = run_impala_shell_cmd(
-        vector, args, expect_success=True, wait_until_connected=False)
+    result = run_impala_shell_cmd(vector, args, expect_success=True)
     assert "WARNING: Option 'config_file' can be only set from shell." in result.stderr
     err_msg = ("WARNING: Unable to read configuration file correctly. "
                "Ignoring unrecognized config option: 'invalid_option'\n")
@@ -742,21 +779,25 @@ class TestImpalaShell(ImpalaTestSuite):
                "'maybe' is not a valid value for a boolean option.")
     assert  err_msg in result.stderr
 
-    # Test the optional configuration file with live_progress and live_summary
-    # Positive test
-    args = ['--config_file=%s/good_impalarc3' % QUERY_FILE_PATH]
-    result = run_impala_shell_cmd(vector, args)
-    assert 'WARNING:' not in result.stderr, \
-      "A valid config file should not trigger any warning: {0}".format(result.stderr)
-    # Negative Tests
-    # specified config file with live_summary enabled for non interactive mode
-    args = ['--config_file=%s/good_impalarc3' % QUERY_FILE_PATH, '--query=select 3']
-    result = run_impala_shell_cmd(vector, args, expect_success=False)
-    assert 'live_summary is available for interactive mode only' in result.stderr
+    # live_progress and live_summary are not supported with strict_hs2_protocol
+    if not vector.get_value('strict_hs2_protocol'):
+      # Test the optional configuration file with live_progress and live_summary
+      # Positive test
+      args = ['--config_file=%s/good_impalarc3' % QUERY_FILE_PATH]
+      result = run_impala_shell_cmd(vector, args)
+      filtered_stderr = [line for line in result.stderr.splitlines()
+                         if 'WARNING: Unable to load command history' not in line]
+      assert 'WARNING:' not in '\n'.join(filtered_stderr), \
+        "A valid config file should not trigger any warning: {0}".format(result.stderr)
+      # Negative Tests
+      # specified config file with live_summary enabled for non interactive mode
+      args = ['--config_file=%s/good_impalarc3' % QUERY_FILE_PATH, '--query=select 3']
+      result = run_impala_shell_cmd(vector, args, expect_success=False)
+      assert 'live_summary is available for interactive mode only' in result.stderr
+
     # testing config file related warning messages
     args = ['--config_file=%s/impalarc_with_warnings2' % QUERY_FILE_PATH]
-    result = run_impala_shell_cmd(
-      vector, args, expect_success=True, wait_until_connected=False)
+    result = run_impala_shell_cmd(vector, args, expect_success=True)
     err_msg = ("WARNING: Unable to read configuration file correctly. "
                "Ignoring unrecognized config option: 'Live_Progress'\n")
     assert err_msg in result.stderr
@@ -1379,16 +1420,17 @@ class TestImpalaShell(ImpalaTestSuite):
     # assign requested address" for both Python 2 and Python 3.
     # Tolerate all three of these variants.
     error_template = (
-      "Caught exception [Errno {0}] {1}, type=<class '{2}'> in OpenSession. "
-      "Num remaining tries: 3")
+      "[Exception] type=<class '{0}'> in OpenSession. Num remaining tries: 3 "
+      "[Errno {1}] {2}")
     expected_err_py2 = \
-        error_template.format(115, "Operation now in progress", "socket.error")
+        error_template.format("socket.error", 115, "Operation now in progress")
     expected_err_py3 = \
-        error_template.format(115, "Operation now in progress", "BlockingIOError")
+        error_template.format("BlockingIOError", 115, "Operation now in progress")
     expected_err_docker = \
-        error_template.format(99, "Cannot assign requested address", "OSError")
+        error_template.format("OSError", 99, "Cannot assign requested address")
     actual_err = result.stderr.splitlines()[0]
-    assert actual_err in [expected_err_py2, expected_err_py3, expected_err_docker]
+    assert actual_err[TS_LEN:] in [expected_err_py2, expected_err_py3,
+                                   expected_err_docker]
 
     # Test http_socket_timeout_s=-1, expect errors
     result = run_impala_shell_cmd(vector, args + ['--http_socket_timeout_s=-1'],
@@ -1544,6 +1586,36 @@ class TestImpalaShell(ImpalaTestSuite):
 
     assert expected_py2 in result.stdout or expected_py3 in result.stdout
 
+  def test_hs2_x_forward(self, vector):
+    """Test that when we use the --hs2_x_forward flag with hs2-http
+    protocol then the X-Forwarded-For http header is set, and this
+    value is reported in the 'Http Origin' field of the profile."""
+    if vector.get_value('strict_hs2_protocol'):
+      pytest.skip("Skip as no profile with strict protocol.")
+    expect_header = vector.get_value('protocol') == 'hs2-http'
+    args = ['-q', 'select 1;profile;',
+            '--hs2_x_forward=a_forwarded_origin_header b_forwarded_origin_header']
+    result = run_impala_shell_cmd(vector, args)
+    found_http_origin = ("Http Origin: a_forwarded_origin_header "
+                         "b_forwarded_origin_header") in result.stdout
+    assert found_http_origin == expect_header
+
+  def test_hs2_x_forward_long(self, vector):
+    """Test that when we use the --hs2_x_forward flag with hs2-http
+    protocol then when passing a very long string, the 'Http Origin'
+    field of the profile is truncated."""
+    if vector.get_value('strict_hs2_protocol'):
+      pytest.skip("Skip as no profile with strict protocol.")
+    expect_header = vector.get_value('protocol') == 'hs2-http'
+    long_string = 'a' * 10000
+    expected_string = 'a' * 8096
+    args = ['-q', 'select 1;profile;',
+            '--hs2_x_forward={0}'.format(long_string)]
+    result = run_impala_shell_cmd(vector, args)
+    found_http_origin = ("Http Origin: {0}".format(expected_string)) in result.stdout
+    assert found_http_origin == expect_header
+    assert long_string not in result.stdout
+
   def test_output_rpc_to_screen_and_file(self, vector, populated_table, tmp_file):
     """Tests the flags that output hs2 rpc call details to both stdout
     and a file.  Asserts the expected text is written."""
