@@ -46,9 +46,11 @@ from tests.common.skip import SkipIfLocal
 from tests.common.test_dimensions import (
   create_client_protocol_dimension, create_client_protocol_strict_dimension,
   create_uncompressed_text_dimension, create_single_exec_option_dimension)
+from tests.common.test_result_verifier import error_msg_expected
 from tests.shell.util import (assert_var_substitution, ImpalaShell, get_impalad_port,
   get_shell_cmd, get_open_sessions_metric, spawn_shell, get_unused_port,
-  create_impala_shell_executable_dimension, get_impala_shell_executable)
+  create_impala_shell_executable_dimension, get_impala_shell_executable,
+  stderr_get_first_error_msg)
 
 QUERY_FILE_PATH = os.path.join(os.environ['IMPALA_HOME'], 'tests', 'shell')
 
@@ -346,6 +348,25 @@ class TestImpalaShellInteractive(ImpalaTestSuite):
     result = p.get_result()
     assert "^C" in result.stderr
 
+  def test_sigusr1_stacktraces(self, vector):
+    if vector.get_value('strict_hs2_protocol'):
+      pytest.skip("Strict HS2 mode doesn't support sleep() function")
+    if vector.get_value('impala_shell') in ['dev', 'python2']:
+      pytest.skip("Python 2 doesn't support faulthandler")
+    command = "select sleep(5000); quit;"
+    p = ImpalaShell(vector)
+    p.send_cmd(command)
+    sleep(2)
+    os.kill(p.pid(), signal.SIGUSR1)
+    result = p.get_result()
+    # The stacktrace is printed to stderr. The main thread starts in impala_shell_main,
+    # so we expect that to be present.
+    assert "Current thread" in result.stderr, result.stderr
+    assert "impala_shell_main" in result.stderr, result.stderr
+    # The SIGUSR1 doesn't harm the running query.
+    assert "Fetched 1 row(s)" in result.stderr, result.stderr
+    assert result.rc == 0, result.stderr
+
   @pytest.mark.execute_serially
   def test_cancellation_mid_command(self, vector):
     """Test that keyboard interrupt cancels multiline query strings"""
@@ -1142,8 +1163,10 @@ class TestImpalaShellInteractive(ImpalaTestSuite):
       assert "ParseException" in result.stderr,\
              result.stderr
     else:
-      assert "ERROR: ParseException: Unmatched string literal" in result.stderr,\
-             result.stderr
+      assert error_msg_expected(
+          stderr_get_first_error_msg(result.stderr),
+          "ParseException: Unmatched string literal"
+      )
 
   def test_utf8_error_message(self, vector):
     if vector.get_value('strict_hs2_protocol'):
@@ -1153,8 +1176,11 @@ class TestImpalaShellInteractive(ImpalaTestSuite):
     query = "select cast(now() as string format 'yyyy年MM月dd日')"
     shell.send_cmd(query)
     result = shell.get_result()
-    assert "ERROR: Bad date/time conversion format: yyyy年MM月dd日" in result.stderr,\
-           result.stderr
+    assert error_msg_expected(
+        stderr_get_first_error_msg(result.stderr),
+        "Bad date/time conversion format: yyyy年MM月dd日"
+    )
+
 
   def test_timezone_validation(self, vector):
     """Test that query option TIMEZONE is validated when executing a query.
