@@ -122,15 +122,14 @@ def assert_pattern(pattern, result, text, message):
 
 
 def run_impala_shell_cmd(vector, shell_args, env=None, expect_success=True,
-                         stdin_input=None, wait_until_connected=True,
-                         stdout_file=None, stderr_file=None):
+                         stdin_input=None, stdout_file=None, stderr_file=None):
   """Runs the Impala shell on the commandline.
 
   'shell_args' is a string which represents the commandline options.
   Returns a ImpalaShellResult.
   """
   result = run_impala_shell_cmd_no_expect(vector, shell_args, env, stdin_input,
-                                          expect_success and wait_until_connected,
+                                          wait_until_connected=expect_success,
                                           stdout_file=stdout_file,
                                           stderr_file=stderr_file)
   if expect_success:
@@ -151,7 +150,11 @@ def run_impala_shell_cmd_no_expect(vector, shell_args, env=None, stdin_input=Non
 
   Does not assert based on success or failure of command.
   """
-  p = ImpalaShell(vector, shell_args, env=env, wait_until_connected=wait_until_connected,
+  # Only wait_until_connected if we also have input to send. Otherwise expect that
+  # the command will run without input and exit. Avoid waiting as ImpalaShell may
+  # exit before it can detect there is a successful connection and assert False.
+  p = ImpalaShell(vector, shell_args, env=env,
+                  wait_until_connected=wait_until_connected and stdin_input,
                   stdout_file=stdout_file, stderr_file=stderr_file)
   result = p.get_result(stdin_input)
   return result
@@ -318,7 +321,7 @@ class ImpalaShell(object):
     stdout_arg = stdout_file if stdout_file is not None else PIPE
     stderr_arg = stderr_file if stderr_file is not None else PIPE
     return Popen(cmd, shell=False, stdout=stdout_arg, stdin=PIPE, stderr=stderr_arg,
-                 env=build_shell_env(env))
+                 universal_newlines=True, env=build_shell_env(env))
 
 
 def get_unused_port():
@@ -397,3 +400,9 @@ def get_impala_shell_executable(vector):
     'python2': [os.path.join(IMPALA_HOME, 'shell/build/python2_venv/bin/impala-shell')],
     'python3': [os.path.join(IMPALA_HOME, 'shell/build/python3_venv/bin/impala-shell')]
   }[vector.get_value_with_default('impala_shell', 'dev')]
+
+
+def stderr_get_first_error_msg(stderr):
+  """Seek to the begining of the first error message in stderr of impala-shell."""
+  PROMPT = "ERROR: "
+  return stderr[(stderr.index(PROMPT) + len(PROMPT)):]
