@@ -115,7 +115,7 @@ def parse_table_constraints(constraints_file):
   if not os.path.isfile(constraints_file):
     LOG.info('No schema constraints file file found')
   else:
-    with open(constraints_file, 'rb') as constraints_file:
+    with open(constraints_file, 'r') as constraints_file:
       for line in constraints_file.readlines():
         line = line.strip()
         if not line or line.startswith('#'):
@@ -164,7 +164,10 @@ def parse_test_file(test_file_name, valid_section_names, skip_unknown_sections=T
   """
   with open(test_file_name, 'rb') as test_file:
     file_data = test_file.read()
-    if encoding: file_data = file_data.decode(encoding)
+    if encoding:
+      file_data = file_data.decode(encoding)
+    else:
+      file_data = file_data.decode('utf-8')
     if os.environ["USE_APACHE_HIVE"] == "true":
       # Remove Hive 4.0 feature for tpcds_schema_template.sql
       if "tpcds_schema_template" in test_file_name:
@@ -364,6 +367,7 @@ def write_test_file(test_file_name, test_file_sections, encoding=None):
           test_file_text.append(section_value)
     test_file_text.append(SECTION_DELIMITER)
     test_file.write(('\n').join(test_file_text))
+    test_file.write('\n')  # end file with a newline
 
 
 def load_tpc_queries(workload, include_stress_queries=False, query_name_filters=[]):
@@ -393,7 +397,7 @@ def load_tpc_queries(workload, include_stress_queries=False, query_name_filters=
   # find. Both workload directories contain other queries that are not part of the TPC
   # spec.
   file_workload = workload
-  if workload == "tpcds":
+  if workload in ["tpcds", "tpcds_partitioned"]:
     # TPCDS is assumed to always use decimal_v2, which is the default since 3.0
     file_workload = "tpcds-decimal_v2"
   if include_stress_queries:
@@ -401,7 +405,11 @@ def load_tpc_queries(workload, include_stress_queries=False, query_name_filters=
   else:
     file_name_pattern = re.compile(r"^{0}-(q.*).test$".format(file_workload))
 
-  query_name_pattern = re.compile(r"^{0}-(.*)$".format(workload.upper()))
+  query_name_prefix = workload.upper()
+  if workload == "tpcds_partitioned":
+    # 'tpcds_partitioned' symlink to the same queries set of 'tpcds'.
+    query_name_prefix = "TPCDS"
+  query_name_pattern = re.compile(r"^{0}-(.*)$".format(query_name_prefix))
   if workload == "tpch_nested":
     query_name_pattern = re.compile(r"^TPCH-(.*)$")
 
@@ -439,3 +447,56 @@ def load_tpc_queries(workload, include_stress_queries=False, query_name_filters=
       raise Exception("Expected exactly 1 query to be in file %s but got %s"
           % (file_path, len(test_cases)))
   return queries
+
+
+def tpc_sort_key(s):
+    """
+    Sorting key for sorting strings in the format "tpch-qxX-yY".
+    The valid format is expected to be split into parts by '-'.
+    If the format doesn't match, 0 or an empty string will be returned
+    as appropriate.
+    Args:
+        s (str): Input string to be sorted.
+    Returns:
+        tuple: A tuple of (int, int, str, str):
+            - The first integer is the value of "x", if present, otherwise 0.
+            - The second integer is the value of "y",
+              included only if "x" is present, otherwise returns an 0.
+            - The third element is the character "X", not numeric,
+              included only if "x" is present, otherwise returns an empty string ('').
+            - The fourth element is the character "Y", not numeric,
+              included only if "y" is present, otherwise returns an empty string ('').
+    """
+    x, y = 0, 0
+    x_char, y_char = '', ''
+    parts = s.split("-")
+    if len(parts) < 2:
+      return x, y, x_char, y_char
+    match = re.search(r"^q(\d+)(\D)?", parts[1])
+    if match:
+      x = int(match.group(1)) if match.group(1) else 0
+      x_char = match.group(2) if match.group(2) else ''
+    if len(parts) == 3 and match:
+      match = re.search(r"^(\d+)(\D)?", parts[2])
+      if match:
+        y = int(match.group(1)) if match.group(1) else 0
+        y_char = match.group(2) if match.group(2) else ''
+    return x, y, x_char, y_char
+
+
+def load_tpc_queries_name_sorted(workload):
+    """
+    Returns a list of queries for the given workload. Only tpch and tpcds are supported,
+    and only the name of the queries will be returned. The names are be sorted and
+    converted to lowercase before being returned.
+    Args:
+        workload (str): tpch or tpcds.
+    """
+    queries = list(load_tpc_queries(workload).keys())
+    queries = [s.lower() for s in queries]
+    queries = sorted(queries, key=tpc_sort_key)
+    if workload == 'tpcds':
+      # TPCDS is assumed to always use decimal_v2, in alignment with load_tpc_queries()
+      substring = "tpcds-decimal_v2"
+      queries = [x.replace(workload, substring) for x in queries]
+    return queries
