@@ -24,6 +24,7 @@ from tests.common.impala_test_suite import ImpalaTestSuite
 from tests.util.parse_util import parse_duration_string_ms
 from datetime import datetime
 from multiprocessing import Process, Queue
+from prometheus_client.parser import text_string_to_metric_families
 from time import sleep, time
 import itertools
 import json
@@ -506,7 +507,7 @@ class TestWebPage(ImpalaTestSuite):
 
   def test_query_stmt(self):
     """Create a long select query then check if it is truncated in the response json."""
-    # The imput query is a select + 450 "x " long, which is long enough to get truncated.
+    # The input query is a select + 450 "x " long, which is long enough to get truncated.
     query = "select \"{0}\"".format("x " * 450)
     # The expected result query should be 253 long and contains the first 250
     # chars + "..."
@@ -515,7 +516,7 @@ class TestWebPage(ImpalaTestSuite):
     response_json = self.__run_query_and_get_debug_page(
       query, self.QUERIES_URL, expected_state=self.client.QUERY_STATES["FINISHED"])
     # Search the json for the expected value.
-    # The query can be in in_filght_queries even though it is in FINISHED state.
+    # The query can be in in_flight_queries even though it is in FINISHED state.
     for json_part in itertools.chain(
       response_json['completed_queries'], response_json['in_flight_queries']):
       if expected_result in json_part['stmt']:
@@ -842,7 +843,7 @@ class TestWebPage(ImpalaTestSuite):
         # the query is in the file.
         responses = self.get_and_check_status(
             self.ROOT_URL + download_link, query, self.IMPALAD_TEST_PORT)
-        # Check the query id is in the content of the reponse.
+        # Check the query id is in the content of the response.
         assert len(responses) == 1
         assert query_id in responses[0].text
       elif profile_format == 'Json':
@@ -870,6 +871,13 @@ class TestWebPage(ImpalaTestSuite):
     # check if metric shows up
     assert 'impala_statestore_subscriber_heartbeat_interval_time_min' in resp[0].text
 
+    for response in resp:
+      # Parse Prometheus text format using prometheus_client library.
+      for _ in text_string_to_metric_families(response.text):
+        # We have to loop through the result to force the lazy parsing function
+        # to parse every line.
+        continue
+
   def test_healthz_endpoint(self):
     """Test to check that the /healthz endpoint returns 200 OK."""
     for port in self.TEST_PORTS_WITH_SS:
@@ -882,11 +890,11 @@ class TestWebPage(ImpalaTestSuite):
     """Checks that the template files conform to the requirements for compatibility with
     the Apache Knox service definition."""
     # Matches all 'a' links with an 'href' that doesn't start with either '#' (which stays
-    # on the same page an so doesn't need to the hostname) or '{{ __common__.host-url }}'
+    # on the same page and so doesn't need to the hostname) or '{{ __common__.host-url }}'
     # Note that if we ever need to add a link that doesn't conform to this, we will
     # probably also have to change the Knox service definition.
     href_regex = "<(a|link) .*? href=['\"](?!({{ __common__.host-url }})|#)"
-    # Matches all 'script' tags that aren't absoluve urls.
+    # Matches all 'script' tags that aren't absolute urls.
     script_regex = "<script .*?src=['\"](?!({{ __common__.host-url }})|http)"
     # Matches all 'form' tags that are not followed by including the hidden inputs.
     form_regex = "<form [^{]*?>(?!{{>www/form-hidden-inputs.tmpl}})"
@@ -894,8 +902,14 @@ class TestWebPage(ImpalaTestSuite):
     javascript_regex = "open\(['\"]GET['\"], (?!make_url)"
     # Matches urls in json parameters passed to DataTables.
     datatables_regex = "url: ['\"](?!make_url)"
-    regex = "(%s)|(%s)|(%s)|(%s)|(%s)" % \
-        (href_regex, script_regex, form_regex, javascript_regex, datatables_regex)
+    # Matches all references of paths that contain '/www/' but are not fully qualified.
+    path_regex = r"((?<!{{ __common__.host-url }})(?<!make_url\(\")/www/.*)"
+    # Matches all links with an 'href' attribute that doesn't start with 'URL', or '`$',
+    # or '`{{ __common__.host-url }}'.
+    link_regex = r"(.*link.*\.href[\s]=[\s](?!(`{{ __common__.host-url }})|(`\$)|URL))"
+    regex = "(%s)|(%s)|(%s)|(%s)|(%s)|(%s)|(%s)" % \
+        (href_regex, script_regex, form_regex, javascript_regex, datatables_regex,
+         path_regex, link_regex)
     results = grep_dir(os.path.join(os.environ['IMPALA_HOME'], "www"), regex, ".*\.tmpl")
     assert len(results) == 0, \
         "All links on the webui must include the webserver host: %s" % results
@@ -1023,8 +1037,8 @@ class TestWebPage(ImpalaTestSuite):
     query = "select count(*) from functional_parquet.alltypes"
     delay_created_action = "impalad_load_tables_delay:SLEEP@1000"
 
-    response_json = self.get_queries()
-    assert response_json['num_in_flight_queries'] == 0
+    response_json = self.try_until("test baseline", self.get_queries,
+        lambda resp: resp['num_in_flight_queries'] == 0)
 
     # Start the query completely async. The server doesn't return a response until
     # the query has exited the CREATED state, so we need to get the query ID another way.
@@ -1061,7 +1075,6 @@ class TestWebPage(ImpalaTestSuite):
     # environments like ASAN.
     response_json = self.try_until("query cancellation", self.get_queries,
         lambda resp: resp['num_in_flight_queries'] == 0)
-    assert response_json['num_in_flight_queries'] == 0
     assert response_json['num_waiting_queries'] == 0
 
     expected_queries = [q for q in response_json['completed_queries']
@@ -1076,8 +1089,8 @@ class TestWebPage(ImpalaTestSuite):
     query = "select *, 1.0/0 from functional_parquet.alltypes limit 10"
     delay_created_action = "impalad_load_tables_delay:SLEEP@1000"
 
-    response_json = self.get_queries()
-    assert response_json['num_in_flight_queries'] == 0
+    response_json = self.try_until("test baseline", self.get_queries,
+        lambda resp: resp['num_in_flight_queries'] == 0)
 
     def run(queue, client, query):
       queue.put(client.execute_async(query))
@@ -1111,9 +1124,10 @@ class TestWebPage(ImpalaTestSuite):
     except Exception as e:
       re.match("UDF ERROR: Cannot divide decimal by zero", str(e))
 
-    # Failed query should be completed.
-    response_json = self.get_queries()
-    assert response_json['num_in_flight_queries'] == 0
+    # Cancel and fetch requests can return before cancellation is finalized. Retry for
+    # slow environments like ASAN.
+    response_json = self.try_until("query failure", self.get_queries,
+        lambda resp: resp['num_in_flight_queries'] == 0)
     assert response_json['num_waiting_queries'] == 0
 
     expected_queries = [q for q in response_json['completed_queries']
@@ -1122,7 +1136,7 @@ class TestWebPage(ImpalaTestSuite):
 
   @pytest.mark.execute_serially
   def test_hadoop_varz_page(self):
-    """test for /hadoop-var to check availablity of haqoop configuration like
+    """test for /hadoop-var to check availability of hadoop configuration like
     hive warehouse dir, fs.defaultFS"""
     responses = self.get_and_check_status(self.HADOOP_VARZ_URL,
         "hive.metastore.warehouse.dir", ports_to_test=self.TEST_PORTS_WITHOUT_SS)
