@@ -16,7 +16,7 @@
 // under the License.
 
 import {profile, set_maxts, maxts, decimals, set_decimals, diagram_width,
-    set_diagram_width, diagram_controls_height, diagram_min_height,
+    set_diagram_width, diagram_min_height,
     margin_header_footer, border_stroke_width, margin_chart_end, clearDOMChildren,
     resizeHorizontalAll} from "./global_members.js";
 import {host_utilization_chart, getUtilizationWrapperHeight}
@@ -37,7 +37,9 @@ var rownum;
 var row_height = 15;
 var integer_part_estimate = 4;
 var char_width = 6;
+var bucket_size = 5;
 var fragment_events_parse_successful = false;
+var decimals_multiplier = Math.pow(10, decimals);
 
 // #phases_header
 var phases = [
@@ -75,32 +77,41 @@ function removeChildIfExists(parentElement, childElement) {
   }
 }
 
-function getSvgRect(fill_color, x, y, width, height, dash, stroke_color) {
+function rtd(num) { // round to decimals
+  return Math.round(num * decimals_multiplier) / decimals_multiplier;
+}
+
+// 'dasharray' and 'stroke_color' are optional parameters
+function getSvgRect(fill_color, x, y, width, height, dasharray, stroke_color) {
   var rect = document.createElementNS("http://www.w3.org/2000/svg", "rect");
-  rect.setAttribute("x", `${x}px`);
-  rect.setAttribute("y", `${y}px`);
-  rect.setAttribute("width", `${width}px`);
-  rect.setAttribute("height", `${height}px`);
+  rect.setAttribute("x", rtd(x));
+  rect.setAttribute("y", rtd(y));
+  rect.setAttribute("width", rtd(width));
+  rect.setAttribute("height", rtd(height));
   rect.setAttribute("fill", fill_color);
-  rect.setAttribute("stroke", stroke_color);
-  if (dash) {
-    rect.setAttribute("stroke-dasharray", "2 2");
+  rect.setAttribute("stroke-width", 0.5);
+  if (stroke_color) {
+    rect.setAttribute("stroke", stroke_color);
+  }
+  if (dasharray) {
+    rect.setAttribute("stroke-dasharray", dasharray);
   }
   return rect;
 }
 
-function getSvgText(text, fill_color, x, y, height, container_center, max_width = 0) {
+// 'max_width' is an optional parameters
+function getSvgText(text, fill_color, x, y, height, container_center, max_width) {
   var text_el = document.createElementNS("http://www.w3.org/2000/svg", "text");
   text_el.appendChild(document.createTextNode(text));
-  text_el.setAttribute("x", `${x}px`);
-  text_el.setAttribute("y", `${y}px`);
+  text_el.setAttribute("x", x);
+  text_el.setAttribute("y", y);
   text_el.style.fontSize = `${height / 1.5}px`;
   if (container_center) {
     text_el.setAttribute("dominant-baseline", "middle");
     text_el.setAttribute("text-anchor", "middle");
   }
   text_el.setAttribute("fill", fill_color);
-  if (max_width != 0) {
+  if (max_width) {
     text_el.setAttribute("textLength", max_width);
     text_el.setAttribute("lengthAdjust", "spacingAndGlyphs");
   }
@@ -109,10 +120,10 @@ function getSvgText(text, fill_color, x, y, height, container_center, max_width
 
 function getSvgLine(stroke_color, x1, y1, x2, y2, dash) {
   var line = document.createElementNS("http://www.w3.org/2000/svg", "line");
-  line.setAttribute("x1", `${x1}px`);
-  line.setAttribute("y1", `${y1}px`);
-  line.setAttribute("x2", `${x2}px`);
-  line.setAttribute("y2", `${y2}px`);
+  line.setAttribute("x1", rtd(x1));
+  line.setAttribute("y1", rtd(y1));
+  line.setAttribute("x2", rtd(x2));
+  line.setAttribute("y2", rtd(y2));
   line.setAttribute("stroke", stroke_color);
   if (dash) {
     line.setAttribute("stroke-dasharray", "2 2");
@@ -131,38 +142,153 @@ function getSvgGroup() {
   return group;
 }
 
+function markMinMaxAvg(event) {
+  if (event.no_bar !== undefined) return;
+  var min = event.ts_list[0];
+  var max = min;
+  var i = 1;
+  for (; i < event.ts_list.length; i++) {
+    if (min > event.ts_list[i]) min = event.ts_list[i];
+    if (max < event.ts_list[i]) max = event.ts_list[i];
+  }
+  var event_span_t = max - min;
+  event.parts = Array(bucket_size).fill(null).map(
+      () => ({count : 0, min : Infinity, max : -Infinity, avg : 0}));
+  var k;
+  var ts;
+  var part;
+  for (i = 0; i < event.ts_list.length; i++) {
+    ts = event.ts_list[i];
+    k = ((ts - min) * bucket_size / event_span_t) | 0;
+    if (k >= bucket_size) k = bucket_size - 1;
+    part = event.parts[k];
+    if (ts < part.min) part.min = ts;
+    if (ts > part.max) part.max = ts;
+    part.avg += ts;
+    part.count += 1;
+  }
+  for (k = 0; k < bucket_size; k++) {
+    event.parts[k].avg /= event.parts[k].count;
+  }
+}
+
+function getPhaseDasharray(dx, dy, l_dx) {
+  return `0 ${rtd(l_dx)} ${rtd(dx - l_dx + dy)} ${rtd(dx)} ${rtd(dy)} 0`;
+}
+
+function attachBucketedPhaseTooltip(rect, part) {
+  rect.appendChild(getSvgTitle(`# of Inst. : ${part.count}\n`
+    + `Min. : ${rtd(part.min / 1e9)}s\n`
+    + `Max. : ${rtd(part.max / 1e9)}s\n`
+    + `Avg. : ${rtd(part.avg / 1e9)}s`));
+  return rect;
+}
+
 function DrawBars(svg, rownum, row_height, events, xoffset, px_per_ns) {
-  var color_idx = 0;
-  var last_end = xoffset;
+  // Structure of each 'events' element -
+  // {
+  //    no_bar : < Boolean value is set, if event is not to be rendered >
+  //    ts_list : < List of this plan node's event's timestamps from all instances >
+  //    // A total of 5 buckets represented as 'parts'
+  //    parts : [{ // Each object contains statistics of a single timestamps bucket
+  //      count : < Number of timestamps in the current bucket >
+  //      avg : < Average of timestamps in the current bucket >
+  //      min : < Minimum timestamp in the current bucket >
+  //      max : < Maximum timestamp in the current bucket >
+  //    }, < ... 4 other objects of the same format > ]
+  // }
   var bar_height = row_height - 2;
+  var last_e_index = events.length - 1;
+  var last_ts_index = events[last_e_index].ts_list.length - 1;
   var plan_node = getSvgGroup();
   plan_node.classList.add("plan_node");
-  events.forEach(function(ev) {
-    if (ev.no_bar == undefined) {
-      var x = last_end;
-      var y = rownum * row_height;
+  // coordinates start with (0,0) in the top-left corner
+  var y = rownum * row_height; // the y-position for the current phase rectangle
+  var cp_y = y; // copy of y, used to bring y-position back to the top(initial value)
+  var dx, dy; // dimensions of the current phase rectangle
+  var bucketed = false;
 
-      var endts = Math.max.apply(null, ev.tslist);
-      var width = xoffset + endts * px_per_ns - last_end;
-      last_end = x + width;
+  if (events[0].ts_list.length > bucket_size) {
+    events.map(markMinMaxAvg);
+    bucketed = true;
+  } else {
+    events.map((ev) => {
+      ev.ts_list.sort((a, b) => a - b);
+    });
+  }
 
-      // Block phase outline
-      plan_node.appendChild(getSvgRect(phases[color_idx].color, x, y, width, bar_height,
-          false, stroke_fill_colors.black));
-      color_idx++;
+  var i;
+  var color_idx = last_e_index;
+  for (i = 0; i <= last_e_index; ++i) {
+    if (events[i].no_bar) --color_idx;
+  }
 
-      // Grey dividers for other instances that finished earlier
-      ev.tslist.forEach(function(ts) {
-        var dx = (endts - ts) * px_per_ns;
-        var ignore_px = 2; // Don't print tiny skews
-        if (Math.abs(dx) > ignore_px) {
-          plan_node.appendChild(getSvgLine(stroke_fill_colors.dark_grey, last_end - dx,
-              y, last_end - dx, y + bar_height, false));
+  var j;
+  var dasharray; // current stroke-dasharray
+  for (var i = last_e_index; i >= 0; --i) {
+    y = cp_y; // reuse calculation, returing to the initial y-position
+    if (events[i].no_bar === undefined) {
+      var l_dx = 0; // previous phase rectangle's width (x-dimension)
+      if (bucketed) {
+        // Represent the aggregate distribution of instances for each phase
+        var part;
+        for (j = 0; j < events[i].parts.length; j++) {
+          part = events[i].parts[j]; // mantain a reference for reuse
+          if (part.count > 0) {
+              dy = (part.count / events[i].ts_list.length) * bar_height;
+          } else {
+            // skip to the next iteration, if the current bucket has no timestamps
+            continue;
+          }
+          dx = part.max * px_per_ns;
+          // 'stroke-dasharray' is a string of alternative lengths for
+          // dashes and no dashes. So, the length of dasharray of must be even.
+          // Width(x-dimension) of previous(upper) phase rectangle is stored to
+          // calculate boundaries such that only open boundaries are stroked with borders
+          // using the 'getPhaseDasharray' function
+          dasharray = getPhaseDasharray(dx, dy, l_dx);
+          plan_node.appendChild(attachBucketedPhaseTooltip(getSvgRect(
+              phases[color_idx].color, xoffset, y, dx, dy, dasharray,
+              stroke_fill_colors.black), part));
+          y += dy;
+          l_dx = dx;
         }
-      });
-      svg.appendChild(plan_node);
+      } else {
+        // Represent phases of each instance seperately, according to the timestamps
+        dy = bar_height / events[i].ts_list.length;
+        for (j = 0; j < events[i].ts_list.length; j++) {
+          dx = events[i].ts_list[j] * px_per_ns;
+          // Calculations for the 'stroke-dasharray' have been done same as above
+          dasharray = getPhaseDasharray(dx, dy, l_dx);
+          plan_node.appendChild(getSvgRect(phases[color_idx].color, xoffset, y, dx, dy,
+              dasharray, stroke_fill_colors.black));
+          y += dy;
+          l_dx = dx;
+        }
+      }
+      --color_idx;
     }
-  });
+  }
+
+  y = rownum * row_height;
+  if (events[last_e_index].no_bar === undefined) {
+    // Plan node's top and bottom outlines
+    var top_edge_ts, bottom_edge_ts;
+    if (bucketed) {
+      top_edge_ts = events[last_e_index].parts[0].max;
+      bottom_edge_ts = events[last_e_index].parts[bucket_size - 1].max;
+    } else {
+      top_edge_ts = events[last_e_index].ts_list[0];
+      bottom_edge_ts = events[last_e_index].ts_list[last_ts_index];
+    }
+    plan_node.appendChild(getSvgLine(stroke_fill_colors.black, xoffset, y,
+        xoffset + top_edge_ts * px_per_ns, y, false));
+    plan_node.appendChild(getSvgLine(stroke_fill_colors.black, xoffset, y + bar_height,
+        xoffset +  bottom_edge_ts * px_per_ns, y + bar_height,
+        false));
+  }
+
+  svg.appendChild(plan_node);
 }
 
 async function renderPhases() {
@@ -174,9 +300,9 @@ async function renderPhases() {
     x = Math.min(x, diagram_width - width);
 
     phases_header.appendChild(getSvgRect(stroke_fill_colors.black, x, 0, width,
-        row_height, false));
+        row_height));
     phases_header.appendChild(getSvgRect(phases[color_idx++].color, x + 1, 1,
-        width - 2, row_height - 2, false));
+        width - 2, row_height - 2));
     phases_header.appendChild(getSvgText(p.label, stroke_fill_colors.black, x + width
         / 2, (row_height + 4) / 2 , row_height, true,
         Math.min(p.label.length * char_width, width / 1.5)));
@@ -187,7 +313,6 @@ async function renderFragmentDiagram() {
   clearDOMChildren(fragment_diagram);
   var px_per_ns = chart_width / maxts;
   var rownum_l = 0;
-  var max_indent = 0;
   var pending_children = 0;
   var pending_senders = 0;
   var text_y = row_height - 4;
@@ -211,27 +336,19 @@ async function renderFragmentDiagram() {
       for (var i = 0; i < fragment.nodes.length; ++i) {
         var node = fragment.nodes[i];
 
-        if (node.events != undefined) {
+        if (node.events !== undefined) {
           // Plan node timing row
           DrawBars(fragment_svg_group, rownum_l, row_height, node.events, name_width,
               px_per_ns);
 
-          fragment_svg_group.id = fragment.name;
-          fragment_svg_group.addEventListener('click', updateFragmentMetricsChartOnClick);
-          fragment_diagram.appendChild(fragment_svg_group);
-
-          if (node.type == "HASH_JOIN_NODE") {
+          if (node.type === "HASH_JOIN_NODE") {
             fragment_diagram.appendChild(getSvgText("X", stroke_fill_colors.black,
-                name_width + Math.min.apply(null, node.events[2].tslist) * px_per_ns,
+                name_width + Math.min.apply(null, node.events[2].ts_list) * px_per_ns,
                 text_y, row_height, false));
             fragment_diagram.appendChild(getSvgText("O", stroke_fill_colors.black,
-                name_width + Math.min.apply(null, node.events[2].tslist) * px_per_ns,
+                name_width + Math.min.apply(null, node.events[2].ts_list) * px_per_ns,
                 text_y, row_height, false));
           }
-        } else {
-          fragment_svg_group.id = fragment.name;
-          fragment_svg_group.addEventListener('click', updateFragmentMetricsChartOnClick);
-          fragment_diagram.appendChild(fragment_svg_group);
         }
 
         if (node.is_receiver) {
@@ -249,21 +366,21 @@ async function renderFragmentDiagram() {
         fragment_diagram.appendChild(getSvgText(node.name, fragment.color,
             label_x, text_y, row_height, false));
 
-        if (node.parent_node != undefined) {
+        if (node.parent_node !== undefined) {
           var y = row_height * node.parent_node.rendering.rownum;
           if (node.is_sender) {
-            var x = name_width + Math.min.apply(null, fevents[3].tslist) * px_per_ns;
+            var x = name_width + Math.min.apply(null, fevents[3].ts_list) * px_per_ns;
             // Dotted horizontal connector to received rows
             fragment_diagram.appendChild(getSvgLine(fragment.color, name_width,
                 y + row_height / 2 - 1, x, y + row_height / 2 - 1, true));
 
             // Dotted rectangle for received rows
-            var x2 = name_width + Math.max.apply(null, fevents[4].tslist) * px_per_ns;
+            var x2 = name_width + Math.max.apply(null, fevents[4].ts_list) * px_per_ns;
             fragment_diagram.appendChild(getSvgRect(stroke_fill_colors.transperent,
-                x, y + 4, x2 - x, row_height - 10, true, fragment.color));
+                x, y + 4, x2 - x, row_height - 10, "2 2", fragment.color));
           }
 
-          if (node.is_sender && node.parent_node.rendering.rownum != rownum_l - 1) {
+          if (node.is_sender && node.parent_node.rendering.rownum !== rownum_l - 1) {
             // DAG edge on right side to distant sender
             var x = name_width - (pending_senders) * char_width - char_width / 2;
             fragment_diagram.appendChild(getSvgLine(fragment.color,
@@ -297,10 +414,11 @@ async function renderFragmentDiagram() {
           }
         }
       }
-
+      fragment_svg_group.id = fragment.name;
+      fragment_svg_group.addEventListener('click', updateFragmentMetricsChartOnClick);
+      fragment_diagram.appendChild(fragment_svg_group);
       // Visit sender fragments in reverse order to avoid dag edges crossing
       pending_fragments.reverse().forEach(printFragment);
-
     }
   });
   fragments.forEach(function(fragment) {
@@ -318,9 +436,9 @@ async function renderTimeticks() {
   var timetick_label;
   for (var i = 1; i <= ntics; ++i) {
     timeticks_footer.appendChild(getSvgRect(stroke_fill_colors.black, x, y, px_per_tic,
-        row_height, false));
+        row_height));
     timeticks_footer.appendChild(getSvgRect(stroke_fill_colors.light_grey, x + 1,
-        y + 1, px_per_tic - 2, row_height - 2, false));
+        y + 1, px_per_tic - 2, row_height - 2));
     timetick_label = (i * sec_per_tic).toFixed(decimals);
     timeticks_footer.appendChild(getSvgText(timetick_label, stroke_fill_colors.black,
         x + px_per_tic - timetick_label.length * char_width + 2, text_y, row_height,
@@ -341,11 +459,11 @@ export function collectFragmentEventsFromProfile() {
     var execution_profile = profile.child_profiles[2];
     var execution_profile_name = execution_profile.profile_name.split(" ").slice(0,2)
         .join(" ");
-    console.assert(execution_profile_name == "Execution Profile");
+    console.assert(execution_profile_name === "Execution Profile");
     execution_profile.child_profiles.forEach(function(fp) {
 
-      if (fp.child_profiles != undefined &&
-          fp.child_profiles[0].event_sequences != undefined) {
+      if (fp.child_profiles !== undefined &&
+          fp.child_profiles[0].event_sequences !== undefined) {
         var cp = fp.child_profiles[0];
         var fevents = fp.child_profiles[0].event_sequences[0].events;
 
@@ -355,15 +473,15 @@ export function collectFragmentEventsFromProfile() {
             fevents[en].no_bar = true;
             continue;
           }
-          fevents[en].tslist = [ fevents[en].timestamp ];
+          fevents[en].ts_list = [ fevents[en].timestamp ];
         }
         for (var instance = 1; instance < fp.child_profiles.length; ++instance) {
-          if (fp.child_profiles[instance].event_sequences != undefined) {
+          if (fp.child_profiles[instance].event_sequences !== undefined) {
             if (fp.child_profiles[instance].event_sequences[0].events.length ==
                 fevents.length) {
               for (var en = 0; en < fevents.length; ++en) {
                 if (fevents[en].no_bar) continue;
-                fevents[en].tslist.push(
+                fevents[en].ts_list.push(
                     fp.child_profiles[instance].event_sequences[0].events[en].timestamp);
               }
             } else {
@@ -371,11 +489,11 @@ export function collectFragmentEventsFromProfile() {
               while(i < fevents.length && en < fp.child_profiles[instance]
                   .event_sequences[0].events.length) {
                 if (fevents[i].no_bar) {
-                  if (fevents[i].label == fp.child_profiles[instance]
+                  if (fevents[i].label === fp.child_profiles[instance]
                       .event_sequences[0].events[en].label) { en++; }
                 } else if (fp.child_profiles[instance].event_sequences[0]
-                    .events[en].label == fevents[i].label) {
-                  fevents[i].tslist.push(fp.child_profiles[instance].event_sequences[0]
+                    .events[en].label === fevents[i].label) {
+                  fevents[i].ts_list.push(fp.child_profiles[instance].event_sequences[0]
                       .events[en].timestamp);
                   ++en;
                 }
@@ -399,92 +517,96 @@ export function collectFragmentEventsFromProfile() {
         var node_stack = [];
         var node_name;
         cp.child_profiles.forEach(function get_plan_nodes(pp, index) {
-          if (pp.node_metadata != undefined) {
-            node_path.push(index);
-            var name_flds = pp.profile_name.split(/[()]/);
-            var node_type = name_flds[0].trim();
-            var node_id = name_flds.length > 1 ? name_flds[1].split(/[=]/)[1] : 0;
-            node_name = pp.profile_name.replace("_NODE", "").replace("_", " ")
-                .replace("KrpcDataStreamSender", "SENDER")
-                .replace("Hash Join Builder", "JOIN BUILD")
-                .replace("join node_", "");
-            if (node_type.indexOf("SCAN_NODE") >= 0) {
-              var table_name = pp.info_strings.find(({ key }) => key === "Table Name")
-                  .value.split(/[.]/);
-              node_name = node_name.replace("SCAN",
-                  `SCAN [${table_name[table_name.length - 1]}]`);
-            }
+          if (pp.node_metadata === undefined) return;
+          node_path.push(index);
+          //  pp.profile_name : "AGGREGATION_NODE (id=52)"
+          var name_flds = pp.profile_name.split(/[()]/);
+          //  name_flds : ["AGGREGATION_NODE ", "id=52", ""]
+          var node_type = name_flds[0].trim();
+          //  node_type: "AGGREGATION_NODE"
+          var node_id = name_flds.length > 1 ?
+              parseInt(name_flds[1].split(/[=]/)[1]) : 0;
+          //  node_id: 52
+          node_name = pp.profile_name.replace("_NODE", "").replace("_", " ")
+              .replace("KrpcDataStreamSender", "SENDER")
+              .replace("Hash Join Builder", "JOIN BUILD")
+              .replace("join node_", "");
+          if (node_type.indexOf("SCAN_NODE") >= 0) {
+            var table_name = pp.info_strings.find(({ key }) => key === "Table Name")
+                .value.split(/[.]/);
+            node_name = node_name.replace("SCAN",
+                `SCAN [${table_name[table_name.length - 1]}]`);
+          }
 
-            var is_receiver = node_type == "EXCHANGE_NODE" ||
-                (node_type == "HASH_JOIN_NODE" && pp.num_children < 3);
+          var is_receiver = node_type === "EXCHANGE_NODE" ||
+              (node_type === "HASH_JOIN_NODE" && pp.num_children < 3);
 
-            var is_sender = (node_type == "Hash Join Builder" ||
-                             node_type == "KrpcDataStreamSender");
-            var parent_node;
-            if (node_type == "PLAN_ROOT_SINK") {
-              parent_node = undefined;
-            } else if (pp.node_metadata.data_sink_id != undefined) {
-              parent_node = receiver_nodes[node_id]; // Exchange sender dst
-            } else if (pp.node_metadata.join_build_id != undefined) {
-              parent_node = receiver_nodes[node_id]; // Join sender dst
-            } else if (node_stack.length > 0) {
-              parent_node = node_stack[node_stack.length - 1];
-            } else if (all_nodes.length) {
-              parent_node = all_nodes[all_nodes.length - 1];
-            }
+          var is_sender = (node_type === "Hash Join Builder" ||
+                            node_type === "KrpcDataStreamSender");
+          var parent_node;
+          if (node_type === "PLAN_ROOT_SINK") {}
+          else if (pp.node_metadata.data_sink_id !== undefined) {
+            parent_node = receiver_nodes[node_id]; // Exchange sender dst
+          } else if (pp.node_metadata.join_build_id !== undefined) {
+            parent_node = receiver_nodes[node_id]; // Join sender dst
+          } else if (node_stack.length > 0) {
+            parent_node = node_stack[node_stack.length - 1];
+          } else if (all_nodes.length) {
+            parent_node = all_nodes[all_nodes.length - 1];
+          }
 
-            max_namelen = Math.max(max_namelen, node_name.length + node_stack.length + 1);
+          max_namelen = Math.max(max_namelen, node_name.length + node_stack.length + 1);
 
-            if (pp.event_sequences != undefined) {
-              var node_events = pp.event_sequences[0].events;
+          if (pp.event_sequences !== undefined) {
+            var node_events = pp.event_sequences[0].events;
 
-              // Start the instance event list for each event with timestamps
-              // from this instance
-              for (var en = 0; en < node_events.length; ++en) {
-                node_events[en].tslist = [ node_events[en].timestamp ];
-                if (node_type == "HASH_JOIN_NODE" && (en == 1 || en == 2)) {
-                  node_events[en].no_bar = true;
-                }
+            // Start the instance event list for each event with timestamps
+            // from this instance
+            for (var en = 0; en < node_events.length; ++en) {
+              node_events[en].ts_list = [ node_events[en].timestamp ];
+              if ((node_type === "HASH_JOIN_NODE" ||
+                  node_type === "NESTED_LOOP_JOIN_NODE") && (en === 1 || en === 2)) {
+                node_events[en].no_bar = true;
               }
             }
-            var node = {
-                name: node_name,
-                type: node_type,
-                node_id: node_id,
-                num_children: 0,
-                child_index: 0,
-                metadata: pp.node_metadata,
-                parent_node: parent_node,
-                events: node_events,
-                path: node_path.slice(0),
-                is_receiver: is_receiver,
-                is_sender: is_sender
-            }
-
-            if (is_sender) {
-              node.parent_node.sender_frag_index = fragments.length;
-            }
-
-            if (is_receiver) {
-              receiver_nodes[node_id] = node;
-            }
-
-            if (parent_node != undefined) {
-              node.child_index = parent_node.num_children++;
-            }
-
-            all_nodes.push(node);
-
-            fragment.nodes.push(node);
-
-            if (pp.child_profiles != undefined) {
-              node_stack.push(node);
-              pp.child_profiles.forEach(get_plan_nodes);
-              node = node_stack.pop();
-            }
-            rownum++;
-            node_path.pop();
           }
+          var node = {
+              name: node_name,
+              type: node_type,
+              node_id: node_id,
+              num_children: 0,
+              child_index: 0,
+              metadata: pp.node_metadata,
+              parent_node: parent_node,
+              events: node_events,
+              path: node_path.slice(0),
+              is_receiver: is_receiver,
+              is_sender: is_sender
+          }
+
+          if (is_sender) {
+            node.parent_node.sender_frag_index = fragments.length;
+          }
+
+          if (is_receiver) {
+            receiver_nodes[node_id] = node;
+          }
+
+          if (parent_node !== undefined) {
+            node.child_index = parent_node.num_children++;
+          }
+
+          all_nodes.push(node);
+
+          fragment.nodes.push(node);
+
+          if (pp.child_profiles !== undefined) {
+            node_stack.push(node);
+            pp.child_profiles.forEach(get_plan_nodes);
+            node = node_stack.pop();
+          }
+          rownum++;
+          node_path.pop();
         });
 
         // For each node, retrieve the instance timestamps for the remaining instances
@@ -497,22 +619,21 @@ export function collectFragmentEventsFromProfile() {
             for (var pi = 0; pi < node.path.length; ++pi) {
               cp = cp.child_profiles[node.path[pi]];
             }
-            console.assert(cp.node_metadata.data_sink_id == undefined ||
+            console.assert(cp.node_metadata.data_sink_id === undefined ||
                            cp.profile_name.indexOf(`(dst_id=${node.node_id})`));
-            console.assert(cp.node_metadata.plan_node_id == undefined ||
-                           cp.node_metadata.plan_node_id == node.node_id);
-
+            console.assert(cp.node_metadata.plan_node_id === undefined ||
+                           cp.node_metadata.plan_node_id === node.node_id);
             // Add instance events to this node
-            if (cp.event_sequences != undefined) {
-              if (node.events.length == cp.event_sequences[0].events.length) {
+            if (cp.event_sequences !== undefined) {
+              if (node.events.length === cp.event_sequences[0].events.length) {
                 for (var en = 0; en < node.events.length; ++en) {
-                  node.events[en].tslist.push(cp.event_sequences[0].events[en].timestamp);
+                  node.events[en].ts_list.push(cp.event_sequences[0].events[en].timestamp);
                 }
               } else {
                 var i = 0, en = 0;
                 while(i < node.events.length && en < cp.event_sequences[0].events.length) {
-                  if (node.events[i].label == cp.event_sequences[0].events[en].label) {
-                    node.events[i].tslist.push(cp.event_sequences[0].events[en].timestamp);
+                  if (node.events[i].label === cp.event_sequences[0].events[en].label) {
+                    node.events[i].ts_list.push(cp.event_sequences[0].events[en].timestamp);
                     ++en; ++i;
                   } else {
                     ++i;
@@ -581,7 +702,7 @@ fragment_diagram.addEventListener('mousemove', function(e) {
   if (e.clientX + scrollable_screen.scrollLeft >= name_width && e.clientX + scrollable_screen.scrollLeft <= name_width + chart_width){
     removeChildIfExists(fragment_diagram, timestamp_gridline);
     timestamp_gridline = getSvgLine(stroke_fill_colors.black, e.clientX + scrollable_screen.scrollLeft, 0, e.clientX + scrollable_screen.scrollLeft,
-        parseInt(fragment_diagram.style.height));
+        parseInt(fragment_diagram.style.height), false);
     fragment_diagram.appendChild(timestamp_gridline);
     var gridline_time = ((maxts * (e.clientX + scrollable_screen.scrollLeft - name_width) / chart_width) / 1e9);
     showTooltip(host_utilization_chart, gridline_time);
@@ -627,11 +748,13 @@ timeticks_footer.addEventListener('wheel', function(e) {
       if (e.wheelDelta <= 0) {
         if (decimals <= 1) return;
         set_decimals(decimals - 1);
+        decimals_multiplier /= 10;
       } else {
         var rendering_constraint = char_width * ((decimals + 1) + integer_part_estimate)
             >= chart_width / ntics;
         if (rendering_constraint) return;
         set_decimals(decimals + 1);
+        decimals_multiplier *= 10;
       }
     } else {
       if (e.wheelDelta <= 0 && ntics <= 10) return;
@@ -648,6 +771,6 @@ timeticks_footer.addEventListener('wheel', function(e) {
 
 plan_order.addEventListener('click', renderFragmentDiagram);
 
-if (typeof process != "undefined" && process.env.NODE_ENV === 'test') {
+if (typeof process !== "undefined" && process.env.NODE_ENV === 'test') {
   exportedForTest = {getSvgRect, getSvgLine, getSvgText, getSvgTitle, getSvgGroup};
 }
